<?php
namespace App\Http\Controllers;

use App\General\GeoLoacation;
use App\Models\EmpPersonalDetails;
use App\Models\EmpCompanyDetails;
use App\Models\Sites;
use App\Models\LeaveRequest;
use App\Models\EmployeeAttendance;
use App\Models\EmpTeamsMember;
use App\Models\EmployeeBreak;
use App\Models\RosterTemplate;
use App\Models\RosterAssign;
use App\Models\AttendanceListChangeCol;
use Illuminate\Support\Facades\Response;
use Illuminate\Support\Facades\Validator;
use Illuminate\Support\Facades\Auth;
use Carbon\Carbon;
use DB;
use Illuminate\Support\Facades\Log;

use Illuminate\Http\Request;


class AttendanceController extends Controller
{
    public function __construct()
    {
        $this->middleware('Permissions:Attendance Maintain')->only(['create', 'edit']);
    }

    public function index(Request $request)
    {
        $query = EmployeeAttendance::query();
        $userTable = $this->getUserTable();
        if ($userTable === "customer") {
            $query->whereHas('EmpCompanyDetails', function ($q) {
                $q->where('customer_id', auth()->id())
                    ->where('workspace_id', auth()->user()->current_workspace_id);
            });
        }
        if ($userTable === "emp") {
            $query->whereHas('EmpCompanyDetails', function ($q) {
                $q->where('customer_id', auth()->user()->customer_id)
                    ->where('workspace_id', auth()->user()->workspace_id);
            });
        }
        if (!$query->exists()) {
            return $this->message('Record not found', 404);
        }
    if ($request->filled('filter')) {
            $query = $this->filter(json_decode($request->filter, true), $query);
        }

        $query->with(['EmpPersonalDetails', 'EmpCompanyDetails', 'Sites', 'empTeamsMembers', 'breaks']);
        $start = $request->input('from', 0);
        $query->offset($start)->limit(10);
        $query->orderBy('id', 'desc');
        $query_result = $query->get();
        $array_filter = json_decode($request->filter, true);
        $filters = [
            'first_name' => $array_filter['first_name'] ?? '',
            'middle_name' => $array_filter['middle_name'] ?? '',
            'last_name' => $array_filter['first_name'] ?? '',
            'employee_email' => $array_filter['last_name'] ?? '',
            'compeleted' => $array_filter['first_name'] ?? '',
            'status' => $array_filter['compeleted'] ?? '',
        ];
        return $this->withFilter($query_result, $filters, 'Get Attendance List Successfully');
    }


    public function check_roster(Request $request){
        $validator = Validator::make($request->all(), [
            'employee_id' => 'required|integer',
            'date' => 'required|date_format:Y-m-d',
        ]);

        if ($validator->fails()) {
            return $this->message($validator->errors()->first(), 422);
        }

        $validatedData = $validator->validated();

        $employeeId = $validatedData['employee_id'];
        $date = $validatedData['date'];

        // Validate that the requested employee_id matches the logged-in user's ID
        if (auth()->id() != $employeeId) {
            return $this->message('Unauthorized access to this employee\'s roster.', 403);
        }

        $userTable = $this->getUserTable();

        $employee = null;
        if ($userTable === 'emp') {
            $employee = EmpCompanyDetails::find($employeeId);
        } elseif ($userTable === 'customer') {
            $employee = User::find($employeeId);
        }

        if (!$employee) {
            return $this->message('Employee record not found.', 404);
        }

        $isAuthorized = false;
        if ($userTable === 'emp') {
            if ($employee->customer_id === auth()->user()->customer_id && $employee->workspace_id === auth()->user()->workspace_id) {
                $isAuthorized = true;
            }
        } elseif ($userTable === 'customer') {

            if ($employee->id === auth()->id() && $employee->current_workspace_id === auth()->user()->current_workspace_id) {
                 $isAuthorized = true;
             }
        } else {
             return $this->message('Unauthorized user type.', 403);
        }

        if (!$isAuthorized) {
            return $this->message('You are not authorized to access this employee\'s roster.', 403);
        }


        $roster = RosterAssign::where('assign_to', $employeeId)
                                ->where('schedule_date', $date)
                                ->with('rosterTemplate')
                                ->get();

        if ($roster && $roster->isNotEmpty()) {
            // Filter out entries where rosterTemplate is null after eager loading
            $filteredRosters = $roster->filter(function ($item) {
                return $item->rosterTemplate !== null;
            });

            if ($filteredRosters->isNotEmpty()) {
                 $roster = $filteredRosters->map(function ($item) {
                     return $item->rosterTemplate;
                 });
                 return $this->success(['roster' => $roster], 'Rosters found successfully.');
            } else {
                 return $this->message('No roster templates found for the assigned rosters on the provided date.', 404);
            }

        } else {
            return $this->message('No roster found for this employee on the provided date.', 404);
        }
    }
    public function filter($filters, $query)
    {

        foreach ($filters as $filterName => $filterValue) {
            if ($filterValue != null ||  $filterValue != "") {
                switch ($filterName) {
                    case 'first_name':
                        $filterValue  = explode(" ", $filterValue);

                        if (count($filterValue) == 1) {
                            $query->whereHas('empPersonalDetails', function ($subquery) use ($filterValue) {
                                $subquery->where('first_name', 'like', '%' . $filterValue[0] . '%');
                                $subquery->orWhere('middle_name', 'like', '%' . $filterValue[0] . '%');
                                $subquery->orWhere('last_name', 'like', '%' . $filterValue[0] . '%');
                            });
                        }

                        if (count($filterValue) == 2) {
                            $query->whereHas('empPersonalDetails', function ($subquery) use ($filterValue) {
                                $subquery->where('first_name', 'like', '%' . $filterValue[0] . '%');
                                $subquery->where('middle_name', 'like', '%' . $filterValue[1] . '%');
                                $subquery->where('last_name', 'like', '%' . $filterValue[1] . '%');
                            });
                        }

                        if (count($filterValue) == 3) {

                            $query->whereHas('empPersonalDetails', function ($subquery) use ($filterValue) {
                                $subquery->where('first_name', 'like', '%' . $filterValue[0] . '%');
                                $subquery->where('middle_name', 'like', '%' . $filterValue[1] . '%');
                                $subquery->where('last_name', 'like', '%' . $filterValue[2] . '%');
                            });
                        }

                        break;
                    case 'teams':
                        $emp_teams_members = EmpTeamsMember::where('team_id', $filterValue)->pluck('emp_id')->toArray();
                        $query->whereIn('employee_id', $emp_teams_members);
                        break;
                    case 'user_type':
                        $query->whereHas('empCompanyDetails', function ($subquery) use ($filterValue) {
                            $filterValue == 0 ? 0 : 1;
                            $subquery->where('user_type', 'like', '%' . $filterValue . '%');
                        });
                        break;
                    case 'title':
                        $query->whereHas('sites', function ($subquery) use ($filterValue) {
                            $subquery->where('title', 'like', '%' . $filterValue . '%');
                        });
                        break;
                    case 'Check_in':
                        $query->where('Check_in', 'like', '%' . $filterValue . '%');
                        break;
                    case 'Check_out':
                        $query->where('Check_out', 'like', '%' . $filterValue . '%');
                        break;
                    case 'working_hours':
                        $query->where('working_hours', 'like', '%' . $filterValue . '%');
                        break;
                    case 'date':
                        $filterValue = explode("/", $filterValue);
                        $query->whereBetween('date', [date("Y-m-d", strtotime(trim($filterValue[0], " "))), date("Y-m-d", strtotime(trim($filterValue[1], " ")))]);
                        break;
                    case 'system_checkout':
                        if ($filterValue != 2) {
                            $query->where('checkout_type', $filterValue);
                        }
                        break;
                }
            }
        }

        return $query;
    }
    public function single_employee_attendance(Request $request)
    {
        // Validate the incoming request inputs.
        $request->validate([
            'to'      => 'required|date',
            'from'    => 'required|date|before_or_equal:to',
            'site_id' => 'nullable'
        ]);

        // Retrieve input values.
        $toDate   = $request->input('to');
        $fromDate = $request->input('from');
        $siteid   = $request->input('site_id');

        // Get the authenticated user's employee id.
        $empid = $request->user()->id;
        
        // Retrieve attendance data conditionally based on site_id.
        if ($siteid) {
            $attendance = EmployeeAttendance::where('site_id', $siteid)
                ->where('employee_id', $empid)
                ->whereBetween('date', [$fromDate, $toDate])
                ->with('RosterAssign.RosterTemplate', 'sites')
                ->with('breaks')
                ->orderBy('date', 'desc')
                ->get();
        } else {
            $attendance = EmployeeAttendance::where('employee_id', $empid)
                ->whereBetween('date', [$fromDate, $toDate])
                ->with('RosterAssign.RosterTemplate', 'sites')
                ->with('breaks')
                ->orderBy('date', 'desc')
                ->get();
        }

        // Return the attendance results.
        return $this->success($attendance, 'Get Employee Attendance List Successfully');
    }
    public function create(Request $request)
    {
        $get_employes =  EmpCompanyDetails::with('EmpPersonalDetails')->where('del', 0)->where('compeleted', 1)->orderBy('id', 'DESC')->get();
        $sites = Sites::where('del', 0)->where('active', 1)->get();
        return view('Attendance.create', compact('get_employes', 'sites'));
    }

    public function store(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'employee' => 'required',
            'site' => 'required',
            'check_in' => 'required|date_format:H:i',
            'check_out' => 'nullable|date_format:H:i|after:check_in',
            'date' => 'required|date',
            'working_hours' => 'nullable',
            'break_in' => ['nullable', 'array'],
            'break_out' => ['nullable', 'array'],
            'break_in.*' => ['nullable', 'date_format:H:i'],
            'break_out.*' => ['nullable', 'date_format:H:i', 'after_or_equal:break_in.*'],
        ], [
            'break_out.*.after_or_equal' => 'The break_out must be after or equal to break_in.',
        ]);

        if ($validator->fails()) {
            $error = $validator->errors()->first();
            return $this->message($error, 422);
        }
        $validatedData = $validator->validated();
        // new validation
        $userTable = $this->getUserTable();
        if ($userTable === "customer") {
            $employeeDetails = EmpCompanyDetails::where('id', $validatedData['employee'])
                ->where('customer_id', auth()->id())
                ->where('workspace_id', auth()->user()->current_workspace_id)
                ->first();
            if (!$employeeDetails) {
                return $this->message('Unauthorized access to this employee.', 403);
            }
        }
        if ($userTable === "emp") {
            $employeeDetails = EmpCompanyDetails::where('id', $validatedData['employee'])
                ->where('customer_id', auth()->user()->customer_id)
                ->where('workspace_id', auth()->user()->workspace_id)
                ->first();
            if (!$employeeDetails) {
                return $this->message('Unauthorized access to this employee.', 403);
            }
        }
        // new validation end
        $roster_assigns =   DB::table('roster_assigns')
            // ->join('roster_templates', 'roster_assigns.roster_template_id', '=', 'roster_templates.id')
            ->where('assign_to', $validatedData['employee'])
            ->where('schedule_date', $validatedData['date'])
            ->count();
        if ($roster_assigns == 0) {
            return $this->message('Roaster not found', 404);
        }
        $check_leave = LeaveRequest::where('employee_id', $request->employee)->where('status', 1)->where('from', ">=", date('Y-m-d'))
            ->where('to', "<=", date('Y-m-d'))->exists();
        if ($check_leave) {
            return $this->message('Your are on leave. You are not allowed to check-in', 422);
        }
        // Calculate total working hours
        $checkIn = Carbon::parse($validatedData['check_in']);
        $checkOut = Carbon::parse($validatedData['check_out']);
        $totalHours = $checkOut->diffInMinutes($checkIn);
        // Calculate total breaks
        $totalBreaks = 0;
        $totalWorkingHours = 0;
        if (isset($validatedData['break_in'])) {
            foreach ($validatedData['break_in'] as $key => $breakIn) {
                if (isset($validatedData['break_out'][$key])) {
                    $breakInTime = Carbon::parse($breakIn);
                    $breakOutTime = Carbon::parse($validatedData['break_out'][$key]);
                    $totalBreaks += $breakOutTime->diffInMinutes($breakInTime);
                }
            }
        }
        $customerId = $userTable == "customer" ? auth()->id() : $employeeDetails->customer_id;

        // Calculate total working hours with breaks
        $totalWorkingHours = $totalHours - $totalBreaks;
        $insertedRecord = EmployeeAttendance::create([
            'employee_id' => $validatedData['employee'],
            'customer_id' => $customerId,
            'workspace_id' => $employeeDetails->workspace_id,
            'site_id' => $validatedData['site'],
            'check_in' => $validatedData['check_in'],
            'working_hours' => $validatedData['check_out'] ? $totalWorkingHours : '',
            'check_out' => $validatedData['check_out'],
            'date' => $validatedData['date'],
            'added_by' => $userTable == "customer" ? 0 : auth()->id(),
        ]);
        $id = $insertedRecord->id;
        if (isset($validatedData['break_in'])) {
            if (is_array($validatedData['break_in'])) {
                foreach ($validatedData['break_in'] as $key => $breakIn) {
                    // Skip if either break_in or break_out is null
                    if ($breakIn !== null && isset($validatedData['break_out'][$key])) {
                        $breakOut = $validatedData['break_out'][$key];

                        EmployeeBreak::create([
                            'emp_attendance_id' => $id,
                            'break_in' => $breakIn,
                            'break_out' => $breakOut,
                        ]);
                    }
                }
            } else {
                // Handle the case where 'break_in' and 'break_out' are single values
                $breakOut = isset($validatedData['break_out'][0]) ? $validatedData['break_out'][0] : null;
                // Skip if either break_in or break_out is null
                if ($validatedData['break_in'][0] !== null) {
                    EmployeeBreak::create([
                        'emp_attendance_id' => $id,
                        'break_in' => $validatedData['break_in'][0],
                        'break_out' => $breakOut,
                    ]);
                }
            }
        }
        return $this->success($insertedRecord, 'Attendance Saved Successfully');
    }

    public function getEmployeeAttendance($id)
    {
        $get_employes =  EmpCompanyDetails::with('EmpPersonalDetails')->where('del', 0)->where('compeleted', 1)->orderBy('id', 'DESC')->get();
        $sites = Sites::where('del', 0)->where('active', 1)->get();
        $emp_attend = EmployeeAttendance::where('id', $id)->first();
        $userTable = $this->getUserTable();
        if (!$emp_attend) {
            return $this->message('Record not found', 404);
        }
        if ($userTable == "customer" && ($emp_attend->workspace_id != auth()->user()->current_workspace_id || $emp_attend->customer_id != auth()->user()->id)) {
            return $this->message('You do not have access to this project', 403);
        }
        if ($userTable == "emp" && ($emp_attend->customer_id != auth()->user()->customer_id || $emp_attend->workspace_id != auth()->user()->workspace_id)) {
            return $this->message('You do not have access to this project', 403);
        }
        $emp_breaks = EmployeeBreak::where('emp_attendance_id', $id)->get();
        $data = [
            'attendance' => $emp_attend,
            'breaks' => $emp_breaks,
            'employees' => $get_employes,
            'sites' => $sites,
        ];
        return $this->success($data, 'Attendance get Successfully');
        // return view('Attendance.edit', compact('get_employes', 'sites', 'emp_attend', 'emp_breaks'));
    }

    public function update(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'employee' => 'required',
            'site' => 'required',
            'check_in' => 'required|date_format:H:i',
            'check_out' => 'nullable|date_format:H:i|after:check_in',
            'date' => 'required|date',
            'status' => 'nullable',
            'break_in' => ['nullable', 'array'],
            'break_out' => ['nullable', 'array'],
            'break_in.*' => ['nullable', 'date_format:H:i'],
            'break_out.*' => ['nullable', 'date_format:H:i', 'after_or_equal:break_in.*'],
        ], [
            'break_out.*.after_or_equal' => 'The break_out must be after or equal to break_in.',
        ]);
        if ($validator->fails()) {
            $error = $validator->errors()->first();
            return $this->message($error, 422);
        }
        $userTable = $this->getUserTable();
        $employeeAttendance = EmployeeAttendance::find($request->id);
        if (!$employeeAttendance) {
            return $this->message('Record not found', 404);
        }
        $validatedData = $validator->validated();
        if ($userTable == "customer" && ($employeeAttendance->workspace_id != auth()->user()->current_workspace_id || $employeeAttendance->customer_id != auth()->user()->id)) {
            return $this->message('You do not have access to this project', 403);
        }
        if ($userTable == "emp" && ($employeeAttendance->customer_id != auth()->user()->customer_id || $employeeAttendance->workspace_id != auth()->user()->workspace_id)) {
            return $this->message('You do not have access to this project', 403);
        }
        // $roster_assigns =   DB::table('roster_assigns')
        //     ->where('assign_to', $validatedData['employee'])
        //     ->where('schedule_date', $validatedData['date'])
        //     ->count();
        // if ($roster_assigns == 0) {
        //     return $this->message('Roaster not found', 404);
        // }

        $checkIn = Carbon::parse($validatedData['check_in']);
        $checkOut = Carbon::parse($validatedData['check_out']);
        $totalHours = $checkOut->diffInMinutes($checkIn);
        // Calculate total breaks
        $totalBreaks = 0;
        $totalWorkingHours = 0;
        if (isset($validatedData['break_in'])) {
            foreach ($validatedData['break_in'] as $key => $breakIn) {
                if (isset($validatedData['break_out'][$key])) {
                    $breakInTime = Carbon::parse($breakIn);
                    $breakOutTime = Carbon::parse($validatedData['break_out'][$key]);
                    $totalBreaks += $breakOutTime->diffInMinutes($breakInTime);
                }
            }
        }
        $customerId = $userTable == "customer" ? auth()->id() : $employeeAttendance->customer_id;

        // Calculate total working hours with breaks
        $totalWorkingHours = $totalHours - $totalBreaks;
        $employeeAttendance->update([
            'employee_id' => $validatedData['employee'],
            'customer_id' => $customerId,
            'workspace_id' => $employeeAttendance->workspace_id,
            'site_id' => $validatedData['site'],
            'working_hours' => $validatedData['check_out'] ? $totalWorkingHours : '',
            'check_in' => $validatedData['check_in'],
            'check_out' => $validatedData['check_out'],
            'status' => $validatedData['status'] ?? 0,
            'date' => $validatedData['date'],
            'added_by' => $userTable == "customer" ? 0 : auth()->id(),
        ]);
        // Delete existing breaks
        $employeeAttendance->breaks()->delete();
        if (isset($validatedData['break_in'])) {
            foreach ($validatedData['break_in'] as $key => $breakIn) {
                $breakOut = isset($validatedData['break_out'][$key]) ? $validatedData['break_out'][$key] : null;
                EmployeeBreak::create([
                    'emp_attendance_id' => $employeeAttendance->id,
                    'break_in' => $breakIn,
                    'break_out' => $breakOut,
                ]);
            }
        }
        return $this->success($employeeAttendance, 'Attendance Updated Successfully');
    }

    public function destroy($id)
    {
        try {
            $attendance = EmployeeAttendance::find($id);
            if (!$attendance) {
                return $this->message('Attendance record not found.', 404);
            }
            $userTable = $this->getUserTable();
            if ($userTable === "customer") {
                $employeeDetails = EmpCompanyDetails::where('id', $attendance->employee_id)
                    ->where('customer_id', auth()->id())
                    ->where('workspace_id', auth()->user()->current_workspace_id)
                    ->first();
                if (!$employeeDetails) {
                    return $this->message('Unauthorized access to this attendance record.', 403);
                }
            }
            if ($userTable === "emp") {
                $employeeDetails = EmpCompanyDetails::where('id', $attendance->employee_id)
                    ->where('customer_id', auth()->user()->customer_id)
                    ->where('workspace_id', auth()->user()->workspace_id)
                    ->first();

                if (!$employeeDetails) {
                    return $this->message('Unauthorized access to this attendance record.', 403);
                }
            }
            EmployeeAttendance::where('id', $id)->delete();
            EmployeeBreak::where('emp_attendance_id', $id)->delete();
            return $this->message('Attendance Deleted Successfully');
        } catch (\Exception $e) {
            return $this->message('Failed to delete attendance. Error: ' . $e->getMessage(), 500);
        }
    }


    public function deletebreak($id)
    {
        try {
            // Fetch the break record
            $empbreak = EmployeeBreak::find($id);
            if (!$empbreak) {
                return $this->message('Break record not found.', 404);
            }
            $attendance = EmployeeAttendance::find($empbreak->emp_attendance_id);
            if (!$attendance) {
                return $this->message('Attendance record not found.', 404);
            }
            // Determine user type and validate access
            $userTable = $this->getUserTable();
            if ($userTable === "customer") {
                $employeeDetails = EmpCompanyDetails::where('id', $attendance->employee_id)
                    ->where('customer_id', auth()->id())
                    ->where('workspace_id', auth()->user()->current_workspace_id)
                    ->first();
                if (!$employeeDetails) {
                    return $this->message('Unauthorized access to this break record.', 403);
                }
            }
            if ($userTable === "emp") {
                $employeeDetails = EmpCompanyDetails::where('id', $attendance->employee_id)
                    ->where('customer_id', auth()->user()->customer_id)
                    ->where('workspace_id', auth()->user()->workspace_id)
                    ->first();
                if (!$employeeDetails) {
                    return $this->message('Unauthorized access to this break record.', 403);
                }
            }
            // Calculate and update working hours
            $breakIn = Carbon::parse($empbreak->break_in);
            $breakOut = Carbon::parse($empbreak->break_out);
            $totalBreakHours = $breakOut->diffInHours($breakIn);
            $totalWorkingHours = $attendance->working_hours + $totalBreakHours;
            $attendance->update(['working_hours' => $totalWorkingHours]);
            $empbreak->delete();
            return $this->message('Break deleted successfully');
        } catch (\Exception $e) {
            return $this->message('Failed to delete break. Error: ' . $e->getMessage(), 500);
        }
    }

    public function attendance_request(Request $request)
    {
        try {
            $query = EmployeeAttendance::query();
            $userTable = $this->getUserTable();
            if ($userTable === "customer") {
                $query->whereHas('EmpCompanyDetails', function ($q) {
                    $q->where('customer_id', auth()->id())
                        ->where('workspace_id', auth()->user()->current_workspace_id);
                });
            }
            if ($userTable === "emp") {
                $query->whereHas('EmpCompanyDetails', function ($q) {
                    $q->where('customer_id', auth()->user()->customer_id)
                        ->where('workspace_id', auth()->user()->workspace_id);
                });
            }
            if ($userTable !== "customer" && $userTable !== "emp") {
                return $this->message('Unauthorized access.', 403);
            }

            if ($request->filled('filter')) {
                $query = $this->filter_request(json_decode($request->filter, true), $query);
            }
            $query->with(['EmpPersonalDetails', 'EmpCompanyDetails', 'Sites', 'addedby', 'breaks', 'empTeamsMembers']);
            $start = $request->input('from', 0);
            $query->offset($start)->limit(10);
            $query->orderBy('id', 'desc');
            $query_result = $query->get();

            // Prepare filters
            $array_filter = json_decode($request->filter, true);
            $filters = [
                'first_name' => $array_filter['first_name'] ?? '',
                'middle_name' => $array_filter['middle_name'] ?? '',
                'last_name' => $array_filter['first_name'] ?? '',
                'employee_email' => $array_filter['last_name'] ?? '',
                'completed' => $array_filter['first_name'] ?? '',
                'status' => $array_filter['completed'] ?? '',
            ];
            return $this->withFilter($query_result, $filters, 'Get Attendance List Successfully');
        } catch (\Exception $e) {
            return $this->message('Failed to retrieve attendance list. Error: ' . $e->getMessage(), 500);
        }
    }


    public function filter_request($filters, $query)
    {
        foreach ($filters as $filterName => $filterValue) {
            if ($filterValue != null ||  $filterValue != "") {
                switch ($filterName) {
                    case 'first_name':
                        $filterValue  = explode(" ", $filterValue);

                        if (count($filterValue) == 1) {
                            $query->whereHas('empPersonalDetails', function ($subquery) use ($filterValue) {
                                $subquery->where('first_name', 'like', '%' . $filterValue[0] . '%');
                                $subquery->orWhere('middle_name', 'like', '%' . $filterValue[0] . '%');
                                $subquery->orWhere('last_name', 'like', '%' . $filterValue[0] . '%');
                            });
                        }

                        if (count($filterValue) == 2) {
                            $query->whereHas('empPersonalDetails', function ($subquery) use ($filterValue) {
                                $subquery->where('first_name', 'like', '%' . $filterValue[0] . '%');
                                $subquery->where('middle_name', 'like', '%' . $filterValue[1] . '%');
                                $subquery->where('last_name', 'like', '%' . $filterValue[1] . '%');
                            });
                        }

                        if (count($filterValue) == 3) {

                            $query->whereHas('empPersonalDetails', function ($subquery) use ($filterValue) {
                                $subquery->where('first_name', 'like', '%' . $filterValue[0] . '%');
                                $subquery->where('middle_name', 'like', '%' . $filterValue[1] . '%');
                                $subquery->where('last_name', 'like', '%' . $filterValue[2] . '%');
                            });
                        }

                        break;
                    case 'user_type':
                        $query->whereHas('empCompanyDetails', function ($subquery) use ($filterValue) {
                            $filterValue == 0 ? 0 : 1;
                            $subquery->where('user_type', 'like', '%' . $filterValue . '%');
                        });
                        break;
                    case 'first_name_add':

                        $filterValue  = explode(" ", $filterValue);
                        if (count($filterValue) == 1) {
                            $query->whereHas('addedby', function ($subquery) use ($filterValue) {
                                $subquery->where('first_name', 'like', '%' . $filterValue[0] . '%');
                            });
                        }

                        if (count($filterValue) == 2) {
                            $query->whereHas('addedby', function ($subquery) use ($filterValue) {
                                $subquery->where('first_name', 'like', '%' . $filterValue[0] . '%');
                                $subquery->where('last_name', 'like', '%' . $filterValue[1] . '%');
                            });
                        }

                        if (count($filterValue) == 3) {
                            $query->whereHas('addedby', function ($subquery) use ($filterValue) {
                                $subquery->where('first_name', 'like', '%' . $filterValue[0] . '%');
                                $subquery->where('middle_name', 'like', '%' . $filterValue[1] . '%');
                                $subquery->where('last_name', 'like', '%' . $filterValue[2] . '%');
                            });
                        }

                        break;
                    case 'title':
                        $query->whereHas('sites', function ($subquery) use ($filterValue) {
                            $subquery->where('title', 'like', '%' . $filterValue . '%');
                        });
                        break;
                    case 'Check_in':
                        $query->where('Check_in', 'like', '%' . $filterValue . '%');
                        break;
                    case 'Check_out':
                        $query->where('Check_out', 'like', '%' . $filterValue . '%');
                        break;
                    case 'working_hours':
                        $query->where('working_hours', 'like', '%' . $filterValue . '%');
                        break;
                    case 'status':
                        $filterValue =   $filterValue ? 1 : 0;
                        $query->where('status', 'like', '%' . $filterValue . '%');
                        break;
                    case 'date':
                        $filterValue = explode("/", $filterValue);
                        $query->whereBetween('date', [date("Y-m-d", strtotime(trim($filterValue[0], " "))), date("Y-m-d", strtotime(trim($filterValue[1], " ")))]);
                        break;
                }
            }
        }

        return $query;
    }

    public function updateStatus(Request $request)
    {
        $EmployeeAttendance = EmployeeAttendance::find($request->id);
        if (!$EmployeeAttendance) {
            return $this->message('Record not found', 404);
        }
        $userTable = $this->getUserTable();
        // Check for customer access
        if ($userTable === "customer") {
            $validAccess = $EmployeeAttendance->customer_id === auth()->id() &&
                $EmployeeAttendance->workspace_id === auth()->user()->current_workspace_id;

            if (!$validAccess) {
                return $this->message('Unauthorized access to this record.', 403);
            }
        }
        // Check for employee access
        if ($userTable === "emp") {
            $validAccess = $EmployeeAttendance->customer_id === auth()->user()->customer_id &&
                $EmployeeAttendance->workspace_id === auth()->user()->workspace_id;

            if (!$validAccess) {
                return $this->message('Unauthorized access to this record.', 403);
            }
        }
        if ($userTable !== "customer" && $userTable !== "emp") {
            return $this->message('Unauthorized access.', 403);
        }
        // Toggle the active status
        $EmployeeAttendance->status = $EmployeeAttendance->status == 1 ? 0 : 1;
        $EmployeeAttendance->save();

        return $this->success($EmployeeAttendance, 'Status Updated Successfully');
    }


    public function approved_all()
    {
        try {
            $userTable = $this->getUserTable();
            $query = EmployeeAttendance::query();
            if ($userTable === "customer") {
                $query->whereHas('EmpCompanyDetails', function ($q) {
                    $q->where('customer_id', auth()->id())
                        ->where('workspace_id', auth()->user()->current_workspace_id);
                });
            }
            if ($userTable === "emp") {
                $query->whereHas('EmpCompanyDetails', function ($q) {
                    $q->where('customer_id', auth()->user()->customer_id)
                        ->where('workspace_id', auth()->user()->workspace_id);
                });
            }
            if ($userTable !== "customer" && $userTable !== "emp") {
                return $this->message('Unauthorized access.', 403);
            }
            $updatedCount = $query->update(['status' => 1]);
            if ($updatedCount === 0) {
                return $this->message('No records found to approve.', 404);
            }
            return $this->success($updatedCount, 'All Requests Approved Successfully');
        } catch (\Exception $e) {
            return $this->message('Failed to approve requests. Error: ' . $e->getMessage(), 500);
        }
    }


    public function approveSelected(Request $request)
    {
        try {
            $selectedIds = $request->input('ids', []);
            if (empty($selectedIds)) {
                return $this->message('No IDs provided.', 422);
            }
            $userTable = $this->getUserTable();
            $query = EmployeeAttendance::whereIn('id', $selectedIds);
            if ($userTable === "customer") {
                $query->whereHas('EmpCompanyDetails', function ($q) {
                    $q->where('customer_id', auth()->id())
                        ->where('workspace_id', auth()->user()->current_workspace_id);
                });
            }
            if ($userTable === "emp") {
                $query->whereHas('EmpCompanyDetails', function ($q) {
                    $q->where('customer_id', auth()->user()->customer_id)
                        ->where('workspace_id', auth()->user()->workspace_id);
                });
            }
            if ($userTable !== "customer" && $userTable !== "emp") {
                return $this->message('Unauthorized access.', 403);
            }
            $updatedCount = $query->update(['status' => 1]);
            if ($updatedCount === 0) {
                return $this->message('No records found to approve.', 404);
            }
            return $this->message('Approved Selected Requests Successfully');
        } catch (\Exception $e) {
            return $this->message('Failed to approve selected requests. Error: ' . $e->getMessage(), 500);
        }
    }



    public function attendance_portal(Request $request)
    {
        $sites = Sites::where('del', 0)->where('active', 1)->get();
        return view('Attendance.attendanceRecord', compact('sites'));
    }

    public function attendance_portal_employee(Request $request)
    {
        $validator = Validator::make(
            $request->all(),
            [
                'from' => 'required',
                'to' => 'required|after_or_equal:from',
                'site_id' => 'nullable',
                'employee_name' => 'nullable'
            ],
            [
                "after_or_equal" => "To date must be bigger or equal to From"
            ]
        );
        if ($validator->fails()) {
            $error = $validator->errors()->first();
            return $this->message($error, 422);
        }
        $validatedData = $validator->validated();
        $userTable = $this->getUserTable();
        // Apply conditions based on user type
        if ($userTable === "customer") {
            $rosterAssignQuery = DB::table('roster_assigns')
                ->where('roster_assigns.customer_id', auth()->id())
                ->where('roster_assigns.workspace_id', auth()->user()->current_workspace_id);
        }

        if ($userTable === "emp") {
            $rosterAssignQuery = DB::table('roster_assigns')
                ->where('roster_assigns.customer_id', auth()->user()->customer_id)
                ->where('roster_assigns.workspace_id', auth()->user()->workspace_id);
        }

        if (!isset($rosterAssignQuery)) {
            return $this->message('Unauthorized access.', 403);
        }

        $required_minutes = $rosterAssignQuery->whereBetween('schedule_date', [$validatedData['from'], $validatedData['to']])
            ->leftjoin('roster_templates', 'roster_assigns.roster_template_id', 'roster_templates.id')
            ->sum('roster_templates.working_hours');

        $employee_attendances = EmployeeAttendance::whereBetween('date', [$validatedData['from'], $validatedData['to']]);
        // Add conditions for customer and emp to the main query
        if ($userTable === "customer") {
            $employee_attendances->whereHas('EmpCompanyDetails', function ($q) {
                $q->where('customer_id', auth()->id())
                    ->where('workspace_id', auth()->user()->current_workspace_id);
            });
        }

        if ($userTable === "emp") {
            $employee_attendances->whereHas('EmpCompanyDetails', function ($q) {
                $q->where('customer_id', auth()->user()->customer_id)
                    ->where('workspace_id', auth()->user()->workspace_id);
            });
        }

        if (!is_null($request->site_id)) {
            $employee_attendances->where('site_id', $validatedData['site_id']);
        }

        if ($request->employee_name) {
            $filterValue = explode(" ", $validatedData['employee_name']);
            $employee_attendances->whereHas('empPersonalDetails', function ($subquery) use ($filterValue) {
                if (count($filterValue) === 1) {
                    $subquery->where(function ($query) use ($filterValue) {
                        $query->where('first_name', 'like', '%' . $filterValue[0] . '%')
                            ->orWhere('middle_name', 'like', '%' . $filterValue[0] . '%')
                            ->orWhere('last_name', 'like', '%' . $filterValue[0] . '%');
                    });
                }

                if (count($filterValue) === 2) {
                    $subquery->where('first_name', 'like', '%' . $filterValue[0] . '%')
                        ->where(function ($query) use ($filterValue) {
                            $query->where('middle_name', 'like', '%' . $filterValue[1] . '%')
                                ->orWhere('last_name', 'like', '%' . $filterValue[1] . '%');
                        });
                }

                if (count($filterValue) === 3) {
                    $subquery->where('first_name', 'like', '%' . $filterValue[0] . '%')
                        ->where('middle_name', 'like', '%' . $filterValue[1] . '%')
                        ->where('last_name', 'like', '%' . $filterValue[2] . '%');
                }
            });
        }

        $working_minutes = (clone $employee_attendances)->sum('working_hours');
        $employee_count = (clone $employee_attendances)
            ->select(DB::raw('COUNT(DISTINCT employee_id) as count'))
            ->first();

        $employee_attendances_lists = (clone $employee_attendances)
            ->distinct()
            ->pluck('employee_id');

        $employee_attendances = DB::table('employee_attendances')
            ->join('emp_personal_details', 'employee_attendances.employee_id', '=', 'emp_personal_details.emp_id')
            ->join('emp_company_details', 'employee_attendances.employee_id', '=', 'emp_company_details.id')
            ->join('roles', 'emp_company_details.access_role', '=', 'roles.code')
            ->whereIn('employee_attendances.employee_id', $employee_attendances_lists)
            ->whereBetween('date', [$validatedData['from'], $validatedData['to']]);

            if (!is_null($request->site_id)) {
                $employee_attendances->where('employee_attendances.site_id', $validatedData['site_id']);
            }
            

        $employee_attendances->groupBy('employee_attendances.employee_id')
            ->select(
                'employee_attendances.employee_id as emp_id',
                DB::raw('MAX(emp_personal_details.first_name) as first_name'),
                DB::raw('MAX(emp_personal_details.middle_name) as middle_name'),
                DB::raw('MAX(emp_personal_details.last_name) as last_name'),
                DB::raw('MAX(emp_personal_details.image) as image'),
                DB::raw('MAX(emp_company_details.user_type) as user_type'),
                DB::raw('MAX(roles.title) as title')
            );

        $count = $employee_count->count;
        $start = $request->input('page', 0);
        $employee_attendances->offset($start)->limit(10);

        $query_result['employees_list'] = $employee_attendances->get();
        $query_result['workinghours'] = convertMinutesToHours($working_minutes);
        $query_result['requirehours'] = convertMinutesToHours($required_minutes);
        $query_result['employee'] = $employee_count->count;

        return $this->success($query_result, 'Get Attendance List Successfully');
    }



    public function employe_attendance(Request $request)
    {
        $validator = Validator::make(
            $request->all(),
            [
                'to' => 'required|date',
                'from' => 'required|date|before_or_equal:to',
                'empid' => 'required',
                'site_id' => 'nullable'
            ],
            [
                'before_or_equal' => 'The from date must be before or equal to the to date.'
            ]
        );

        if ($validator->fails()) {
            return response()->json(['error' => $validator->errors()->first()], 422);
        }
        $validatedData = $validator->validated();
        $siteid = $validatedData['site_id'];
        $userTable = $this->getUserTable();

        // Add conditions based on user type
        if ($userTable === "customer") {

            $attendanceQuery = EmployeeAttendance::whereHas('EmpCompanyDetails', function ($q) {
                $q->where('customer_id', auth()->id())
                    ->where('workspace_id', auth()->user()->current_workspace_id);
            });
        }
        if ($userTable === "emp") {
            $attendanceQuery = EmployeeAttendance::whereHas('EmpCompanyDetails', function ($q) {
                $q->where('customer_id', auth()->user()->customer_id)
                    ->where('workspace_id', auth()->user()->workspace_id);
            });
        }
        if (!$attendanceQuery->exists()) {
            return $this->message('Record not found', 404);
        };
        // Add additional conditions
        $attendanceQuery->where('employee_id', $validatedData['empid'])
            ->whereBetween('date', [$validatedData['from'], $validatedData['to']]);

        if ($siteid) {
            $attendanceQuery->where('site_id', $siteid);
        }

        $data = $attendanceQuery
            ->with('RosterAssign.RosterTemplate', 'sites', 'breaks')
            ->get();
        if ($data->isEmpty()) {
            return $this->message('Record not found', 404);
        }
        return $this->success($data, 'Get Employee Attendance List Successfully');
    }


    ////////////////Attendance Api///////////////


    public function check_in(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'employee_id' => 'required',
            'longitude' => 'required',
            'latitude' => 'required',
            'date' => 'required|date',
            'check_in' => 'nullable|date_format:H:i',
            'check_out' => 'nullable|date_format:H:i|after:check_in',
            'site_id' => 'nullable',
        ]);

        if ($validator->fails()) {
            return $this->message($validator->errors()->first(), 422);
        }

        $validatedData = $validator->validated();
        $userTable = $this->getUserTable();
        if ($userTable === "customer") {
            $authorized = EmployeeAttendance::whereHas('EmpCompanyDetails', function ($q) use ($validatedData) {
                $q->where('customer_id', auth()->id())
                    ->where('workspace_id', auth()->user()->current_workspace_id)
                    ->where('employee_id', $validatedData['employee_id']);
            })->exists();

            if (!$authorized) {
                return response()->json(['message' => 'Unauthorized access to this employee.'], 403);
            }
        }
        if ($userTable === "emp") {
            $authorized = EmployeeAttendance::whereHas('EmpCompanyDetails', function ($q) use ($validatedData) {
                $q->where('customer_id', auth()->user()->customer_id)
                    ->where('workspace_id', auth()->user()->workspace_id)
                    ->where('employee_id', $validatedData['employee_id']);
            })->exists();

            if (!$authorized) {
                return response()->json(['message' => 'Unauthorized access to this employee.'], 403);
            }
        }
        $check_leave = LeaveRequest::where('employee_id', $request->employee_id)
            ->where('status', 1)
            ->whereBetween('from', [date('Y-m-d'), date('Y-m-d')])
            ->exists();
        if ($check_leave) {
            return response()->json([
                'message' => 'You are on leave. You are not allowed to check-in.'
            ], 422);
        }
        $site = Sites::whereRaw("CONVERT(longitude, CHAR) LIKE CONCAT('%', ?, '%')", [$validatedData['longitude']])
            ->whereRaw("CONVERT(latitude, CHAR) LIKE CONCAT('%', ?, '%')", [$validatedData['latitude']])
            ->select('id')
            ->first();

        $site_id = $site ? $site->id : null;
        $data = [
            'employee_id' => $validatedData['employee_id'],
            'longitude' => $validatedData['longitude'],
            'latitude' =>  $validatedData['latitude'],
            'date' => $validatedData['date'],
            'site_id' => $site_id,
            'customer_id' => $authorized->customer_id,
            'workspace_id' => $authorized->workspace_id
        ];
        if ($request->has('check_in')) {
            $data['check_in'] = $request->input('check_in');
        }
        if ($request->has('check_out')) {
            $data['check_out'] = $request->input('check_out');
        }
        $existingData = EmployeeAttendance::where('employee_id', $data['employee_id'])
            ->where('date', $data['date'])
            ->first();
        if ($request->has('check_out')) {
            if ($existingData && !$existingData->check_in) {
                return $this->message('Cannot check out without checking in first.', 422);
            }

            if ($existingData) {
                $checkIn = Carbon::parse($existingData->check_in);
                $checkOut = Carbon::parse($validatedData['check_out']);
                $totalHours = $checkOut->diffInMinutes($checkIn);
                $existingData->update([
                    'check_out' => $data['check_out'],
                    'working_hours' => $totalHours
                ]);
                $message = 'Check Out successfully';
            } else {
                return $this->message('Cannot check out without checking in first.', 422);
            }
        } else {
            if ($existingData) {
                $message = 'Check In already exists';
            } else {
                EmployeeAttendance::create($data);
                $message = 'Check In successfully';
            }
        }
        return $this->success($existingData, $message);
    }



    public function _check_in(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'employee_id' => 'required',
            'longitude' => 'required',
            'latitude' => 'required',
            'date' => 'required|date',
            'check_in' => 'date_format:H:i',
            'site_id' => 'nullable',
            'image' => 'nullable|mimes:jpeg,png,jpg,gif|max:7168',
        ]);
        if ($validator->fails()) {
            $error = $validator->errors()->first();
            return $this->message($error, 422);
        }
        $validatedData = $validator->validated();
        $userTable = $this->getUserTable();
        if ($userTable === "customer") {
            $employeeDetails = EmpCompanyDetails::where('id', $validatedData['employee_id'])
                ->where('customer_id', auth()->id())
                ->where('workspace_id', auth()->user()->current_workspace_id)
                ->first();
            if (!$employeeDetails) {
                return $this->message('Unauthorized access to this employee.', 403);
            }
        }
        if ($userTable === "emp") {
            $employeeDetails = EmpCompanyDetails::where('id', $validatedData['employee_id'])
                ->where('customer_id', auth()->user()->customer_id)
                ->where('workspace_id', auth()->user()->workspace_id)
                ->first();
            if (!$employeeDetails) {
                return $this->message('Unauthorized access to this employee.', 403);
            }
        }
        if (Auth::user()->id != $validatedData['employee_id']) {
            return $this->message('Invalid Request.', 422);
        }

        $check_roster = RosterAssign::join('roster_templates', 'roster_assigns.roster_template_id', 'roster_templates.id')
            ->where('roster_assigns.assign_to', Auth::user()->id)
            ->where('schedule_date', date('Y-m-d'))
            ->where('roster_templates.start_time', "<=", $request->check_in)
            ->where('roster_templates.end_time', ">=", $request->check_in)
            ->exists();

        if (!$check_roster) {
            $message = "Please clock in according to your scheduled roster time.";

            /////////////////// ATTENDANCE LOGS ////////////////////
            $log = [
                'employee_id' => Auth::user()->id,
                'site_id' => null,
                'check_in' => $request->check_in,
                'check_out' => null,
                'date' => formatDate($request->date),
                'longitude' => $validatedData['longitude'],
                'latitude' => $validatedData['latitude'],
                'message' => $message,
                'error_type' => 'invalid-roster-time',
                'request_type' => 'attendance',
                'customer_id' => $employeeDetails->customer_id,
                'workspace_id' => $employeeDetails->workspace_id
            ];
            storeAttendanceLogs($log);
            /////////////////// ATTENDANCE LOGS END ////////////////////
            return $this->message($message, 422);
        }
        $point = [
            'longitude' => $validatedData['longitude'],
            'latitude' => $validatedData['latitude'],
        ];
        $sites = Sites::all();
        $foundSite = null;
        foreach ($sites as $site) {
            $center = [
                'longitude' => $site->longitude,
                'latitude' => $site->latitude,
            ];
            $radius = $site->area_radius;
            if (GeoLoacation::isPointWithinRadius($point, $center, $radius)) {
                $foundSite = $site;
                break;
            }
        }
        if (!$foundSite) {
            $data = [
                'employee_location' => $point,
                'all_sites' => $sites
            ];
            $message = 'The provided location is outside the permitted radius of any site.';
            /////////////////// ATTENDANCE LOGS ////////////////////
            $log = [
                'employee_id' => Auth::user()->id,
                'site_id' => null,
                'check_in' => $request->check_in,
                'check_out' => null,
                'date' => formatDate($request->date),
                'longitude' => $validatedData['longitude'],
                'latitude' => $validatedData['latitude'],
                'message' => $message,
                'error_type' => 'invalid-location',
                'request_type' => 'attendance',
                'customer_id' => $employeeDetails->customer_id,
                'workspace_id' => $employeeDetails->workspace_id
            ];
            storeAttendanceLogs($log);
            /////////////////// ATTENDANCE LOGS END ////////////////////
            return $this->message($message, 422);
        }

        $previousRecord = EmployeeAttendance::where('employee_id', Auth::user()->id)
            ->where('date', $validatedData['date'])
            ->latest()
            ->first();
        if ($previousRecord && !$previousRecord->check_out) {
            $message = 'You are already checked in at a site. Please check out first before checking in again.';

            /////////////////// ATTENDANCE LOGS ////////////////////
            $log = [
                'employee_id' => Auth::user()->id,
                'site_id' => $foundSite->id,
                'check_in' => $request->check_in,
                'check_out' => null,
                'date' => formatDate($request->date),
                'longitude' => $validatedData['longitude'],
                'latitude' => $validatedData['latitude'],
                'message' => $message,
                'error_type' => 'already-check-in',
                'request_type' => 'attendance',
                'customer_id' => $employeeDetails->customer_id,
                'workspace_id' => $employeeDetails->workspace_id
            ];
            storeAttendanceLogs($log);
            /////////////////// ATTENDANCE LOGS END ////////////////////
            return $this->success($previousRecord->id, $message, 422);
        }
        // $validatedData['image'] = "";
        if ($request->hasFile('image')) {

            $validatedData['image'] = $this->handleFileImageUpload($request, 'AttendanceLog')['path'] ?? null;;
        }
        $data = [
            'employee_id' => Auth::user()->id,
            'date'       => $validatedData['date'],
            'check_in'   => $validatedData['check_in'],
            'longitude'  => $validatedData['longitude'],
            'latitude'   => $validatedData['latitude'],
            'status'     => 1,
            'site_id'    => $foundSite->id,
            'added_by'   => Auth::user()->id,
            'image'      => $validatedData['image'],
            'customer_id' => $employeeDetails->customer_id,
            'workspace_id' => $employeeDetails->workspace_id
        ];

        $checkin = EmployeeAttendance::create($data);
        $message = 'Check-in Successfully';
        ///////////////// ATTENDANCE LOGS ////////////////////
        $log = [
            'employee_id' => Auth::user()->id,
            'site_id' => $foundSite->id,
            'check_in' => $request->check_in,
            'check_out' => null,
            'date' => formatDate($request->date),
            'longitude' => $validatedData['longitude'],
            'latitude' => $validatedData['latitude'],
            'customer_id' => $employeeDetails->customer_id,
            'workspace_id' => $employeeDetails->workspace_id,
            'message' => $message,
            'error_type' => 'success',
            'request_type' => 'attendance'
        ];
        storeAttendanceLogs($log);
        /////////////////// ATTENDANCE LOGS END ////////////////////
        return $this->success($checkin, $message);
    }

    public function _check_out(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'checkin_id' => 'required',
            'check_out' => 'date_format:H:i',
            'longitude' => 'required',
            'latitude' => 'required',
            'date' => 'required|date',
        ]);

        if ($validator->fails()) {
            $error = $validator->errors()->first();
            return $this->message($error, 422);
        }
        $validatedData = $validator->validated();
        $userTable = $this->getUserTable();
        if ($userTable === "customer") {
            $employeeDetails = EmpCompanyDetails::where('id', $validatedData['checkin_id'])
                ->where('customer_id', auth()->id())
                ->where('workspace_id', auth()->user()->current_workspace_id)
                ->first();
            if (!$employeeDetails) {
                return $this->message('Unauthorized access to this employee.', 403);
            }
        }
        if ($userTable === "emp") {
            $employeeDetails = EmpCompanyDetails::where('id', $validatedData['checkin_id'])
                ->where('customer_id', auth()->user()->customer_id)
                ->where('workspace_id', auth()->user()->workspace_id)
                ->first();
            if (!$employeeDetails) {
                return $this->message('Unauthorized access to this employee.', 403);
            }
        }
        $attendanceRecord = EmployeeAttendance::where([
            ['id', $validatedData['checkin_id']],
            ['check_out', null]
        ])->first();

        if (!$attendanceRecord) {
            $attendanceRecordWithCheckout = EmployeeAttendance::where('id', $validatedData['checkin_id'])->where('check_out', '!=', null)->first();
            if ($attendanceRecordWithCheckout) {
                $message = 'Your shift has been auto logout from the system at ' . formatTime($attendanceRecordWithCheckout->check_out);
                $is_checkout = 1;
            } else {
                $message = 'Check-in required before check-out. Please complete your check-in before proceeding with check-out.';
                $is_checkout = 1;
            }
            /////////////////// ATTENDANCE LOGS ////////////////////
            $log = [
                'employee_id' => Auth::user()->id,
                'site_id' => null,
                'check_in' => null,
                'customer_id' => $employeeDetails->customer_id,
                'workspace_id' => $employeeDetails->workspace_id,
                'check_out' => $validatedData['check_out'],
                'date' => formatDate($validatedData['date']),
                'longitude' => $validatedData['longitude'],
                'latitude' => $validatedData['latitude'],
                'message' => $message,
                'error_type' => 'check-in-missed',
                'request_type' => 'attendance'
            ];
            storeAttendanceLogs($log);
            /////////////////// ATTENDANCE LOGS END ////////////////////
            return $this->success($is_checkout, $message, 422);
        }

        //////////////////////
        $point = [
            'longitude' => $validatedData['longitude'],
            'latitude' => $validatedData['latitude'],
        ];

        $site = Sites::find($attendanceRecord->site_id);
        // return  $site;
        if (!$site) {
            $message = "The site you are trying to check out from is not available now.";
            /////////////////// ATTENDANCE LOGS ////////////////////
            $log = [
                'employee_id' => Auth::user()->id,
                'customer_id' => $employeeDetails->customer_id,
                'workspace_id' => $employeeDetails->workspace_id,
                'site_id' => $attendanceRecord->site_id,
                'check_in' => $attendanceRecord->check_in,
                'check_out' => $validatedData['check_out'],
                'date' => formatDate($validatedData['date']),
                'longitude' => $validatedData['longitude'],
                'latitude' => $validatedData['latitude'],
                'message' => $message,
                'error_type' => 'site-not-available',
                'request_type' => 'attendance'
            ];
            storeAttendanceLogs($log);
            /////////////////// ATTENDANCE LOGS END ////////////////////
            return $this->message($message, 422);
        }

        $center = [
            'longitude' => $site->longitude,
            'latitude' => $site->latitude,
        ];
        $radius = $site->area_radius;
        $allSites = Sites::all();
        $data = [];
        $data = [
            'employee_location' => $point,
            'site_location' => $center,
            'site_radius' => $radius,
            'all_sites' => $allSites
        ];

        $foundSite = null;

        foreach ($allSites as $allSite) {
            $center = [
                'longitude' => $allSite->longitude,
                'latitude' => $allSite->latitude,
            ];
            $radius = $allSite->area_radius;

            if (GeoLoacation::isPointWithinRadius($point, $center, $radius)) {
                $foundSite = $allSite;
                break;
            }
        }
        if (!$foundSite) {
            $message = 'You are trying to check out outside from the site. Your assigned site is: ' . $site->title . '.';

            /////////////////// ATTENDANCE LOGS ////////////////////
            $log = [
                'employee_id' => Auth::user()->id,
                'customer_id' => $employeeDetails->customer_id,
                'workspace_id' => $employeeDetails->workspace_id,
                'site_id' => $attendanceRecord->site_id,
                'check_in' => $attendanceRecord->check_in,
                'check_out' => $validatedData['check_out'],
                'date' => formatDate($validatedData['date']),
                'longitude' => $validatedData['longitude'],
                'latitude' => $validatedData['latitude'],
                'message' => $message,
                'error_type' => 'site-area-invalid',
                'request_type' => 'attendance'
            ];
            storeAttendanceLogs($log);
            /////////////////// ATTENDANCE LOGS END ////////////////////
            return $this->message($message, 422);
        }

        if ($foundSite && $foundSite->id !== $site->id) {
            $message = 'You are trying to check out from the site: ' . $foundSite->title . '. Your assigned site is: ' . $site->title . '.';
            /////////////////// ATTENDANCE LOGS ////////////////////
            $log = [
                'employee_id' => Auth::user()->id,
                'customer_id' => $employeeDetails->customer_id,
                'workspace_id' => $employeeDetails->workspace_id,
                'site_id' => $attendanceRecord->site_id,
                'check_in' => $attendanceRecord->check_in,
                'check_out' => $validatedData['check_out'],
                'date' => formatDate($validatedData['date']),
                'longitude' => $validatedData['longitude'],
                'latitude' => $validatedData['latitude'],
                'message' => $message,
                'error_type' => 'site-area-invalid',
                'request_type' => 'attendance'
            ];
            storeAttendanceLogs($log);
            /////////////////// ATTENDANCE LOGS END ////////////////////
            return $this->message($message, 422);
        }

        /////////////////////


        // $checkIn = Carbon::parse($attendanceRecord->check_in);
        // $checkOut = Carbon::parse($validatedData['check_out']);
        $checkIn = Carbon::parse($attendanceRecord->date . ' ' . $attendanceRecord->check_in);
        $checkOut = Carbon::parse($validatedData['date'] . ' ' . $validatedData['check_out']);

        $totalHours = $checkOut->diffInMinutes($checkIn);

        if ($checkOut->lte($checkIn)) {
            $message = 'Check-out time must come after check-in time.';
            /////////////////// ATTENDANCE LOGS ////////////////////
            $log = [
                'employee_id' => Auth::user()->id,
                'customer_id' => $employeeDetails->customer_id,
                'workspace_id' => $employeeDetails->workspace_id,
                'site_id' => $attendanceRecord->site_id,
                'check_in' => $attendanceRecord->check_in,
                'check_out' => $validatedData['check_out'],
                'date' => formatDate($validatedData['date']),
                'longitude' => $validatedData['longitude'],
                'latitude' => $validatedData['latitude'],
                'message' => $message,
                'error_type' => 'check-out-time-invalid',
                'request_type' => 'attendance'
            ];
            storeAttendanceLogs($log);
            /////////////////// ATTENDANCE LOGS END ////////////////////
            return $this->message($message, 422);
        }

        $breaks = EmployeeBreak::where('emp_attendance_id', $attendanceRecord->id)->get();
        // Calculate total breaks
        $totalBreaks = 0;
        $totalWorkingHours = 0;
        if (isset($breaks)) {
            foreach ($breaks as $break) {
                $breakInTime = Carbon::parse($break->break_in);
                $breakOutTime = Carbon::parse($break->break_out);
                $totalBreaks += $breakOutTime->diffInMinutes($breakInTime);
            }
        }
        // Calculate total working hours with breaks
        $totalWorkingHours = $totalHours - $totalBreaks;

        // $checkIn = Carbon::parse($attendanceRecord->check_in);
        // $checkOut = Carbon::parse($validatedData['check_out']);
        // $totalHours = $checkOut->diffInMinutes($checkIn);

        $attendanceRecord->update([
            'check_out' => $validatedData['check_out'],
            'working_hours' => $totalWorkingHours
        ]);

        $message = 'success';
        ///////////////// ATTENDANCE LOGS ////////////////////
        $log = [
            'employee_id' => Auth::user()->id,
            'site_id' => $attendanceRecord->site_id,
            'customer_id' => $employeeDetails->customer_id,
            'workspace_id' => $employeeDetails->workspace_id,
            'check_in' => $attendanceRecord->check_in,
            'check_out' => $validatedData['check_out'],
            'date' => formatDate($validatedData['date']),
            'longitude' => $validatedData['longitude'],
            'latitude' => $validatedData['latitude'],
            'message' => $message,
            'error_type' => 'success',
            'request_type' => 'attendance'
        ];
        /////////////////// ATTENDANCE LOGS END ////////////////////
        storeAttendanceLogs($log);
        return $this->success($attendanceRecord, $message);
    }


    public function last_check_in(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'employee_id' => 'required',
        ]);

        if ($validator->fails()) {
            return $this->message($validator->errors()->first(), 422);
        }
        $validatedData = $validator->validated();
        $userTable = $this->getUserTable();
        $query = EmployeeAttendance::query();
        if ($userTable === "customer") {
            $query->whereHas('EmpCompanyDetails', function ($q) use ($validatedData) {
                $q->where('customer_id', auth()->id())
                    ->where('workspace_id', auth()->user()->current_workspace_id)
                    ->where('id', $validatedData['employee_id']);
            });
        }

        if ($userTable === "emp") {
            $query->whereHas('EmpCompanyDetails', function ($q) use ($validatedData) {
                $q->where('customer_id', auth()->user()->customer_id)
                    ->where('workspace_id', auth()->user()->workspace_id)
                    ->where('id', $validatedData['employee_id']);
            });
        }

        if ($userTable !== "customer" && $userTable !== "emp") {
            return $this->message('Unauthorized access.', 403);
        }
        if (!$query->exists()) {
            return $this->message('Records not found', 404);
        }
        $lastCheckin = $query->where('employee_id', $validatedData['employee_id'])
            ->latest()
            ->first();

        if (!$lastCheckin) {
            return $this->message('No check-in records found for the employee.', 404);
        }

        $responseMessage = isset($lastCheckin->check_out)
            ? 'User Check-In and Check-Out Details'
            : 'User Check-In Detail';
        return $this->success($lastCheckin, $responseMessage);
    }

    public function break_in(Request $request)
    {
        // dd($request->all());
        $validator = Validator::make($request->all(), [
            'emp_attendance_id' => 'required|exists:employee_attendances,id',
            'break_in' => 'required|date_format:H:i',
            'date' => 'required|date',
        ]);
        if ($validator->fails()) {
            return $this->message($validator->errors()->first(), 422);
        }
        $validatedData = $validator->validated();
        $userTable = $this->getUserTable();
        $attendanceRecord = EmployeeAttendance::where('id', $validatedData['emp_attendance_id']);

        if ($userTable === "customer") {
            $attendanceRecord->where('customer_id', auth()->id())
                ->where('workspace_id', auth()->user()->current_workspace_id);
        }
        if ($userTable === "emp") {
            $attendanceRecord->where('customer_id', auth()->user()->customer_id)
                ->where('workspace_id', auth()->user()->workspace_id);
        }
        if ($userTable !== "customer" && $userTable !== "emp") {
            return $this->message('Unauthorized access ', 403);
        }
        $attendanceRecord = $attendanceRecord->first();
        if (!$attendanceRecord) {
            return $this->message('Attendance record not found ', 404);
        }
        $data = [
            'emp_attendance_id' => $validatedData['emp_attendance_id'],
            'break_in' => $validatedData['break_in'],
            'date' => $validatedData['date'],
        ];
        $breakIn = EmployeeBreak::create($data);
        return $this->success($breakIn, 'Break In successfully');
    }


    public function last_break_in(Request $request)
    {
        // dd($request->all());
        $validator = Validator::make($request->all(), [
            'emp_attendance_id' => 'required|exists:employee_attendances,id',
        ]);
        if ($validator->fails()) {
            return $this->message($validator->errors()->first(), 422);
        }
        $validatedData = $validator->validated();
        $userTable = $this->getUserTable();
        $attendanceRecord = EmployeeAttendance::where('id', $validatedData['emp_attendance_id']);

        if ($userTable === "customer") {
            $attendanceRecord->where('customer_id', auth()->id())
                ->where('workspace_id', auth()->user()->current_workspace_id);
        }
        if ($userTable === "emp") {
            $attendanceRecord->where('customer_id', auth()->user()->customer_id)
                ->where('workspace_id', auth()->user()->workspace_id);
        }
        if ($userTable !== "customer" && $userTable !== "emp") {
            return $this->message('Unauthorized access.', 403);
        }
        $attendanceRecord = $attendanceRecord->first();
        if (!$attendanceRecord) {
            return $this->message('Attendance record not found', 404);
        }
        $lastBreakIn = EmployeeBreak::where('emp_attendance_id', $validatedData['emp_attendance_id'])
            ->latest()
            ->first();

        if ($lastBreakIn) {
            if (is_null($lastBreakIn->break_out)) {
                return $this->success($lastBreakIn, 'Last open break found.', 200);
            } else {
                return $this->message('No open break found.', 404);
            }
        }

        return $this->message('No break records found for this attendance.', 404);
    }


    public function break_out(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'breakin_id' => 'required|exists:employee_breaks,id',
            'break_out' => 'required|date_format:H:i',
        ]);
        if ($validator->fails()) {
            return $this->message($validator->errors()->first(), 404);
        }
        $validatedData = $validator->validated();
        $attendanceRecord = EmployeeBreak::find($validatedData['breakin_id']);
        if (!$attendanceRecord) {
            return $this->message(' Break-in No break found', 404);
        }
        $userTable = $this->getUserTable();
        $employeeAttendance = $attendanceRecord->employeeAttendance;

        if (!$employeeAttendance) {
            return $this->message('Associated attendance record not found', 404);
        }
        if (
            ($userTable === "customer" &&
                ($employeeAttendance->customer_id !== auth()->id() ||
                    $employeeAttendance->workspace_id !== auth()->user()->current_workspace_id)) ||
            ($userTable === "emp" &&
                ($employeeAttendance->customer_id !== auth()->user()->customer_id ||
                    $employeeAttendance->workspace_id !== auth()->user()->workspace_id))
        ) {
            return $this->message('Unauthorized access to this record', 403);
        }
        // dd($employeeAttendance);
        // Update the break-out time
        $attendanceRecord->update([
            'break_out' => $validatedData['break_out'],
        ]);
        return $this->success($attendanceRecord, 'Break Out successfully');
    }



    // public function checkin_history(Request $request)
    // {
    //     $validator = Validator::make($request->all(), [
    //         'employee_id' => 'required',
    //         'start_date' => 'required|date',
    //         'emp_attendance_id' => 'required',
    //         // 'end_date' => 'required|date'
    //     ]);

    //     if ($validator->fails()) {
    //         $error = $validator->errors()->first();

    //         return response()->json([
    //             'message' => $error
    //         ], 422);
    //     } else {
    //         $validatedData = $validator->validated();

    //         // $empattendance = EmployeeAttendance::where('employee_id', $validatedData['employee_id'])
    //         //     ->whereBetween('date', [$validatedData['start_date'], $validatedData['end_date']])
    //         //     ->get();
    //         //       $empattendance = EmployeeAttendance::where('employee_id', $validatedData['employee_id'])
    //         // ->where('date',$validatedData['start_date'])
    //         // ->with('breaks') 
    //         // ->get();
    //         $empattendance = EmployeeAttendance::where('employee_id', $validatedData['employee_id'])
    //         ->where('date', $validatedData['start_date'])
    //         ->whereHas('breaks', function ($query) use ($validatedData) {
    //             $query->where('emp_attendance_id', $validatedData['emp_attendance_id']);
    //         })
    //         ->with('breaks')
    //         ->get();



    //         if ($empattendance->isNotEmpty()) {
    //             return response([
    //                 'message' => 'User Check In History',
    //                 'data' => $empattendance
    //             ], 200);
    //         } else {
    //             return response([
    //                 'message' => 'No records found for the specified date range'
    //             ], 200);
    //         }
    //     }
    // }
    public function checkin_history(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'employee_id' => 'required',
            'start_date' => 'required|date',
            'emp_attendance_id' => 'required|exists:employee_attendances,id',
        ]);

        if ($validator->fails()) {
            $error = $validator->errors()->first();
            return $this->message($error, 422);
        }

        $validatedData = $validator->validated();

        $userTable = $this->getUserTable();

        $empattendanceQuery = EmployeeAttendance::where('employee_id', $validatedData['employee_id'])
            ->where('date', $validatedData['start_date'])
            ->where(function ($query) use ($validatedData) {
                $query->whereHas('breaks', function ($innerQuery) use ($validatedData) {
                    $innerQuery->where('emp_attendance_id', $validatedData['emp_attendance_id']);
                })
                    ->orWhereDoesntHave('breaks');
            });

        // Apply conditions for customer and emp
        if ($userTable === "customer") {
            $empattendanceQuery->where('customer_id', auth()->id())
                ->where('workspace_id', auth()->user()->current_workspace_id);
        }

        if ($userTable === "emp") {
            $empattendanceQuery->where('customer_id', auth()->user()->customer_id)
                ->where('workspace_id', auth()->user()->workspace_id);
        }

        if ($userTable !== "customer" && $userTable !== "emp") {
            return $this->message('Unauthorized access.', 403);
        }

        $empattendance = $empattendanceQuery->with([
            'breaks' => function ($query) use ($validatedData) {
                $query->where('emp_attendance_id', $validatedData['emp_attendance_id']);
            },
            'sites' => function ($query) {
                $query->select('id', 'title', 'street_address');
            }
        ])
            ->latest('id')
            ->first();
        if ($empattendance) {
            return $this->success($empattendance, 'User Check In History');
        }

        return $this->message('No records found for the specified date range', 404);
    }


    public function attendance_list(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'employee_id' => 'required|exists:employee_attendances,employee_id',
        ]);
        if ($validator->fails()) {
            return $this->message($validator->errors()->first(), 422);
        }
        $validatedData = $validator->validated();
        $userTable = $this->getUserTable();
        $firstDayOfPreviousMonth = Carbon::now()->subMonth()->startOfMonth();
        $lastDayOfPreviousMonth = Carbon::now()->subMonth()->endOfMonth();
        $empattendanceQuery = EmployeeAttendance::where('employee_id', $validatedData['employee_id'])
            ->whereBetween('date', [$firstDayOfPreviousMonth, $lastDayOfPreviousMonth]);
        if ($userTable === "customer") {
            $empattendanceQuery->where('customer_id', auth()->id())
                ->where('workspace_id', auth()->user()->current_workspace_id);
        }
        if ($userTable === "emp") {
            $empattendanceQuery->where('customer_id', auth()->user()->customer_id)
                ->where('workspace_id', auth()->user()->workspace_id);
        }
        if ($userTable !== "customer" && $userTable !== "emp") {
            return $this->message('Unauthorized access.', 403);
        }
        $empattendance = $empattendanceQuery->get();

        if ($empattendance->count() > 0) {
            return $this->success($empattendance, 'User Attendance List for the previous month');
        }
        return $this->message('No Record Found for the previous month.', 404);
    }

    ////////////////Attendance Api///////////////

    ///////////////// CRON JOB //////////////////
    public function autoCheckout(Request $request)
    {
        $date_from = Carbon::now()->subDays(2)->format('Y-m-d');
        $current_date = Carbon::now()->format('Y-m-d');
        $current_time = Carbon::now()->format('H:i:s');
        $userTable = $this->getUserTable();

        $employee_ids_query = EmployeeAttendance::whereNotNull('check_in')
            ->whereNull('check_out')
            ->where('date', ">=", $date_from);
        if ($userTable === "customer") {
            $employee_ids_query->where('customer_id', auth()->id())
                ->where('workspace_id', auth()->user()->current_workspace_id);
        } elseif ($userTable === "emp") {
            $employee_ids_query->where('customer_id', auth()->user()->customer_id)
                ->where('workspace_id', auth()->user()->workspace_id);
        } else {
            return $this->message('Unauthorized access.', 403);
        }
        $employee_ids = $employee_ids_query->select('id', 'employee_id', 'date')->get();
        Log::info(date('d-m-y H:i:s') . " Employees who did not checkout: " . $employee_ids);
        $update_attendance = $delete_attendance = $check = 0;
        $all_employee_id = [];

        foreach ($employee_ids as $value) {
            $roster = RosterAssign::join('roster_templates', 'roster_assigns.roster_template_id', 'roster_templates.id')
                ->where('roster_assigns.assign_to', $value->employee_id)
                ->where('roster_assigns.schedule_date', $value->date)
                ->first();

            if ($roster) {
                $employee_attendance = EmployeeAttendance::where('id', $value->id)->first();
                $total_working_minutes = $this->calculateWorkingHours($employee_attendance, $roster->end_time);

                if ($value->date < $current_date) {
                    EmployeeAttendance::where('employee_id', $value->employee_id)
                        ->where('date', $value->date)
                        ->update([
                            'check_out' => formatTime($roster->end_time),
                            'working_hours' => $total_working_minutes,
                            'checkout_type' => 1
                        ]);
                    $update_attendance += 1;
                    $check = 1;
                } elseif ($value->date == $current_date && $current_time >= $roster->end_time) {
                    EmployeeAttendance::where('employee_id', $value->employee_id)
                        ->where('date', $value->date)
                        ->update([
                            'check_out' => formatTime($roster->end_time),
                            'working_hours' => $total_working_minutes,
                            'checkout_type' => 1
                        ]);
                    $update_attendance += 1;
                    $check = 1;
                }
                $all_employee_id[] = $value->employee_id;
            } else {
                EmployeeAttendance::where('employee_id', $value->employee_id)
                    ->where('date', $value->date)
                    ->delete();
                $delete_attendance += 1;
                $check = 1;
            }
        }

        $employee_details = EmpCompanyDetails::whereIn('id', $all_employee_id)
            ->select('id', 'employee_email')
            ->get();

        Log::info(date('d-m-y H:i:s') . " Auto Checkout API run successfully. Employee details: " . $employee_details);

        $data = [
            'total_updated_attendance' => $update_attendance,
            'total_deleted_attendance' => $delete_attendance,
        ];

        if (!$check) {
            return $this->message('Nothing to update and delete');
        }

        return $this->success($data, 'All employees checked-out successfully');
    }



    public function attendanceDownload(Request $request)
    {
        $query = EmployeeAttendance::with(['EmpPersonalDetails', 'EmpCompanyDetails', 'Sites', 'empTeamsMembers']);

        if ($request->filled('filter')) {
            $query = $this->filter(json_decode($request->filter, true), $query);
        }

        $attendances = $query->get();

        $cols = DB::table('attendance_list_change_cols')->pluck('col')->toArray();

        $columns = [
            'Employee',
            'Type',
            'Teams',
            'Site',
            'Check In',
            'Check Out',
            'Working Hours',
            'Date'
        ];

        $col_list[] = "Employee Id";
        $col_list[] = "External Id";
        $indexs = [];
        $i = 2;
        foreach ($columns as $column) {
            if (in_array($column, $cols)) {
                $col_list[] = $column;
            } else {
                $indexs[] = $i;
            }
            $i++;
        }

        $csvData[] = $col_list;

        $data_list  = [];


        $user_meta = '';
        foreach ($attendances as $attendance) {
            $team_list = "";
            foreach ($attendance->empTeamsMembers as $et) {
                $team_list = $et->empTeamsList->title . ",";
            }

            $user_meta =  DB::table('user_meta')->where('emp_id', $attendance->employee_id)->where('option', 'employee_payroll_or_external_id')->first();
            $data_list = [
                $attendance->employee_id,
                $user_meta ? $user_meta->value : '',
                $attendance->emppersonaldetails->first_name . ' ' . $attendance->emppersonaldetails->middle_name . ' ' . $attendance->emppersonaldetails->last_name,
                $attendance->empcompanydetails->user_type == 0 ?  'Internal' : 'External',
                $team_list  ? rtrim($team_list, ',') : "",
                $attendance->sites->title,
                $this->AM_PM($attendance->check_in),
                $attendance->check_out ? $this->AM_PM($attendance->check_out) : $attendance->check_out,
                $this->minutesToHours($attendance->working_hours),
                date("d-m-Y", strtotime($attendance->date))
            ];

            $csvData[] = array_values(array_diff_key($data_list, array_flip($indexs)));
        }

        $headers = [
            'Content-Type' => 'text/csv',
            'Content-Disposition' => 'attachment; filename="attendance.csv"',
        ];


        $callback = function () use ($csvData) {
            $file = fopen('php://output', 'w');
            foreach ($csvData as $row) {
                fputcsv($file, $row);
            }
            fclose($file);
        };

        return Response::stream($callback, 200, $headers);
    }


    function AM_PM($TIME)
    {
        $time = Carbon::parse($TIME);
        $time = $time->format('g:i A');
        $time = str_replace(' ', '', $time);
        $time = str_replace(':00', '', $time);
        return $time;
    }


    function minutesToHours($minutes)
    {
        $isNegative = $minutes < 0;
        $minutes = $minutes == '' ?  0 :   abs($minutes);

        $hours = floor($minutes / 60);
        $remainingMinutes = $minutes % 60;

        $result = ($isNegative ? '-' : '') . $hours . 'h:' . ($remainingMinutes < 10 ? '0' : '') . $remainingMinutes . 'm';

        return $result;
    }

    public function changeCol(Request $request)
    {
        $arr = [];
        $col = $request->col;
        if ($col) {
            DB::table('attendance_list_change_cols')->delete();
            for ($i = 0; $i < count($col); $i++) {
                $arr[$i]['col'] = $col[$i];
            }
            AttendanceListChangeCol::insert($arr);
        }
        return back();
    }

    public function calculateWorkingHours($employee_attendance, $check_out)
    {
        $checkIn = Carbon::parse($employee_attendance->date . ' ' . $employee_attendance->check_in);
        $checkOut = Carbon::parse($employee_attendance->date . ' ' . $check_out);

        $totalHours = $checkOut->diffInMinutes($checkIn);

        $breaks = EmployeeBreak::where('emp_attendance_id', $employee_attendance->id)->get();
        // Calculate total breaks
        $totalBreaks = 0;
        $totalWorkingHours = 0;
        if (isset($breaks)) {
            foreach ($breaks as $break) {
                if (is_null($break->break_out)) {
                    $break_update = EmployeeBreak::where('id', $break->id)->update(['break_out' => $check_out]);
                    $breakOutTime = Carbon::parse($check_out);
                } else {
                    $breakOutTime = Carbon::parse($break->break_out);
                }
                $breakInTime = Carbon::parse($break->break_in);
                $totalBreaks += $breakOutTime->diffInMinutes($breakInTime);
            }
        }
        // Calculate total working hours with breaks
        $totalWorkingHours = $totalHours - $totalBreaks;
        return $totalWorkingHours;
    }
}
