<?php

namespace App\Http\Controllers;

use Illuminate\Http\Request;
use App\Models\RosterAssign;
use App\Models\EmployeeAttendance;
use App\Models\LeaveRequest;
use App\Models\EmpCompanyDetails;
use App\Models\Sites;
use App\Models\Project;
use App\Models\Company;
use Illuminate\Support\Collection;
use App\Models\EmpTeam;
use App\Models\EmpPersonalDetails;
use Carbon\Carbon;
use App\Models\EmpDocuments;
use App\Models\RequiredDocument;
use App\Models\User;
use DB;
use Illuminate\Support\Facades\Auth;

class DashboardController extends Controller
{

    public $current_date;

    public function __construct()
    {
        $this->current_date = date('Y-m-d');
    }

    public function index(Request $request) 
    {
        $userTable = $this->getUserTable();
        $workspace_id = 0;
        $customer_id = 0;
        if ($userTable === "customer") {
            $customer_id = Auth::user()->id;
            $workspace_id = Auth::user()->current_workspace_id;  
            $authPersonalDetails = User::where('id', Auth::user()->id)->first();
        }
        if ($userTable === "emp") {
            $customer_id = auth()->user()->customer_id;
            $workspace_id = auth()->user()->workspace_id;
            $authPersonalDetails = EmpPersonalDetails::where('emp_id', Auth::user()->id)->first();  
        }
  
        $filter = $request->query('filter', 'ALL'); 
        $start_date = null;
        $end_date = date('Y-m-d'); 
        
        switch ($filter) {
            case '1M': 
                $start_date = date('Y-m-d', strtotime('-1 month'));
                break;
            case '6M': 
                $start_date = date('Y-m-d', strtotime('-6 months'));
                break;
            case '1Y':
                $start_date = date('Y-m-d', strtotime('-1 year'));
                break;
            case 'All':
            default:
                $start_date = null;
                break;
        }
        $data['chart_data'] = $this->generateChartData($filter, $start_date, $end_date, $customer_id, $workspace_id);

        $data['total_employees']  =   EmpCompanyDetails::where('del',0)->where('compeleted','1')->where('customer_id',$customer_id)->where('workspace_id',$workspace_id)->count();
        
        $data['total_clients']    =   Company::where('del',0)->where('customer_id',$customer_id)->where('workspace_id',$workspace_id)->count();

        $data['total_sites']      =   Sites::where('del',0)->where('customer_id',$customer_id)->where('workspace_id',$workspace_id)->count();

        $data['sites']      =   Sites::where('del',0)->where('customer_id',$customer_id)->where('workspace_id',$workspace_id)->get();

        $data['total_projects']   =   Project::where('is_deleted',0)->where('customer_id',$customer_id)->where('workspace_id',$workspace_id)->count();

        $employees_ids = RosterAssign::join('emp_company_details','roster_assigns.assign_to','emp_company_details.id')
                                    ->where('emp_company_details.del',0)
                                    ->where('emp_company_details.workspace_id',$workspace_id)
                                    ->where('emp_company_details.customer_id',$customer_id)
                                    ->where('emp_company_details.status',1)
                                    ->where('roster_assigns.schedule_date', $this->current_date)
                                    ->groupBy('roster_assigns.assign_to')
                                    ->pluck('roster_assigns.assign_to')
                                    ->toArray();

        $data['total_roster_employees'] = count($employees_ids);

        $present_employee_ids = EmployeeAttendance::where('date',$this->current_date)
                                    ->wherein('employee_id',$employees_ids)
                                    ->groupBy('employee_id')
                                    ->where('workspace_id',$workspace_id)
                                    ->where('customer_id',$customer_id)
                                    ->pluck('employee_id')
                                    ->toArray();

        $data['total_present'] = count($present_employee_ids);

        $leave_employee_ids = LeaveRequest::where(function ($query) {
            $query->where('from', '<=', $this->current_date)
                  ->where('to', '>=', $this->current_date);
        })
        ->orWhere(function ($query) {
            $query->where('from', '<=', $this->current_date)
                  ->where('to', '>=', $this->current_date);
        })
        ->where('status', 1)
        ->where('workspace_id', $workspace_id)
        ->where('customer_id', $customer_id)
        ->pluck('employee_id')
        ->toArray();    

                                
        $data['total_leaves'] = count($leave_employee_ids);

        $data['total_absent'] = $data['total_roster_employees'] - $data['total_present'] - $data['total_leaves'];

        $data['today_present'] = EmployeeAttendance::where('date',$this->current_date)
                                    ->where('workspace_id',$workspace_id)
                                    ->where('customer_id',$customer_id)
                                    ->with('present_employee_details','employee_site','present_employee_company_details','breaks')
                                    ->get();

        $employeeCountBySiteName = $data['today_present']->groupBy(function ($attendance) {
            return $attendance->employee_site->title; // Group by site name
        })->map(function ($site) {
            return [
                'employee_count' => $site->count(),  // Count of employees
                'check_in_count' => $site->whereNotNull('check_in')->count(),  // Count of check-ins
                'check_out_count' => $site->whereNotNull('check_out')->count()  // Count of check-outs
            ];
        });
        $data['sites_employee'] = $employeeCountBySiteName->toArray();
        
        $data['today_leaves'] = LeaveRequest::with('leave_employee_details','leave_employee_company_details')
        ->where(function ($query) {
            $query->where('from', '<=', $this->current_date)
                  ->where('to', '>=', $this->current_date);
        })
        ->where('workspace_id', $workspace_id)
        ->where('customer_id', $customer_id)
        ->where('status', 1)
        ->get();
    
        
        
        $emp_teams = EmpTeam::with(['emp_teams_member' => function ($query) {
            $query->whereHas('roster_assigns');
        }])->where('del', 0)
        ->where('workspace_id',$workspace_id)
        ->where('customer_id',$customer_id)
        ->get();
        
        $teams_response = [];
        
        foreach ($emp_teams as $team) {
            $total_employees_with_roster = $team->emp_teams_member->count();
            $total_employees_with_attendance = 0;
            $total_employees_without_attendance = 0;
        
            foreach ($team->emp_teams_member as $team_member) {
                if ($team_member->roster_assigns && $team_member->roster_assigns->team_employee_attendance) {
                    $total_employees_with_attendance++;
                } else {
                    $total_employees_without_attendance++;
                }
            }
        
            $teams_response[] = [
                'team_id' => $team->id,
                'team_title' => $team->title,
                'total_employees_with_roster' => $total_employees_with_roster,
                'total_employees_with_attendance' => $total_employees_with_attendance,
                'total_employees_without_attendance' => $total_employees_without_attendance,
            ];
        }
        $data['teams'] = $teams_response;

        return response()->json([
            'message' => 'Dashboard data Fetched Successfully.',
            'data' => $data
        ]);
    }
    private function generateChartData($filter, $start_date, $end_date, $customer_id, $workspace_id)
    {
        $chartData = [];
        
        if ($start_date === null && $filter === 'All') {
            $earliest_client = Company::where('del', 0)
                ->where('customer_id', $customer_id)
                ->where('workspace_id', $workspace_id)
                ->min('created_at');
                
            $earliest_site = Sites::where('del', 0)
                ->where('customer_id', $customer_id)
                ->where('workspace_id', $workspace_id)
                ->min('created_at');
                
            $earliest_project = Project::where('is_deleted', 0)
                ->where('customer_id', $customer_id)
                ->where('workspace_id', $workspace_id)
                ->min('created_at');
                
            $earliest_dates = array_filter([$earliest_client, $earliest_site, $earliest_project]);
            $earliest_date = !empty($earliest_dates) ? min($earliest_dates) : date('Y-m-d', strtotime('-1 year'));
            
            $start_date = date('Y-m-d', strtotime($earliest_date));
        }
        
        if ($filter == '1M') {
            $period = new \DatePeriod(
                new \DateTime($start_date),
                new \DateInterval('P1D'),
                new \DateTime($end_date . ' +1 day')
            );
            $format = 'd M';
            $query_format = 'Y-m-d';
            $interval = 'day';
        } elseif ($filter == '6M' || $filter == '1Y') {
            $period = new \DatePeriod(
                new \DateTime($start_date),
                new \DateInterval('P1M'),
                new \DateTime($end_date . ' +1 day')
            );
            $format = 'M';
            $query_format = 'Y-m';
            $interval = 'month';
        } else { // All
            $start_datetime = new \DateTime($start_date);
            $end_datetime = new \DateTime($end_date);
            $diff_months = $start_datetime->diff($end_datetime)->m + ($start_datetime->diff($end_datetime)->y * 12);
            
            if ($diff_months > 24) {
                $period = new \DatePeriod(
                    new \DateTime(date('Y-01-01', strtotime($start_date))),
                    new \DateInterval('P1Y'),
                    new \DateTime(date('Y-12-31', strtotime($end_date . ' +1 year')))
                );
                $format = 'Y';
                $query_format = 'Y';
                $interval = 'year';
            } else {
                $period = new \DatePeriod(
                    new \DateTime($start_date),
                    new \DateInterval('P1M'),
                    new \DateTime($end_date . ' +1 month')
                );
                $format = 'M Y';
                $query_format = 'Y-m';
                $interval = 'month';
            }
        }
        
        foreach ($period as $key => $date) {
            $date_formatted = $date->format($format);
            $date_query = $date->format($query_format);
            
            $period_start_date = clone $date;
            $period_end_date = clone $date;
            
            if ($interval == 'day') {
                $period_end_date->modify('+1 day');
            } elseif ($interval == 'month') {
                $period_end_date->modify('last day of this month');
                $period_start_date->modify('first day of this month');
            } elseif ($interval == 'year') {
                $period_start_date->modify('first day of january this year');
                $period_end_date->modify('last day of december this year');
            }
            
            $period_start_str = $period_start_date->format('Y-m-d');
            $period_end_str = $period_end_date->format('Y-m-d');
            
            $clients_count = Company::where('del', 0)
                ->where('customer_id', $customer_id)
                ->where('workspace_id', $workspace_id)
                ->whereDate('created_at', '>=', $period_start_str)
                ->whereDate('created_at', '<=', $period_end_str)
                ->count();
            
            $sites_count = Sites::where('del', 0)
                ->where('customer_id', $customer_id)
                ->where('workspace_id', $workspace_id)
                ->whereDate('created_at', '>=', $period_start_str)
                ->whereDate('created_at', '<=', $period_end_str)
                ->count();
            
            $projects_count = Project::where('is_deleted', 0)
                ->where('customer_id', $customer_id)
                ->where('workspace_id', $workspace_id)
                ->whereDate('created_at', '>=', $period_start_str)
                ->whereDate('created_at', '<=', $period_end_str)
                ->count();
            
            $chartData[] = [
                'name' => $date_formatted,
                'Clients' => $clients_count,
                'Sites' => $sites_count,
                'Projects' => $projects_count
            ];
        }
        
        if ($filter == '1M') {
            $chartData = array_slice($chartData, -30);
        } elseif ($filter == '6M') {
            $chartData = array_slice($chartData, -6);
        } elseif ($filter == '1Y') {
            $chartData = array_slice($chartData, -12);
        } else { // All
            if (count($chartData) > 12) {
                $step = ceil(count($chartData) / 12);
                $temp = [];
                for ($i = 0; $i < count($chartData); $i += $step) {
                    if (isset($chartData[$i])) {
                        $temp[] = $chartData[$i];
                    }
                }
                $chartData = $temp;
            }
        }
        
        if (count($chartData) < 3) {
            $months = ['Jan', 'Feb', 'Mar', 'Apr', 'May', 'Jun', 'Jul', 'Aug', 'Sep', 'Oct', 'Nov', 'Dec'];
            $current_month = (int)date('m') - 1;
            
            while (count($chartData) < 3) {
                $chartData[] = [
                    'name' => $months[$current_month % 12],
                    'Clients' => 0,
                    'Sites' => 0,
                    'Projects' => 0
                ];
                $current_month--;
            }
            
            $chartData = array_reverse($chartData);
        }
        
        if (count($chartData) > 0) {
            $total_clients = Company::where('del', 0)
                ->where('customer_id', $customer_id)
                ->where('workspace_id', $workspace_id)
                ->count();
                
            $total_sites = Sites::where('del', 0)
                ->where('customer_id', $customer_id)
                ->where('workspace_id', $workspace_id)
                ->count();
                
            $total_projects = Project::where('is_deleted', 0)
                ->where('customer_id', $customer_id)
                ->where('workspace_id', $workspace_id)
                ->count();
                
            $chartData[count($chartData) - 1]['Clients'] = $total_clients;
            $chartData[count($chartData) - 1]['Sites'] = $total_sites;
            $chartData[count($chartData) - 1]['Projects'] = $total_projects;
        }
        
        return $chartData;
    }

    public function expiringDocuments(Request $request) {
        $customer_id = auth()->user()->id;
        $workspace_id = auth()->user()->current_workspace_id;
        
        $today = Carbon::today();
        $near_expiry_date = $today->addDays(30); // Documents expiring in the next 30 days
    
        // Base query for fetching documents
        $query = DB::table('emp_documents as expiry_doc')
            ->join('emp_personal_details', 'emp_personal_details.emp_id', '=', 'expiry_doc.emp_id')
            ->join('emp_company_details', 'emp_personal_details.emp_id', '=', 'emp_company_details.id')
            ->join('emp_teams_members', 'emp_personal_details.emp_id', '=', 'emp_teams_members.emp_id')
            ->join('emp_teams', 'emp_teams_members.team_id', '=', 'emp_teams.id')
            ->join('required_documents', 'required_documents.id', '=', 'expiry_doc.required_document')
            ->join('required_document_fields as expiry_field', 'expiry_field.id', '=', 'expiry_doc.required_document_field')
            ->leftJoin('emp_documents as file_doc', function($join) {
                $join->on('expiry_doc.emp_id', '=', 'file_doc.emp_id')
                     ->on('expiry_doc.required_document', '=', 'file_doc.required_document');
            })
            ->join('required_document_fields as file_field', 'file_field.id', '=', 'file_doc.required_document_field')
            ->where('expiry_field.field_name', 'Expiry Date')  // Ensure we're filtering for expiry date field
            ->where('file_field.field_type', 'file')  // Ensure we're filtering for file fields
            ->whereDate('expiry_doc.value', '<=', $near_expiry_date)  // Filter based on expiry date
            ->where('expiry_doc.del', '0')  // Filter non-deleted expiry records
            ->where('file_doc.del', '0')    // Filter non-deleted file records
            ->where('emp_company_details.del', '0')
            ->where('emp_company_details.compeleted', '1')
            ->where('emp_company_details.customer_id', $customer_id)
            ->where('emp_company_details.workspace_id', $workspace_id)
            ->select(
                'emp_personal_details.emp_id',
                'emp_personal_details.first_name',
                'emp_personal_details.middle_name',
                'emp_personal_details.last_name',
                DB::raw("GROUP_CONCAT(emp_teams.title SEPARATOR ', ') as team_titles"),  // Combine multiple team titles
                'required_documents.id as document_id',
                'required_documents.title as document_title',
                'expiry_doc.value as expiry_date',  // Expiry date
                'file_doc.value as document_path',   // File path from the file_doc
                DB::raw("DATEDIFF(expiry_doc.value, CURDATE()) as remaining_days")  // Calculate remaining days
            )
            ->groupBy(
                'emp_personal_details.emp_id',
                'emp_personal_details.first_name',
                'emp_personal_details.middle_name',
                'emp_personal_details.last_name',
                'required_documents.id',
                'required_documents.title',
                'expiry_doc.value',
                'file_doc.value'
            );
    
        $count = DB::table(DB::raw("({$query->toSql()}) as sub"))->mergeBindings($query)->count();
    
        // Apply pagination
        $start = $request->input('from', 0);
        $limit = 10;  // Set the limit for pagination
        $query->offset($start)->limit($limit);
    
        // Get the paginated results
        $query_result = $query->get();
    
        $data =  [
            'expiring_documents' => $query_result,
        ];
    
        return response()->json([
            'message' => 'Get employees with expiring documents successfully.',
            'data' => $data,
            'count' => $count,
        ], 200);
    }
    
    public function missingDocuments(Request $request)
    {
        $customer_id = auth()->user()->id;
        $workspace_id = auth()->user()->current_workspace_id;

        $limit = 10;  
        $start = $request->input('from', 0);

        // Subquery to get the uploaded documents for each employee
        $uploadedDocuments = DB::table('emp_documents')
            ->select('emp_id', DB::raw('GROUP_CONCAT(required_document) as uploaded_docs'))
            ->groupBy('emp_id');

        $query = DB::table('emp_personal_details')
            ->join('emp_teams_members', 'emp_personal_details.emp_id', '=', 'emp_teams_members.emp_id')
            ->join('emp_teams', 'emp_teams_members.team_id', '=', 'emp_teams.id')
            ->join('emp_company_details', 'emp_personal_details.emp_id', '=', 'emp_company_details.id') // Join employee company details
            ->crossJoin('required_documents')
            ->join('required_document_fields', 'required_documents.id', '=', 'required_document_fields.doc_id')
            ->leftJoinSub($uploadedDocuments, 'uploaded_docs', function ($join) {
                $join->on('emp_personal_details.emp_id', '=', 'uploaded_docs.emp_id');
            })
            ->whereRaw('FIND_IN_SET(required_documents.id, IFNULL(uploaded_docs.uploaded_docs, "")) = 0')
            ->where('emp_personal_details.del', '0')
            ->where('required_documents.del', '0')
            ->where('required_document_fields.field_required', 1)
            ->where(function ($query) {
                $query->where('required_documents.for_who', '2')
                    ->orWhere(function ($query) {
                        $query->whereColumn('required_documents.for_who', 'emp_company_details.user_type');
                    });
            })
            ->where('emp_company_details.customer_id', $customer_id)
            ->where('emp_company_details.workspace_id', $workspace_id)
            ->select(
                'emp_personal_details.emp_id',
                'emp_personal_details.first_name',
                'emp_personal_details.middle_name',
                'emp_personal_details.last_name',
                DB::raw("GROUP_CONCAT(DISTINCT emp_teams.title SEPARATOR ', ') as team_titles"),
                'required_documents.title as document_title',
                'emp_company_details.user_type'
            )
            ->groupBy(
                'emp_personal_details.emp_id',
                'emp_personal_details.first_name',
                'emp_personal_details.middle_name',
                'emp_personal_details.last_name',
                'required_documents.title',
                'emp_company_details.user_type',
            );

        $count = DB::table(DB::raw("({$query->toSql()}) as sub"))
            ->mergeBindings($query)
            ->count();

        $results = $query->offset($start)->limit($limit)->get();

        return response()->json([
            'message' => 'Get employees with missing documents successfully.',
            'data' => ['missing_documents' => $results],
            'count' => $count,
        ], 200);
    }

    public function absentEmployees(Request $request){

        $userTable = $this->getUserTable();
        $workspace_id = 0;
        $customer_id = 0;
        if ($userTable === "customer") {
            $customer_id = Auth::user()->id;
            $workspace_id = Auth::user()->current_workspace_id;  
            $authPersonalDetails = User::where('id', Auth::user()->id)->first();
        }
        if ($userTable === "emp") {
            $customer_id = auth()->user()->customer_id;
            $workspace_id = auth()->user()->workspace_id;
            $authPersonalDetails = EmpPersonalDetails::where('emp_id', Auth::user()->id)->first();  
        }

        $employees_ids = RosterAssign::join('emp_company_details','roster_assigns.assign_to','emp_company_details.id')
                            ->where('emp_company_details.del',0)
                            ->where('emp_company_details.status',1)
                            ->where('roster_assigns.schedule_date', $this->current_date)
                            ->where('roster_assigns.customer_id', $customer_id) 
                            ->where('roster_assigns.workspace_id', $workspace_id)
                            ->groupBy('roster_assigns.assign_to')
                            ->pluck('roster_assigns.assign_to')
                            ->toArray();
        $present_employee_ids = EmployeeAttendance::where('date',$this->current_date)
                                    ->wherein('employee_id',$employees_ids)
                                    ->where('customer_id', $customer_id)
                                    ->where('workspace_id', $workspace_id)
                                    ->groupBy('employee_id')
                                    ->pluck('employee_id')
                                    ->toArray();
        $leave_employee_ids = LeaveRequest::where('from',">=",$this->current_date)
                                ->where('to',"<=",$this->current_date)
                                ->where('status',1)
                                ->where('customer_id', $customer_id)
                                ->where('workspace_id', $workspace_id)
                                ->pluck('employee_id')
                                ->toArray();

        $absent_employee_id = array_values(array_diff($employees_ids,array_merge($present_employee_ids,$leave_employee_ids)));

$absent_employees = EmpCompanyDetails::join('emp_personal_details','emp_company_details.id','emp_personal_details.emp_id')
    ->join('roles','emp_company_details.access_role','roles.code')
    ->whereIn('emp_company_details.id', $absent_employee_id)
    ->where('emp_company_details.customer_id', $customer_id)
    ->where('emp_company_details.workspace_id', $workspace_id)
    ->select(
        'emp_company_details.employee_email',
        'emp_personal_details.*',
        'roles.title',
        'emp_company_details.access_role'
    )
    ->distinct(); // ✅ This is the only addition

        $count = $absent_employees->count();
        $start = $request->input('from', 0);
        $limit = 10;  // Set the limit for pagination
        $absent_employees->offset($start)->limit($limit);
    
        // Get the paginated results
        $query_result = $absent_employees->get();
    
        $data =  [
            'absent_employees' => $query_result,
        ];

        return response()->json([
            'message' => 'Get absent employees successfully.',
            'data' => $data,
            'count' => $count,
        ], 200);
    }

    public function presentEmployees(Request $request){
        // dd("Heloo buddy");
        $customer_id = auth()->user()->id;
        $workspace_id = auth()->user()->current_workspace_id;

        $present_employees = EmployeeAttendance::where('date',$this->current_date)
        ->where('customer_id', $customer_id)
        ->where('workspace_id', $workspace_id)
        ->with('present_employee_details','employee_site','present_employee_company_details');

        $count = $present_employees->count();
        $start = $request->input('from', 0);
        $limit = 10;  // Set the limit for pagination
        $present_employees->offset($start)->limit($limit);
    
        // Get the paginated results
        $query_result = $present_employees->get();
        
        foreach ($query_result as &$present) {
            $present['formatted_date'] = formatDate($present['date']);
            $present['formatted_check_in'] = formatTime($present['check_in']);
            $present['formatted_check_out'] = formatTime($present['check_out']);
        }
        
        $data =  [
            'today_present' => $query_result,
        ];

        return response()->json([
            'message' => 'Get employees with expiring documents successfully.',
            'data' => $data,
            'count' => $count,
        ], 200);
    }
}
