<?php

namespace App\Http\Controllers;

use DB;
use App\Models\User;
use App\Models\EmpTeam;
use Illuminate\Http\Request;
use App\Models\EmpWorkerType;
use App\Models\EmpPersonalDetails;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Validator;

class EmpWorkerTypeController extends Controller
{
    public function index()
    {
        $userTable = $this->getUserTable();
        $query = EmpWorkerType::query();
        if ($userTable === "customer") {
            $query->where('del', '0')->where('customer_id', auth()->id())
                ->where('workspace_id', auth()->user()->current_workspace_id);
        }
        if ($userTable === "emp") {
            $query->where('del', '0')->where('customer_id', auth()->user()->customer_id) // Assuming customer_id links employee to projects
                ->where('workspace_id', auth()->user()->workspace_id);
        }
        
        $emp_worker_types =  $query->get();

       return $this->success($emp_worker_types,'Worker Type Get Successfully');
    }

    public function create()
    {
        //
    }

    public function store(Request $request)
    {
        $user = auth()->user();
        $validator = Validator::make($request->all(), [
            'title' => 'required',
        ]);
        
        if ($validator->fails()) {

            $errors = $validator->errors();
            return $this->error($errors,422);

        } else {
          
           $validatedData =  $validator->validated();
           $userTable = $this->getUserTable();
           $auth_id =0;
           $workspace_id = 0;
           if ($userTable === "customer") {
               $auth_id = Auth::user()->id;
               $workspace_id = Auth::user()->current_workspace_id;  
               $authPersonalDetails = User::where('id', Auth::user()->id)->first();
               
               
           }

           if ($userTable === "emp") {
               $auth_id = auth()->user()->customer_id;
               $workspace_id = auth()->user()->workspace_id;
               $authPersonalDetails = EmpPersonalDetails::where('emp_id', Auth::user()->id)->first();  
           }

           $validatedData['customer_id'] = $auth_id;
           $validatedData['workspace_id'] = $workspace_id;
           $validatedData['created_by'] = Auth::user()->id;
           
           EmpWorkerType::insert($validatedData);

           $emp_worker_types =  EmpWorkerType::where('customer_id', $auth_id)
                                                ->where('workspace_id', $workspace_id)
                                                ->where('del', '0') 
                                                ->get();

           $data['emp_worker_types'] = $emp_worker_types;
    
         return $this->success($data,'Worker Type Saved Successfully');
        }        

    }

    public function show($id)
    {
        //
    }

    public function worker_type_edit(Request $request)
    {
        $id=$request->id;
        $userTable = $this->getUserTable();
        $emp_worker_type = EmpWorkerType::find($id);
        if (
            $userTable == "customer" && ($emp_worker_type->workspace_id != auth()->user()->current_workspace_id || $emp_worker_type->customer_id != auth()->user()->id)
        ) {
            return $this->message('You do not have access to this Employee worker Type', 403);
        }
        if ($userTable == "emp" && ($emp_worker_type->customer_id != auth()->user()->customer_id || $emp_worker_type->workspace_id != auth()->user()->workspace_id)) {
            
            return $this->message('You do not have access to this Employee worker Type', 403);
        }
        $data['emp_worker_types'] = $emp_worker_type;
 
        return $this->success($data, 'Worker Type Get Successfully');
    }

    public function worker_type_update(Request $request)
    {
        $user = auth()->user();
        $validator = Validator::make($request->all(), [
            'title' => 'required'
        ]);
        
        if ($validator->fails()) {

            $errors = $validator->errors();
            return $this->error($errors,422);

        } else {
            $validatedData =  $validator->validated();
            $id=$request->id;
            $emp_worker_types = EmpWorkerType::where('id', $id)->first();
            $userTable = $this->getUserTable();
        
            if ($userTable == "customer" && ($emp_worker_types->workspace_id != auth()->user()->current_workspace_id || $emp_worker_types->customer_id != auth()->user()->id)
            ) {
                return $this->message('You do not have access to this Employee worker Types', 403);
            }

            if ($userTable == "emp" && ($emp_worker_types->customer_id != auth()->user()->customer_id || $emp_worker_types->workspace_id != auth()->user()->workspace_id)) {
                return $this->message('You do not have access to this Employee worker Types', 403);
            }
            if (!$emp_worker_types) {
                return $this->notFound('Employee worker Types Not Found', 404);
            }
            $emp_worker_types->update($validatedData);
            if($userTable == "customer"){
                $data['emp_worker_types'] = EmpWorkerType::where('customer_id', $user->id)
                ->where('workspace_id', $user->current_workspace_id)
                ->where('del', '0')
                ->get();
            }

            elseif($userTable == "emp"){
                $data['emp_worker_types'] = EmpWorkerType::where('customer_id',auth()->user()->customer_id)
                ->where('workspace_id', auth()->user()->workspace_id)
                ->where('del', '0')
                ->get();
            }
               
            return $this->success($data,'Worker Type Updated Successfully');
        }        

    }

    public function worker_type_delete(Request $request)
    {
        $id=$request->id;
        $userTable = $this->getUserTable();
        $EmpWorkerType = EmpWorkerType::find($id);
        if ($userTable == "customer" && ($EmpWorkerType->workspace_id != auth()->user()->current_workspace_id || $EmpWorkerType->customer_id != auth()->user()->id)) {
            return $this->message('You do not have access to this Employee Worker', 403);
        }
        if ($userTable == "emp" && ($EmpWorkerType->customer_id != auth()->user()->customer_id || $EmpWorkerType->workspace_id != auth()->user()->workspace_id)) {
            return $this->message('You do not have access to this Employee Worker', 403);
        }
        $EmpWorkerType->del='1';
        $EmpWorkerType->save();
         
        $user = auth()->user();
        if($userTable == "customer"){
                $data['emp_worker_types'] = EmpWorkerType::where('customer_id', $user->id)
                ->where('workspace_id', $user->current_workspace_id)
                ->where('del', '0')
                ->get();
            }
            elseif($userTable == "emp"){
                $data['emp_worker_types'] = EmpWorkerType::where('customer_id',auth()->user()->customer_id)
                ->where('workspace_id', auth()->user()->workspace_id)
                ->where('del', '0')
                ->get();
            }
        return $this->success($data,'Worker Type Deleted Successfully');
    }
}
