<?php


namespace App\Http\Controllers;

use App\Models\Fund;
use App\Models\Role;
use App\Models\User;
use App\Models\EmpType;
use App\Models\EmpAccess;
use App\Models\EmpPermit;
use App\Models\FundsMeta;
use App\General\MetaClass;
use App\Models\EmpDocuments;
use Illuminate\Http\Request;
use App\Models\EmpTeamsMember;
use Illuminate\Validation\Rule;
use App\Models\EmpListChangeCol;
use App\Models\RequiredDocument;
use App\Models\EmpCompanyDetails;
use App\Models\EmpPersonalDetails;
use Illuminate\Support\Facades\DB;
use App\Models\EmpEmergencyContacts;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Hash;
use Illuminate\Support\Facades\Response;
use Illuminate\Support\Facades\Validator;



class EmployeeController extends Controller
{
    
public function index(Request $request)
{
    $user = Auth::user();
    if (!$user) {
        return $this->error('Unauthorized', 401);
    }

    // Corrected: removed ->query()
    $query = EmpCompanyDetails::withoutGlobalScope(\App\Scopes\NotDeletedScope::class);

    $userTable = $this->getUserTable();
    $auth_id = 0;
    $workspace_id = 0;

    if ($userTable === "customer") {
        $auth_id = $user->id;
        $workspace_id = $user->current_workspace_id;
        $authPersonalDetails = User::find($auth_id);
    }

    if ($userTable === "emp") {
        $auth_id = $user->customer_id;
        $workspace_id = $user->workspace_id;
        $authPersonalDetails = EmpPersonalDetails::where('emp_id', $user->id)->first();
    }

    // ❌ Removed this line because you're intentionally bypassing the 'del = 0' global scope
    // $query->where('del', '0');

    $query->where('customer_id', $auth_id)
          ->where('workspace_id', $workspace_id);

    if ($request->filled('invited')) {
        $query->where('invited', $request->invited);
    }

    if ($request->filled('filter')) {
        $filters = json_decode($request->filter, true);
        $query = $this->filter($filters, $query);
    }

    $query->with([
        'empPersonalDetails',
        'empDocuments',
        'empEmergencyContacts',
        'empAccess',
        'accessRole',
        'empTeamsMembers',
        'empMetaData',
        'funds_meta'
    ]);

    $count = $query->count();
    $start = $request->input('from', 0);
    $query->orderBy('id', 'desc');
    $query_result = $query->get();

    $users = $query_result;

    $array_filter = $request->filled('filter') ? json_decode($request->filter, true) : [];
    $filters = [
        'first_name'     => $array_filter['first_name'] ?? '',
        'middle_name'    => $array_filter['middle_name'] ?? '',
        'last_name'      => $array_filter['last_name'] ?? '',
        'employee_email' => $array_filter['employee_email'] ?? '',
        'completed'      => $array_filter['completed'] ?? '',
        'status'         => $array_filter['status'] ?? '',
        'access_role'    => $array_filter['access_role'] ?? '',
    ];

    return $this->WithFilter($users, $filters, 'Get Employees list Successfully');
}


    public function filter($filters,$query)
    {
    
        foreach ($filters as $filterName => $filterValue) {
              if ($filterValue != null ||  $filterValue !="" ) {
                  switch ($filterName) {
                  case 'first_name':
                    $query->whereHas('empPersonalDetails', function ($subquery) use ($filterValue) {
                    $subquery->where('first_name', 'like', '%' . $filterValue . '%');
                  });
                  break;
                  case 'middle_name':
                    $query->whereHas('empPersonalDetails', function ($subquery) use ($filterValue) {
                    $subquery->where('middle_name', 'like', '%' . $filterValue . '%');
                  });
                  break;
                  case 'last_name':
                    $query->whereHas('empPersonalDetails', function ($subquery) use ($filterValue) {
                    $subquery->where('last_name', 'like', '%' . $filterValue . '%');
                  });
                  break;
                  case 'employee_email':
                    $query->where('employee_email', 'like', '%' . $filterValue . '%');
                  break;
                  case 'streat_address':
                    $query->whereHas('empPersonalDetails', function ($subquery) use ($filterValue) {
                      $subquery->where('streat_address', 'like', '%' . $filterValue . '%');
                    });
                    break;
                  case 'mobile':
                    $query->whereHas('empPersonalDetails', function ($subquery) use ($filterValue) {
                      $subquery->where('mobile', 'like', '%' . $filterValue . '%');
                    });
                  break;
                  case 'tax_file_no':
                    $query->where('tax_file_no', 'like', '%' . $filterValue . '%');
                  break;
                  case 'bank_name':
                    $query->whereHas('empMetaData', function ($subquery) use ($filterValue) {
                      $subquery->where('option', 'bank_name')
                               ->where('value', 'like', '%' . $filterValue . '%');
                  });
                  break;
                  case 'account_number':
                    $query->whereHas('empMetaData', function ($subquery) use ($filterValue) {
                      $subquery->where('option', 'account_number')
                      ->where('value', 'like', '%' . $filterValue . '%');
                  });
                  break;

                  case 'teams':
                    $emp_teams_members = EmpTeamsMember::where('team_id', $filterValue)->pluck('emp_id')->toArray();
                    $query->whereIn('id', $emp_teams_members);
                  break;
                  case 'access_role':
                    $query->where('access_role', 'like', '%' . $filterValue . '%');
                  break;
                  case 'completed':
                      $filterValue =   $filterValue ? 1 : 0;
                      $query->where('compeleted', 'like', '%' . $filterValue . '%');
                  break;
                  case 'status':
                      $filterValue =   $filterValue ? 1 : 0;
                      $query->where('status', 'like', '%' . $filterValue . '%');
                  break;
                  case 'user_type':
                    $filterValue =   $filterValue ? 1 : 0;
                    $query->where('user_type', 'like', '%' . $filterValue . '%');
                  break;
                  case 'link_key':
                    $query->where('link_key', 'like', '%' . $filterValue . '%');
                  break;                        
                  }
              }
        }

        return $query;
    }

    public function empDownload(Request $request)
    {
        $query = EmpCompanyDetails::with(['empPersonalDetails','empDocuments','empEmergencyContacts','empAccess','accessRole','empTeamsMembers']);

        if($request->filled('invited'))
        {
          $query->where('invited',$request->invited);
        }
        
        if($request->filled('filter'))
        {
          $query = $this->filter(json_decode($request->filter, true),$query);
        }
        $userTable = $this->getUserTable();
        $auth_id =0;
        $workspace_id = 0;
        if ($userTable === "customer") {
            $auth_id = Auth::user()->id;
            $workspace_id = Auth::user()->current_workspace_id;  
            $authPersonalDetails = User::where('id', Auth::user()->id)->first(); 
        }
        if ($userTable === "emp") {
            $auth_id = auth()->user()->customer_id;
            $workspace_id = auth()->user()->workspace_id;
            $authPersonalDetails = EmpPersonalDetails::where('emp_id', Auth::user()->id)->first();  
        }
        $query->where('customer_id',$auth_id);
        $query->where('workspace_id',$workspace_id);
        $query->where('del','0');
        $query->orderBy('id', 'desc');
        $emps = $query->get();

        $cols = DB::table('emp_list_change_cols')->pluck('col')->toArray();
      
          $columns = [
              'First Name',
              'Middle Name',
              'Last Name',
              'Employee Email',
              'Teams',
              'Role',
              'Mobile',
              'Address',
              'Tax File No',
              'Account Number',
              'BSB',
              'Registration',
              'Status',
              'Type',
          ];

          $col_list[] = "Employee Id";
          $col_list[] = "External Id";
          $indexs = [];
          $i = 2;
          foreach ($columns as $column) {
            if (in_array($column, $cols))
            {
              $col_list[] = $column;
            }
            else
            {
              $indexs[] = $i;
            }
            $i++;
          }

          $csvData[] = $col_list;

          $data_list  = [];

          foreach ($emps as $emp) {
              $team_list = "";
              foreach ($emp->empTeamsMembers as $et) {
                  $team_list .= $et->empTeamsList->title . ",";
              }
              
              $team_list = rtrim($team_list, ',');
          
              $user_meta = DB::table('user_meta')
                              ->where('emp_id', $emp->id)
                              ->get();   
                                        
              $data_list = [
                $emp->id,
                $this->searchInMeta($user_meta,'employee_payroll_or_external_id'),
                $emp->empPersonalDetails ? $emp->empPersonalDetails->first_name : '',
                $emp->empPersonalDetails ? $emp->empPersonalDetails->middle_name : '',
                $emp->empPersonalDetails ? $emp->empPersonalDetails->last_name : '',
                $emp->employee_email,
                $team_list,
                $emp->accessRole ? $emp->accessRole->title : '',
                $emp->empPersonalDetails ? $emp->empPersonalDetails->mobile : '',
                $emp->empPersonalDetails ? $emp->empPersonalDetails->streat_address : '',
                $emp->tax_file_no,
                $this->searchInMeta($user_meta,'account_number'),
                $this->searchInMeta($user_meta,'bank_name'),
                $emp->registration == 0 ? 'InProcess' : 'Completed',
                $emp->status == 0 ? 'InActive' : 'Active',
                $emp->user_type == 0 ? 'Internal' : 'External'
              ];
          
              $csvData[] = array_values(array_diff_key($data_list, array_flip($indexs)));
            
          }
        $headers = [
            'Content-Type' => 'text/csv',
            'Content-Disposition' => 'attachment; filename="emp.csv"', 
        ];

        
        $callback = function () use ($csvData) {
            $file = fopen('php://output', 'w');
            foreach ($csvData as $row) {
                fputcsv($file, $row);
            }
            fclose($file);
        };

        return Response::stream($callback, 200, $headers);
    }

    function searchInMeta($dataArray, $option) {
      $value = '';
      foreach ($dataArray as $element) {
          if ($element->option === $option) {
              $value = $element->value;
              break;
          }
      }
      return $value;
  }
    public function profile(Request $request)
{
    // Check existence without global scope
    $exists = EmpCompanyDetails::withoutGlobalScope(\App\Scopes\NotDeletedScope::class)
        ->where('id', $request->id)
        ->exists();

    if (!$exists) {
        return $this->message('Employee not found');
    }

    $userTable = $this->getUserTable();

    // Get full employee record without global scope
    $employee = EmpCompanyDetails::withoutGlobalScope(\App\Scopes\NotDeletedScope::class)
        ->find($request->id);

    if (
        $userTable === "customer" &&
        ($employee->workspace_id != auth()->user()->current_workspace_id || $employee->customer_id != auth()->user()->id)
    ) {
        return $this->message('You do not have access to this Employee Profile', 403);
    }

    if (
        $userTable === "emp" &&
        ($employee->customer_id != auth()->user()->customer_id || $employee->workspace_id != auth()->user()->workspace_id)
    ) {
        return $this->message('You do not have access to this Employee Profile', 403);
    }

    // Build profile query without global scope
    $query = EmpCompanyDetails::withoutGlobalScope(\App\Scopes\NotDeletedScope::class)
        ->where('id', $request->id)
        ->with([
            'empPersonalDetails',
            'empEmergencyContacts',
            'empAccess',
            'empworkExperience',
            'workerType'
        ]);

    $query_result = $query->first();

    // Add related data
    $query_result['empType'] = EmpType::where('id', $query_result->employment_type)
        ->select('id', 'title')
        ->first();

    $query_result['empWorkPermit'] = EmpPermit::where('id', $query_result->work_permit_type)
        ->select('id', 'title')
        ->first();

    $query_result['empAccessRole'] = Role::where('code', $query_result->access_role)
        ->select('id', 'title', 'code')
        ->first();

    $data['profile'] = $query_result;

    // Funds
    $data['funds_meta'] = FundsMeta::with('fund')
        ->where('emp_id', $request->id)
        ->get();

    // Medical files
    $data['medicalAttachFiles'] = DB::table('medical_attach_files')
        ->where('emp_id', $request->id)
        ->get();

    // Employee documents
    $data['emp_documents'] = EmpDocuments::where('emp_id', $request->id)
        ->where('del', '0')
        ->get();

    // Required documents
    $data['required_documents'] = RequiredDocument::with('requiredDocumentField')
        ->where('customer_id', $employee->customer_id)
        ->where('workspace_id', $employee->workspace_id)
        ->where('del', '0')
        ->get();

    // Meta
    $user_meta = DB::table('user_meta')
        ->where('emp_id', $request->id)
        ->get();

    $employee_payroll_or_external_id = MetaClass::getOptionValue('employee_payroll_or_external_id', $user_meta);
    $data['meta']['employee_payroll_or_external_id'] = $employee_payroll_or_external_id;

    return $this->success($data, 'Get Profile Successfully');
}

    
    public function show($id)
    {
      //
    }

    public function edit($id)
    {
      //
    }

    public function update(Request $request, $id)
    {
      //
    }
    
    public function store(Request $request)
    {
      //
    }

    public function employe_delete(Request $request)
    {
      $id=$request->id;
      $userTable = $this->getUserTable();
        $EmpCompanyDetails = EmpCompanyDetails::withoutGlobalScope(\App\Scopes\NotDeletedScope::class)->find($id);
    
        if ($userTable == "customer" && ($EmpCompanyDetails->workspace_id != auth()->user()->current_workspace_id || $EmpCompanyDetails->customer_id != auth()->user()->id)) {
            return $this->message('You do not have access to this Employee ', 403);
        }

        if ($userTable == "emp" && ($EmpCompanyDetails->customer_id != auth()->user()->customer_id || $EmpCompanyDetails->workspace_id != auth()->user()->workspace_id)) {
            return $this->message('You do not have access to this Employee ', 403);
        }
        $EmpCompanyDetails->del='1';
        $EmpCompanyDetails->save();

        return $this->message('Employee  Deleted Successfully');
         
    }

  public function updateStatus(Request $request)
{
    $id = $request->id;
    $userTable = $this->getUserTable();
    
    $emp = EmpCompanyDetails::withoutGlobalScope(\App\Scopes\NotDeletedScope::class)
        ->where('id', $id)
        ->first();
        
    if (!$emp) {
        return $this->message('Employee not found', 404);
    }
    
    if (
        ($userTable === "customer" &&
            ($emp->customer_id != auth()->user()->id || $emp->workspace_id != auth()->user()->current_workspace_id)) ||
        ($userTable === "emp" &&
            ($emp->customer_id != auth()->user()->customer_id || $emp->workspace_id != auth()->user()->workspace_id))
    ) {
        return $this->message('You do not have access to this Employee', 403);
    }
    
    // Fix 1: Initialize $status variable and handle edge cases
    $status = null;
    if ($emp->status == 1) {
        $status = 0;
    } elseif ($emp->status == 0) {
        $status = 1;
    } else {
        // Handle case where status is neither 0 nor 1
        $status = 1; // Default to active
    }
    
    // Fix 2: Use the same scope and add error handling
    try {
        $updated = EmpCompanyDetails::withoutGlobalScope(\App\Scopes\NotDeletedScope::class)
            ->where('id', $id)
            ->update(['status' => $status]);
            
        // Fix 3: Check if update was successful
        if ($updated === 0) {
            return $this->message('Failed to update employee status', 500);
        }
        
        return $this->success(['status' => $status], 'Employee status updated successfully');
        
    } catch (\Exception $e) {
        // Fix 4: Add error logging
        \Log::error('Failed to update employee status: ' . $e->getMessage());
        return $this->message('Failed to update employee status', 500);
    }
}

    public function changeCol(Request $request)
    {
      $arr = [];
      $col = $request->col;
      if($col){
        DB::table('emp_list_change_cols')->delete();
        for($i = 0; $i < count($col); $i++)
        {
          $arr[$i]['col'] = $col[$i];
        }
        EmpListChangeCol::insert($arr);
      }
      return back();
    }

}
