<?php

namespace App\Http\Controllers;

use DB;
use App\Models\Fund;
use App\Models\User;
use Illuminate\Http\Request;
use App\Models\EmpPersonalDetails;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Validator;

class FundsController extends Controller
{

    public function index(Request $request)
    {
      
        $userTable = $this->getUserTable();
        $auth_id =0;
        $workspace_id = 0;
        if ($userTable === "customer") {
            $auth_id = Auth::user()->id;
            $workspace_id = Auth::user()->current_workspace_id;  
            $authPersonalDetails = User::where('id', Auth::user()->id)->first(); 
        }
        if ($userTable === "emp") {
            $auth_id = auth()->user()->customer_id;
            $workspace_id = auth()->user()->workspace_id;
            $authPersonalDetails = EmpPersonalDetails::where('emp_id', Auth::user()->id)->first();  
        }
        $query = Fund::query();
        $query->where('customer_id',  $auth_id)
        ->where('workspace_id',  $workspace_id)
        ->where('del', '0') 
        ->get();


        if($request->filled('filter'))
        {
            $query =  $this->filter(json_decode($request->filter, true),$query);
        }
        $count = $query->count();
        $start = $request->input('from', 0);
        $query->offset($start)->limit(20);
        $query->orderBy('id', 'desc');
        $query_result = $query->get();

        $i= 0 ;

        $funds =$query_result;

        $array_filter =  json_decode($request->filter, true);
        $filters = [
            'title' => $array_filter['title'] ?? '', 
            'percentage' => $array_filter['percentage'] ?? '',
            'usi' => $array_filter['usi'] ?? '',
            'usi_number' => $array_filter['usi_number'] ?? ''
        ];

        return $this->WithFilter($funds,$filters,'Get Funds List Successfully');
    }

    public function filter($filters,$query)
    {
     
        foreach ($filters as $filterName => $filterValue) {
              if ($filterValue != null ||  $filterValue !="" ) {
                  switch ($filterName) {
                  case 'title':
                      $query->where('title', 'like', '%' . $filterValue . '%');
                  break;
                  case 'percentage':
                      $query->where('percentage', 'like', '%' . $filterValue . '%');
                  break;
                  case 'usi':
                      $filterValue =   $filterValue ? 1 : 0;
                      $query->where('usi', 'like', '%' . $filterValue . '%');
                  break;
                  case 'usi_number':
                    $query->where('usi_number', 'like', '%' . $filterValue . '%');
                  break;                       
                  }
              }
        }

        return $query;
    }

    public function create()
    {
        //
    }

    public function store(Request $request)
    {
        $user = auth()->user();
        $validator = Validator::make($request->all(), [
            'title' => 'required',
            'percentage' => 'required|numeric|between:0,999999',
            'usi' => 'required',
            'usi_number' => 'required_if:usi,1|nullable',
        ],
        [
            'usi_number.required_if' => "The usi number field is required.",
        ]);
        
        if ($validator->fails()) {

            $error = $validator->errors()->first();
            return $this->error($error ,422);

        } else {
          
            $userTable = $this->getUserTable();
            $auth_id =0;
            $workspace_id = 0;
            if ($userTable === "customer") {
                $auth_id = Auth::user()->id;
                $workspace_id = Auth::user()->current_workspace_id;  
                $authPersonalDetails = User::where('id', Auth::user()->id)->first(); 
            }
            if ($userTable === "emp") {
                $auth_id = auth()->user()->customer_id;
                $workspace_id = auth()->user()->workspace_id;
                $authPersonalDetails = EmpPersonalDetails::where('emp_id', Auth::user()->id)->first();  
            }
           $validatedData =  $validator->validated();
           $validatedData['customer_id'] =  $auth_id;
           $validatedData['workspace_id'] =  $workspace_id;
           Fund::insert($validatedData);
    
           return $this->message('Funds save successfully');
        }        

    }

    public function show($id)
    {
        //
    }

    public function funds_edit(Request $request)
    {
        $id=$request->id;
        $userTable = $this->getUserTable();
        $Fund = Fund::find($id);
        if (
            $userTable == "customer" && ($Fund->workspace_id != auth()->user()->current_workspace_id || $Fund->customer_id != auth()->user()->id)
        ) {
            return $this->message('You do not have access to this Fund', 403);
        }
        if ($userTable == "emp" && ($Fund->customer_id != auth()->user()->customer_id || $Fund->workspace_id != auth()->user()->workspace_id)) {
            
            return $this->message('You do not have access to this Fund', 403);
        }
        $data['Funds'] = $Fund;
        return $this->success($data,'Fund get successfully');
    }

    public function funds_update(Request $request)
    {
        $user = auth()->user();
        $validator = Validator::make($request->all(), [
            'id' => 'required',
            'title' => 'required',
            'percentage' => 'required|numeric',
            'usi' => 'required',
            'usi_number' => 'required_if:usi,1|nullable',
        ],
        [
            'usi_number.required_if' => "The usi number field is required.",
        ]);
        
        if ($validator->fails()) {

            $error = $validator->errors()->first();
            return $this->error($error ,422);

        } else {

            $id=$request->id;
            $userTable = $this->getUserTable();
            $Fund = Fund::find($id);
            if (
                $userTable == "customer" && ($Fund->workspace_id != auth()->user()->current_workspace_id || $Fund->customer_id != auth()->user()->id)
            ) {
                return $this->message('You do not have access to this Fund', 403);
            }
            if ($userTable == "emp" && ($Fund->customer_id != auth()->user()->customer_id || $Fund->workspace_id != auth()->user()->workspace_id)) {
                
                return $this->message('You do not have access to this Fund', 403);
            }
            $validatedData =  $validator->validated();

            Fund::where('id', $id)->update($validatedData);

            return $this->message('Funds Updated Successfully');
        }        

    }

    public function funds_delete(Request $request)
    {
        $id=$request->id;
          $userTable = $this->getUserTable();
        $Fund = Fund::find($id);
        if ($userTable == "customer" && ($Fund->workspace_id != auth()->user()->current_workspace_id || $Fund->customer_id != auth()->user()->id)) {
            return $this->message('You do not have access to this Fund', 403);
        }
        if ($userTable == "emp" && ($Fund->customer_id != auth()->user()->customer_id || $Fund->workspace_id != auth()->user()->workspace_id)) {
            return $this->message('You do not have access to this Fund', 403);
        }
        $Fund->del='1';
        $Fund->save();

        return $this->message('Funds Deleted Successfully');
    }
}
