<?php

namespace App\Http\Controllers;

use DB;
use PDF;
use Illuminate\Support\Facades\File;
use Session;
use Carbon\Carbon;
use App\Models\Role;
use App\Models\Swms;
use App\Models\Sites;
use App\Models\EmpType;
use App\Models\Project;
use App\Models\Training;
use App\General\MetaClass;
// use Barryvdh\DomPDF\Facade\Pdf;
use Illuminate\Http\Request;
use App\Models\IncidentImage;
use App\Models\IncidentReport;
use App\Models\InspectionPlan;
use Illuminate\Validation\Rule;
use App\Models\EmpCompanyDetails;
use App\Models\EmpPersonalDetails;
use App\Models\GeneratedPdfReport;
use App\Models\IncidentClassificationReport;
use App\Models\IncidentDocument;
use App\Models\IncidentEmployeeInjuryType;
use App\Models\IncidentEmployerDetail;
use App\Models\IncidentFinalClassification;
use Spatie\Browsershot\Browsershot;
use Illuminate\Pagination\Paginator;
use Illuminate\Support\Facades\Hash;
use App\Models\IncidentReportQuestion;
use Illuminate\Support\Facades\Validator;
use App\Models\IncidentInjuryClassification;
use App\Models\IncidentInjuryManagement;
use App\Models\IncidentInvolvedPersonDetail;
use App\Models\IncidentInvolvedPersonTraining;
use Illuminate\Pagination\LengthAwarePaginator;
use App\Models\IncidentNotifiableClassification;
use App\Models\IncidentNotifiedTo;
use App\Models\IncidentReportedBy;
use App\Models\IncidentReportedTo;
use App\Models\IncidentReportMeta;
use App\Models\IncidentReportNotifiableClassification;
use App\Models\IncidentReportQuestionAnswer;
use App\Models\IncidentRiskCategoryDetail;
use App\Models\IncidentSignoff;
use App\Models\IncidentWitnes;
use App\Models\InspectionPlanSignature;
use App\Models\MeetingNoteSignature;
use App\Models\SwmsApprovedBy;
use App\Models\SwmsComplianceResponsible;
use App\Models\swmsDevelopedBy;
use App\Models\SwmsMonitoringResponsible;
use App\Models\SwmsReviewResponsible;
use App\Models\SwmsSignature;
use App\Models\WhsReport;
use App\Models\WhsSignature;

class IncidentReportController extends Controller
{
    public function index(Request $request)
    {
        // $auth_id = auth()->user()->id;
        $userTable = $this->getUserTable(); // Get whether user is a customer or employee
        $query = IncidentReport::query()
            ->orderBy('id', 'DESC')
            ->where('del', '0')
            ->with('authorisedBy');
        // Filter based on user type
        if ($userTable === 'customer') {
            $query->where('customer_id', auth()->id())
                ->where('workspace_id', auth()->user()->current_workspace_id);
        } elseif ($userTable === 'emp') {
            $query->where('customer_id', auth()->user()->customer_id)
                ->where('workspace_id', auth()->user()->workspace_id);
        }
    
        //  Restrict non-admin users to their assigned sites
        // if ($auth_id != 1) {
        //     $sites = Sites::where('sso_id', $auth_id)->pluck('id');
        //     $query->whereIn('site_id', $sites);
        // }
    
        // Apply filters if provided
        if ($request->filled('filter')) {
            $query = $this->filter(json_decode($request->filter, true), $query);
        }
        $start = $request->input('from', 0);
        $query->offset($start)->limit(20);
        $query_result = $query->get();
        return $this->withCount($query_result , 'Get Incident Reports List Successfully');
    }
    

    public function filter($filters, $query)
    {
        foreach ($filters as $filterName => $filterValue) {
            if ($filterValue != null ||  $filterValue != "") {
                switch ($filterName) {
                    case 'authorised_by':
                        $filterValue  = explode(" ", $filterValue);
                        if (count($filterValue) == 1) {
                            $query->whereHas('authorizedBy', function ($subquery) use ($filterValue) {
                                $subquery->where('first_name', 'like', '%' . $filterValue[0] . '%');
                            });
                        }
                        if (count($filterValue) == 2) {
                            $query->whereHas('authorizedBy', function ($subquery) use ($filterValue) {
                                $subquery->where('first_name', 'like', '%' . $filterValue[0] . '%');
                                $subquery->where('last_name', 'like', '%' . $filterValue[1] . '%');
                            });
                        }
                        if (count($filterValue) == 3) {

                            $query->whereHas('authorizedBy', function ($subquery) use ($filterValue) {
                                $subquery->where('first_name', 'like', '%' . $filterValue[0] . '%');
                                $subquery->where('middle_name', 'like', '%' . $filterValue[1] . '%');
                                $subquery->where('last_name', 'like', '%' . $filterValue[2] . '%');
                            });
                        }
                        break;
                    case 'title':
                        $query->where('title', 'like', '%' . $filterValue . '%');
                        break;
                    case 'document_number':
                        $query->where('document_number', 'like', '%' . $filterValue . '%');
                        break;
                    case 'revision_number':
                        $query->where('revision_number', 'like', '%' . $filterValue . '%');
                        break;
                    case 'issue_date':
                        $query->where('issue_date', 'like', '%' . $filterValue . '%');
                        break;
                    case 'process':
                        $query->where('process', 'like', '%' . $filterValue . '%');
                        break;
                    case 'published':
                        $query->where('is_published', 'like', '%' . $filterValue . '%');
                        break;
                }
            }
        }
        return $query;
    }
    public function storeStep1(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'incident_report_id' => 'required',
            'title' => 'required',
            'document_number' => 'nullable',
            'authorised_by' => 'nullable',
            'revision_number' => 'nullable',
            'issue_date' => 'nullable|date',
            'revision_date' => 'nullable|date|after_or_equal:issue_date',
            'site_id' => 'required',
        ]);
        if ($validator->fails()) {
            storeReportsLogs([
                'employee_id' => auth()->user()->id,
                'message' => $validator->errors()->first(),
                'report_id' => $request->incident_report_id,
                'report_type' => 'incident_report',
                'error_type' => 'validation error'
            ]);
            return $this->message($validator->errors()->first(), 422);
        }
        try {
            $validatedData = $validator->validated();
            $document_number = 'ITP-' . ($validatedData['document_number'] ?? '0');
            $userTable = $this->getUserTable(); // Get the user type (customer or emp)
            if ($userTable == "customer") {
                $customer_id = auth()->id();
                $workspace_id = auth()->user()->current_workspace_id;
            } elseif ($userTable == "emp") {
                $customer_id = auth()->user()->customer_id;
                $workspace_id = auth()->user()->workspace_id;
            }
            // Prepare data for incident report
            $incidentReportData = [
                'title' => $validatedData['title'],
                'document_number' => $document_number,
                'authorised_by' => $validatedData['authorised_by'] ?? 0,
                'revision_number' => $validatedData['revision_number'] ?? 0,
                'issue_date' => $validatedData['issue_date'] ?? null,
                'revision_date' => $validatedData['revision_date'] ?? null,
                'site_id' => $validatedData['site_id'] ?? null,
                'customer_id' => $customer_id,
                'workspace_id' => $workspace_id,
            ];
            $incidentReport = IncidentReport::find($validatedData['incident_report_id']);
            if (!$incidentReport) {
                // Validate issue & revision date only when creating a new report
                $validator = Validator::make($request->all(), [
                    'issue_date' => 'nullable|date|after_or_equal:today',
                    'revision_date' => 'nullable|date|after_or_equal:today|after_or_equal:issue_date',
                ]);
                if ($validator->fails()) {
                    storeReportsLogs([
                        'employee_id' => auth()->user()->id,
                        'message' => $validator->errors()->first(),
                        'report_id' => $request->incident_report_id,
                        'report_type' => 'incident_report',
                        'error_type' => 'validation error'
                    ]);
                    return $this->message($validator->errors()->first(), 422);
                }
                // Create new incident report
                $incidentReport = IncidentReport::create($incidentReportData);
                $validatedData['incident_report_id'] = $incidentReport->id;
                $msg = 'Report Created Successfully';
                storeReportsLogs([
                    'employee_id' => auth()->user()->id,
                    'message' => $msg . " with id " . $incidentReport->id,
                    'report_id' => $incidentReport->id,
                    'report_type' => 'incident_report'
                ]);
            } else {
                $current_revision_number = IncidentReport::where([
                    ['id', $validatedData['incident_report_id']],
                    ['document_number', $document_number],
                    ['del', 0]
                ])->value('revision_number');
                if ($current_revision_number != $validatedData['revision_number']) {
                    $existingRevision = IncidentReport::where([
                        ['id', '!=', $validatedData['incident_report_id']],
                        ['document_number', $document_number],
                        ['revision_number', $validatedData['revision_number']],
                        ['del', 0]
                    ])->exists();
                    if ($existingRevision) {
                        return $this->message('This version number for this document already exists.' ,422);
                    }
                    // Create a new revision (child report)
                    $incidentReportData['parent_number'] = $validatedData['incident_report_id'];
                    $incidentReportData['revision_number'] = $validatedData['revision_number'];
                    $childIncidentReport = IncidentReport::create($incidentReportData);
                    $validatedData['incident_report_id'] = $childIncidentReport->id;
                    if ($childIncidentReport) {
                        // Copy existing incident report data
                        $incidentReport->replicate()->fill([
                            'parent_number' => $incidentReport->id,
                            'revision_number' => $validatedData['revision_number']
                        ])->save();
                        // Copy related data to child report
                        $this->copyIncidentReport($incidentReport->id, $childIncidentReport->id);
                    }
                } else {
                    $incidentReport->update($incidentReportData);
                    $msg = 'Report Updated Successfully';
                }
            }
            // Logging
            storeReportsLogs([
                'employee_id' => auth()->user()->id,
                'message' => 'Step 1 ' . $msg,
                'report_id' => $validatedData['incident_report_id'],
                'report_type' => 'incident_report'
            ]);
            // Fetch employee access role
            $userAuthorizedBy = EmpCompanyDetails::with(['EmpPersonalDetails', 'accessRole', 'accessTier'])
                ->find($validatedData['authorised_by']);
            $accessRole = $userAuthorizedBy->accessRole->code ?? null;
            // Create or update incident signoff
            IncidentSignoff::updateOrCreate(
                [
                    'incident_report_id' => $validatedData['incident_report_id'],
                    'emp_id' => $validatedData['authorised_by'],
                ],
                [
                    'role_code' => $accessRole,
                    'emp_id' => $validatedData['authorised_by'],
                ]
            );
            return $this->success(['incident_report_id' => $validatedData['incident_report_id']], $msg);
        } catch (\Exception $e) {
            storeReportsLogs([
                'employee_id' => auth()->user()->id,
                'message' => 'Step 1 encountered an error: ' . $e->getMessage(),
                'report_id' => $validatedData['incident_report_id'],
                'report_type' => 'incident_report',
                'error_type' => 'exception error'
            ]);
            return $this->error('An error occurred: ' . $e->getMessage());
        }
    }
    public function editStep1(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'incident_report_id' => 'required'
        ]);
        if ($validator->fails()) {
            return $this->message($validator->errors()->first(), 422);
        }
        try {
            $validatedData = $validator->validated();
            // Fetch the incident report along with site data
            $incident_report = IncidentReport::with('site')->find($validatedData['incident_report_id']);
            if (!$incident_report) {
                return $this->message('Incident report not found', 404);
            }
            $userTable = $this->getUserTable();
            if ($userTable === 'customer' && ($incident_report->customer_id !== auth()->id() || $incident_report->workspace_id !== auth()->user()->current_workspace_id)) {
                return $this->message('You do not have access to this report.', 403);
            }
            if ($userTable === 'emp' && ($incident_report->customer_id !== auth()->user()->customer_id || $incident_report->workspace_id !== auth()->user()->workspace_id)) {
                return $this->message('You do not have access to this report.', 403);
            }
            // Fetch authorized employee list
            $emp_list = EmpCompanyDetails::where([
                ['compeleted', '1'],
                ['approved', '1'],
                ['status', '1'],
                ['del', '0']
            ])
                ->with([
                    'empPersonalDetails:emp_id,first_name,middle_name,last_name',
                    'accessRole:id,title,code',
                    'empTier:id,title',
                    'accessTier:id,title,tier_key'
                ])
                ->whereIn('tier_id', function ($query) {
                    $query->select('id')->from('tiers')->whereIn('tier_key', ['T-1', 'T-2']);
                })
                ->select('id', 'access_role', 'tier_id')
                ->get();
            $data['authorised_by_list'] = $emp_list;
            // Fetch sites
            // $siteQuery = Sites::where('active', '1')->select('id', 'title');
            // if ($validatedData['incident_report_id'] == 0) {
            //     $data['sites'] = $siteQuery->where('del', '0')->get();
            // } else {
            //     if ($incident_report && $incident_report->site_id) {
            //         $siteQuery->orWhere('id', $incident_report->site_id)->where('del', '0');
            //     }
            //     $data['sites'] = $siteQuery->get();
            // }
            $data['incident_report'] = $incident_report;
            return $this->success($data, 'Get Successfully');
        } catch (\Exception $e) {
            storeReportsLogs([
                'employee_id' => auth()->user()->id,
                'message' => 'Step 1 edit: ' . $e->getMessage(),
                'report_id' => $validatedData['incident_report_id'],
                'report_type' => 'incident_report',
                'error_type' => 'exception error'
            ]);
            return $this->error('An error occurred: ' . $e->getMessage());
        }
    }
    public function storeStep2(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'incident_report_id' => 'required|integer|exists:incident_reports,id',
            'investigation_number' => 'nullable|string',
            'office_number' => 'nullable|string',
            'project_id' => 'nullable|integer',
            'project_number' => 'nullable|integer',
            'report_compiled_by' => 'nullable|integer|max:255',
            'position_id' => 'nullable|integer',
            'date' => 'nullable|date',
            'incident_date' => 'nullable|date',
            'incident_time' => 'nullable|date_format:H:i',
            'date_incident_reported' => 'nullable|date|after_or_equal:incident_date',
            'time_incident_reported' => 'nullable',
            'reported_by' => 'nullable|integer|max:255',
            'company_name_by' => 'nullable|string|max:255',
            'position_by' => 'nullable|integer|max:255',
            'contact_number_by' => 'nullable',
            'reported_to' => 'nullable|integer|max:255',
            'company_name_to' => 'nullable|string|max:255',
            'position_to' => 'nullable|integer|max:255',
            'contact_number_to' => 'nullable',
            'injury_classification_id' => 'nullable|array|min:1',
            'injury_classification_id.*' => 'nullable|integer',
            'details' => 'nullable|string'
        ]);
        if ($validator->fails()) {
            storeReportsLogs([
                'employee_id' => auth()->user()->id,
                'message' => 'Step 2 An error occurred: ' . $validator->errors()->first(),
                'report_id' => $request->incident_report_id,
                'report_type' => 'incident_report',
                'error_type' => 'validation error'
            ]);
            return $this->message($validator->errors()->first(), 422);
        }
        try {
            $validatedData = $validator->validated();
            // Fetch the incident report
            $incidentReport = IncidentReport::find($validatedData['incident_report_id']);
            if (!$incidentReport) {
                return $this->message('Incident report not found', 404);
            }
            $userTable = $this->getUserTable();
            if ($userTable === 'customer' && ($incidentReport->customer_id !== auth()->id() || $incidentReport->workspace_id !== auth()->user()->current_workspace_id)) {
                return $this->message('You do not have access to this report.', 403);
            }
            if ($userTable === 'emp' && ($incidentReport->customer_id !== auth()->user()->customer_id || $incidentReport->workspace_id !== auth()->user()->workspace_id)) {
                return $this->message('You do not have access to this report.', 403);
            }
            // Update Injury Classification
            if (!empty($validatedData['injury_classification_id'])) {
                if (in_array(1, $validatedData['injury_classification_id']) && empty($validatedData['details'])) {
                    storeReportsLogs([
                        'employee_id' => auth()->user()->id,
                        'message' => 'Step 2 The Details field is required when injury classification is Other',
                        'report_id' => $validatedData['incident_report_id'],
                        'report_type' => 'incident_report'
                    ]);
                    return $this->message('The Details field is required when injury classification is Other', 422);
                }
                // Delete existing classifications for this report
                IncidentClassificationReport::where([
                    'incident_report_id' => $validatedData['incident_report_id'],
                    'type' => '0'
                ])->delete();
                // Insert new classifications
                $incidentClassifications = array_map(function ($c_id) use ($validatedData) {
                    return [
                        "incident_report_id" => $validatedData['incident_report_id'],
                        "injury_classification_id" => $c_id,
                        "details" => $c_id == 1 ? $validatedData['details'] : "", // if selected option is Other
                        "type" => "0"
                    ];
                }, $validatedData['injury_classification_id']);
                IncidentClassificationReport::insert($incidentClassifications);
            }
            // Update Incident Report Data
            $incidentReport->update(array_filter([
                'investigation_number' => $validatedData['investigation_number'] ?? null,
                'office_number' => $validatedData['office_number'] ?? null,
                'project_id' => $validatedData['project_id'] ?? null,
                'project_number' => $validatedData['project_number'] == 'undefined' ? 0 : $validatedData['project_number'],
                'report_compiled_by' => $validatedData['report_compiled_by'] ?? null,
                'position_id' => $validatedData['position_id'] ?? null,
                'date' => $validatedData['date'] ?? null,
                'incident_date' => $validatedData['incident_date'] ?? null,
                'incident_time' => $validatedData['incident_time'] ?? null,
                'date_incident_reported' => $validatedData['date_incident_reported'] ?? null,
                'time_incident_reported' => $validatedData['time_incident_reported'] ?? null
            ]));
            // Update Reported To Data
            if (!empty($validatedData['reported_to']) || !empty($validatedData['company_name_to']) || !empty($validatedData['position_to']) || !empty($validatedData['contact_number_to'])) {
                IncidentReportedTo::where('incident_report_id', $validatedData['incident_report_id'])->delete();
                IncidentReportedTo::create(array_filter([
                    'incident_report_id'  => $validatedData['incident_report_id'],
                    'reported_to'  => $validatedData['reported_to'] ?? null,
                    'company_name'  => $validatedData['company_name_to'] ?? null,
                    'position'  => $validatedData['position_to'] ?? null,
                    'contact_number'  => $validatedData['contact_number_to'] ?? null
                ]));
            }
            // Update Reported By Data
            if (!empty($validatedData['reported_by']) || !empty($validatedData['company_name_by']) || !empty($validatedData['position_by']) || !empty($validatedData['contact_number_by'])) {
                IncidentReportedBy::where('incident_report_id', $validatedData['incident_report_id'])->delete();
                IncidentReportedBy::create(array_filter([
                    'incident_report_id'  => $validatedData['incident_report_id'],
                    'reported_by'  => $validatedData['reported_by'] ?? null,
                    'company_name'  => $validatedData['company_name_by'] ?? null,
                    'position'  => $validatedData['position_by'] ?? null,
                    'contact_number'  => $validatedData['contact_number_by'] ?? null
                ]));
            }
            // Set success message based on injury classification presence
            $msg = IncidentClassificationReport::where(['incident_report_id' => $validatedData['incident_report_id'], 'type' => '0'])->exists()
                ? 'Updated Successfully'
                : 'Saved Successfully';
            storeReportsLogs([
                'employee_id' => auth()->user()->id,
                'message' => 'Step 2 ' . $msg,
                'report_id' => $request->incident_report_id,
                'report_type' => 'incident_report'
            ]);
            return $this->success(['incident_report_id' => $validatedData['incident_report_id']], $msg);
        } catch (\Exception $e) {
            storeReportsLogs([
                'employee_id' => auth()->user()->id,
                'message' => 'Step 2 store: ' . $e->getMessage(),
                'report_id' => $validatedData['incident_report_id'],
                'report_type' => 'incident_report',
                'error_type' => 'Exception error'
            ]);
            return $this->error($e->getMessage());
        }
    }

    public function editStep2(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'incident_report_id' => 'required|integer|exists:incident_reports,id'
        ]);
        if ($validator->fails()) {
            storeReportsLogs([
                'employee_id' => auth()->user()->id,
                'message' => 'Step 2 Error occurred: ' . $validator->errors()->first(),
                'report_id' => $request->incident_report_id,
                'report_type' => 'incident_report',
                'error_type' => 'validation error'
            ]);
            return $this->message($validator->errors()->first(), 422);
        }
        try {
            $validatedData = $validator->validated();
            // Fetch Incident Report with related data
            $incident_report = IncidentReport::with(['incidentClassificationReports' => function ($query) {
                $query->where('type', '0');
            }])->find($validatedData['incident_report_id']);
            $userTable = $this->getUserTable();
            if ($userTable === 'customer' && ($incident_report->customer_id !== auth()->id() || $incident_report->workspace_id !== auth()->user()->current_workspace_id)) {
                return $this->message('You do not have access to this report.', 403);
            }
            if ($userTable === 'emp' && ($incident_report->customer_id !== auth()->user()->customer_id || $incident_report->workspace_id !== auth()->user()->workspace_id)) {
                return $this->message('You do not have access to this report.', 403);
            }

            // Fetch Related Incident Injury Classifications
            // $incident_injury_classification_list = IncidentInjuryClassification::where('status', '1')
            //     ->where('del', '0')
            //     ->orderBy('id', 'DESC')
            //     ->get();
            // Fetch Active Projects
            // $project_list = Project::where('active', '1')
            //     ->where('is_deleted', '0')
            //     ->get();
            // Fetch Employee List for various roles
            // $emp_list = EmpCompanyDetails::where([
            //         ['compeleted', '1'],
            //         ['approved', '1'],
            //         ['status', '1'],
            //         ['del', '0']
            //     ])
            //     ->with([
            //         'empPersonalDetails:emp_id,first_name,middle_name,last_name',
            //         'accessRole:id,title,code',
            //         'empTier:id,title',
            //         'accessTier:id,title,tier_key'
            //     ])
            //     ->whereIn('tier_id', function ($query) {
            //         $query->select('id')->from('tiers')
            //             ->whereIn('tier_key', ['T-1', 'T-2']);
            //     })
            //     ->select('id', 'access_role', 'tier_id')
            //     ->get();
            // Fetch Employees for `compiled_list`
            // $compiled_list = EmpCompanyDetails::where([
            //         ['compeleted', '1'],
            //         ['approved', '1'],
            //         ['status', '1'],
            //         ['del', '0']
            //     ])
            //     ->with([
            //         'empPersonalDetails:emp_id,first_name,middle_name,last_name',
            //         'accessRole:id,title,code',
            //         'empTier:id,title',
            //         'accessTier:id,title,tier_key'
            //     ])
            //     ->whereIn('tier_id', function ($query) {
            //         $query->select('id')->from('tiers')
            //             ->whereIn('tier_key', ['T-1', 'T-2', 'T-3', 'T-4', 'T-5']);
            //     })
            //     ->select('id', 'access_role', 'tier_id')
            //     ->get();
            // Fetch Employees for `incident_reported_by_list`
            // $incident_reported_by_list = EmpCompanyDetails::where([
            //         ['compeleted', '1'],
            //         ['approved', '1'],
            //         ['status', '1'],
            //         ['del', '0']
            //     ])
            //     ->with([
            //         'empPersonalDetails:emp_id,first_name,middle_name,last_name',
            //         'accessRole:id,title,code',
            //         'empTier:id,title',
            //         'accessTier:id,title,tier_key'
            //     ])
            //     ->whereIn('tier_id', function ($query) {
            //         $query->select('id')->from('tiers')
            //             ->whereIn('tier_key', ['T-1', 'T-2', 'T-3', 'T-4', 'T-5', 'T-6']);
            //     })
            //     ->select('id', 'access_role', 'tier_id')
            //     ->get();

            // Fetch Reported By & Reported To Data
            $incident_reported_by = IncidentReportedBy::where('incident_report_id', $validatedData['incident_report_id'])->first();
            $incident_reported_to = IncidentReportedTo::where('incident_report_id', $validatedData['incident_report_id'])->first();

            // Assign additional data to incident report object
            $incident_report->incident_reported_by = $incident_reported_by;
            $incident_report->incident_reported_to = $incident_reported_to;

            // Prepare Response Data
            $data = [
                'incident_report' => $incident_report,
                // 'incident_injury_classification_list' => $incident_injury_classification_list,
                // 'report_compiled_list' => $compiled_list,
                // 'incident_reported_by_list' => $incident_reported_by_list,
                // 'incident_reported_to_list' => $emp_list,
                // 'project_list' => $project_list
            ];

            return $this->success($data, 'Data retrieved successfully.');
        } catch (\Exception $e) {
            storeReportsLogs([
                'employee_id' => auth()->user()->id,
                'message' => 'Step 2 Edit Error: ' . $e->getMessage(),
                'report_id' => $validatedData['incident_report_id'],
                'report_type' => 'incident_report',
                'error_type' => 'exception error'
            ]);
            return $this->error($e->getMessage());
        }
    }

    public function storeStep3(Request $request)
    {
        $dateValidation = ['required_if:is_notified_to,0', 'nullable'];
        $validator = Validator::make($request->all(), [
            'incident_report_id' => 'required|exists:incident_reports,id',
            'is_notified' => 'integer',
            'notifiable_classification_id' => 'required_if:is_notified,0',
            'is_notified_to' => 'required',
            'is_victim' => 'nullable',
            'notified_to' => ['required_if:is_notified_to,1', 'integer'],
            'ohs_regulator' => 'required_if:is_notified_to,0',
            'reference_number' => ['nullable', 'integer'],
            'date' => $dateValidation,
            'time' => 'required_if:is_notified_to,0',
            'contact_name' => 'required_if:is_notified_to,0',
            'involved_persons' => 'required_if:is_victim,0'
        ], [
            'incident_report_id.required' => 'Incident Report ID is required.',
            'is_notified.required_unless' => 'Please specify if the incident was notified.',
            'is_notified_to.required' => 'Please specify if the incident was notified to someone.',
            'notified_to.required_if' => 'The field "Notified To" is required when "Is Notified To" is checked.',
            'notified_to.integer' => 'The "Notified To" field must be an integer.',
            'ohs_regulator.required_if' => 'OHS Regulator is required when applicable indication is not checked.',
            'reference_number.integer' => 'Reference Number must be an integer.',
            'date.required_if' => 'Date is required when applicable indication is not checked.',
            'time.required_if' => 'Time is required when applicable indication is not checked.',
            'notifiable_classification_id.required_if' => 'The Classification of Incident must be selected when applicable indication is not checked.',
            'contact_name.required_if' => 'Contact Name is required when applicable indication is not checked.',
            'involved_persons.required_if' => 'Involved persons are required.'
        ]);
        if ($validator->fails()) {
            storeReportsLogs([
                'employee_id' => auth()->user()->id,
                'message' => 'Step 3 Error: ' . $validator->errors()->first(),
                'report_id' => $request->incident_report_id,
                'report_type' => 'incident_report',
                'error_type' => 'validation error'
            ]);
            return $this->message($validator->errors()->first(), 422);
        }
        try {
            $validatedData = $validator->validated();
            $incidentReport = IncidentReport::find($validatedData['incident_report_id']);
            if (!$incidentReport) {
                storeReportsLogs([
                    'employee_id' => auth()->user()->id,
                    'message' => 'Step 3 Incident Report Record Not Found',
                    'report_id' => $request->incident_report_id,
                    'report_type' => 'incident_report',
                    'error_type' => 'Not found error'
                ]);
                return $this->message('Incident Report Record Not Found', 404);
            }
            $userTable = $this->getUserTable();
            if ($userTable === 'customer' && ($incidentReport->customer_id !== auth()->id() || $incidentReport->workspace_id !== auth()->user()->current_workspace_id)) {
                return $this->message('You do not have access to this report.', 403);
            }
            if ($userTable === 'emp' && ($incidentReport->customer_id !== auth()->user()->customer_id || $incidentReport->workspace_id !== auth()->user()->workspace_id)) {
                return $this->message('You do not have access to this report.', 403);
            }
            // Update step number if no notifiable classifications exist
            if (!IncidentReportNotifiableClassification::where('incident_report_id', $validatedData['incident_report_id'])->exists()) {
                $incidentReport->update(['step_no' => 4]);
                $msg = 'Saved Successfully';
            } else {
                $msg = 'Updated Successfully';
            }
            // Update Incident Report
            $incidentReport->update([
                'is_notified' => $validatedData['is_notified'],
                'is_notified_to' => $validatedData['is_notified_to'],
                'is_victim' => $validatedData['is_victim'] ?? null,
                'involved_persons' => $validatedData['involved_persons'] ?? null
            ]);
            // Update or Insert Notifiable Classification
            if (isset($validatedData['notifiable_classification_id'])) {
                IncidentReportNotifiableClassification::updateOrCreate(
                    ['incident_report_id' => $validatedData['incident_report_id']],
                    ['notifiable_classification_id' => $validatedData['notifiable_classification_id']]
                );
            }
            // Update or Insert Incident Notified To
            IncidentNotifiedTo::updateOrCreate(
                ['incident_report_id' => $validatedData['incident_report_id']],
                [
                    'notified_to' => $validatedData['notified_to'] ?? null,
                    'ohs_regulator' => $validatedData['ohs_regulator'] ?? null,
                    'reference_number' => $validatedData['reference_number'] ?? 0,
                    'date' => $validatedData['date'] ?? null,
                    'time' => $validatedData['time'] ?? null,
                    'contact_name' => $validatedData['contact_name'] ?? null
                ]
            );
            storeReportsLogs([
                'employee_id' => auth()->user()->id,
                'message' => 'Step 3 ' . $msg,
                'report_id' => $request->incident_report_id,
                'report_type' => 'incident_report'
            ]);
            return $this->success(['incident_report_id' => $validatedData['incident_report_id']], $msg);
        } catch (\Exception $e) {
            storeReportsLogs([
                'employee_id' => auth()->user()->id,
                'message' => 'Step 3 Store Error: ' . $e->getMessage(),
                'report_id' => $validatedData['incident_report_id'],
                'report_type' => 'incident_report',
                'error_type' => 'Exception error'
            ]);
            return $this->error('Error: ' . $e->getMessage());
        }
    }

    public function editStep3(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'incident_report_id' => 'required|exists:incident_reports,id'
        ]);
        if ($validator->fails()) {
            return $this->message($validator->errors()->first(), 422);
        }
        try {
            $validatedData = $validator->validated();
            // Fetch Incident Report with related data
            $incidentReport = IncidentReport::with(['notifiableClassification', 'notifiedTos'])
                ->where('id', $validatedData['incident_report_id'])
                ->firstOrFail();
            if (!$incidentReport) {
                return $this->message('Incident Report Record Not Found', 404);
            }
            $userTable = $this->getUserTable();
            if ($userTable === 'customer' && ($incidentReport->customer_id !== auth()->id() || $incidentReport->workspace_id !== auth()->user()->current_workspace_id)) {
                return $this->message('You do not have access to this report.', 403);
            }
            if ($userTable === 'emp' && ($incidentReport->customer_id !== auth()->user()->customer_id || $incidentReport->workspace_id !== auth()->user()->workspace_id)) {
                return $this->message('You do not have access to this report.', 403);
            }
            // Fetch Notifiable Classification List
            $incidentNotifiableClassificationList = IncidentNotifiableClassification::where('status', '1')
                ->where('del', '0')
                ->get();
            // Construct response data
            $data = [
                'incident_report' => $incidentReport,
                'incident_notifiable_classification_list' => $incidentNotifiableClassificationList
            ];
            return $this->success($data, 'Data Retrieved Successfully');
        } catch (\Exception $e) {
            storeReportsLogs([
                'employee_id' => auth()->user()->id,
                'message' => 'Step 3 edit Error: ' . $e->getMessage(),
                'report_id' => $validatedData['incident_report_id'],
                'report_type' => 'incident_report',
                'error_type' => 'Exception error'
            ]);
            return $this->error($e->getMessage());
        }
    }

    public function storeStep4(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'incident_report_id' => 'required|exists:incident_reports,id',
            'id' => 'nullable|exists:incident_involved_person_details,id',
            'employee_id' => 'nullable|exists:emp_personal_details,emp_id',
            'surname' => 'nullable|string|max:255',
            'contact_number' => 'nullable|string',
            'gender' => 'nullable|integer',
            'dob' => [
                'nullable',
                'date',
                function ($attribute, $value, $fail) {
                    if ($value && Carbon::parse($value)->isAfter(Carbon::now()->subYears(10))) {
                        $fail("The {$attribute} must be at least 10 years old.");
                    }
                },
            ],
            'age' => 'nullable|integer',
            'occupation' => 'nullable|string|max:255',
            'language' => 'nullable|string|max:255',
            'supervisor' => 'nullable|string|max:255',
            'experience_years' => 'nullable|integer|min:0',
            'experience_months' => 'nullable|integer|min:0|max:11',
            'is_employer_worker' => 'nullable|string',
            'is_employer_worker_details' => 'nullable|string',
            'employment_type' => 'nullable|integer|max:255',
            'employee_type' => 'nullable|string|max:255',
            'training_id' => 'nullable|exists:trainings,id',
            'nature_of_injury' => 'nullable|string|max:255',
            'bodily_location' => 'nullable|string|max:255',
            'nature_of_disease' => 'nullable|string|max:255',
            'breakdown_of_injury' => 'nullable|string|max:255',
            'mechanism_of_injury' => 'nullable|string|max:255',
        ]);
        if ($validator->fails()) {
            storeReportsLogs([
                'employee_id' => auth()->user()->id,
                'message' => 'Step 4 Validation Error: ' . $validator->errors()->first(),
                'report_id' => $request->incident_report_id,
                'report_type' => 'incident_report',
                'error_type' => 'validation error'
            ]);
            return $this->message($validator->errors()->first(), 422);
        }
        try {
            $validatedData = $validator->validated();
            $incidentReport = IncidentReport::find($validatedData['incident_report_id']);
            if (!$incidentReport) {
                return $this->message('Incident Report Record Not Found', 404);
            }
            $userTable = $this->getUserTable();
            if ($userTable === 'customer' && ($incidentReport->customer_id !== auth()->id() || $incidentReport->workspace_id !== auth()->user()->current_workspace_id)) {
                return $this->message('You do not have access to this report.', 403);
            }
            if ($userTable === 'emp' && ($incidentReport->customer_id !== auth()->user()->customer_id || $incidentReport->workspace_id !== auth()->user()->workspace_id)) {
                return $this->message('You do not have access to this report.', 403);
            }
            // Check if the employee is already involved
            $existingPerson = IncidentInvolvedPersonDetail::where([
                'incident_report_id' => $validatedData['incident_report_id'],
                'employee_id' => $validatedData['employee_id']
            ])->first();

            if (!$validatedData['id'] && $existingPerson) {
                storeReportsLogs([
                    'employee_id' => auth()->user()->id,
                    'message' => 'Step 4 This person is already involved',
                    'report_id' => $validatedData['incident_report_id'],
                    'report_type' => 'incident_report'
                ]);
                return $this->message('This person is already involved' ,400);
            }
            // Store or Update Involved Person Details
            $involvedPerson = IncidentInvolvedPersonDetail::updateOrCreate(
                ['id' => $validatedData['id'] ?? null],
                [
                    'incident_report_id' => $validatedData['incident_report_id'],
                    'employee_id' => $validatedData['employee_id'] ?? 0,
                    'surname' => $validatedData['surname'],
                    'contact_number' => $validatedData['contact_number'],
                    'gender' => $validatedData['gender'],
                    'dob' => $validatedData['dob'],
                    'age' => $validatedData['age'] ?? 0,
                    'occupation' => $validatedData['occupation'],
                    'language' => $validatedData['language'],
                    'experience_years' => $validatedData['experience_years'] ?? 0,
                    'experience_months' => $validatedData['experience_months'] ?? 0,
                    'employment_type' => $validatedData['employment_type'],
                    'employee_type' => $validatedData['employee_type']
                ]
            );
            // Store Training Details
            if ($validatedData['training_id']) {
                IncidentInvolvedPersonTraining::updateOrCreate(
                    [
                        'incident_involved_person_id' => $involvedPerson->id,
                        'incident_report_id' => $validatedData['incident_report_id']
                    ],
                    ['training_id' => $validatedData['training_id']]
                );
            }
            // Store Injury Types
            $injuryOptions = ['nature_of_injury', 'bodily_location', 'nature_of_disease', 'breakdown_of_injury', 'mechanism_of_injury'];
            foreach ($injuryOptions as $option) {
                if (!empty($validatedData[$option])) {
                    IncidentEmployeeInjuryType::updateOrCreate(
                        [
                            'incident_report_id' => $validatedData['incident_report_id'],
                            'employer_id' => $validatedData['employee_id'] ?? 0,
                            'option' => $option
                        ],
                        ['value' => $validatedData[$option]]
                    );
                }
            }
            // Store Employer Details
            IncidentEmployerDetail::updateOrCreate(
                [
                    'incident_report_id' => $validatedData['incident_report_id'],
                    'employer_id' => $validatedData['employee_id'] ?? 0
                ],
                [
                    'supervisor' => $validatedData['supervisor'],
                    'is_employer_worker' => $validatedData['is_employer_worker'],
                    'is_employer_worker_details' => $validatedData['is_employer_worker_details']
                ]
            );
            // Update step number if it's a new entry
            if (!$validatedData['id']) {
                $incidentReport->update(['step_no' => 5]);
            }
            storeReportsLogs([
                'employee_id' => auth()->user()->id,
                'message' => 'Step 4 ' . ($validatedData['id'] ? 'Updated' : 'Saved') . ' Successfully',
                'report_id' => $validatedData['incident_report_id'],
                'report_type' => 'incident_report'
            ]);
            $msg = $validatedData['id'] ? 'Updated Successfully' : 'Saved Successfully';
            return $this->success(['incident_report_id' => $validatedData['incident_report_id']], $msg);
        } catch (\Exception $e) {
            storeReportsLogs([
                'employee_id' => auth()->user()->id,
                'message' => 'Step 4 Store Error: ' . $e->getMessage(),
                'report_id' => $validatedData['incident_report_id'],
                'report_type' => 'incident_report',
                'error_type' => 'Exception error'
            ]);
            return $this->error('Error: ' . $e->getMessage());
        }
    }

    public function editStep4(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'incident_report_id'  => 'required|exists:incident_reports,id',
            'employee_id' => 'required|exists:emp_personal_details,emp_id'
        ]);
        if ($validator->fails()) {
            storeReportsLogs([
                'employee_id' => auth()->user()->id,
                'message' => 'Step 4 Validation Error: ' . $validator->errors()->first(),
                'report_id' => $request->incident_report_id,
                'report_type' => 'incident_report',
                'error_type' => 'Validation error'
            ]);
            return $this->message($validator->errors()->first(), 422);
        }
        try {
            $validatedData = $validator->validated();
            // Fetch Incident Report and apply access control check
            $incidentReport = IncidentReport::find($validatedData['incident_report_id']);
            if (!$incidentReport) {
                return $this->message('Incident Report Record Not Found', 404);
            }
            // Access Control Check
            $userTable = $this->getUserTable();
            if (
                ($userTable === 'customer' && ($incidentReport->customer_id !== auth()->id() || $incidentReport->workspace_id !== auth()->user()->current_workspace_id)) ||
                ($userTable === 'emp' && ($incidentReport->customer_id !== auth()->user()->customer_id || $incidentReport->workspace_id !== auth()->user()->workspace_id))
            ) {
                return $this->message('You do not have access to this report.', 403);
            }
            // Fetch Incident Report with all related data using ORM
            $incidentReport->load([
                'involvedPersons' => function ($query) use ($validatedData) {
                    $query->where('employee_id', $validatedData['employee_id']);
                },
                'involvedTrainings' => function ($query) use ($validatedData) {
                    $query->where('incident_involved_person_id', $validatedData['employee_id']);
                },
                'injuryTypes' => function ($query) use ($validatedData) {
                    $query->where('employer_id', $validatedData['employee_id']);
                },
                'employerDetails' => function ($query) use ($validatedData) {
                    $query->where('employer_id', $validatedData['employee_id']);
                }
            ]);
            return $this->success($incidentReport, 'Get Successfully');
        } catch (\Exception $e) {
            $shortMessage = substr($e->getMessage(), 0, 200) . (strlen($e->getMessage()) > 200 ? '...' : '');
            storeReportsLogs([
                'employee_id' => auth()->user()->id,
                'message' => 'Step 4 Retrieval Error: ' . $shortMessage,
                'report_id' => $validatedData['incident_report_id'],
                'report_type' => 'incident_report',
                'error_type' => 'Exception error'
            ]);
            return $this->error($shortMessage);
        }
    }

    public function listingStep4(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'incident_report_id' => 'required|exists:incident_reports,id',
            'listing' => 'required|boolean',
            'fields_list' => 'required|boolean'
        ]);
        if ($validator->fails()) {
            storeReportsLogs([
                'employee_id' => auth()->user()->id,
                'message' => 'Step 4 Validation Error: ' . $validator->errors()->first(),
                'report_id' => $request->incident_report_id,
                'report_type' => 'incident_report',
                'error_type' => 'Validation error'
            ]);
            return $this->message($validator->errors()->first(), 422);
        }
        try {
            $validatedData = $validator->validated();
            $incidentReport = IncidentReport::find($validatedData['incident_report_id']);
            if (!$incidentReport) {
                return $this->message('Incident Report Record Not Found', 404);
            }
            // Access Control Check
            $userTable = $this->getUserTable();
            if (
                ($userTable === 'customer' && ($incidentReport->customer_id !== auth()->id() || $incidentReport->workspace_id !== auth()->user()->current_workspace_id)) ||
                ($userTable === 'emp' && ($incidentReport->customer_id !== auth()->user()->customer_id || $incidentReport->workspace_id !== auth()->user()->workspace_id))
            ) {
                return $this->message('You do not have access to this report.', 403);
            }
            $data = [];
            // **Fetching Involved Persons Listing**
            if ($validatedData['listing']) {
                $incidentInvolvedPersons = IncidentInvolvedPersonDetail::where('incident_report_id', $validatedData['incident_report_id'])
                    ->with([
                        'empCompanyDetails:id,tier_id',
                        'empCompanyDetails.empTier:id,title',
                        'employee:emp_id,first_name,middle_name,last_name' // Changed 'empPersonalDetails' to 'employee'
                    ])
                    ->get(['id', 'incident_report_id', 'employee_id']);
                $data['incident_involved_person_details'] = $incidentInvolvedPersons;
            }
            // need to improve this query according to customer id 
            if ($validatedData['fields_list']) {
                $employeesList = EmpCompanyDetails::where([
                    ['compeleted', '1'],
                    ['approved', '1'],
                    ['status', '1'],
                    ['del', '0']
                ])
                    ->with([
                        'empPersonalDetails:emp_id,first_name,middle_name,last_name',
                        'empTier:id,title'
                    ])
                    ->select('id', 'tier_id')
                    ->get();

                $supervisorList = EmpCompanyDetails::where([
                    ['compeleted', '1'],
                    ['approved', '1'],
                    ['status', '1'],
                    ['del', '0'],
                    ['access_role', 'spv']
                ])
                    ->with([
                        'empPersonalDetails:emp_id,first_name,middle_name,last_name',
                        'accessRole:id,title,code',
                        'empTier:id,title'
                    ])
                    ->select('id', 'access_role', 'tier_id')
                    ->get();
                $data['employees_list'] = $employeesList;
                $data['supervisor_list'] = $supervisorList;
                $data['type_of_employment_list'] = EmpType::where('del', '0')->get();
                $data['training_list'] = Training::where([['status', '1'], ['del', '0']])->orderByDesc('id')->get();
                $data['languages_list'] = [
                    "English",
                    "Spanish",
                    "French",
                    "German",
                    "Chinese",
                    "Hindi",
                    "Arabic",
                    "Portuguese",
                    "Bengali",
                    "Russian",
                    "Japanese",
                    "Punjabi",
                    "Korean",
                    "Turkish"
                ];
                $data['months_list'] = ["January", "February", "March", "April", "May", "June", "July", "August", "September", "October", "November", "December"];
                $data['type_of_employee_list'] = ['Trainee', 'Apprentice', 'Carpenter', 'Off-sider', 'Labourer', 'Other'];
                $data['employer_of_worker_list'] = ['Subcontractor', 'Supplier', 'Other', 'N/A'];
                $data['mechanism_of_injury_list'] = ['Fall', 'Trip', 'Hit by object', 'Overexertion'];
                $data['breakdown_of_injury_list'] = ['Fracture', 'Sprain', 'Burn', 'Concussion'];
                $data['nature_of_disease_list'] = ['Respiratory', 'Cardiovascular', 'Musculoskeletal'];
                $data['bodily_location_list'] = ['Head', 'Arm', 'Leg', 'Torso'];
                $data['nature_of_injury_list'] = ['Bruise', 'Cut', 'Dislocation', 'Laceration'];
            }
            return $this->success($data, 'Get Successfully');
        } catch (\Exception $e) {
            $shortMessage = substr($e->getMessage(), 0, 200) . (strlen($e->getMessage()) > 200 ? '...' : '');
            storeReportsLogs([
                'employee_id' => auth()->user()->id,
                'message' => 'Step 4 Retrieval Error: ' . $shortMessage,
                'report_id' => $validatedData['incident_report_id'],
                'report_type' => 'incident_report',
                'error_type' => 'Exception error'
            ]);
            return $this->error($shortMessage);
        }
    }


    public function deleteStep4(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'incident_report_id' => 'required|exists:incident_reports,id',
            'employee_id' => 'required|exists:incident_involved_person_details,employee_id'
        ]);
        if ($validator->fails()) {
            storeReportsLogs([
                'employee_id' => auth()->user()->id,
                'message' => 'Step 4 Delete Error: ' . $validator->errors()->first(),
                'report_id' => $request->incident_report_id,
                'report_type' => 'incident_report',
                'error_type' => 'Validation error'
            ]);
            return $this->message($validator->errors()->first(), 422);
        }
        try {
            $validatedData = $validator->validated();
            $incidentReport = IncidentReport::find($validatedData['incident_report_id']);
            if (!$incidentReport) {
                return $this->message( 'Incident Report Not Found', 404);
            }
            // 🔹 Check user permissions to delete
            $userTable = $this->getUserTable();
            if ($userTable === 'customer' && ($incidentReport->customer_id !== auth()->id() || $incidentReport->workspace_id !== auth()->user()->current_workspace_id)) {
                return $this->message('You do not have access to delete this record.', 403);
            }
            if ($userTable === 'emp' && ($incidentReport->customer_id !== auth()->user()->customer_id || $incidentReport->workspace_id !== auth()->user()->workspace_id)) {
                return $this->message('You do not have access to delete this record.', 403);
            }
            // 🔹 Find & Delete the related records
            $incidentPerson = IncidentInvolvedPersonDetail::where([
                'incident_report_id' => $validatedData['incident_report_id'],
                'employee_id' => $validatedData['employee_id']
            ])->first();
            if (!$incidentPerson) {
                return $this->message('Person not found in the report', 404);
            }
            // ❌ Delete related records (Trainings, Injury Types)
            IncidentInvolvedPersonTraining::where([
                'incident_involved_person_id' => $validatedData['employee_id'],
                'incident_report_id' => $validatedData['incident_report_id']
            ])->delete();
            IncidentEmployeeInjuryType::where([
                'incident_report_id' => $validatedData['incident_report_id'],
                'employer_id' => $validatedData['employee_id']
            ])->delete();
            // ❌ Delete Person Details
            $incidentPerson->delete();
            // 🔹 Log deletion
            storeReportsLogs([
                'employee_id' => auth()->user()->id,
                'message' => 'Step 4 Deleted Successfully',
                'report_id' => $request->incident_report_id,
                'report_type' => 'incident_report'
            ]);
            return $this->message('Deleted Successfully');
        } catch (\Exception $e) {
            storeReportsLogs([
                'employee_id' => auth()->user()->id,
                'message' => 'Step 4 Delete Error: ' . $e->getMessage(),
                'report_id' => $validatedData['incident_report_id'],
                'report_type' => 'incident_report',
                'error_type' => 'Exception error'
            ]);
            return $this->error('Error: ' . $e->getMessage());
        }
    }

    public function storeStep5(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'incident_report_id' => 'required|exists:incident_reports,id',
            'shift_type' => 'nullable|string',
            'answer' => 'nullable',
            'answer.*.question_id' => 'nullable|exists:incident_report_questions,id',
            'answer.*.text' => 'nullable|string',
        ]);
        if ($validator->fails()) {
            storeReportsLogs([
                'employee_id' => auth()->user()->id,
                'message' => 'Step 5 Validation Error: ' . $validator->errors()->first(),
                'report_id' => $request->incident_report_id,
                'report_type' => 'incident_report',
                'error_type' => 'Validation error'
            ]);
            return $this->error($validator->errors()->first(), 422);
        }
        try {
            $validatedData = $validator->validated();
            // 🔹 Fetch Incident Report with security check
            $incidentReport = IncidentReport::find($validatedData['incident_report_id']);
            // 🔹 Access Control Check
            $userTable = $this->getUserTable();
            if (
                ($userTable === 'customer' && ($incidentReport->customer_id !== auth()->id() || $incidentReport->workspace_id !== auth()->user()->current_workspace_id)) ||
                ($userTable === 'emp' && ($incidentReport->customer_id !== auth()->user()->customer_id || $incidentReport->workspace_id !== auth()->user()->workspace_id))
            ) {
                return $this->error('You do not have access to this report.', 403);
            }
            // 🔹 Save or Update Report Meta Data
            $reportMeta = IncidentReportMeta::updateOrCreate(
                ['incident_report_id' => $validatedData['incident_report_id']],
                ['shift_type' => $validatedData['shift_type']]
            );
            // 🔹 Update Step Number if it's a new meta entry
            if ($reportMeta->wasRecentlyCreated) {
                $incidentReport->update(['step_no' => 6]);
            }
            // 🔹 Store Report Question Answers
            if (!empty($validatedData['answer'])) {
                IncidentReportQuestionAnswer::where('incident_report_id', $validatedData['incident_report_id'])->delete();
                $reportAnswers = collect($validatedData['answer'])->map(function ($item) use ($validatedData) {
                    return [
                        'incident_report_id' => $validatedData['incident_report_id'],
                        'question_id' => $item['question_id'],
                        'answer' => $item['text'],
                        'created_at' => now(),
                        'updated_at' => now()
                    ];
                })->toArray();
                IncidentReportQuestionAnswer::insert($reportAnswers);
            }
            // 🔹 Log the operation
            storeReportsLogs([
                'employee_id' => auth()->user()->id,
                'message' => 'Step 5 ' . ($reportMeta->wasRecentlyCreated ? 'Saved' : 'Updated') . ' Successfully',
                'report_id' => $validatedData['incident_report_id'],
                'report_type' => 'incident_report'
            ]);
            return $this->success([
                'incident_report_id' => $validatedData['incident_report_id']
            ], 'Step 5 ' . ($reportMeta->wasRecentlyCreated ? 'Saved' : 'Updated') . ' Successfully');
        } catch (\Exception $e) {
            storeReportsLogs([
                'employee_id' => auth()->user()->id,
                'message' => 'Step 5 Store Error: ' . $e->getMessage(),
                'report_id' => $validatedData['incident_report_id'],
                'report_type' => 'incident_report',
                'error_type' => 'Exception error'
            ]);
            return $this->error('Error: ' . $e->getMessage(), 500);
        }
    }


    public function editStep5(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'incident_report_id' => 'required|exists:incident_reports,id',
        ]);
        if ($validator->fails()) {
            storeReportsLogs([
                'employee_id' => auth()->user()->id,
                'message' => 'Step 5 Edit Validation Error: ' . $validator->errors()->first(),
                'report_id' => $request->incident_report_id,
                'report_type' => 'incident_report',
                'error_type' => 'Validation error'
            ]);
            return $this->message($validator->errors()->first(), 422);
        }
        try {
            $validatedData = $validator->validated();
            $incidentReport = IncidentReport::find($validatedData['incident_report_id']);
            // 🔹 Access Control Check
            $userTable = $this->getUserTable();
            if (
                ($userTable === 'customer' && ($incidentReport->customer_id !== auth()->id() || $incidentReport->workspace_id !== auth()->user()->current_workspace_id)) ||
                ($userTable === 'emp' && ($incidentReport->customer_id !== auth()->user()->customer_id || $incidentReport->workspace_id !== auth()->user()->workspace_id))
            ) {
                return $this->error('You do not have access to this report.', 403);
            }
            // 🔹 Fetch related data using Eloquent ORM
            $incidentReport->load([
                'incidentReportMetas',  // Relationship for incident_report_metas
                'questionAnswers', // Relationship for incident_report_question_answers
            ]);
            // 🔹 Fetch additional required data
            $reportQuestions = IncidentReportQuestion::where('del', '0')->where('status', '1')->get();
            $employerList = EmpCompanyDetails::where('compeleted', '1')
                ->where('approved', '1')
                ->where('status', '1')
                ->where('del', '0')
                ->with([
                    'empPersonalDetails:id,first_name,middle_name,last_name',
                    'accessRole:id,title,code',
                    'empTier:id,title',
                ])
                ->select('id', 'access_role', 'tier_id')
                ->get();
            // 🔹 Shift types and worker categories
            $shiftTypeAndTimeList = [
                'Day' => ['06:00 AM - 11:59 PM', '07:00 AM - 11:59 PM'],
                'Evening' => ['12:00 AM - 15:59 PM'],
                'Night' => ['16:00 AM - 05:59 PM']
            ];
            $employerOfWorkerList = ['N/A', 'Other', 'Supplier', 'Subcontractor'];
            // 🔹 Construct response data
            $data = [
                'incident_report' => $incidentReport,
                'shift_type_and_time_list' => $shiftTypeAndTimeList,
                'employer_list' => $employerList,
                'report_questions_list' => $reportQuestions,
                'employer_of_worker_list' => $employerOfWorkerList,
            ];
            return $this->success($data, 'Step 5 Data Retrieved Successfully');
        } catch (\Exception $e) {
            storeReportsLogs([
                'employee_id' => auth()->user()->id,
                'message' => 'Step 5 Edit Error: ' . $e->getMessage(),
                'report_id' => $validatedData['incident_report_id'],
                'report_type' => 'incident_report',
                'error_type' => 'Exception error'
            ]);
            return $this->error('Error: ' . $e->getMessage(), 500);
        }
    }


    public function storeStep6(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'incident_report_id' => 'required|exists:incident_reports,id',
            'is_plant_involved' => 'nullable|boolean',
            'plant_name' => 'nullable|integer',
            'equipment_type' => 'nullable|string',
            'registration_number' => 'nullable|integer',
            'damage' => 'nullable|integer',
            'cost' => 'nullable|integer',
            'is_correct_action_required' => 'nullable|boolean'
        ]);
        if ($validator->fails()) {
            storeReportsLogs([
                'employee_id' => auth()->user()->id,
                'message' => 'Step 6 Validation Error: ' . $validator->errors()->first(),
                'report_id' => $request->incident_report_id,
                'report_type' => 'incident_report',
                'error_type' => 'Validation error'
            ]);
            return $this->message($validator->errors()->first(), 422);
        }
        try {
            $validatedData = $validator->validated();
            // 🔹 Fetch Incident Report with security check
            $incidentReport = IncidentReport::find($validatedData['incident_report_id']);
            // 🔹 Access Control Check
            $userTable = $this->getUserTable();
            if (
                ($userTable === 'customer' && ($incidentReport->customer_id !== auth()->id() || $incidentReport->workspace_id !== auth()->user()->current_workspace_id)) ||
                ($userTable === 'emp' && ($incidentReport->customer_id !== auth()->user()->customer_id || $incidentReport->workspace_id !== auth()->user()->workspace_id))
            ) {
                return $this->error('You do not have access to this report.', 403);
            }
            // 🔹 Update Incident Report fields
            $incidentReport->update([
                'is_plant_involved' => $validatedData['is_plant_involved'] ?? null,
                'plant_name' => $validatedData['plant_name'] ?? null,
                'is_correct_action_required' => $validatedData['is_correct_action_required'] ?? null
            ]);
            // 🔹 Update or Create Incident Report Meta
            $incidentReportMeta = IncidentReportMeta::updateOrCreate(
                ['incident_report_id' => $validatedData['incident_report_id']],
                [
                    'equipment_type' => $validatedData['equipment_type'] ?? null,
                    'registration_number' => $validatedData['registration_number'] ?? 0,
                    'damage' => $validatedData['damage'] ?? 0,
                    'cost' => $validatedData['cost'] ?? 0
                ]
            );
            // 🔹 Update step number if new entry
            if (!$incidentReportMeta->wasRecentlyCreated) {
                $msg = 'Updated Successfully';
            } else {
                $incidentReport->update(['step_no' => 7]);
                $msg = 'Saved Successfully';
            }
            // 🔹 Log the action
            storeReportsLogs([
                'employee_id' => auth()->user()->id,
                'message' => 'Step 6 ' . $msg,
                'report_id' => $validatedData['incident_report_id'],
                'report_type' => 'incident_report'
            ]);
            return $this->success(['incident_report_id' => $validatedData['incident_report_id']], $msg);
        } catch (\Exception $e) {
            storeReportsLogs([
                'employee_id' => auth()->user()->id,
                'message' => 'Step 6 Store Error: ' . $e->getMessage(),
                'report_id' => $validatedData['incident_report_id'],
                'report_type' => 'incident_report',
                'error_type' => 'Exception error'
            ]);
            return $this->error('Error: ' . $e->getMessage(), 500);
        }
    }

    public function editStep6(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'incident_report_id' => 'required|exists:incident_reports,id'
        ]);
        if ($validator->fails()) {
            storeReportsLogs([
                'employee_id' => auth()->user()->id,
                'message' => 'Step 6 Validation Error: ' . $validator->errors()->first(),
                'report_id' => $request->incident_report_id,
                'report_type' => 'incident_report',
                'error_type' => 'Validation error'
            ]);
            return $this->message($validator->errors()->first(), 422);
        }
        try {
            $validatedData = $validator->validated();
            $incidentReport = IncidentReport::with('incidentReportMetas')->find($validatedData['incident_report_id']);
            // 🔹 Access Control Check
            $userTable = $this->getUserTable();
            if (
                ($userTable === 'customer' && ($incidentReport->customer_id !== auth()->id() || $incidentReport->workspace_id !== auth()->user()->current_workspace_id)) ||
                ($userTable === 'emp' && ($incidentReport->customer_id !== auth()->user()->customer_id || $incidentReport->workspace_id !== auth()->user()->workspace_id))
            ) {
                return $this->error('You do not have access to this report.', 403);
            }
            // 🔹 Plant Equipment List
            $plantList = ['Fastform Plant / Equipment', 'Contractor Plant / Equipment', 'Third Party / Client'];
            $data = [
                'incident_report' => $incidentReport,
                'plant_list' => $plantList
            ];
            return $this->success($data, 'Get Successfully');
        } catch (\Exception $e) {
            storeReportsLogs([
                'employee_id' => auth()->user()->id,
                'message' => 'Step 6 Retrieval Error: ' . $e->getMessage(),
                'report_id' => $validatedData['incident_report_id'],
                'report_type' => 'incident_report',
                'error_type' => 'Exception error'
            ]);
            return $this->error('Error: ' . $e->getMessage(), 500);
        }
    }

    public function storeStep7(Request $request)
    {
        $validator = Validator::make($request->all(), [
            "incident_report_id" => "required|exists:incident_reports,id",
            "incident_response"  => "nullable|string",
            "is_injury_management" => "nullable|integer",
            "date" => "nullable|date",
            "time" => "nullable|date_format:H:i",
            "representative_name" => "nullable|string",
            "injured_names" => "nullable|string",
            "is_first_aid" => "nullable|integer",
            "is_referred" => "nullable|integer",
            "is_doctor" => "nullable|integer",
            "is_clinic" => "nullable|integer",
            "is_admit" => "nullable|integer",
            "is_doctor_details" => "nullable|integer",
            "doctor_name" => "nullable|string",
            "doctor_number" => "nullable|string",
            "doctor_address" => "nullable|string",
            "hospital_name" => "nullable|string",
            "hospital_number" => "nullable|string",
            "hospital_address" => "nullable|string",
            "is_recurrence_injury" => "nullable|integer",
            "previous_injury_date" => "nullable|date",
            "previous_report_number" => "nullable|integer"
        ]);
        if ($validator->fails()) {
            storeReportsLogs([
                'employee_id' => auth()->user()->id,
                'message' => 'Step 7 Validation Error: ' . $validator->errors()->first(),
                'report_id' => $request->incident_report_id,
                'report_type' => 'incident_report',
                'error_type' => 'Validation error'
            ]);
            return $this->message($validator->errors()->first(), 422);
        }
        try {
            $validatedData = $validator->validated();
            $incidentReport = IncidentReport::find($validatedData['incident_report_id']);
            // 🔹 Access Control Check
            $userTable = $this->getUserTable();
            if (
                ($userTable === 'customer' && ($incidentReport->customer_id !== auth()->id() || $incidentReport->workspace_id !== auth()->user()->current_workspace_id)) ||
                ($userTable === 'emp' && ($incidentReport->customer_id !== auth()->user()->customer_id || $incidentReport->workspace_id !== auth()->user()->workspace_id))
            ) {
                return $this->error('You do not have access to this report.', 403);
            }
            $incidentReport->update([
                'is_injury_management' => $validatedData['is_injury_management'] ?? 0,
                'incident_response' => $validatedData['incident_response'] ?? '',
            ]);

            // 🔹 Update or Create Injury Management Details
            $injuryManagementData = array_merge(
                $request->except(['is_injury_management', 'incident_response', 'incident_report_id']),
                ['incident_report_id' => $validatedData['incident_report_id']]
            );
            $injuryManagement = IncidentInjuryManagement::updateOrCreate(
                ['incident_report_id' => $validatedData['incident_report_id']],
                $injuryManagementData
            );
            // 🔹 Update Step Number if First Time
            if (!$injuryManagement->wasRecentlyCreated) {
                $msg = 'Updated Successfully';
            } else {
                $incidentReport->update(['step_no' => 8]);
                $msg = 'Saved Successfully';
            }
            // 🔹 Log Action
            storeReportsLogs([
                'employee_id' => auth()->user()->id,
                'message' => 'Step 7 ' . $msg,
                'report_id' => $request->incident_report_id,
                'report_type' => 'incident_report'
            ]);
            return $this->success(['incident_report_id' => $validatedData['incident_report_id']], $msg);
        } catch (\Exception $e) {
            storeReportsLogs([
                'employee_id' => auth()->user()->id,
                'message' => 'Step 7 Store Error: ' . $e->getMessage(),
                'report_id' => $validatedData['incident_report_id'],
                'report_type' => 'incident_report',
                'error_type' => 'Exception error'
            ]);
            return $this->error('Error: ' . $e->getMessage(), 500);
        }
    }


    public function editStep7(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'incident_report_id' => 'required|exists:incident_reports,id'
        ]);

        if ($validator->fails()) {
            storeReportsLogs([
                'employee_id' => auth()->user()->id,
                'message' => 'Step 7 Edit Error: ' . $validator->errors()->first(),
                'report_id' => $request->incident_report_id,
                'report_type' => 'incident_report',
                'error_type' => 'Validation error'
            ]);
            return $this->error($validator->errors()->first(), 422);
        }

        try {
            $validatedData = $validator->validated();

            // 🔹 Fetch Incident Report with Security Check
            $incidentReport = IncidentReport::with('injuryManagements')
                ->where('id', $validatedData['incident_report_id'])
                ->firstOrFail();

            // 🔹 Access Control Check
            $userTable = $this->getUserTable();
            if (
                ($userTable === 'customer' && ($incidentReport->customer_id !== auth()->id() || $incidentReport->workspace_id !== auth()->user()->current_workspace_id)) ||
                ($userTable === 'emp' && ($incidentReport->customer_id !== auth()->user()->customer_id || $incidentReport->workspace_id !== auth()->user()->workspace_id))
            ) {
                return $this->error('You do not have access to this report.', 403);
            }

            return $this->success($incidentReport, 'Get Successfully');
        } catch (\Exception $e) {
            storeReportsLogs([
                'employee_id' => auth()->user()->id,
                'message' => 'Step 7 Edit Error: ' . $e->getMessage(),
                'report_id' => $validatedData['incident_report_id'],
                'report_type' => 'incident_report',
                'error_type' => 'Exception error'
            ]);
            return $this->error('Error: ' . $e->getMessage(), 500);
        }
    }

    public function storeStep8(Request $request)
    {
        $validator = Validator::make($request->all(), [
            "incident_report_id" => 'required|exists:incident_reports,id',
            "is_final_injury_classification" => 'required|boolean',
            "injury_classification_id" => 'nullable|array',
            'details' => 'nullable|string',
            "is_final_incident_classification" => 'nullable|boolean',
            "final_incident_name" => 'nullable|integer',
            "final_incident_details" => 'nullable|string',
            "lti_or_rwi" => 'nullable|integer',
            "is_rehabilitation" => 'nullable|boolean',
            "return_to_duty_date" => 'nullable|date',
            "rtw_date" => 'nullable|date',
            "total_lost_days" => 'nullable|integer',
            "is_worker_notified" => 'nullable|boolean',
            "notified_by_whom" => 'nullable|integer',
            "notified_by_whom_date" => 'nullable|date',
            "notified_by_whom_time" => 'nullable|string',
            "notified_by_whom_details" => 'nullable|string',
            "sign_off" => 'nullable|array'
        ]);
        if ($validator->fails()) {
            storeReportsLogs([
                'employee_id' => auth()->user()->id,
                'message' => 'Step 8 Validation Error: ' . $validator->errors()->first(),
                'report_id' => $request->incident_report_id,
                'report_type' => 'incident_report',
                'error_type' => 'Validation error'
            ]);
            return $this->message($validator->errors()->first(), 422);
        }
        try {
            $validatedData = $validator->validated();
            $incidentReport = IncidentReport::find($validatedData['incident_report_id']);
            // 🔹 Access Control Check
            $userTable = $this->getUserTable();
            if (
                ($userTable === 'customer' && ($incidentReport->customer_id !== auth()->id() || $incidentReport->workspace_id !== auth()->user()->current_workspace_id)) ||
                ($userTable === 'emp' && ($incidentReport->customer_id !== auth()->user()->customer_id || $incidentReport->workspace_id !== auth()->user()->workspace_id))
            ) {
                return $this->error('You do not have access to this report.', 403);
            }
            // 🔹 Update Incident Report
            $incidentReport->update([
                "is_final_incident_classification" => $validatedData['is_final_incident_classification'],
                "is_final_injury_classification" => $validatedData['is_final_injury_classification'],
                "lti_or_rwi" => $validatedData['lti_or_rwi'],
            ]);
            // 🔹 Update or Create Final Classification
            $incidentFinalClassification = IncidentFinalClassification::updateOrCreate(
                ['incident_report_id' => $validatedData['incident_report_id']],
                [
                    "final_incident_name" => $validatedData['final_incident_name'],
                    "final_incident_details" => $validatedData['final_incident_details'],
                    "is_rehabilitation" => $validatedData['is_rehabilitation'],
                    "return_to_duty_date" => $validatedData['return_to_duty_date'],
                    "rtw_date" => $validatedData['rtw_date'],
                    "total_lost_days" => $validatedData['total_lost_days'],
                    "is_worker_notified" => $validatedData['is_worker_notified'],
                    "notified_by_whom" => $validatedData['notified_by_whom'] ?? null,
                    "date" => $validatedData['notified_by_whom_date'] ?? null,
                    "time" => $validatedData['notified_by_whom_time'] ?? null,
                    "details" => $validatedData['notified_by_whom_details'] ?? null
                ]
            );
            // 🔹 Update Injury Classification
            if (isset($validatedData['injury_classification_id'])) {
                IncidentClassificationReport::where('incident_report_id', $validatedData['incident_report_id'])
                    ->where("type", 1)
                    ->delete();
                foreach ($validatedData['injury_classification_id'] as $c_id) {
                    IncidentClassificationReport::create([
                        "incident_report_id" => $validatedData['incident_report_id'],
                        "injury_classification_id" => $c_id,
                        "details" => $c_id == 1 ? $validatedData['details'] : "",
                        "type" => "1"
                    ]);
                }
            }
            // 🔹 Sign-Off Handling
            if (isset($validatedData['sign_off'])) {
                foreach ($validatedData['sign_off'] as $incident_signoff) {
                    $incident_signoff['incident_report_id'] = $validatedData['incident_report_id'];
                    $incident_signoff['emp_id'] = $incident_signoff['emp_id'] ?? 0;
                    IncidentSignoff::updateOrCreate(
                        [
                            'incident_report_id' => $validatedData['incident_report_id'],
                            'role_code' => $incident_signoff['role_code']
                        ],
                        $incident_signoff
                    );
                }
            }
            // 🔹 Update Step No if needed
            if (!$incidentFinalClassification->wasRecentlyCreated) {
                $incidentReport->update(['step_no' => 9]);
            }
            storeReportsLogs([
                'employee_id' => auth()->user()->id,
                'message' => 'Step 8 Updated Successfully',
                'report_id' => $validatedData['incident_report_id'],
                'report_type' => 'incident_report'
            ]);
            return $this->success(['incident_report_id' => $validatedData['incident_report_id']], 'Updated Successfully');
        } catch (\Exception $e) {
            storeReportsLogs([
                'employee_id' => auth()->user()->id,
                'message' => 'Step 8 Store Error: ' . $e->getMessage(),
                'report_id' => $validatedData['incident_report_id'],
                'report_type' => 'incident_report',
                'error_type' => 'Exception error'
            ]);
            return $this->error('Error: ' . $e->getMessage(), 500);
        }
    }

    public function editStep8(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'incident_report_id' => 'required|exists:incident_reports,id'
        ]);
        if ($validator->fails()) {
            storeReportsLogs([
                'employee_id' => auth()->user()->id,
                'message' => 'Step 8 Edit Validation Error: ' . $validator->errors()->first(),
                'report_id' => $request->incident_report_id,
                'report_type' => 'incident_report',
                'error_type' => 'Validation error'
            ]);
            return $this->error($validator->errors()->first(), 422);
        }

        try {
            $validatedData = $validator->validated();
            // 🔹 Fetch Incident Report with Security Check
            $incidentReport = IncidentReport::with([
                'incidentClassificationReports' => function ($query) {
                    $query->where('type', 1);
                },
                'finalClassifications',
                'signoffs'
            ])->find($validatedData['incident_report_id']);

            // 🔹 Access Control Check
            $userTable = $this->getUserTable();
            if (
                ($userTable === 'customer' && ($incidentReport->customer_id !== auth()->id() || $incidentReport->workspace_id !== auth()->user()->current_workspace_id)) ||
                ($userTable === 'emp' && ($incidentReport->customer_id !== auth()->user()->customer_id || $incidentReport->workspace_id !== auth()->user()->workspace_id))
            ) {
                return $this->error('You do not have access to this report.', 403);
            }

            // 🔹 Fetch Employee Details
            $empList = EmpCompanyDetails::where('compeleted', '1')
                ->where('approved', '1')
                ->where('status', '1')
                ->where('del', '0')
                ->with([
                    'empPersonalDetails:emp_id,first_name,middle_name,last_name',
                    'accessRole:id,title,code',
                    'empTier:id,title',
                    'accessTier:id,title,tier_key'
                ])
                ->select('id', 'access_role', 'tier_id')
                ->get();

            // 🔹 Filter Employee Lists Based on Roles
            $formanList = $empList->where('access_role', 'FM')->values();
            $directorList = $empList->where('access_role', 'Dir')->values();
            $safetyManagerList = $empList->where('access_role', '01')->values();
            $siteSafetyOfficerList = $empList->where('access_role', 'SSO')->values();

            // 🔹 Fetch Additional Data
            $byWhom = EmpCompanyDetails::where('compeleted', '1')
                ->where('approved', '1')
                ->where('status', '1')
                ->where('del', '0')
                ->with([
                    'empPersonalDetails:emp_id,first_name,middle_name,last_name',
                    'accessRole:id,title,code',
                    'empTier:id,title',
                    'accessTier:id,title,tier_key'
                ])
                ->select('id', 'access_role', 'tier_id')
                ->whereHas('empTier', function ($query) {
                    $query->whereIn('tier_key', ['T-1', 'T-2']);
                })
                ->get();

            $incidentInjuryClassificationList = IncidentInjuryClassification::where('status', '1')
                ->where('del', '0')
                ->orderBy('id', 'DESC')
                ->get();

            $data = [
                'incident_report' => $incidentReport,
                'incident_injury_classification_list' => $incidentInjuryClassificationList,
                'by_whom' => $byWhom,
                'director_list' => $directorList,
                'safety_manager_list' => $safetyManagerList,
                'site_safety_officer_list' => $siteSafetyOfficerList,
                'forman_list' => $formanList
            ];

            return $this->success($data, 'Get Successfully');
        } catch (\Exception $e) {
            storeReportsLogs([
                'employee_id' => auth()->user()->id,
                'message' => 'Step 8 Edit Error: ' . $e->getMessage(),
                'report_id' => $validatedData['incident_report_id'],
                'report_type' => 'incident_report',
                'error_type' => 'Exception error'
            ]);
            return $this->error('Error: ' . $e->getMessage(), 500);
        }
    }


    public function storeStep9(Request $request)
    {
        $validator = Validator::make($request->all(), [
            "incident_report_id" => 'required|exists:incident_reports,id',
            "incident_risk_category_id" => 'nullable|exists:incident_risk_categories,id',
            "details" => 'nullable|string',
            "name" => 'nullable|array',
            "time" => 'nullable|array',
            "date" => 'nullable|array',
            "statement_number" => 'nullable|array',
            "title" => 'nullable|array',
            "number" => 'nullable|array'
        ]);
        if ($validator->fails()) {
            storeReportsLogs([
                'employee_id' => auth()->user()->id,
                'message' => 'Step 9 Validation Error: ' . $validator->errors()->first(),
                'report_id' => $request->incident_report_id,
                'report_type' => 'incident_report',
                'error_type' => 'Validation error'
            ]);
            return $this->error($validator->errors()->first(), 422);
        }
        try {
            $validatedData = $validator->validated();
            $incidentReport = IncidentReport::find($validatedData['incident_report_id']);
            // 🔹 Access Control Check
            $userTable = $this->getUserTable();
            if (
                ($userTable === 'customer' && ($incidentReport->customer_id !== auth()->id() || $incidentReport->workspace_id !== auth()->user()->current_workspace_id)) ||
                ($userTable === 'emp' && ($incidentReport->customer_id !== auth()->user()->customer_id || $incidentReport->workspace_id !== auth()->user()->workspace_id))
            ) {
                return $this->error('You do not have access to this report.', 403);
            }
            // 🔹 Update or Create Risk Category Details
            IncidentRiskCategoryDetail::updateOrCreate(
                ['incident_report_id' => $validatedData['incident_report_id']],
                [
                    "incident_risk_category_id" => $validatedData['incident_risk_category_id'] ?? null,
                    "details" => $validatedData['details'] ?? null
                ]
            );
            // 🔹 Process Witness Data
            if (!empty($validatedData['name'])) {
                IncidentWitnes::where('incident_report_id', $validatedData['incident_report_id'])->delete();
                foreach ($validatedData['name'] as $index => $witnessName) {
                    IncidentWitnes::create([
                        "incident_report_id" => $validatedData['incident_report_id'],
                        "name" => $witnessName,
                        "date" => $validatedData['date'][$index] ?? null,
                        "time" => $validatedData['time'][$index] ?? null,
                        "statement_number" => $validatedData['statement_number'][$index] ?? null
                    ]);
                }
            }
            // 🔹 Process Document Data
            if (!empty($validatedData['title'])) {
                IncidentDocument::where('incident_report_id', $validatedData['incident_report_id'])->delete();
                foreach ($validatedData['title'] as $index => $docTitle) {
                    IncidentDocument::create([
                        "incident_report_id" => $validatedData['incident_report_id'],
                        "title" => $docTitle,
                        "number" => $validatedData['number'][$index] ?? null
                    ]);
                }
            }
            storeReportsLogs([
                'employee_id' => auth()->user()->id,
                'message' => 'Step 9 Successfully Processed',
                'report_id' => $validatedData['incident_report_id'],
                'report_type' => 'incident_report'
            ]);
            return $this->success(['incident_report_id' => $validatedData['incident_report_id']], 'Saved Successfully');
        } catch (\Exception $e) {
            storeReportsLogs([
                'employee_id' => auth()->user()->id,
                'message' => 'Step 9 Store Error: ' . $e->getMessage(),
                'report_id' => $validatedData['incident_report_id'],
                'report_type' => 'incident_report',
                'error_type' => 'Exception error'
            ]);
            return $this->error('Error: ' . $e->getMessage(), 500);
        }
    }

    public function editStep9(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'incident_report_id' => 'required|exists:incident_reports,id'
        ]);
        if ($validator->fails()) {
            return $this->error($validator->errors()->first(), 422);
        }
        try {
            $validatedData = $validator->validated();
            // 🔹 Fetch Incident Report with Security Check
            $incidentReport = IncidentReport::find($validatedData['incident_report_id']);
            // 🔹 Access Control Check
            $userTable = $this->getUserTable();
            if (
                ($userTable === 'customer' && ($incidentReport->customer_id !== auth()->id() || $incidentReport->workspace_id !== auth()->user()->current_workspace_id)) ||
                ($userTable === 'emp' && ($incidentReport->customer_id !== auth()->user()->customer_id || $incidentReport->workspace_id !== auth()->user()->workspace_id))
            ) {
                return $this->error('You do not have access to this report.', 403);
            }
            // 🔹 Load Related Data
            $incidentReport->load([
                'incidentRiskCategoryDetails',
                'incidentWitnesses',
                'incidentDocuments'
            ]);
            return $this->success(['incident_report' => $incidentReport], 'Get Successfully');
        } catch (\Exception $e) {
            storeReportsLogs([
                'employee_id' => auth()->user()->id,
                'message' => 'Step 9 Retrieval Error: ' . $e->getMessage(),
                'report_id' => $validatedData['incident_report_id'],
                'report_type' => 'incident_report',
                'error_type' => 'Exception error'
            ]);
            return $this->error('Error: ' . $e->getMessage(), 500);
        }
    }


    public function storeStep10(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'incident_report_id' => 'required|exists:incident_reports,id',
        ]);

        if ($validator->fails()) {
            return $this->message($validator->errors()->first(), 422);
        }
        $isUpdate = $request->incident_report_id;
        $message = $isUpdate ? 'Updated Successfully' : 'Saved Successfully';
        try {
            $incidentReport = IncidentReport::find($request->incident_report_id);
            if (!$incidentReport) {
                return $this->message('Record not found.', 404);
            }
            $userTable = $this->getUserTable();
            if ($userTable === 'customer' && ($incidentReport->customer_id !== auth()->id() || $incidentReport->workspace_id !== auth()->user()->current_workspace_id)) {
                return $this->message('You do not have access to this report.', 403);
            }
            if ($userTable === 'emp' && ($incidentReport->customer_id !== auth()->user()->customer_id || $incidentReport->workspace_id !== auth()->user()->workspace_id)) {
                return $this->message('You do not have access to this report.', 403);
            }
            $incidentReport->process = 1;
            $incidentReport->step_no = 10;
            $incidentReport->save();
            $log = [
                'employee_id' => auth()->user()->id,
                'message' => 'WHS  Step 5 :' . $message,
                'report_id' =>   $request->incident_report_id,
                'report_type' => 'incident_report'
            ];
            storeReportsLogs($log);
            return $this->message($message);
        } catch (\Exception $e) {
            // Handle exceptions and return an error response
            $shortMessage = substr($e->getMessage(), 0, 200) . (strlen($e->getMessage()) > 200 ? '...' : '');
            $log = [
                'employee_id' => auth()->user()->id,
                'message' => 'WHS Step 5 error:' . $shortMessage,
                'report_id' =>  $request->incident_report_id,
                'report_type' => 'incident_report'
            ];
            storeReportsLogs($log);
            return $this->error('An error occurred while saving  data: ' . $e->getMessage());
        }
    }
    public function destroy($id)
    {
        try {
            // 🔹 Find the Incident Report
            $incidentReport = IncidentReport::find($id);
            if (!$incidentReport) {
                return $this->message('Record not found', 404);
            }
            if ($incidentReport->del == 1) {
                return $this->message('Record already deleted', 403);
            }
            $userTable = $this->getUserTable();
            if (
                ($userTable === 'customer' && ($incidentReport->customer_id !== auth()->id() || $incidentReport->workspace_id !== auth()->user()->current_workspace_id)) ||
                ($userTable === 'emp' && ($incidentReport->customer_id !== auth()->user()->customer_id || $incidentReport->workspace_id !== auth()->user()->workspace_id))
            ) {
                return $this->error('You do not have access to delete this report.', 403);
            }
            // 🔹 Soft Delete by Updating `del` Field
            $incidentReport->update(['del' => 1]);
            // 🔹 Log Action
            storeReportsLogs([
                'employee_id' => auth()->user()->id,
                'message' => 'Incident Report Deleted Successfully',
                'report_id' => $id,
                'report_type' => 'incident_report'
            ]);
            return $this->message('Incident Report Deleted Successfully');
        } catch (\Exception $e) {
            storeReportsLogs([
                'employee_id' => auth()->user()->id,
                'message' => 'Incident Report Delete Failed: ' . $e->getMessage(),
                'report_id' => $id,
                'report_type' => 'incident_report',
                'error_type' => 'Exception error'
            ]);
            return $this->error('Incident Report Deletion Failed. Please retry.', 500);
        }
    }

    public function checkSignOffApi(Request $request)
    {
        try {
            // 🔹 Validate Request
            $validator = Validator::make($request->all(), [
                'employee_id' => 'required|exists:emp_personal_details,emp_id',
                'incident_report_id' => 'required|exists:incident_reports,id'
            ]);
            if ($validator->fails()) {
                storeReportsLogs([
                    'employee_id' => auth()->user()->id,
                    'message' => 'Step 9 Validation Error: ' . $validator->errors()->first(),
                    'report_id' => $request->incident_report_id,
                    'report_type' => 'incident_report',
                    'error_type' => 'Validation error'
                ]);
                return $this->error($validator->errors()->first(), 422);
            }
            $validatedData = $validator->validated();
            $incidentReport = IncidentReport::find($validatedData['incident_report_id']);
            if (!$incidentReport) {
                return $this->error('Incident Report Record Not Found', 404);
            }
            // 🔹 Check User Permissions
            $userTable = $this->getUserTable();
            if (
                ($userTable === 'customer' && ($incidentReport->customer_id !== auth()->id() || $incidentReport->workspace_id !== auth()->user()->current_workspace_id)) ||
                ($userTable === 'emp' && ($incidentReport->customer_id !== auth()->user()->customer_id || $incidentReport->workspace_id !== auth()->user()->workspace_id))
            ) {
                return $this->error('You do not have access to this report.', 403);
            }
            // 🔹 Fetch Sign-Off Records Using ORM
            $incidentSignoffs = IncidentSignoff::where([
                'emp_id' => $validatedData['employee_id'],
                'incident_report_id' => $validatedData['incident_report_id']
            ])->get();
            return $this->success($incidentSignoffs, 'Data retrieved successfully.');
        } catch (\Exception $e) {
            // 🔹 Log the error
            storeReportsLogs([
                'employee_id' => auth()->user()->id,
                'message' => 'Step 9 Check Sign-Off Error: ' . $e->getMessage(),
                'report_id' => $validatedData['incident_report_id'] ?? null,
                'report_type' => 'incident_report',
                'error_type' => 'Exception error'
            ]);
            return $this->error('An error occurred while retrieving data.', 500);
        }
    }

    public function signOffApi(Request $request)
    {
        try {
            $validator = Validator::make($request->all(), [
                'report_id' => 'required',
                'employee_id' => 'required|integer|exists:emp_personal_details,emp_id',
                'report_type' => 'required|string|in:incident_report,swms_report,inspection_report,whs_report,meeting_note_report',
                'signature' => 'required|file|mimes:png,jpg,jpeg,pdf|max:10240',
                'key' => 'required|string'
            ]);
            if ($validator->fails()) {
                storeReportsLogs([
                    'employee_id' => $request->employee_id ?? null,
                    'message' => 'Sign-Off API Validation Error: ' . $validator->errors()->first(),
                    'report_id' => $request->report_id,
                    'report_type' => $request->report_type,
                    'error_type' => 'Validation error'
                ]);
                return $this->error($validator->errors()->first(), 422);
            }
            $userTable = $this->getUserTable();
            $validatedData = $validator->validated();
            $employeeId = $validatedData['employee_id'];
            $reportType = $validatedData['report_type'];
            $key = $validatedData['key'];
            // 🔹 Use Global File Upload Function
            if ($request->hasFile('signature')) {
                $validatedData['signature'] = $this->handleFileImageUpload($request, 'SignoffSignature')['path'] ?? null;
            } else {
                return $this->error('No signature file uploaded.', 433);
            }
            // 🔹 Process Signature Based on Report Type
            switch ($reportType) {
                case 'incident_report':
                    $incidentReport = IncidentReport::find($validatedData['report_id']);
                    if (!$incidentReport) {
                        return $this->error('Incident Report Record Not Found', 404);
                    }
                    if ($userTable === 'customer' && ($incidentReport->customer_id !== auth()->id() || $incidentReport->workspace_id !== auth()->user()->current_workspace_id)) {
                        return $this->message('You do not have access to this report.', 403);
                    }
                    if ($userTable === 'emp' && ($incidentReport->customer_id !== auth()->user()->customer_id || $incidentReport->workspace_id !== auth()->user()->workspace_id)) {
                        return $this->message('You do not have access to this report.', 403);
                    }
                    IncidentSignoff::where('incident_report_id', $validatedData['report_id'])
                        ->update(['signature' => $validatedData['signature']]);
                    break;
                case 'swms_report':
                    // before adding need to confirm logic from TL
                    $userLogin = EmpCompanyDetails::with(['accessRole', 'accessTier'])->find($employeeId);
                    $roleAccessRole = $userLogin->accessRole->id ?? 0;
                    $updateTableMap = [
                        'signatures' => SwmsSignature::class,
                        'developed_bies' => swmsDevelopedBy::class,
                        'approved_bies' => SwmsApprovedBy::class,
                        'review_responsibles' => SwmsReviewResponsible::class,
                        'compliance_responsibles' => SwmsComplianceResponsible::class,
                        'monitoring_responsibles' => SwmsMonitoringResponsible::class,
                    ];
                    if (isset($updateTableMap[$key])) {
                        $updateTableMap[$key]::where('id', $validatedData['report_id'])
                            ->where('employee_id', $employeeId)
                            ->update(['signature' => $validatedData['signature']]);
                    }
                    break;
                case 'inspection_report':
                    $inspectionReport = InspectionPlan::find($validatedData['report_id']);
                    if (!$inspectionReport) {
                        return $this->error('Inspection Report Record Not Found', 404);
                    }
                    if ($userTable === 'customer' && ($inspectionReport->customer_id !== auth()->id() || $inspectionReport->workspace_id !== auth()->user()->current_workspace_id)) {
                        return $this->message('You do not have access to this report.', 403);
                    }
                    if ($userTable === 'emp' && ($inspectionReport->customer_id !== auth()->user()->customer_id || $inspectionReport->workspace_id !== auth()->user()->workspace_id)) {
                        return $this->message('You do not have access to this report.', 403);
                    }
                    InspectionPlanSignature::where('inspection_plan_id', $validatedData['report_id'])
                        ->update(['signature' => $validatedData['signature']]);
                    break;
                case 'whs_report':
                    $whsReport = WhsReport::find($validatedData['report_id']);
                    if (!$whsReport) {
                        return $this->error('Warehouse Report Record Not Found', 404);
                    }
                    if ($userTable === 'customer' && ($whsReport->customer_id !== auth()->id() || $whsReport->workspace_id !== auth()->user()->current_workspace_id)) {
                        return $this->message('You do not have access to this report.', 403);
                    }
                    if ($userTable === 'emp' && ($whsReport->customer_id !== auth()->user()->customer_id || $whsReport->workspace_id !== auth()->user()->workspace_id)) {
                        return $this->message('You do not have access to this report.', 403);
                    }
                    WhsSignature::create([
                        'whs_id' => $validatedData['report_id'],
                        'employee_id' => $employeeId,
                        'signatures' => $validatedData['signature']
                    ]);
                    break;
                case 'meeting_note_report':
                    // before adding need to confirm logic from TL
                    $updated = MeetingNoteSignature::where([
                        'employee_note_id' => $validatedData['report_id'],
                        'employee_id' => $employeeId
                    ])->update([
                        'signature' => $validatedData['signature'],
                        'date' => now()->format('Y-m-d')
                    ]);
                    if (!$updated) {
                        storeReportsLogs([
                            'employee_id' => $request->employee_id ?? null,
                            'message' => 'Meeting Note Signature Not Uploaded',
                            'report_id' => $request->report_id,
                            'report_type' => $reportType,
                            'error_type' => 'Save status 0 returned'
                        ]);
                    }
                    break;
                default:
                    return $this->error('Invalid report type.', 400);
            }
            storeReportsLogs([
                'employee_id' => $employeeId,
                'message' => 'Sign-Off API Success for ' . strtoupper($reportType),
                'report_id' => $validatedData['report_id'],
                'report_type' => $reportType
            ]);
            return $this->message('Saved Successfully');
        } catch (\Exception $e) {
            storeReportsLogs([
                'employee_id' => $request->employee_id ?? null,
                'message' => 'Sign-Off API Error: ' . $e->getMessage(),
                'report_id' => $request->report_id,
                'report_type' => $request->report_type,
                'error_type' => 'Exception error'
            ]);
            return $this->error('An error occurred while saving the signature.', 500);
        }
    }

    // 
    public function SignOffAllFiles(Request $request)
    {

        $validator = Validator::make($request->all(), [
            'employe_id' => 'required',
        ]);
        $authUser = auth()->user();
        if ($validator->fails()) {
            $log = [
                'employee_id' => $authUser->id,
                'message' => 'API Signoff files ' . $validator->errors()->first(),
                'report_id' => 0,
                'report_type' => 'incident_report'
            ];
            storeReportsLogs($log);
            return $this->message($validator->errors()->first(), 422);
        }
        $userLogin = EmpCompanyDetails::with(['EmpPersonalDetails', 'accessRole', 'accessTier'])
            ->where('id', $request->employe_id)
            ->first();
        if (!$userLogin) {
            return $this->message('Employee not found.', 404);
        }
        $userTable = $this->getUserTable();
        if ($userTable === 'customer' && ($userLogin->customer_id !== $authUser->id || $userLogin->workspace_id !== $authUser->current_workspace_id)) {
            return $this->message('You do not have access to this report.', 403);
        }
        if ($userTable === 'emp' && ($userLogin->customer_id !== $authUser->customer_id || $userLogin->workspace_id !== $authUser->workspace_id)) {
            return $this->message('You do not have access to this report.', 403);
        }
        // Get validated data
        $validatedData = $validator->validated();
        $incidentFile_urls = [];
        $inspectionFile_urls = [];
        $swmsFile_urls = [];
        $whsFile_urls = [];
        $meeting_documents = [];
        if ($userLogin && isset($userLogin->accessRole) && is_object($userLogin->accessRole)) {
            $access_role_code = $userLogin->accessRole->code;
            if ($access_role_code) {
                // dd($access_role_code);
                $incident_signoffs = IncidentSignoff::where('role_code', $access_role_code)->where('emp_id', $request->employe_id)->get();
                if ($incident_signoffs->isNotEmpty()) {
                    $incident_report_ids = $incident_signoffs->pluck('incident_report_id');

                    $incident_files = DB::table('generated_pdf_reports')
                        ->whereIn('report_id', $incident_report_ids)
                        ->where('report_type', 'incident_report')
                        ->get(['report_id', 'path']);
                    $file_mapping = [];
                    foreach ($incident_files as $file) {
                        if (!empty($file->path)) {
                            $file_mapping[$file->report_id] = url($file->path);
                        }
                    }
                    foreach ($incident_signoffs as $signoff) {
                        $incident_record = IncidentReport::where('id', $signoff->incident_report_id)->where('customer_id', $userLogin->customer_id)
                            ->where('is_published', '1')->first();
                        $incident_report_file_url = $file_mapping[$signoff->incident_report_id] ?? '';
                        if ($incident_report_file_url !== '' && $incident_record) {
                            $incidentFile_urls[] = [
                                'id' => $signoff->id,
                                'title' => $incident_record->title ?? '',
                                'document_number' => $incident_record->document_number ?? '',
                                'revision_number' => $incident_record->revision_number ?? '',
                                'url' => $file_mapping[$signoff->incident_report_id] ?? '',
                                'is_signed' => $signoff->signature === null ? '0' : '1',
                                'type' => 'incident_report',
                                'key' => 'incident_signoffs'
                            ];
                        }
                    }
                }
            }
        }
        $inspection_plan_signatures = DB::table('inspection_plan_signatures')
            ->where('employee_id', $request->employe_id)
            ->get(['id', 'inspection_plan_id', 'signature']);
        if ($inspection_plan_signatures->isNotEmpty()) {
            $inspection_plan_ids = $inspection_plan_signatures->pluck('inspection_plan_id');
            $inspection_files = DB::table('generated_pdf_reports')
                ->whereIn('report_id', $inspection_plan_ids)
                ->where('report_type', 'inspection_report', 'path')
                ->get();
            $file_mapping = [];
            foreach ($inspection_files as $file) {
                if (!empty($file->path)) {
                    $file_mapping[$file->report_id] = url($file->path);
                }
            }
            foreach ($inspection_plan_signatures as $signature) {
                $inspection_reports_record = InspectionPlan::where('id', $signature->inspection_plan_id)->where('customer_id', $userLogin->customer_id)
                    ->where('is_published', '1')->first();
                $inspection_report_file = $file_mapping[$signature->inspection_plan_id] ?? '';
                if ($inspection_report_file !== '' && $inspection_reports_record) {
                    $inspectionFile_urls[] = [
                        'id' => $signature->id ?? '',
                        'title' => $inspection_reports_record->title ?? '',
                        'document_number' => $inspection_reports_record->document_number ?? '',
                        'revision_number' => $inspection_reports_record->revision_number ?? '',
                        'url' => $inspection_report_file,
                        'is_signed' => $signature->signature === null ? '0' : '1',
                        'type' => 'inspection_report',
                        'key' => 'inspection_signatures'
                    ];
                }
            }
        }
        //   $accessRoleId=$userLogin->accessRole->id;
        if (isset($userLogin->accessRole->id)) {
            $role_access_role = $userLogin->accessRole->id;
            $swms_signatures = DB::table('swms_signatures')
                ->where('tier_id', $role_access_role)
                ->get(['id', 'swms_id', 'signatures']);
            $swms_swms_developed_bies = DB::table('swms_developed_bies')
                ->where('employee_id', $request->employe_id)
                ->get(['id', 'swms_id', 'signature']);
            $swms_swms_approved_bies = DB::table('swms_approved_bies')
                ->where('employee_id', $request->employe_id)
                ->get(['id', 'swms_id', 'signature']);
            $swms_swms_review_responsibles_bies = DB::table('swms_review_responsibles')
                ->where('employee_id', $request->employe_id)
                ->get(['id', 'swms_id', 'signature']);
            $swms_compliance_responsibles_bies = DB::table('swms_compliance_responsibles')
                ->where('employee_id', $request->employe_id)
                ->get(['id', 'swms_id', 'signature']);
            $monitoring_responsibles_bies = DB::table('swms_monitoring_responsibles')
                ->where('employee_id', $request->employe_id)
                ->get(['id', 'swms_id', 'signature']);
            $swmsFile_urls = [];
            if ($swms_signatures->isNotEmpty()) {
                $swms_files = DB::table('generated_pdf_reports')
                    ->whereIn('report_id', $swms_signatures->pluck('swms_id'))
                    ->where('report_type', 'swms_report')
                    ->get(['report_id', 'path']);
                foreach ($swms_signatures as $signature) {
                    $swms_reports_record = DB::table('swms')->where('id', $signature->swms_id)->where('published', '1')
                        ->where('customer_id', $userLogin->customer_id)->first();
                    $generated_signature_url = $swms_files->firstWhere('report_id', $signature->swms_id)->path ?? '';
                    if ($generated_signature_url !== '' && $swms_reports_record) {
                        $swmsFile_urls[] = [
                            'id' => $signature->id,
                            'title' => $swms_reports_record->title,
                            'document_number' => $swms_reports_record->swms_number ?? '',
                            'revision_number' => $swms_reports_record->revision_number ?? '',
                            'url' => $generated_signature_url,
                            'is_signed' => $signature->signatures === null ? 0 : 1,
                            'type' => 'swms_report',
                            'key' => 'signatures',
                        ];
                    }
                }
            }
            if ($swms_swms_developed_bies->isNotEmpty()) {
                $swms_developed_by_files = DB::table('generated_pdf_reports')
                    ->whereIn('report_id', $swms_swms_developed_bies->pluck('swms_id'))
                    ->where('report_type', 'swms_report')
                    ->get(['report_id', 'path']);
                foreach ($swms_swms_developed_bies as $developed) {
                    $swms_reports_record = DB::table('swms')->where('id', $developed->swms_id)->where('customer_id', $userLogin->customer_id)
                        ->where('published', '1')->first();
                    $generate_report_url = $swms_developed_by_files->firstWhere('report_id', $developed->swms_id)->path ?? '';
                    if ($generate_report_url !== '' && $swms_reports_record) {
                        $swmsFile_urls[] = [
                            'id' => $developed->id,
                            'title' => $swms_reports_record->title,
                            'document_number' => $swms_reports_record->swms_number ?? '',
                            'revision_number' => $swms_reports_record->revision_number ?? '',
                            'url' => $swms_developed_by_files->firstWhere('report_id', $developed->swms_id)->path ?? '',
                            'is_signed' => $developed->signature === null ? 0 : 1,
                            'type' => 'swms_report',
                            'key' => 'developed_bies',

                        ];
                    }
                }
            }
            if ($swms_swms_approved_bies->isNotEmpty()) {
                $swms_approved_by_files = DB::table('generated_pdf_reports')
                    ->whereIn('report_id', $swms_swms_approved_bies->pluck('swms_id'))
                    ->where('report_type', 'swms_report')
                    ->get(['report_id', 'path']);
                foreach ($swms_swms_approved_bies as $approved) {
                    $swms_reports_record = DB::table('swms')->where('id', $approved->swms_id)->where('customer_id', $userLogin->customer_id)
                        ->where('published', '1')->first();
                    $approved_report_url = $swms_approved_by_files->firstWhere('report_id', $approved->swms_id)->path ?? '';
                    if ($approved_report_url !== '' && $swms_reports_record) {
                        $swmsFile_urls[] = [
                            'id' => $approved->id,
                            'title' => $swms_reports_record->title,
                            'document_number' => $swms_reports_record->swms_number ?? '',
                            'revision_number' => $swms_reports_record->revision_number ?? '',
                            'url' => $swms_approved_by_files->firstWhere('report_id', $approved->swms_id)->path ?? '',
                            'is_signed' => $approved->signature === null ? 0 : 1, // Check if signature is null
                            'type' => 'swms_report',
                            'key' => 'approved_bies',

                        ];
                    }
                }
            }
            if ($swms_swms_review_responsibles_bies->isNotEmpty()) {
                $swms_review_responsibles_bies_files = DB::table('generated_pdf_reports')
                    ->whereIn('report_id', $swms_swms_review_responsibles_bies->pluck('swms_id'))
                    ->where('report_type', 'swms_report')
                    ->get(['report_id', 'path']);
                foreach ($swms_swms_review_responsibles_bies as $reviewed) {
                    $swms_reports_record = DB::table('swms')->where('id', $reviewed->swms_id)->where('customer_id', $userLogin->customer_id)
                        ->where('published', '1')->first();
                    $revied_report_url = $swms_review_responsibles_bies_files->firstWhere('report_id', $reviewed->swms_id)->path ?? '';
                    if ($revied_report_url !== '' && $swms_reports_record) {
                        $swmsFile_urls[] = [
                            'id' => $reviewed->id, // SWMS approved by ID
                            'title' => $swms_reports_record->title,
                            'document_number' => $swms_reports_record->swms_number ?? '',
                            'revision_number' => $swms_reports_record->revision_number ?? '',
                            'url' => $revied_report_url,
                            'is_signed' => $reviewed->signature === null ? 0 : 1,
                            'type' => 'swms_report',
                            'key' => 'review_responsibles',

                        ];
                    }
                }
            }
            if ($swms_compliance_responsibles_bies->isNotEmpty()) {
                $swms_compliance_responsibles_bies_files = DB::table('generated_pdf_reports')
                    ->whereIn('report_id', $swms_compliance_responsibles_bies->pluck('swms_id'))
                    ->where('report_type', 'swms_report')
                    ->get(['report_id', 'path']);
                foreach ($swms_compliance_responsibles_bies as $compliance) {
                    $swms_reports_record = DB::table('swms')->where('id', $compliance->swms_id)->where('customer_id', $userLogin->customer_id)
                        ->where('published', '1')->first();
                    $report_url = $swms_compliance_responsibles_bies_files->firstWhere('report_id', $compliance->swms_id)->path ?? '';
                    if ($report_url !== '' && $swms_reports_record) {
                        $swmsFile_urls[] = [
                            'id' => $compliance->id, // SWMS approved by ID
                            'title' => $swms_reports_record->title,
                            'document_number' => $swms_reports_record->swms_number ?? '',
                            'revision_number' => $swms_reports_record->revision_number ?? '',
                            'url' => $swms_compliance_responsibles_bies_files->firstWhere('report_id', $compliance->swms_id)->path ?? '',
                            'is_signed' => $compliance->signature === null ? 0 : 1,
                            'type' => 'swms_report',
                            'key' => 'compliance_responsibles',

                        ];
                    }
                }
            }
            if ($monitoring_responsibles_bies->isNotEmpty()) {
                $monitoring_responsibles_files = DB::table('generated_pdf_reports')
                    ->whereIn('report_id', $monitoring_responsibles_bies->pluck('swms_id'))
                    ->where('report_type', 'swms_report')
                    ->get(['report_id', 'path']);
                foreach ($monitoring_responsibles_bies as $monitoring) {
                    $swms_monitoring_reports_record = DB::table('swms')->where('id', $monitoring->swms_id)->where('customer_id', $userLogin->customer_id)
                        ->where('published', '1')->first();
                    $report_monitor_url = $monitoring_responsibles_files->firstWhere('report_id', $monitoring->swms_id)->path ?? '';
                    if ($report_monitor_url !== '' && $swms_monitoring_reports_record) {
                        $swmsFile_urls[] = [
                            'id' => $monitoring->id, // SWMS approved by ID
                            'title' => $swms_monitoring_reports_record->title,
                            'document_number' => $swms_monitoring_reports_record->swms_number ?? '',
                            'revision_number' => $swms_monitoring_reports_record->revision_number ?? '',
                            'url' => $report_monitor_url,
                            'is_signed' => $monitoring->signature === null ? 0 : 1,
                            'type' => 'swms_report',
                            'key' => 'monitoring_responsibles',
                        ];
                    }
                }
            }
        }
        // WHS
        if (isset($userLogin->accessTier->id)) {
            // return $request->employee_id;
            $whs_conducted_by_report_id = DB::table('whs_conducted_by')
                ->join('whs_reports', 'whs_conducted_by.whs_report_id', '=', 'whs_reports.id')
                ->where('whs_conducted_by.employee_id', $request->employe_id)
                ->where('whs_reports.is_published', 1)
                ->where('whs_reports.del', 0)
                ->pluck('whs_reports.id')
                ->toArray();
            $whs_additional_persons_report_id = DB::table('whs_additional_persons')
                ->join('whs_reports', 'whs_additional_persons.whs_report_id', '=', 'whs_reports.id')
                ->where('whs_additional_persons.employee_id', $request->employe_id)
                ->where('whs_reports.is_published', 1)
                ->where('whs_reports.del', 0)
                ->pluck('whs_reports.id')
                ->toArray();
            $whs_responsible_report_id = DB::table('whs_responsible')
                ->join('whs_reports', 'whs_responsible.whs_report_id', '=', 'whs_reports.id')
                ->where('whs_responsible.role_id', $userLogin->accessRole->id)
                ->where('whs_reports.is_published', 1)
                ->where('whs_reports.del', 0)
                ->pluck('whs_reports.id')
                ->toArray();
            // Need to discuss Add or Nots
            $whs_hazards_report_id = DB::table('whs_report_hazards')
                ->join('whs_reports', 'whs_report_hazards.whs_report_id', '=', 'whs_reports.id')
                ->where('whs_report_hazards.role_id', $userLogin->accessRole->id)
                ->where('whs_reports.is_published', 1)
                ->where('whs_reports.del', 0)   // need to discuss
                ->pluck('whs_reports.id')
                ->toArray();
            $authorised_by_report_id = DB::table('whs_reports')
                ->where('authorised_by', $request->employe_id)
                ->where('whs_reports.is_published', 1)
                ->where('whs_reports.del', 0)
                ->pluck('whs_reports.id')
                ->toArray();
            $authorised_by_report_ids = array_merge($whs_conducted_by_report_id, $whs_additional_persons_report_id,  $whs_responsible_report_id, $authorised_by_report_id, $whs_hazards_report_id);
            $authorised_by_report_ids = array_unique($authorised_by_report_ids);
            if ($authorised_by_report_ids) {
                // Fetch corresponding PDF reports for the SWMS signatures
                $whs_files = DB::table('generated_pdf_reports')
                    ->whereIn('report_id', $authorised_by_report_ids) // Use pluck to get IDs
                    ->where('report_type', 'whs_report') // Ensure the correct report type
                    ->get(['report_id', 'path']);
                if (!empty($whs_files)) {
                    foreach ($whs_files as $file) {
                        $status = DB::table('whs_signatures')
                            ->where('employee_id', $request->employe_id)
                            ->where('whs_id', $file->report_id) // Access report_id using object property
                            ->exists() ? 1 : 0;
                        $report_record = DB::table('whs_reports')
                            ->where('id', $file->report_id)
                            ->where('whs_reports.is_published', 1)
                            ->where('whs_reports.del', 0)
                            ->first();
                        $whsFile_urls[] = [
                            'id' => $file->report_id,
                            'title' => $report_record->title ?? '',
                            'document_number' => $report_record->number ?? '',
                            'revision_number' => $report_record->revision_number ?? '',
                            'url' => $file->path, // Use the path from the object
                            'is_signed' => $status, // Get signature status
                            'type' => 'whs_report',
                            'key' => 'whs_signatures'

                        ];
                    }
                }
            }
        }
        //  Meeting Manager Notes PDF
        $meeting_manager_notes_pdf = DB::table('meeting_note_signatures')
            ->join('meeting_note_documents_pdfs', 'meeting_note_signatures.employee_note_id', '=', 'meeting_note_documents_pdfs.meeting_note_id')
            ->where('meeting_note_signatures.employee_id', $request->employe_id)
            ->where('meeting_note_signatures.del', "0")
            ->select(
                'meeting_note_signatures.*',
                'meeting_note_documents_pdfs.*'
            )
            ->get();
        if (!empty($meeting_manager_notes_pdf)) {
            foreach ($meeting_manager_notes_pdf as $file) {
                $meeting_document = DB::table('meetings')
                    ->where('id', $file->employee_note_id)
                    ->first();
                $meeting_documents[] = [
                    'id' => $file->meeting_note_id,
                    'title' => $meeting_document->agenda ?? '',
                    'document_number' => $meeting_document->id ?? '',
                    'revision_number' => '',
                    'url' => url($file->document_path),
                    'is_signed' => $file->signature ? 1 : 0, // Get signature status
                    'type' => 'meeting_note_report',
                    'key' => 'meeting_notes_signatures'
                ];
            }
        }
        $log = [
            'employee_id' => auth()->user()->id,
            'message' => 'API SignOff Files fetched successfully',
            'report_id' => 0,
            'report_type' => 'incident_report',

        ];
        storeReportsLogs($log);
        $data = [
            'incident_report' => $incidentFile_urls ?? [],
            'inspection_report' => $inspectionFile_urls ?? [],
            'swms_report' => $swmsFile_urls ?? [],
            'whs_report' => $whsFile_urls ?? [],
            'meeting_documents' => $meeting_documents ?? []
        ];
        return $this->success($data, 'SignOff Files fetched successfully');
    }

    public function copyIncidentReport($parent_incident_id, $child_incident_id)
    {

        try {
            $incident_reported_tos = DB::table('incident_reported_tos')->where('incident_report_id', $parent_incident_id)->get();
            $incident_reported_bys = DB::table('incident_reported_bys')->where('incident_report_id', $parent_incident_id)->get();
            $incident_classification_reports = DB::table('incident_classification_reports')->where('incident_report_id', $parent_incident_id)->get();
            $incident_notified_tos = DB::table('incident_notified_tos')->where('incident_report_id', $parent_incident_id)->get();
            $incident_involved_person_details = DB::table('incident_involved_person_details')->where('incident_report_id', $parent_incident_id)->get();
            $incident_involved_person_trainings = DB::table('incident_involved_person_trainings')->where('incident_report_id', $parent_incident_id)->get();
            $incident_employee_injury_types = DB::table('incident_employee_injury_types')->where('incident_report_id', $parent_incident_id)->get();
            $incident_employer_details = DB::table('incident_employer_details')->where('incident_report_id', $parent_incident_id)->get();
            $incident_report_question_answers = DB::table('incident_report_question_answers')->where('incident_report_id', $parent_incident_id)->get();
            $incident_report_metas = DB::table('incident_report_metas')->where('incident_report_id', $parent_incident_id)->get();
            $incident_injury_managements = DB::table('incident_injury_managements')->where('incident_report_id', $parent_incident_id)->get();
            $incident_final_classifications = DB::table('incident_final_classifications')->where('incident_report_id', $parent_incident_id)->get();
            $incident_signoffs = DB::table('incident_signoffs')->where('incident_report_id', $parent_incident_id)->get();
            $incident_risk_category_details = DB::table('incident_risk_category_details')->where('incident_report_id', $parent_incident_id)->get();
            $incident_witness = DB::table('incident_witness')->where('incident_report_id', $parent_incident_id)->get();
            $incident_documents = DB::table('incident_documents')->where('incident_report_id', $parent_incident_id)->get();
            $tosData = $incident_reported_tos->map(function ($item) use ($child_incident_id) {
                return [
                    'incident_report_id' => $child_incident_id,
                    'reported_to' => $item->reported_to,
                    'company_name' => $item->company_name,
                    'position' => $item->position,
                    'contact_number' => $item->contact_number
                ];
            })->toArray();

            $bysData = $incident_reported_bys->map(function ($item) use ($child_incident_id) {
                return [
                    'incident_report_id' => $child_incident_id,
                    'reported_by' => $item->reported_by,
                    'company_name' => $item->company_name,
                    'position' => $item->position,
                    'contact_number' => $item->contact_number
                ];
            })->toArray();
            $incident_classification_reportsData = $incident_classification_reports->map(function ($item) use ($child_incident_id) {
                return [
                    'incident_report_id' => $child_incident_id,
                    'injury_classification_id' => $item->injury_classification_id,
                    'details' => $item->details,
                    'type' => $item->type,
                    'created_at' => $item->created_at,
                    'updated_at' => $item->updated_at,


                ];
            })->toArray();
            $incident_notified_tos = $incident_notified_tos->map(function ($item) use ($child_incident_id) {
                return [
                    'incident_report_id' => $child_incident_id,
                    'notified_to' => $item->notified_to,
                    'ohs_regulator' => $item->ohs_regulator,
                    'reference_number' => $item->reference_number,
                    'date' => $item->date,
                    'time' => $item->time,
                    'contact_name' => $item->contact_name,

                ];
            })->toArray();
            $incident_involved_person_details = $incident_involved_person_details->map(function ($item) use ($child_incident_id) {
                return [
                    'incident_report_id' => $child_incident_id,
                    'employee_id' => $item->employee_id,
                    'surname' => $item->surname,
                    'contact_number' => $item->contact_number,
                    'gender' => $item->gender,
                    'dob' => $item->dob,
                    'age' => $item->age,
                    'occupation' => $item->occupation,
                    'language' => $item->language,
                    'experience_years' => $item->experience_years,
                    'experience_months' => $item->experience_months,
                    'employment_type' => $item->employment_type,
                    'employee_type' => $item->employee_type,
                    'shift_type' => $item->shift_type,
                    'shift_time' => $item->shift_time,

                ];
            })->toArray();
            $incident_involved_person_trainings = $incident_involved_person_trainings->map(function ($item) use ($child_incident_id) {
                return [
                    'incident_report_id' => $child_incident_id,
                    'training_id' => $item->training_id,
                    'incident_involved_person_id' => $item->incident_involved_person_id,
                    'created_at' => $item->created_at,
                    'updated_at' => $item->updated_at,


                ];
            })->toArray();
            $incident_employee_injury_types = $incident_employee_injury_types->map(function ($item) use ($child_incident_id) {
                return [
                    'incident_report_id' => $child_incident_id,
                    'employer_id' => $item->employer_id,
                    'option' => $item->option,
                    'value' => $item->value,
                    'created_at' => $item->created_at,
                    'updated_at' => $item->updated_at,


                ];
            })->toArray();
            $incident_employer_details = $incident_employer_details->map(function ($item) use ($child_incident_id) {
                return [
                    'incident_report_id' => $child_incident_id,
                    'employer_id' => $item->employer_id,
                    'employer_name' => $item->employer_name,
                    'employer_address' => $item->employer_address,
                    'employer_number' => $item->employer_number,
                    'supervisor' => $item->supervisor,
                    'contact_number' => $item->contact_number,
                    'is_employer_worker' => $item->is_employer_worker,
                    'is_employer_worker_details' => $item->is_employer_worker_details,
                    'created_at' => $item->created_at,
                    'updated_at' => $item->updated_at,


                ];
            })->toArray();
            $incident_report_question_answersData = $incident_report_question_answers->map(function ($item) use ($child_incident_id) {
                return [
                    'incident_report_id' => $child_incident_id,
                    'question_id' => $item->question_id,
                    'answer' => $item->answer,
                    'created_at' => $item->created_at,
                    'updated_at' => $item->updated_at,


                ];
            })->toArray();

            $incident_report_metas_array = $incident_report_metas->map(function ($item) use ($child_incident_id) {
                return [
                    'incident_report_id' => $child_incident_id,
                    'equipment_type' => $item->equipment_type,
                    'registration_number' => $item->registration_number,
                    'damage' => $item->damage,
                    'cost' => $item->cost,
                    'incident_response' => $item->incident_response,
                    'shift_type' => $item->shift_type,
                    'created_at' => $item->created_at,
                    'updated_at' => $item->updated_at,
                ];
            })->toArray();
            $incident_injury_managements = $incident_injury_managements->map(function ($item) use ($child_incident_id) {
                return [
                    'incident_report_id' => $child_incident_id,
                    'date' => $item->date,
                    'time' => $item->time,
                    'representative_name' => $item->representative_name,
                    'previous_injury_date' => $item->previous_injury_date,
                    'injured_names' => $item->injured_names,
                    'is_first_aid' => $item->is_first_aid,
                    'is_referred' => $item->is_referred,
                    'is_doctor' => $item->is_doctor,
                    'is_clinic' => $item->is_clinic,
                    'is_admit' => $item->is_admit,
                    'is_doctor_details' => $item->is_doctor_details,
                    'doctor_name' => $item->doctor_name,
                    'doctor_number' => $item->doctor_number,
                    'doctor_address' => $item->doctor_address,
                    'hospital_name' => $item->hospital_name,
                    'hospital_number' => $item->hospital_number,
                    'hospital_address' => $item->hospital_address,
                    'is_recurrence_injury' => $item->is_recurrence_injury,
                    'previous_report_number' => $item->previous_report_number,
                    'created_at' => $item->created_at,
                    'updated_at' => $item->updated_at,
                ];
            })->toArray();
            $incident_final_classifications = $incident_final_classifications->map(function ($item) use ($child_incident_id) {
                return [
                    'incident_report_id' => $child_incident_id,
                    'final_incident_name' => $item->final_incident_name,
                    'final_incident_details' => $item->final_incident_details,
                    'is_rehabilitation' => $item->is_rehabilitation,
                    'return_to_duty_date' => $item->return_to_duty_date,
                    'rtw_date' => $item->rtw_date,
                    'total_lost_days' => $item->total_lost_days,
                    'is_worker_notified' => $item->is_worker_notified,
                    'notified_by_whom' => $item->notified_by_whom,
                    'date' => $item->date,
                    'time' => $item->time,
                    'details' => $item->details,
                    'created_at' => $item->created_at,
                    'updated_at' => $item->updated_at,
                ];
            })->toArray();
            $incident_signoffs = $incident_signoffs->map(function ($item) use ($child_incident_id) {
                return [
                    'incident_report_id' => $child_incident_id,
                    'role_code' => $item->role_code,
                    'emp_id' => $item->emp_id,
                    'name' => $item->name,
                    'signature' => $item->signature,
                    'time' => $item->time,
                    'date' => $item->date,
                    'created_at' => $item->created_at,
                    'updated_at' => $item->updated_at,
                ];
            })->toArray();
            $incident_risk_category_details = $incident_risk_category_details->map(function ($item) use ($child_incident_id) {
                return [
                    'incident_report_id' => $child_incident_id,
                    'incident_risk_category_id' => $item->incident_risk_category_id,
                    'details' => $item->details,
                    'created_at' => $item->created_at,
                    'updated_at' => $item->updated_at,
                ];
            })->toArray();
            $incident_witness = $incident_witness->map(function ($item) use ($child_incident_id) {
                return [
                    'incident_report_id' => $child_incident_id,
                    'name' => $item->name,
                    'date' => $item->date,
                    'time' => $item->time,
                    'statement_number' => $item->statement_number,
                    'created_at' => $item->created_at,
                    'updated_at' => $item->updated_at,
                ];
            })->toArray();
            $incident_documents = $incident_documents->map(function ($item) use ($child_incident_id) {
                return [
                    'incident_report_id' => $child_incident_id,
                    'title' => $item->title,
                    'number' => $item->number,
                    'created_at' => $item->created_at,
                    'updated_at' => $item->updated_at,
                ];
            })->toArray();


            DB::table('incident_documents')->insert($incident_documents);
            DB::table('incident_reported_tos')->insert($tosData);
            DB::table('incident_reported_bys')->insert($bysData);
            DB::table('incident_classification_reports')->insert($incident_classification_reportsData);
            DB::table('incident_notified_tos')->insert($incident_notified_tos);
            DB::table('incident_involved_person_details')->insert($incident_involved_person_details);
            DB::table('incident_involved_person_trainings')->insert($incident_involved_person_trainings);
            DB::table('incident_employee_injury_types')->insert($incident_employee_injury_types);
            DB::table('incident_employer_details')->insert($incident_employer_details);
            DB::table('incident_report_question_answers')->insert($incident_report_question_answersData);
            DB::table('incident_report_metas')->insert($incident_report_metas_array);
            DB::table('incident_injury_managements')->insert($incident_injury_managements);
            DB::table('incident_final_classifications')->insert($incident_final_classifications);
            DB::table('incident_signoffs')->insert($incident_signoffs);
            DB::table('incident_risk_category_details')->insert($incident_risk_category_details);
            DB::table('incident_witness')->insert($incident_witness);
            $log = [
                'employee_id' => auth()->user()->id,
                'message' => 'Updated Report version to ' . $child_incident_id . ' from parent id' . $parent_incident_id,
                'report_id' => $parent_incident_id,
                'report_type' => 'incident_report'
            ];
            storeReportsLogs($log);
            $this->incidentReportDownloadPdf($child_incident_id, 1);
        } catch (\Exception $e) {
            $shortMessage = substr($e->getMessage(), 0, 200) . (strlen($e->getMessage()) > 200 ? '...' : '');
            $log = [
                'employee_id' => auth()->user()->id,
                'message' => 'Updated Report version to ' . $child_incident_id . ' from parent id' . $parent_incident_id . "error: " .   $shortMessage,
                'report_id' =>  $child_incident_id,
                'report_type' => 'incident_report',
                'error_type' => 'Exception error'

            ];
            storeReportsLogs($log);
            return $this->error($shortMessage);
          
        }
    }
    public function incidentReportDownloadPdf($id, $response = 0)
    {
        if (!$id) {
            $log = [
                'employee_id' => auth()->user()->id,
                'message' => 'Report ID is required ',
                'report_id' => 0,
                'report_type' => 'incident_report',
                'error_type' => 'Not found error'

            ];
            storeReportsLogs($log);
            return $this->message('Report ID is required', 422);
        }
        try {
            $userTable = $this->getUserTable(); // Get user type (customer or emp)
            $dataClassifications = IncidentReport::where('id', $id)
                ->with([
                    'injuryClassifications' => function ($query) {
                        $query->select('incident_injury_classifications.id as injury_id', 'incident_injury_classifications.title');
                    }
                ])
                ->get()
                ->pluck('injuryClassifications')
                ->flatten()
                ->map(function ($classification) {
                    return ['incident_injury_classifications_title' => $classification->title];
                });
            $data = IncidentReport::with([
                'site:id,title,supervisor_id',
                'project:id,title',
                'authorisedBy:id,first_name,last_name,emp_id',
                'reportedTos' => function ($query) {
                    $query->with([
                        'reportedToEmployee:id,emp_id,first_name,last_name',
                        'positionDetail:id,title'
                    ]);
                },
                'reportedToDetails:emp_id,first_name,last_name',
                'reportedBys' => function ($query) {
                    $query->with([
                        'reportedByEmployee:id,emp_id,first_name,last_name',
                        'positionDetail:id,title'
                    ]);
                },
                'reportedByDetails:emp_id,first_name,last_name',
                'classifications:id,incident_report_id,injury_classification_id,details',
                'injuryClassifications',
                'notifiableClassifications' => function ($query) use ($userTable) {
                    if ($userTable === 'customer') {
                        $query->where('customer_id', auth()->id())
                            ->where('workspace_id', auth()->user()->current_workspace_id);
                    } elseif ($userTable === 'emp') {
                        $query->where('customer_id', auth()->user()->customer_id)
                            ->where('workspace_id', auth()->user()->workspace_id);
                    } else {
                        $query->whereRaw('1 = 0');
                    }
                },
                'notifiedTos' => function ($query) {
                    $query->select('incident_notified_tos.*');
                },
                'involvedPersons' => function ($query) {
                    $query->select('incident_involved_person_details.*')
                        ->with('employmentType:id,title'); // Ensure employmentType is loaded
                },
                'involvedTrainings:id,title',
                'injuryTypes:id,incident_report_id,option',
                'employerDetails:id,incident_report_id,employer_name,is_employer_worker',
                'questions' => function ($query) use ($userTable) {
                    if ($userTable === 'customer') {
                        $query->where('customer_id', auth()->id())
                            ->where('workspace_id', auth()->user()->current_workspace_id);
                    } elseif ($userTable === 'emp') {
                        $query->where('customer_id', auth()->user()->customer_id)
                            ->where('workspace_id', auth()->user()->workspace_id);
                    } else {
                        $query->whereRaw('1 = 0');
                    }
                },
                // 'questionAnswers:id,incident_report_id,question_id,answer',
                'incidenteportMetas:id,incident_report_id,shift_type,equipment_type,registration_number,damage,cost',
                'injuryManagements',
                'finalClassifications:id,incident_report_id,final_incident_details,is_rehabilitation,return_to_duty_date,rtw_date,total_lost_days,is_worker_notified,notified_by_whom,date,time,details',
                'incidentRiskCategoryDetails:id,incident_report_id,incident_risk_category_id,details',
                'incidentWitnesses:id,incident_report_id,name,date,time,statement_number',
                'incidentDocuments:id,incident_report_id,title,number',
                'supervisor:emp_id,first_name,last_name',
                'forman:emp_id,first_name,last_name',
            ])->find($id);
            // $supervisorIds = explode(',', $data->site_supervisor_id);
            $supervisors = EmpPersonalDetails::where('emp_id', $data->site->supervisor_id)
                ->select('first_name as emp_personal_supervisor_detail_f_name', 'last_name as emp_personal_supervisor_detail_last_name')
                ->get();
            $questionsAndAnswers = IncidentReportQuestionAnswer::where('incident_report_id', $id)
                ->with('question:id,question')
                ->get()
                ->map(function ($qa) {
                    return [
                        'question' => $qa->question->question ?? null,
                        'answer' => $qa->answer,
                    ];
                });
            $incidentSignoffs = IncidentSignoff::where('incident_report_id', $id)
                ->with([
                    'employee:emp_id,first_name,last_name' // Load related employee details
                ])
                ->get(['emp_id', 'signature', 'date', 'time', 'role_code'])
                ->map(function ($signoff) {
                    return [
                        'emp_personal_details_first_name' => $signoff->employee->first_name ?? null,
                        'emp_personal_details_last_name' => $signoff->employee->last_name ?? null,
                        'inc_signoffs_signature' => $signoff->signature,
                        'inc_signoffs_date' => $signoff->date,
                        'inc_signoffs_time' => $signoff->time,
                        'inc_signoffs_role_code' => $signoff->role_code,
                    ];
                });
            $reportImages = IncidentImage::where('incident_report_id', $id)->get();
            $viewPath = 'Incident.incident_reports_pdf';
            $html = view($viewPath, compact('data', 'questionsAndAnswers', 'incidentSignoffs', 'dataClassifications', 'reportImages', 'supervisors'))->render();
            $filename = sprintf(
                "%s_%s_R%s.pdf",
                preg_replace('/[^A-Za-z0-9_]+/', '_', trim(str_replace(' ', '_', $data->title ?? 'incident_report'))),
                preg_replace('/[^A-Za-z0-9_]+/', '', trim(str_replace(' ', '_', $data->document_number ?? rand(99999, 999999)))),
                preg_replace('/[^A-Za-z0-9_]+/', '', trim(str_replace(' ', '_', $data->revision_number ?? '00')))
            );
            // Define Storage Path
            $folderPath = public_path('IncidentReportDocuments');
            if (!File::exists($folderPath)) {
                File::makeDirectory($folderPath, 0755, true);
            }
            $pdfPath = $folderPath . '/' . $filename;
            // Check if the PDF already exists
            $existingReport = GeneratedPdfReport::where([
                ['report_type', 'incident_report'],
                ['report_id', $id],
                ['path', url('IncidentReportDocuments/' . $filename)]
            ])->first();
            if ($existingReport) {
                return $this->success(['downloadUrl' => url('IncidentReportDocuments/' . $filename)], 'Report already exists..');
            }
            // Generate PDF
            $pdf = PDF::loadHTML($html)->setPaper('A4', 'portrait');
            $pdf->save($pdfPath);
            // Save Report in Database
            if ($userTable === "customer") {
                $customerId = auth()->id();
                $workspaceId = auth()->user()->current_workspace_id;
            } elseif ($userTable === "emp") {
                $customerId = auth()->user()->customer_id;
                $workspaceId = auth()->user()->workspace_id;
            }
            $generatedReport = new GeneratedPdfReport();
            $generatedReport->report_id = $id;
            $generatedReport->path = url('IncidentReportDocuments/' . $filename);
            $generatedReport->report_type = 'incident_report';
            $generatedReport->customer_id = $customerId;
            $generatedReport->workspace_id = $workspaceId;
            $generatedReport->save();
            return $this->success(['downloadUrl' => url('IncidentReportDocuments/' . $filename)], 'Incident report generated successfully.');
        } catch (\Exception $e) {
            $shortMessage = substr($e->getMessage(), 0, 200) . (strlen($e->getMessage()) > 200 ? '...' : '');
            $log = [
                'employee_id' => auth()->user()->id,
                'message' => 'Download report id' . $id . ' error ' .   $shortMessage,
                'report_id' =>  $id,
                'report_type' => 'incident_report',
                'error_type' => 'Exception error'

            ];
            storeReportsLogs($log);
            return $this->error($shortMessage);
        }
    }

    public function IncidentReportPdfFiles(Request $request)
    {
        // Get user type (customer or emp)
        $userTable = $this->getUserTable();
        $query = GeneratedPdfReport::where('report_type', 'incident_report');
        if (!$query) {
            return $this->message('No records found.', 404);
        }
        if ($userTable === 'customer') {
            $query->where('customer_id', auth()->user()->id)
                ->where('workspace_id', auth()->user()->current_workspace_id);
        } elseif ($userTable === 'emp') {
            $query->where('customer_id', auth()->user()->customer_id)
                ->where('workspace_id', auth()->user()->workspace_id);
        } else {
            return $this->message('You do not have permission to access these records.', 403);
        }
        return $this->withCount($query, 'Retrieve all records Successfully');
    }
    public function downloadPdfFile($filename)
    {
        $path = public_path("Files_manager/inspection_reports/");

        if (!file_exists($path . $filename)) {
            $html = '<html>
            <head>
                <style>
                    body {
                        font-family: Arial, sans-serif;
                        text-align: center;
                        padding: 50px;
                        background-color: #f8f9fa;
                    }
                    .container {
                        max-width: 600px;
                        margin: 0 auto;
                        border: 1px solid #ddd;
                        padding: 20px;
                        border-radius: 5px;
                        background-color: #fff;
                    }
                    h1 {
                        color: #dc3545;
                    }
                    p {
                        color: #333;
                    }
                </style>
            </head>
            <body>
                <div class="container">
                    <h1>File Not Found</h1>
                    <p>The file is not found or has been deleted.</p>
                </div>
            </body>
        </html>';

            return response($html, 404)
                ->header('Content-Type', 'text/html');
        }

        return response()->download($path . $filename);
    }

    public function  incidentReportPublishReport(Request $request)
    {
        $incidentReport = IncidentReport::find($request->incident_report_id);
        if (!$incidentReport) {
            return $this->message('Report Not Found please try again.', 404);
        }
        $userTable = $this->getUserTable();
        if ($userTable === 'customer' && ($incidentReport->customer_id !== auth()->id() || $incidentReport->workspace_id !== auth()->user()->current_workspace_id)) {
            return $this->message('You do not have access to this report.', 403);
        }
        if ($userTable === 'emp' && ($incidentReport->customer_id !== auth()->user()->customer_id || $incidentReport->workspace_id !== auth()->user()->workspace_id)) {
            return $this->message('You do not have access to this report.', 403);
        }
        $incidentReport->is_published = $incidentReport->is_published == '1' ? '0' : '1';
        $incidentReport->save();
        if (!$incidentReport) {
            return $this->error('Failed to update report status');
        }
        $log = [
            'employee_id' => auth()->user()->id,
            'message' => 'Report status updated successfully',
            'report_id' =>  $request->incident_report_id,
            'report_type' => 'incident_report'
        ];
        storeReportsLogs($log);
        if ($incidentReport->is_published == '1') {
            $response = $this->incidentReportDownloadPdf($request->incident_report_id);
            $responseData = json_decode($response->getContent(), true);
            return $this->success($responseData['data'], 'Report status updated successfully');
        }
        return $this->message('Report status updated successfully');
    }
    public function incidentReportDownloadPdfRegenerate($id)
    {
        if (!$id) {
            $log = [
                'employee_id' => auth()->user()->id,
                'message' => 'Report ID is required ',
                'report_id' => 0,
                'report_type' => 'incident_report',
                'error_type' => 'Not found error'

            ];
            storeReportsLogs($log);
            return $this->message('Report ID is required', 422);
        }
        try {
            $userTable = $this->getUserTable(); // Get user type (customer or emp)
            $dataClassifications = IncidentReport::where('id', $id)
                ->with([
                    'injuryClassifications' => function ($query) {
                        $query->select('incident_injury_classifications.id as injury_id', 'incident_injury_classifications.title');
                    }
                ])
                ->get()
                ->pluck('injuryClassifications')
                ->flatten()
                ->map(function ($classification) {
                    return ['incident_injury_classifications_title' => $classification->title];
                });
            $data = IncidentReport::with([
                'site:id,title,supervisor_id',
                'project:id,title',
                'authorisedBy:id,first_name,last_name,emp_id',
                'reportedTos' => function ($query) {
                    $query->with([
                        'reportedToEmployee:id,emp_id,first_name,last_name',
                        'positionDetail:id,title'
                    ]);
                },
                'reportedToDetails:emp_id,first_name,last_name',
                'reportedBys' => function ($query) {
                    $query->with([
                        'reportedByEmployee:id,emp_id,first_name,last_name',
                        'positionDetail:id,title'
                    ]);
                },
                'reportedByDetails:emp_id,first_name,last_name',
                'classifications:id,incident_report_id,injury_classification_id,details',
                'injuryClassifications',
                'notifiableClassifications' => function ($query) use ($userTable) {
                    if ($userTable === 'customer') {
                        $query->where('customer_id', auth()->id())
                            ->where('workspace_id', auth()->user()->current_workspace_id);
                    } elseif ($userTable === 'emp') {
                        $query->where('customer_id', auth()->user()->customer_id)
                            ->where('workspace_id', auth()->user()->workspace_id);
                    } else {
                        $query->whereRaw('1 = 0');
                    }
                },
                'notifiedTos' => function ($query) {
                    $query->select('incident_notified_tos.*');
                },
                'involvedPersons' => function ($query) {
                    $query->select('incident_involved_person_details.*')
                        ->with('employmentType:id,title'); // Ensure employmentType is loaded
                },
                'involvedTrainings:id,title',
                'injuryTypes:id,incident_report_id,option',
                'employerDetails:id,incident_report_id,employer_name,is_employer_worker',
                'questions' => function ($query) use ($userTable) {
                    if ($userTable === 'customer') {
                        $query->where('customer_id', auth()->id())
                            ->where('workspace_id', auth()->user()->current_workspace_id);
                    } elseif ($userTable === 'emp') {
                        $query->where('customer_id', auth()->user()->customer_id)
                            ->where('workspace_id', auth()->user()->workspace_id);
                    } else {
                        $query->whereRaw('1 = 0');
                    }
                },
                // 'questionAnswers:id,incident_report_id,question_id,answer',
                'incidenteportMetas:id,incident_report_id,shift_type,equipment_type,registration_number,damage,cost',
                'injuryManagements',
                'finalClassifications:id,incident_report_id,final_incident_details,is_rehabilitation,return_to_duty_date,rtw_date,total_lost_days,is_worker_notified,notified_by_whom,date,time,details',
                'incidentRiskCategoryDetails:id,incident_report_id,incident_risk_category_id,details',
                'incidentWitnesses:id,incident_report_id,name,date,time,statement_number',
                'incidentDocuments:id,incident_report_id,title,number',
                'supervisor:emp_id,first_name,last_name',
                'forman:emp_id,first_name,last_name',
            ])->find($id);
            // $supervisorIds = explode(',', $data->site_supervisor_id);
            $supervisors = EmpPersonalDetails::where('emp_id', $data->site->supervisor_id)
                ->select('first_name as emp_personal_supervisor_detail_f_name', 'last_name as emp_personal_supervisor_detail_last_name')
                ->get();
            $questionsAndAnswers = IncidentReportQuestionAnswer::where('incident_report_id', $id)
                ->with('question:id,question')
                ->get()
                ->map(function ($qa) {
                    return [
                        'question' => $qa->question->question ?? null,
                        'answer' => $qa->answer,
                    ];
                });
            $incidentSignoffs = IncidentSignoff::where('incident_report_id', $id)
                ->with([
                    'employee:emp_id,first_name,last_name' // Load related employee details
                ])
                ->get(['emp_id', 'signature', 'date', 'time', 'role_code'])
                ->map(function ($signoff) {
                    return [
                        'emp_personal_details_first_name' => $signoff->employee->first_name ?? null,
                        'emp_personal_details_last_name' => $signoff->employee->last_name ?? null,
                        'inc_signoffs_signature' => $signoff->signature,
                        'inc_signoffs_date' => $signoff->date,
                        'inc_signoffs_time' => $signoff->time,
                        'inc_signoffs_role_code' => $signoff->role_code,
                    ];
                });
            $reportImages = IncidentImage::where('incident_report_id', $id)->get();
            $viewPath = 'Incident.incident_reports_pdf';
            $html = view($viewPath, compact('data', 'questionsAndAnswers', 'incidentSignoffs', 'dataClassifications', 'reportImages', 'supervisors'))->render();

            $filename = sprintf(
                "%s_%s_R%s.pdf",
                preg_replace('/[^A-Za-z0-9_]+/', '_', trim(str_replace(' ', '_', $data->title ?? 'incident_report'))),
                preg_replace('/[^A-Za-z0-9_]+/', '', trim(str_replace(' ', '_', $data->document_number ?? rand(99999, 999999)))),
                preg_replace('/[^A-Za-z0-9_]+/', '', trim(str_replace(' ', '_', $data->revision_number ?? '00')))
            );

            $folderPath = public_path('IncidentReportDocuments');
            if (!File::exists($folderPath)) {
                File::makeDirectory($folderPath, 0755, true);
            }

            $report_full_path = public_path('IncidentReportDocuments/' . $filename);
            $path = public_path("IncidentReportDocuments/{$filename}");
            $already_file_status = GeneratedPdfReport::where('report_type', 'incident_report')
                ->where('report_id', $id)
                ->where('path', url('IncidentReportDocuments/' . $filename))
                ->first();

            if (file_exists($report_full_path) && $already_file_status && $already_file_status->path == url('IncidentReportDocuments/' . $filename)) {
                GeneratedPdfReport::where('report_type', 'incident_report')
                    ->where('report_id', $id)
                    ->where('path', url('IncidentReportDocuments/' . $filename))
                    ->delete();
                unlink($report_full_path);
            }

            if ($userTable === "customer") {
                $customerId = auth()->id();
                $workspaceId = auth()->user()->current_workspace_id;
            } elseif ($userTable === "emp") {
                $customerId = auth()->user()->customer_id;
                $workspaceId = auth()->user()->workspace_id;
            }

            GeneratedPdfReport::updateOrCreate(
                ['report_id' => $id],
                [
                    'report_id' => $id,
                    'path' => url('IncidentReportDocuments/' . $filename),
                    'report_type' => 'incident_report',
                    'customer_id' => $customerId,
                    'workspace_id' => $workspaceId
                ]
            );
            $pdf = PDF::loadHTML($html)
                ->setPaper('A4', 'portrait');
            $pdf->save($path);
           return $this->success([ 'downloadUrl' => url('IncidentReportDocuments/' . $filename)] , 'PDF Regenerated successfully');
          
        } catch (\Exception $e) {
            $shortMessage = substr($e->getMessage(), 0, 200) . (strlen($e->getMessage()) > 200 ? '...' : '');
            $log = [
                'employee_id' => auth()->user()->id,
                'message' => 'Download report id' . $id . ' error ' .   $shortMessage,
                'report_id' =>  $id,
                'report_type' => 'incident_report',
                'error_type' => 'Exception error'

            ];
            storeReportsLogs($log);
            return $this->error($shortMessage);
        }
    }
    public function incidentReportViewSignature(Request $request)
    {
        $user = auth()->user();
        $userTable = $this->getUserTable();
        // Define conditions based on user type
        $customerCondition = fn($query) => $query->where([
            ['customer_id', $user->id],
            ['workspace_id', $user->current_workspace_id]
        ]);
        $employeeCondition = fn($query) => $query->where([
            ['customer_id', $user->customer_id],
            ['workspace_id', $user->workspace_id]
        ]);
        // Fetch Incident Report with necessary relationships & conditions
        $incidentReport = IncidentReport::where('id', $request->incident_report_id)
            ->when($userTable === 'customer', $customerCondition)
            ->when($userTable === 'emp', $employeeCondition)
            ->whereHas('authorisedByCompany', function ($query) use ($userTable, $customerCondition, $employeeCondition) {
                $query->where([['status', 1], ['approved', 1], ['compeleted', 1]]);
                if ($userTable === 'customer') {
                    $customerCondition($query);
                } elseif ($userTable === 'emp') {
                    $employeeCondition($query);
                }
            })
            ->whereHas('authorisedBy', fn($query) => $query->where('del', 0)) // Ensure personal details are not deleted
            ->with([
                'authorisedByCompany:id,access_role,customer_id,workspace_id',
                'authorisedBy:emp_id,first_name,last_name'
            ])
            ->first();
        if (!$incidentReport) {
            return $this->error('Incident Report Not Found', 404);
        }
        // Fetch incident signoffs
        $incidentSignoffs = IncidentSignoff::where('incident_report_id', $request->incident_report_id)
            ->whereHas('employee', fn($query) => $query->where('del', 0)) // Ensure employees are not deleted
            ->whereHas('companyDetails', function ($query) use ($userTable, $customerCondition, $employeeCondition) {
                $query->where([['status', 1], ['approved', 1], ['compeleted', 1]]);
                if ($userTable === 'customer') {
                    $customerCondition($query);
                } elseif ($userTable === 'emp') {
                    $employeeCondition($query);
                }
            })
            ->with([
                'employee:emp_id,first_name,last_name',
                'companyDetails:id,status,approved,compeleted'
            ])
            ->get();
        return $this->success(['incident_report' => $incidentReport, 'incident_signoffs' => $incidentSignoffs], 'Data retrieved successfully');
    }




    public function imageStore(Request $request)
    {
        // Validate request
        $validator = Validator::make($request->all(), [
            'incident_report_id' => 'required|exists:incident_reports,id',
            'image' => 'required|image|mimes:jpeg,png,jpg,gif,svg|max:10240',
        ]);
        if ($validator->fails()) {
            storeReportsLogs([
                'employee_id' => auth()->user()->id,
                'message' => 'WHS Step 3 store: ' . $validator->errors()->first(),
                'report_id' => $request->incident_report_id,
                'report_type' => 'incident_report',
                'error_type' => 'Validation error',
            ]);
            return $this->message($validator->errors()->first(), 422);
        }
        $incidentReport = IncidentReport::findOrFail($request->incident_report_id);
        $userTable = $this->getUserTable();
        $user = auth()->user();
        if (
            ($userTable === 'customer' && ($incidentReport->customer_id !== $user->id || $incidentReport->workspace_id !== $user->current_workspace_id)) ||
            ($userTable === 'emp' && ($incidentReport->customer_id !== $user->customer_id || $incidentReport->workspace_id !== $user->workspace_id))
        ) {
            return $this->message('You do not have access to this report.', 403);
        }
        // Upload Image
        if ($request->hasFile('image') && $request->file('image')->isValid()) {
            $imagePath = $this->handleFileImageUpload($request, 'IncidentReportDocuments')['path'] ?? null;
            if (!$imagePath) {
                return $this->message('Image upload failed.', 422);
            }
            $image = IncidentImage::create([
                'incident_report_id' => $request->incident_report_id,
                'image' => $imagePath,
            ]);
            return $this->success($image, 'Image uploaded successfully');
        }

        return $this->message('No valid image provided.', 422);
    }


    public function getImages(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'incident_report_id' => 'required|exists:incident_reports,id',
        ]);
        if ($validator->fails()) {
            storeReportsLogs([
                'employee_id' => auth()->user()->id,
                'message' => 'WHS Step 3 store: ' . $validator->errors()->first(),
                'report_id' => $request->incident_report_id,
                'report_type' => 'incident_report',
                'error_type' => 'Validation error',
            ]);
            return $this->message($validator->errors()->first(), 422);
        }
        $incidentReport = IncidentReport::find($request->incident_report_id);
        if (!$incidentReport) {
            return $this->message('Incident report not found.', 404);
        }
        $userTable = $this->getUserTable();
        $user = auth()->user();
        if (
            ($userTable === 'customer' && ($incidentReport->customer_id !== $user->id || $incidentReport->workspace_id !== $user->current_workspace_id)) ||
            ($userTable === 'emp' && ($incidentReport->customer_id !== $user->customer_id || $incidentReport->workspace_id !== $user->workspace_id))
        ) {
            return $this->message('You do not have access to this report.', 403);
        }
        // Fetch Images
        $images = IncidentImage::where('incident_report_id', $request->incident_report_id)->get();
        if (!$images) {
            return $this->message('No images found.', 404);
        }
        return $this->withCount($images, 'Images retrieved successfully!');
    }

    public function deleteImage(Request $request)
    {
        // Validate request
        $validator = Validator::make($request->all(), [
            'incident_report_id' => 'required|exists:incident_reports,id',
            'id' => 'required', // Can be a single ID or an array of IDs
        ]);
        if ($validator->fails()) {
            return $this->message($validator->errors()->first(), 422);
        }
        $imageIds = is_array($request->id) ? $request->id : [$request->id];
        $incidentReport = IncidentReport::findOrFail($request->incident_report_id);
        $userTable = $this->getUserTable();
        $user = auth()->user();
        if (
            ($userTable === 'customer' && ($incidentReport->customer_id !== $user->id || $incidentReport->workspace_id !== $user->current_workspace_id)) ||
            ($userTable === 'emp' && ($incidentReport->customer_id !== $user->customer_id || $incidentReport->workspace_id !== $user->workspace_id))
        ) {
            return $this->message('You do not have permission to delete these images.', 403);
        }
        // Fetch all images matching the provided IDs
        $images = IncidentImage::whereIn('id', $imageIds)
            ->where('incident_report_id', $request->incident_report_id)
            ->get();
        if ($images->isEmpty()) {
            return $this->message('No matching images found.', 404);
        }
        foreach ($images as $image) {
            $imagePath = public_path($image->image);
            if ($image->image && file_exists($imagePath)) {
                unlink($imagePath); // Remove from folder
            }
            $image->delete(); // Remove from database
        }
        return $this->message(count($imageIds) > 1 ? 'Images deleted successfully.' : 'Image deleted successfully.');
    }
}
