<?php

namespace App\Http\Controllers;

use PDF;
use Exception;
use App\Models\Role;
use App\Models\Sites;
use App\Models\Project;
use Illuminate\Http\Request;
use App\Models\IncidentReport;
use App\Models\InspectionPlan;
// use Spatie\Browsershot\Browsershot;
use App\Models\InspectionImage;
use App\Models\EmpCompanyDetails;
use App\Models\GeneratedPdfReport;
use App\Models\InspectionPlanMeta;
use Illuminate\Support\Facades\DB;
use App\Models\InspectionPlanTable;
use App\Models\InspectionPlanGeneral;
use App\Models\InspectionPlanChecklist;
use App\Models\InspectionPlanSignature;
use Illuminate\Support\Facades\Validator;
use Illuminate\Pagination\LengthAwarePaginator;

class InspectionPlanController extends Controller
{
    function Home()
    {
        return view('InspectionPlan.index');
    }
    function CreateNewInspectionPlan($step, $inspection_plan_id)
    {
        $inspection_plan_checklist = InspectionPlanChecklist::where('inspection_plan_id', $inspection_plan_id)->where('status', "1")->get('name');
        return view('InspectionPlan.NewInspectionReport.newInspectionReport', compact('inspection_plan_checklist'));
    }
    public function index(Request $request)
    {
        $userTable = $this->getUserTable(); // Get user type (customer or emp)
        $authId = auth()->user()->id;
        $query = InspectionPlan::query();
        $query->orderBy('id', 'desc')
            ->where('del', '0')
            ->with('authorizedBy');
        // Apply customer and employee access checks
        if ($userTable === 'customer') {
            $query->where('customer_id', $authId)
                ->where('workspace_id', auth()->user()->current_workspace_id);
        }
        if ($userTable === 'emp') {
            $query->whereHas('site', function ($q) {
                $q->where('customer_id', auth()->user()->customer_id)
                    ->where('workspace_id', auth()->user()->workspace_id);
            });
        }
        // Apply SSO-specific site filtering
        if ($authId != 1) {
            $sites = Sites::where('sso_id', $authId)->pluck('id');
            $query->whereIn('site_id', $sites);
        }
        // Apply filters if provided
        if ($request->filled('filter')) {
            $query = $this->filter(json_decode($request->filter, true), $query);
        }
        $start = $request->input('from', 0);
        $query->offset($start)->limit(20);
        $queryResult = $query->get();
        return $this->withFilter($queryResult, $request->input('filter', []), 'Get Incident Reports List Successfully');
    }


    public function filter($filters, $query)
    {

        foreach ($filters as $filterName => $filterValue) {
            if ($filterValue != null ||  $filterValue != "") {
                switch ($filterName) {
                    case 'authorised_by':
                        $filterValue  = explode(" ", $filterValue);

                        if (count($filterValue) == 1) {
                            $query->whereHas('authorizedBy', function ($subquery) use ($filterValue) {
                                $subquery->where('first_name', 'like', '%' . $filterValue[0] . '%');
                            });
                        }
                        if (count($filterValue) == 2) {
                            $query->whereHas('authorizedBy', function ($subquery) use ($filterValue) {
                                $subquery->where('first_name', 'like', '%' . $filterValue[0] . '%');
                                $subquery->where('last_name', 'like', '%' . $filterValue[1] . '%');
                            });
                        }
                        if (count($filterValue) == 3) {

                            $query->whereHas('authorizedBy', function ($subquery) use ($filterValue) {
                                $subquery->where('first_name', 'like', '%' . $filterValue[0] . '%');
                                $subquery->where('middle_name', 'like', '%' . $filterValue[1] . '%');
                                $subquery->where('last_name', 'like', '%' . $filterValue[2] . '%');
                            });
                        }
                        break;
                    case 'title':
                        $query->where('title', 'like', '%' . $filterValue . '%');
                        break;
                    case 'document_number':
                        $query->where('document_number', 'like', '%' . $filterValue . '%');
                        break;
                    case 'issue_date':
                        $query->where('issue_date', 'like', '%' . $filterValue . '%');
                        break;
                    case 'process':
                        $query->where('process', 'like', '%' . $filterValue . '%');
                        break;
                    case 'published':
                        $query->where('is_published', 'like', '%' . $filterValue . '%');
                        break;
                }
            }
        }

        return $query;
    }
    public function destroy($id)
    {
        $userTable = $this->getUserTable(); // Get user type (customer or emp)
        $inspectionPlan = InspectionPlan::where('id', $id)->where('del', '0')->first();
        if (!$inspectionPlan) {
            $log = [
                'employee_id' => auth()->user()->id,
                'message' => 'Inspection Plan Delete Failed: ID ' . $id . ' not found',
                'report_id' => $id ?? 0,
                'report_type' => 'inspection_report',
                'error_type' => 'Not found error'
            ];
            storeReportsLogs($log);
            return $this->message('Inspection Plan not found or already deleted', 404);
        }
        // Apply customer and employee checks
        if ($userTable === 'customer' && ($inspectionPlan->customer_id != auth()->id() || $inspectionPlan->workspace_id != auth()->user()->current_workspace_id)) {
            return $this->message('Unauthorized access to this inspection plan', 403);
        }

        if ($userTable === 'emp' && ($inspectionPlan->customer_id != auth()->user()->customer_id || $inspectionPlan->workspace_id != auth()->user()->workspace_id)) {
            return $this->message('Unauthorized access to this inspection plan', 403);
        }
        // Mark the inspection plan as deleted
        $status = $inspectionPlan->update(['del' => '1']);
        if (!$status) {
            $log = [
                'employee_id' => auth()->user()->id,
                'message' => 'Inspection Plan Delete Failed: ID ' . $id,
                'report_id' => $id ?? 0,
                'report_type' => 'inspection_report',
                'error_type' => 'Update failed'
            ];
            storeReportsLogs($log);
            return $this->message('Inspection Plan delete failed. Please try again.', 500);
        }
        // Log the successful deletion
        $log = [
            'employee_id' => auth()->user()->id,
            'message' => 'Inspection Plan Deleted: ID ' . $id . ' successfully',
            'report_id' => $id ?? 0,
            'report_type' => 'inspection_report'
        ];
        storeReportsLogs($log);
        return $this->message('Inspection Plan deleted successfully');
    }

    public function destroySignature($id)
    {
        try {
            $signature = InspectionPlanSignature::where('id', $id)->where('del', '0')->first();
            if (!$signature) {
                $log = [
                    'employee_id' => auth()->user()->id,
                    'message' => 'Inspection Plan Signature Deletion Failed: ID ' . $id . ' not found',
                    'report_id' => $id ?? 0,
                    'report_type' => 'inspection_report',
                    'error_type' => 'Not found error',
                ];
                storeReportsLogs($log);
                return $this->message('Inspection Plan Signature not found or already deleted', 404);
            }
            // Customer and Employee checks
            $userTable = $this->getUserTable();
            $inspectionPlan = InspectionPlan::find($signature->inspection_plan_id);
            if (!$inspectionPlan) {
                return $this->message('Associated Inspection Plan not found', 404);
            }
            if ($userTable === 'customer' && ($inspectionPlan->customer_id != auth()->id() || $inspectionPlan->workspace_id != auth()->user()->current_workspace_id)) {
                return $this->message('Unauthorized access to this inspection plan signature', 403);
            }
            if ($userTable === 'emp' && ($inspectionPlan->customer_id != auth()->user()->customer_id || $inspectionPlan->workspace_id != auth()->user()->workspace_id)) {
                return $this->message('Unauthorized access to this inspection plan signature', 403);
            }
            $status = $signature->update(['del' => '1']);
            if (!$status) {
                $log = [
                    'employee_id' => auth()->user()->id,
                    'message' => 'Inspection Plan Signature Deletion Failed: ID ' . $id,
                    'report_id' => $id ?? 0,
                    'report_type' => 'inspection_report',
                    'error_type' => 'Update failed',
                ];
                storeReportsLogs($log);
                return $this->message('Failed to delete signature. Please try again.', 500);
            }
            $log = [
                'employee_id' => auth()->user()->id,
                'message' => 'Inspection Plan Signature Deleted: ID ' . $id . ' successfully',
                'report_id' => $id ?? 0,
                'report_type' => 'inspection_report',
            ];
            storeReportsLogs($log);
            return $this->message('Signature deleted successfully');
        } catch (\Exception $e) {
            $shortMessage = substr($e->getMessage(), 0, 200) . (strlen($e->getMessage()) > 200 ? '...' : '');
            $log = [
                'employee_id' => auth()->user()->id,
                'message' => 'Inspection Signature Deletion error: ' . $shortMessage,
                'report_id' => $id ?? 0,
                'report_type' => 'inspection_report',
                'error_type' => 'Exception error',
            ];
            storeReportsLogs($log);
            return $this->message($shortMessage, 422);
        }
    }

    public function copyInspectionPlanReport($old_inspection_plan_report_id, $new_inspection_plan_id)
    {
        if (!$old_inspection_plan_report_id) {
            return response()->json(['message' => 'New report id is required .please retry' . $old_inspection_plan_report_id], 422);
        }
        // Fetch the checklist associated with the old inspection plan
        $inspection_plan_checklist = InspectionPlanChecklist::where('inspection_plan_id', $old_inspection_plan_report_id)->get();
        // Check if there are any checklist items
        if ($inspection_plan_checklist->isEmpty()) {
            return response()->json(['message' => 'No checklist items found for the specified inspection plan.'], 422);
        } else {
            // Duplicate each checklist item
            foreach ($inspection_plan_checklist as $checklist) {
                $checklistClone = $checklist->replicate();
                $checklistClone->inspection_plan_id = $new_inspection_plan_id;
                $checklistClone->save();
                // Get the cloned checklist ID for future references
                $new_checklist_id = $checklistClone->id;
                // Duplicate Inspection Plan General Data associated with the old checklist
                $inspection_general_data = InspectionPlanGeneral::where('inspection_plan_id', $old_inspection_plan_report_id)
                    ->where('iptc_id', $checklist->id) // Use current checklist ID
                    ->get();
                foreach ($inspection_general_data as $general) {
                    $generalClone = $general->replicate();
                    $generalClone->inspection_plan_id = $new_inspection_plan_id;
                    $generalClone->iptc_id = $new_checklist_id; // Update with new checklist ID
                    $generalClone->save();
                }
                // Duplicate Inspection Plan Meta
                $inspection_plan_meta = InspectionPlanMeta::where('itpc_id', $checklist->id) // Use current checklist ID
                    ->where('inspection_plan_id', $old_inspection_plan_report_id)
                    ->where('is_table', 0)
                    ->get();

                foreach ($inspection_plan_meta as $meta) {
                    $metaClone = $meta->replicate();
                    $metaClone->inspection_plan_id = $new_inspection_plan_id;
                    $metaClone->itpc_id = $new_checklist_id; // Update with new checklist ID
                    $metaClone->save();
                }
                // Duplicate Inspection Plan Signatures
                $inspection_plan_signatures = InspectionPlanSignature::where('inspection_plan_id', $old_inspection_plan_report_id)
                    ->where('itpc_id', $checklist->id)
                    ->where('del', 0)
                    ->get();
                foreach ($inspection_plan_signatures as $signature) {
                    $signatureClone = $signature->replicate();
                    $signatureClone->inspection_plan_id = $new_inspection_plan_id;
                    $signatureClone->itpc_id = $new_checklist_id; // Update with new checklist ID
                    $signatureClone->save();
                }
            }
            return response()->json(['message' => 'Inspection Plan Report duplicated successfully.'], 200);
        }
        return response()->json(['message' => 'Inspection Plan Report failed to duplicate.'], 422);
    }
    public function saveStep1(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'inspection_plan_id' => 'required',
            'title' => 'required|string|max:255',
            'document_number' => 'nullable',
            'authorised_by' => 'nullable|integer',
            'revision_number' => 'nullable|integer',
            'site_id' => 'required',
        ], [
            'document_number.integer' => 'The document number must be an integer.',
            'revision_number.integer' => 'The revision number must be an integer.',
            'site_id.required' => 'Site is required',
        ]);
        if ($request->inspection_plan_id === "0") {
            $validator_date = Validator::make($request->all(), [
                'issue_date' => 'nullable|date|after_or_equal:today',
                'revision_date' => 'nullable|date|after_or_equal:issue_date',
            ]);
            if ($validator_date->fails()) {
                return $this->message($validator_date->errors()->first(), 422);
            }
        }
        if ($validator->fails()) {
            $log = [
                'employee_id' => auth()->user()->id,
                'message' => 'Inspection step 1 error:' .   $validator->errors()->first(),
                'report_id' => $request->inspection_plan_id,
                'report_type' => 'inspection_report',
                'error_type' => 'Validation error'
            ];
            storeReportsLogs($log);
            return $this->message($validator_date->errors()->first(), 422);
        }
        try {
            $validatedData =  $validator->validated();
            $msg = 'Saved Successfully';
            $userTable = $this->getUserTable(); // Get the user type (customer or emp)
            if ($userTable == "customer") {
                $validatedData['customer_id'] = auth()->id();
                $validatedData['workspace_id'] = auth()->user()->current_workspace_id;
            } elseif ($userTable == "emp") {
                $validatedData['customer_id'] = auth()->user()->customer_id;
                $validatedData['workspace_id'] = auth()->user()->workspace_id;
            }
            $inspection_plan_data = [
                'title' => $validatedData['title'],
                'document_number' => 'ITP-' . $validatedData['document_number'],
                'authorised_by' => $validatedData['authorised_by'] ?? null,
                'revision_number' => $validatedData['revision_number'],
                'customer_id' => $validatedData['customer_id'],
                'workspace_id' => $validatedData['workspace_id'],
                'issue_date' => $request->issue_date ?? null,
                'revision_date' => $request->revision_date ?? null,
                'site_id' => $request->site_id ?? 0,
            ];
            if (DB::table('inspection_plans')->where('id', $validatedData['inspection_plan_id'])->count() == 0) {
                $validatedData['inspection_plan_id'] = DB::table('inspection_plans')->insertGetId($inspection_plan_data);
                $msg = 'Saved Successfully';
            }
            $document_number_updated = 'ITP-' . $validatedData['document_number'];
            $already_report = DB::table('inspection_plans')->where('id', $validatedData['inspection_plan_id'])->where('document_number', $document_number_updated)->first();
            if ($already_report && $already_report->revision_number != $validatedData['revision_number']) {
                $already_report_to_other_reports = DB::table('inspection_plans')->where('id', '!=', $validatedData['inspection_plan_id'])->where('document_number', $document_number_updated)->first();
                if ($already_report_to_other_reports && $already_report_to_other_reports->revision_number == $validatedData['revision_number']) {
                    return $this->message('Revision number already exists to this document number', 422);
                } else {
                    $inspection_plan_data['parent_number'] = $validatedData['inspection_plan_id'] ?? null;
                    $inspection_plan_data['revision_number'] = $validatedData['revision_number'];
                    $inspection_plan_data['itp_reference_number'] = $already_report->itp_reference_number;
                    $inspection_plan_data['principal_contractor'] = $already_report->principal_contractor;
                    $inspection_plan_data['project_name'] = $already_report->project_name;
                    $inspection_plan_data['job_number'] = $already_report->job_number;
                    $inspection_plan_data['building'] = $already_report->building;
                    $inspection_plan_data['open_date'] = $already_report->open_date;
                    $inspection_plan_data['close_date'] = $already_report->close_date;
                    $inspection_plan_data['process'] = $already_report->process;
                    $inspection_plan_data['activity'] = $already_report->activity;
                    $inspection_plan_data['site_id'] = $already_report->site_id;

                    $inspection_plan_data['is_published'] = $already_report->is_published;
                    $new_id =  DB::table('inspection_plans')->insertGetId($inspection_plan_data);
                    $res = $this->copyInspectionPlanReport($validatedData['inspection_plan_id'], $new_id);
                    $validatedData['inspection_plan_id'] = $new_id;
                    $msg = 'Saved Successfully';
                }
            }
            $this_report_data = DB::table('inspection_plans')->where('id', $validatedData['inspection_plan_id'])->first();
            $inspection_plan_data['parent_number'] = $this_report_data->parent_number;
            $inspection_plan_data['revision_number'] = $this_report_data->revision_number;
            DB::table('inspection_plans')->where('id', $validatedData['inspection_plan_id'])->update($inspection_plan_data);
            $data['inspection_plan_id'] =  $validatedData['inspection_plan_id'];
            $user_authorized_by = EmpCompanyDetails::with(['EmpPersonalDetails', 'accessRole', 'accessTier'])
                ->find($validatedData['authorised_by']);
            $access_role = $user_authorized_by->accessRole->code ?? null;
            $inspection_isignoff_data = [
                'inspection_plan_id' => $validatedData['inspection_plan_id'],
                'employee_type' => $access_role,
                'employee_id' => $validatedData['authorised_by'],
            ];
            $result =   DB::table('inspection_plan_signatures')->updateOrInsert(
                [
                    'inspection_plan_id' => $validatedData['inspection_plan_id'],
                    'employee_id' => $validatedData['authorised_by'],
                ],
                $inspection_isignoff_data
            );
            $log = [
                'employee_id' => auth()->user()->id,
                'message' => 'Inspection step 1:' .   $msg,
                'report_id' => $request->inspection_plan_id,
                'report_type' => 'inspection_report'
            ];
            storeReportsLogs($log);
            return $this->success($data, $msg);
        } catch (\Exception $e) {
            $shortMessage = substr($e->getMessage(), 0, 200) . (strlen($e->getMessage()) > 200 ? '...' : '');
            $log = [
                'employee_id' => auth()->user()->id,
                'message' => 'Inspection step 1  error:' .   $shortMessage,
                'report_id' => $request->inspection_plan_id,
                'report_type' => 'inspection_report',
                'error_type' => 'Exception error'
            ];
            storeReportsLogs($log);
            return $this->message($shortMessage, 422);
        }
    }
    public function editStep1(Request $request)
    {
        $userTable = $this->getUserTable();
        $validator = Validator::make($request->all(), [
            'inspection_plan_id' => 'required'
        ]);
        if ($validator->fails()) {
            storeReportsLogs([
                'employee_id' => auth()->user()->id,
                'message' => 'Inspection step 1 edit error: ' . $validator->errors()->first(),
                'report_id' => $request->inspection_plan_id,
                'report_type' => 'inspection_report',
                'error_type' => 'Validation error'
            ]);
            return $this->message($validator->errors()->first(), 422);
        }
        $validatedData = $validator->validated();
        try {
            $inspectionPlans = InspectionPlan::with('siteData')
                ->where('id', $validatedData['inspection_plan_id'])
                ->first();
            if (!$inspectionPlans) {
                return $this->message('Inspection Plan not found', 404);
            }
            if (
                $userTable == "customer" && (
                    $inspectionPlans->workspace_id != auth()->user()->current_workspace_id ||
                    $inspectionPlans->customer_id != auth()->id()
                )
            ) {
                return $this->message('You do not have access to this project', 403);
            }
            if (
                $userTable == "emp" && (
                    $inspectionPlans->customer_id != auth()->user()->customer_id ||
                    $inspectionPlans->workspace_id != auth()->user()->workspace_id
                )
            ) {
                return $this->message('You do not have access to this project', 403);
            }
            $empList = EmpCompanyDetails::where('compeleted', '1')
                ->where('approved', '1')
                ->where('status', '1')
                ->where('del', '0')
                ->with([
                    'empPersonalDetails' => fn($query) => $query->select('emp_id', 'first_name', 'middle_name', 'last_name'),
                    'accessRole' => fn($query) => $query->select('id', 'title', 'code'),
                    'empTier' => fn($query) => $query->select('id', 'title'),
                    'accessTier' => fn($query) => $query->select('id', 'title', 'tier_key'),
                ])
                ->select('id', 'access_role', 'tier_id')
                ->whereIn('tier_id', fn($query) => $query->select('id')
                    ->from('tiers')
                    ->whereIn('tier_key', ['T-1', 'T-2', 'T-3']))
                ->get();
            $authorisedByList = $empList;
            $data = [];
            if ($validatedData['inspection_plan_id'] == 0) {
                $data['sites'] = DB::table('sites')
                    ->where('del', '0')
                    ->where('active', '1')
                    ->get(['id', 'title']);
            }
            if ($validatedData['inspection_plan_id'] != 0) {
                $inspectionPlan = InspectionPlan::with('siteData')
                    ->where('id', $validatedData['inspection_plan_id'])
                    ->first();
                $data['sites'] = DB::table('sites')
                    ->where(function ($query) use ($inspectionPlan) {
                        $query->where('active', '1')
                            ->orWhere(function ($subQuery) use ($inspectionPlan) {
                                if ($inspectionPlan && isset($inspectionPlan->site_id)) {
                                    $subQuery->where('id', $inspectionPlan->site_id)
                                        ->where('del', '0');
                                }
                            });
                    })
                    ->get(['id', 'title']);
            }
            $data['inspection_plans'] = $inspectionPlans;
            $data['authorised_by_list'] = $authorisedByList;
            return $this->success($data, 'Get Successfully');
        } catch (\Exception $e) {
            $shortMessage = substr($e->getMessage(), 0, 200) . (strlen($e->getMessage()) > 200 ? '...' : '');
            storeReportsLogs([
                'employee_id' => auth()->user()->id,
                'message' => 'Inspection step 1 error: ' . $shortMessage,
                'report_id' => $request->inspection_plan_id,
                'report_type' => 'inspection_report',
                'error_type' => 'Exception error'
            ]);
            return $this->message($shortMessage, 422);
        }
    }

    public function saveStep2(Request $request)
    {
        $userTable = $this->getUserTable();
        $validator = Validator::make($request->all(), [
            'inspection_plan_id' => 'required',
            'principal_contractor' => 'nullable',
            'project_name' => 'nullable',
            'itp' => 'nullable',
        ]);
        if ($validator->fails()) {
            storeReportsLogs([
                'employee_id' => auth()->user()->id,
                'message' => 'Inspection step 2 errors: ' . $validator->errors()->first(),
                'report_id' => $request->inspection_plan_id,
                'report_type' => 'inspection_report',
                'error_type' => 'Validation error'
            ]);
            return $this->message($validator->errors()->first(), 422);
        }
        try {
            $data = $request->all();
            $inspectionPlanId = $request->inspection_plan_id;
            $inspectionPlan = InspectionPlan::find($inspectionPlanId);
            if (!$inspectionPlan) {
                return $this->message('Inspection Plan not found', 404);
            }
            // Customer and Employee Access Check
            if ($userTable === 'customer' && ($inspectionPlan->customer_id !== auth()->id() || $inspectionPlan->workspace_id !== auth()->user()->current_workspace_id)) {
                return $this->message('You do not have access to this project', 403);
            }
            if ($userTable === 'emp' && ($inspectionPlan->customer_id !== auth()->user()->customer_id || $inspectionPlan->workspace_id !== auth()->user()->workspace_id)) {
                return $this->message('You do not have access to this project', 403);
            }
            $inspectionPlanData = [
                'principal_contractor' => $request->principal_contractor,
                'project_name' => $request->project_name,
                'job_number' => $request->job_number,
                'activity' => $request->activity,
                'building' => $request->building,
                'level' => $request->level,
                'to_level' => $request->to_level,
                'zone' => $request->zone,
                'pour' => $request->pour,
                'itp_reference_number' => 'ITP-' . $request->itp,
                'open_date' => $request->open_date,
                'close_date' => $request->close_date,
            ];
            if (is_null($inspectionPlan->open_date)) {
                $validatorDate = Validator::make($request->all(), [
                    'open_date' => 'nullable|date|after_or_equal:today',
                ]);
                if ($validatorDate->fails()) {
                    storeReportsLogs([
                        'employee_id' => auth()->user()->id,
                        'message' => 'Inspection step 2 error: ' . $validatorDate->errors()->first(),
                        'report_id' => $request->inspection_plan_id,
                        'report_type' => 'inspection_report',
                        'error_type' => 'Validation error'
                    ]);
                    return $this->message($validatorDate->errors()->first(), 422);
                }
            }
            if (is_null($inspectionPlan->close_date)) {
                $validatorDate = Validator::make($request->all(), [
                    'close_date' => 'nullable|date|after_or_equal:open_date',
                ]);
                if ($validatorDate->fails()) {
                    storeReportsLogs([
                        'employee_id' => auth()->user()->id,
                        'message' => 'Inspection step 2 error: ' . $validatorDate->errors()->first(),
                        'report_id' => $request->inspection_plan_id,
                        'report_type' => 'inspection_report',
                        'error_type' => 'Validation error'
                    ]);
                    return $this->message($validatorDate->errors()->first(), 422);
                }
            }
            InspectionPlan::updateOrCreate(['id' => $inspectionPlanId], $inspectionPlanData);
            $checklistItems = [];
            foreach ($data as $key => $value) {
                if (strpos($key, '_status') !== false) {
                    $name = str_replace('_status', '', $key);
                    $itpcKey = 'itpc_' . $name;
                    $checklistItems[] = [
                        'inspection_plan_id' => $inspectionPlanId,
                        'itp_reference_number' => $request->itp ?? 0,
                        'itpc_number' => 'ITPC-' . ($data[$itpcKey] ?? 0),
                        'name' => $name,
                        'status' => (string)$value,
                    ];
                }
            }
            $hasChecklistWithStatusOne = collect($checklistItems)->contains('status', '1');
            if (!$hasChecklistWithStatusOne) {
                storeReportsLogs([
                    'employee_id' => auth()->user()->id,
                    'message' => 'Inspection step 2 error: At least one checklist form must have a status of yes to proceed.',
                    'report_id' => $request->inspection_plan_id,
                    'report_type' => 'inspection_report',
                    'error_type' => 'Validation error'
                ]);
                return $this->message('At least one checklist form must have a status of yes to proceed.', 422);
            }
            foreach ($checklistItems as $item) {
                InspectionPlanChecklist::updateOrCreate(
                    [
                        'inspection_plan_id' => $item['inspection_plan_id'],
                        'name' => $item['name'],
                    ],
                    $item
                );
            }
            storeReportsLogs([
                'employee_id' => auth()->user()->id,
                'message' => 'Inspection plan saved successfully',
                'report_id' => $request->inspection_plan_id,
                'report_type' => 'inspection_report'
            ]);
            return $this->success(['inspection_plan_id' => $inspectionPlanId], 'Inspection plan saved successfully');
        } catch (\Exception $e) {
            $shortMessage = substr($e->getMessage(), 0, 200) . (strlen($e->getMessage()) > 200 ? '...' : '');
            storeReportsLogs([
                'employee_id' => auth()->user()->id,
                'message' => 'Inspection step 2 error: ' . $shortMessage,
                'report_id' => $request->inspection_plan_id,
                'report_type' => 'inspection_report',
                'error_type' => 'Exception error'
            ]);
            return $this->message($shortMessage, 422);
        }
    }

    public function editStep2(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'inspection_plan_id' => 'required'
        ]);
        if ($validator->fails()) {
            storeReportsLogs([
                'employee_id' => auth()->user()->id,
                'message' => 'Inspection plan step 2 validation error: ' . $validator->errors()->first(),
                'report_id' => $request->inspection_plan_id,
                'report_type' => 'inspection_report',
                'error_type' => 'Validation error',
            ]);
            return $this->message($validator->errors()->first(), 422);
        }
        $validatedData = $validator->validated();
        try {
            $inspectionPlan = InspectionPlan::find($validatedData['inspection_plan_id']);
            if (!$inspectionPlan) {
                return $this->message('Inspection Plan not found', 404);
            }
            // Customer and Employee Access Check
            $userTable = $this->getUserTable();
            if (
                $userTable === 'customer' &&
                ($inspectionPlan->customer_id !== auth()->id() || $inspectionPlan->workspace_id !== auth()->user()->current_workspace_id)
            ) {
                return $this->message('You do not have access to this inspection plan', 403);
            }
            if (
                $userTable === 'emp' &&
                ($inspectionPlan->customer_id !== auth()->user()->customer_id || $inspectionPlan->workspace_id !== auth()->user()->workspace_id)
            ) {
                return $this->message('You do not have access to this inspection plan', 403);
            }
            $data = [
                'inspection_plan' => $inspectionPlan,
                'inspection_plan_checklist' => InspectionPlanChecklist::where('inspection_plan_id', $validatedData['inspection_plan_id'])->get(),
                'inspection_plan_signature' => InspectionPlanSignature::where('inspection_plan_id', $validatedData['inspection_plan_id'])->get(),
                'inspection_plan_settings' => InspectionPlanTable::select(['title', 'step_order'])->distinct('title')->orderBy('step_order', 'asc')->get(),
                'inspection_plan_project' => Project::where('is_deleted', 0)->get(['id', 'title']),
            ];
            if ($data['inspection_plan_settings']->isEmpty()) {
                $message = 'Please create checklist steps to proceed further';
                storeReportsLogs([
                    'employee_id' => auth()->user()->id,
                    'message' => 'Inspection plan step 2: ' . $message,
                    'report_id' => $request->inspection_plan_id,
                    'report_type' => 'inspection_report'
                ]);
                return $this->message($message, 200, $data);
            }
            storeReportsLogs([
                'employee_id' => auth()->user()->id,
                'message' => 'Inspection plan step 2 fetched successfully',
                'report_id' => $request->inspection_plan_id,
                'report_type' => 'inspection_report'
            ]);
            return $this->success($data, 'Inspection Plan Step 2 fetched successfully');
        } catch (\Exception $e) {
            $shortMessage = substr($e->getMessage(), 0, 200) . (strlen($e->getMessage()) > 200 ? '...' : '');
            storeReportsLogs([
                'employee_id' => auth()->user()->id,
                'message' => 'Inspection step 2 error: ' . $shortMessage,
                'report_id' => $request->inspection_plan_id,
                'report_type' => 'inspection_report',
                'error_type' => 'Exception error'
            ]);
            return $this->message($shortMessage, 422);
        }
    }

    public function inspectionReportDownloadPdf(Request $request)
    {
        $id = $request->id;
        if (!$id) {
            $log = [
                'employee_id' => auth()->user()->id,
                'message' => 'Inspection plan report PDF download ID is required',
                'report_id' => $id,
                'report_type' => 'inspection_report',
                'error_type' => 'Validation error',
            ];
            storeReportsLogs($log);
            return $this->message('ID of plan is required', 422);
        }
        try {
            // Fetch the report with the required joins
            $report = InspectionPlan::leftJoin('sites', 'inspection_plans.site_id', '=', 'sites.id')
                ->leftJoin('emp_personal_details as authorised_by_details', 'inspection_plans.authorised_by', '=', 'authorised_by_details.id')
                ->leftJoin('emp_personal_details as emp_personal_supervisor_detail', 'sites.supervisor_id', '=', 'emp_personal_supervisor_detail.id')
                ->leftJoin('emp_personal_details as emp_personal_forman_detail', 'sites.forman_id', '=', 'emp_personal_forman_detail.id')
                ->where('inspection_plans.id', $id)
                ->select(
                    'inspection_plans.*',
                    'sites.title as sites_title',
                    'authorised_by_details.first_name as authorised_by_first_name',
                    'authorised_by_details.last_name as authorised_by_last_name',
                    'emp_personal_supervisor_detail.first_name as emp_personal_supervisor_detail_f_name',
                    'emp_personal_supervisor_detail.last_name as emp_personal_supervisor_detail_last_name',
                    'emp_personal_forman_detail.first_name as emp_personal_forman_detail_f_name',
                    'emp_personal_forman_detail.last_name as emp_personal_forman_detail_last_name'
                )
                ->first();
            if (!$report) {
                $log = [
                    'employee_id' => auth()->user()->id,
                    'message' => 'Inspection plan to this ID not found',
                    'report_id' => $id,
                    'report_type' => 'inspection_report',
                    'error_type' => 'Validation error',
                ];
                storeReportsLogs($log);
                return $this->message('Inspection plan to this ID not found', 404);
            }
            // Add customer and employee checks
            $userTable = $this->getUserTable();
            if (
                $userTable === "customer" &&
                ($report->customer_id != auth()->id() || $report->workspace_id != auth()->user()->current_workspace_id)
            ) {
                return $this->message('Unauthorized access to this inspection plan', 403);
            }

            if (
                $userTable === "emp" &&
                ($report->customer_id != auth()->user()->customer_id || $report->workspace_id != auth()->user()->workspace_id)
            ) {
                return $this->message('Unauthorized access to this inspection plan', 403);
            }
            // Fetch forms and report images
            $forms = InspectionPlanChecklist::leftJoin('inspection_plan_generals', 'inspection_plan_checklists.id', '=', 'inspection_plan_generals.iptc_id')
                ->leftJoin('inspection_plan_metas', 'inspection_plan_checklists.id', '=', 'inspection_plan_metas.itpc_id')
                ->leftJoin('inspection_plan_signatures', 'inspection_plan_checklists.id', '=', 'inspection_plan_signatures.itpc_id')
                ->leftJoin('emp_personal_details as inspection_plan_signatures_details', 'inspection_plan_signatures.employee_id', '=', 'inspection_plan_signatures_details.emp_id')
                ->where('inspection_plan_checklists.inspection_plan_id', $id)
                ->where('inspection_plan_checklists.status', '=', '1')
                ->select(
                    'inspection_plan_checklists.name as inspection_plan_step_name',
                    'inspection_plan_generals.*',
                    'inspection_plan_metas.title as meta_title',
                    'inspection_plan_metas.value as meta_value',
                    'inspection_plan_metas.is_table',
                    'inspection_plan_signatures.signature as inspection_plan_signature',
                    'inspection_plan_signatures.date as inspection_plan_date',
                    'inspection_plan_signatures.time as inspection_plan_time',
                    'inspection_plan_signatures_details.id as inspection_plan_employee_id',
                    'inspection_plan_signatures_details.first_name as inspection_plan_first_name',
                    'inspection_plan_signatures_details.last_name as inspection_plan_last_name'
                )
                ->get()
                ->groupBy('inspection_plan_step_name');
            $report_images = InspectionImage::where('inspection_plan_id', $id)->get();
            // Generate PDF content
            $viewPath = 'InspectionPlan.InspectionPlanPDF.InspectionPlanPdf';
            $html = view($viewPath, compact('report', 'forms', 'report_images'))->render();
            // Prepare file details
            $report_title = preg_replace('/[^A-Za-z0-9_]+/', '_', trim(str_replace(' ', '_', $report->title ?? 'No_Title')));
            $report_document_number = preg_replace('/[^A-Za-z0-9_]+/', '', trim(str_replace(' ', '_', $report->document_number ?? 'No_Document_Number')));
            $report_revision_number = preg_replace('/[^A-Za-z0-9_]+/', '', trim(str_replace(' ', '_', $report->revision_number ?? '00')));
            $filename = $report_title . $report_document_number . "R" . $report_revision_number . ".pdf";
            $directory = 'inspectionPlanReport';
            $full_path = public_path("$directory/$filename");
            $file_url = url("$directory/$filename");
            // Check if file exists
            $existingFile = GeneratedPdfReport::where('report_type', 'inspection_report')
                ->where('report_id', $id)
                ->where('path', $file_url)
                ->first();

            if (file_exists($full_path) && $existingFile && $existingFile->path === $file_url) {
                return $this->success($file_url, 'PDF already exists and is available for download');
            }
            // Generate and save the PDF
            if (!file_exists(public_path($directory))) {
                mkdir(public_path($directory), 0777, true);
            }
            $pdf = PDF::loadHTML($html)->setPaper('A4', 'portrait');
            $pdf->save($full_path);
            // Save file details to the database
            GeneratedPdfReport::create([
                'report_id' => $id,
                'path' => $file_url,
                'report_type' => 'inspection_report',
            ]);
            return $this->success($file_url, 'PDF generated and uploaded successfully');
        } catch (\Exception $e) {
            $shortMessage = substr($e->getMessage(), 0, 200) . (strlen($e->getMessage()) > 200 ? '...' : '');
            $log = [
                'employee_id' => auth()->user()->id,
                'message' => 'Inspection plan report download error: ' . $shortMessage,
                'report_id' => $id,
                'report_type' => 'inspection_report',
                'error_type' => 'Exception error'
            ];
            storeReportsLogs($log);
            return $this->message($shortMessage, 422);
        }
    }
    // need to conform this fucniton is needed or not 
    public function inspectionPlanFormPages()
    {
        $emp_list = EmpCompanyDetails::where('compeleted', '1')
            ->where('approved', '1')
            ->where('status', '1')
            ->where('del', '0')
            ->with([
                'empPersonalDetails' => function ($query) {
                    $query->select('emp_id', 'first_name', 'middle_name', 'last_name');
                },
                'accessRole' => function ($query) {
                    $query->select('id', 'title', 'code');
                },
                'empTier' => function ($query) {
                    $query->select('id', 'title');
                },
                'accessTier' => function ($query) {
                    $query->select('id', 'title', 'tier_key');
                },
            ])
            ->select('id', 'access_role', 'tier_id')
            ->whereIn('tier_id', function ($query) {
                $query->select('id')
                    ->from('tiers')
                    ->whereIn('tier_key', ['T-1', 'T-2', 'T-3']);
            })
            ->get();
        $employees = $emp_list;
        return view('InspectionPlan.NewInspectionReport.form_index', compact('employees'));
    }
    public function inspectionPlanSaveStep(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'inspection_plan_id' => 'required',
            'name' => 'required|string|max:255',
            'level' => 'nullable|string|max:255',
            'architectural_drawing_number' => 'nullable',
            'structectural_drawing_number' => 'nullable',
            'Post_Tension_Drawing_Number' => 'nullable',
            'partial_Handover_date' => 'nullable|date',
            'complete_handover_date' => 'nullable|date',
            'complete_handover_time' => 'nullable',
        ], [
            'partial_Handover_date.after_or_equal' => 'The partial handover date must be today or later.',
            'complete_handover_date.after_or_equal' => 'The complete handover date must be today or later.',
            'complete_handover_time.after_or_equal' => 'The complete handover time must be equal to or later than the current time.',
        ]);
        if ($validator->fails()) {
            $log = [
                'employee_id' => auth()->user()->id,
                'message' => 'Inspection plan step ' . $request->name . ' save error: ' . $validator->errors()->first(),
                'report_id' => $request->inspection_plan_id,
                'report_type' => 'inspection_report',
                'error_type' => 'Validation error',
            ];
            storeReportsLogs($log);
            return $this->message($validator->errors()->first(), 422);
        }
        try {
            $validatedData = $validator->validated();
            $inspection_plan_id = $validatedData['inspection_plan_id'];
            $name = $validatedData['name'];
            $inspectionPlan = InspectionPlan::where('id', $inspection_plan_id)->first();
            if (!$inspectionPlan) {
                return $this->message('Inspection Plan not found', 404);
            }
            // Customer and Employee Checks
            $userTable = $this->getUserTable();
            if (
                $userTable === "customer" &&
                ($inspectionPlan->customer_id != auth()->id() || $inspectionPlan->workspace_id != auth()->user()->current_workspace_id)
            ) {
                return $this->message('Unauthorized access to this inspection plan', 403);
            }
            if (
                $userTable === "emp" &&
                ($inspectionPlan->customer_id != auth()->user()->customer_id || $inspectionPlan->workspace_id != auth()->user()->workspace_id)
            ) {
                return $this->message('Unauthorized access to this inspection plan', 403);
            }
            $inspection_plan_checklist_columns = InspectionPlanChecklist::where('inspection_plan_id', $inspection_plan_id)
                ->where('name', $name)
                ->first();
            if (!$inspection_plan_checklist_columns) {
                return $this->message('Checklist column not found for the given name', 404);
            }
            $inspection_plan_checklist_columns_id = $inspection_plan_checklist_columns->id;
            // Prepare data for deck handover
            $inspection_plan_deck_handover = [
                'inspection_plan_id' => $inspection_plan_id,
                'itp_reference_number' => $inspection_plan_checklist_columns->itp_reference_number ?? null,
                'iptc_id' => $inspection_plan_checklist_columns_id ?? null,
                'level' => $request->level,
                'to_level' => $request->to_level,
                'pour' => $request->pour,
                'zone' => $request->zone,
                'partial_Handover' => (string)($request->partial_Handover),
                'partial_Handover_date' => $request->partial_Handover_date,
                'partial_Handover_time' => $request->partial_Handover_time,
                'partial_Handover_location' => $request->partial_Handover_location,
                'complete_handover' => (string)($request->complete_handover),
                'complete_handover_date' => $request->complete_handover_date,
                'complete_handover_time' => $request->complete_handover_time,
                'complete_handover_Location' => $request->complete_handover_location,
                'architectural_drawing_number' => $request->architectural_drawing_number,
                'structectural_drawing_number' => $request->structectural_drawing_number,
                'Post_Tension_Drawing_Number' => $request->Post_Tension_Drawing_Number,
                "description_box_for_message" => $request->description_box_for_message,
            ];
            $existingGeneralStatus = InspectionPlanGeneral::where('inspection_plan_id', $inspection_plan_id)
                ->where('iptc_id', $inspection_plan_checklist_columns->id ?? null)
                ->first();
            if (!$existingGeneralStatus || $existingGeneralStatus->partial_Handover_date === null) {
                $validator_date = Validator::make($request->all(), ['partial_Handover_date' => 'nullable|date']);
                if ($validator_date->fails()) {
                    return $this->message($validator_date->errors()->first(), 422);
                }
            }
            if (!$existingGeneralStatus || $existingGeneralStatus->complete_handover_date === null) {
                $validator_date = Validator::make($request->all(), ['complete_handover_date' => 'nullable|date']);
                if ($validator_date->fails()) {
                    return $this->message($validator_date->errors()->first(), 422);
                }
            }
            if ($existingGeneralStatus) {
                $existingGeneralStatus->update($inspection_plan_deck_handover);
            } else {
                $data['inspection_plan_general'] = InspectionPlanGeneral::create($inspection_plan_deck_handover)->id;
            }

            InspectionPlanChecklist::where('id', $inspection_plan_checklist_columns_id)->update(['is_complete' => '1']);
            $remainingChecklist = InspectionPlanChecklist::where('inspection_plan_id', $inspection_plan_id)->where('is_complete', 0)->first();
            if (!$remainingChecklist) {
                InspectionPlan::where('id', $inspection_plan_id)->update(['process' => '1']);
            }
            $signature_employee_ids = $request->input('column_employee_id', []);
            $signature_files = $request->file('column_signature', []);
            $signature_dates = $request->input('column_date', []);
            foreach ($signature_employee_ids as $index => $employee_id) {
                if (!$employee_id) continue;
                $signature_path = null;
                if (isset($signature_files[$index]) && $signature_files[$index]->isValid()) {
                    $imageName = $employee_id . time() . '.' . $signature_files[$index]->getClientOriginalExtension();
                    $signature_files[$index]->move(public_path('uploads/inspection_document_signatures'), $imageName);
                    $signature_path = 'uploads/inspection_document_signatures/' . $imageName;
                }
                InspectionPlanSignature::create([
                    'inspection_plan_id' => $inspection_plan_id,
                    'itpc_id' => $inspection_plan_checklist_columns_id,
                    'employee_name' => $employee_id,
                    'employee_type' => $employee_id,
                    'signature' => $signature_path,
                    'date' => $signature_dates[$index] ?? null,
                ]);
            }
            $log = [
                'employee_id' => auth()->user()->id,
                'message' => 'Inspection plan step ' . $name . ' saved successfully',
                'report_id' => $inspection_plan_id,
                'report_type' => 'inspection_report',
            ];
            storeReportsLogs($log);
            return $this->message('Saved successfully');
        } catch (\Exception $e) {
            $log = [
                'employee_id' => auth()->user()->id,
                'message' => 'Inspection plan step save error: ' . $e->getMessage(),
                'report_id' => $request->inspection_plan_id,
                'report_type' => 'inspection_report',
                'error_type' => 'Exception error',
            ];
            storeReportsLogs($log);
            return $this->message($e->getMessage(), 500);
        }
    }

    public function inspectionPlanFormStepData(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'inspection_plan_id' => 'required',
            'name' => 'required'
        ]);
        if ($validator->fails()) {
            return $this->message($validator->errors()->first(), 422);
        }
        try {
            $validatedData =  $validator->validated();
            $inspectionPlan = InspectionPlan::find($validatedData['inspection_plan_id']);
            $userTable = $this->getUserTable();
            if (
                $userTable === "customer" &&
                ($inspectionPlan->customer_id != auth()->id() || $inspectionPlan->workspace_id != auth()->user()->current_workspace_id)
            ) {
                return $this->message('Unauthorized access to this inspection plan', 403);
            }
            if (
                $userTable === "emp" &&
                ($inspectionPlan->customer_id != auth()->user()->customer_id || $inspectionPlan->workspace_id != auth()->user()->workspace_id)
            ) {
                return $this->message('Unauthorized access to this inspection plan', 403);
            }
            $data = [];
            $inspection_plan_id = $validatedData['inspection_plan_id'];
            $name = $request->name;
            $inplection_plan_checklist_columns = InspectionPlanChecklist::where('inspection_plan_id', $inspection_plan_id)->where('name', $name)->first();
            $data['inspection_plan_step_data'] = InspectionPlanGeneral::where('inspection_plan_id', $inspection_plan_id)->where('iptc_id', $inplection_plan_checklist_columns->id)->first();
            $data['inpection_plan_signature'] = InspectionPlanSignature::where('inspection_plan_id', $inspection_plan_id)->where('itpc_id', $inplection_plan_checklist_columns->id)->where('del', 0)->get();
            $emp_list = EmpCompanyDetails::where('compeleted', '1')
                ->where('approved', '1')
                ->where('status', '1')
                ->where('del', '0')
                ->with([
                    'empPersonalDetails' => function ($query) {
                        $query->select('emp_id', 'first_name', 'middle_name', 'last_name');
                    },
                    'accessRole' => function ($query) {
                        $query->select('id', 'title', 'code');
                    },
                    'empTier' => function ($query) {
                        $query->select('id', 'title');
                    },
                    'accessTier' => function ($query) {
                        $query->select('id', 'title', 'tier_key');
                    },
                ])
                ->select('id', 'access_role', 'tier_id')
                ->whereIn('tier_id', function ($query) {
                    $query->select('id')
                        ->from('tiers')
                        ->whereIn('tier_key', ['T-1', 'T-2', 'T-3']);
                })
                ->get();
            $authorised_by_list = $emp_list;
            $data['authorised_by_list'] = $authorised_by_list;
            return $this->success($data, 'fetch data successfully');
        } catch (\Exception $e) {
            $shortMessage = substr($e->getMessage(), 0, 200) . (strlen($e->getMessage()) > 200 ? '...' : '');
            $log = [
                'employee_id' => auth()->user()->id,
                'message' => 'Inspection plan step save error:' .   $shortMessage,
                'report_id' => $request->inspection_plan_id,
                'report_type' => 'inspection_report',
                'error_type' => 'Exception error'
            ];
            storeReportsLogs($log);
            return $this->message($shortMessage, 500);
        }
    }
    public function inspectionPlanSettingaEmployeeList()
    {
        try {
            $userTable = $this->getUserTable(); // Get user type (customer or employee)
            // Base query for employees
            $emp_list_query = EmpCompanyDetails::where('compeleted', '1')
                ->where('approved', '1')
                ->where('status', '1')
                ->where('del', '0')
                ->with([
                    'empPersonalDetails' => function ($query) {
                        $query->select('emp_id', 'first_name', 'middle_name', 'last_name');
                    },
                    'accessRole' => function ($query) {
                        $query->select('id', 'title', 'code');
                    },
                    'empTier' => function ($query) {
                        $query->select('id', 'title');
                    },
                    'accessTier' => function ($query) {
                        $query->select('id', 'title', 'tier_key');
                    },
                ])
                ->select('id', 'access_role', 'tier_id')
                ->whereIn('tier_id', function ($query) {
                    $query->select('id')
                        ->from('tiers')
                        ->whereIn('tier_key', ['T-1', 'T-2', 'T-3']);
                });
            // Add customer or employee-based filtering
            if ($userTable === "customer") {
                $emp_list_query->whereHas('empPersonalDetails', function ($query) {
                    $query->where('customer_id', auth()->id())
                        ->where('workspace_id', auth()->user()->current_workspace_id);
                });
            }
            if ($userTable === "emp") {
                $emp_list_query->whereHas('empPersonalDetails', function ($query) {
                    $query->where('customer_id', auth()->user()->customer_id)
                        ->where('workspace_id', auth()->user()->workspace_id);
                });
            }
            // Execute the query
            $emp_list = $emp_list_query->get();
            $data['authorised_by_list'] = $emp_list;
            // Check if the list is empty
            if ($emp_list->isEmpty()) {
                return $this->message('No employees found. Please create.', 422);
            }
            // Return success response
            return $this->success($data, 'Records retrieved successfully');
        } catch (\Exception $e) {
            $shortMessage = substr($e->getMessage(), 0, 200) . (strlen($e->getMessage()) > 200 ? '...' : '');
            $log = [
                'employee_id' => auth()->user()->id,
                'message' => 'Inspection plan employee list error: ' . $shortMessage,
                'report_id' => null,
                'report_type' => 'inspection_report',
                'error_type' => 'Exception error'
            ];
            storeReportsLogs($log);
            return $this->message($shortMessage, 500);
        }
    }

    public function InspectionReportPdfFiles(Request $request)
    {
        try {
            $userTable = $this->getUserTable(); // Get the user type (customer or emp)
            $query = GeneratedPdfReport::where('report_type', 'inspection_report');
            // Add customer or employee-based filtering
            if ($userTable === "customer") {
                $query->whereHas('inspectionPlan', function ($q) {
                    $q->where('customer_id', auth()->id())
                        ->where('workspace_id', auth()->user()->current_workspace_id);
                });
            }
            if ($userTable === "emp") {
                $query->whereHas('inspectionPlan', function ($q) {
                    $q->where('customer_id', auth()->user()->customer_id)
                        ->where('workspace_id', auth()->user()->workspace_id);
                });
            }
            if (!$query->exists()) {
                return $this->message('Record Not Found', 404);
            }
            $query->orderBy('created_at', 'desc');
            // Pagination logic
            $perPage = $request->input('per_page', 10); // Default is 10 items per page
            $currentPage = LengthAwarePaginator::resolveCurrentPage();
            $totalReports = $query->count();
            $currentReports = $query->skip(($currentPage - 1) * $perPage)->take($perPage)->get();
            // Prepare paginated data
            $paginator = new LengthAwarePaginator(
                $currentReports,
                $totalReports,
                $perPage,
                $currentPage,
                ['path' => LengthAwarePaginator::resolveCurrentPath()]
            );
            $data = [
                'paginators' => $paginator->items(),
                'total' => $paginator->total(),
                'per_page' => $paginator->perPage(),
                'current_page' => $paginator->currentPage(),
                'last_page' => $paginator->lastPage()
            ];
            return $this->success($data, 'Inspection report PDF files retrieved successfully');
        } catch (\Exception $e) {
            $shortMessage = substr($e->getMessage(), 0, 200) . (strlen($e->getMessage()) > 200 ? '...' : '');
            $log = [
                'employee_id' => auth()->user()->id,
                'message' => 'Error retrieving inspection report PDF files: ' . $shortMessage,
                'report_type' => 'inspection_report',
                'error_type' => 'Exception error'
            ];
            storeReportsLogs($log);
            return $this->message($shortMessage, 500);
        }
    }


    public function downloadPdfFile($filename)
    {
        // Ensure the filename has the correct extension
        if (pathinfo($filename, PATHINFO_EXTENSION) === '') {
            $filename .= '.pdf'; // Default to PDF if no extension provided
        }
        $path = public_path("inspectionPlanReport/");
        $fileUrl = url("inspectionPlanReport/" . $filename);
        // Check if the file exists
        if (!file_exists($path . $filename)) {
            return $this->message('File not found. The requested file may have been deleted or is not available.', 404);
        }
        return $this->success($fileUrl, 'File retrieved successfully');
    }

    public function inspectionReportPublishReport(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'id' => 'required|exists:inspection_plans,id'
        ]);
        if ($validator->fails()) {
            $log = [
                'employee_id' => auth()->user()->id,
                'message' => 'Validation error: ' . $validator->errors()->first(),
                'report_id' => $request->id,
                'report_type' => 'inspection_report',
                'error_type' => 'Validation error',
            ];
            storeReportsLogs($log);
            return $this->message($validator->errors()->first(), 422);
        }
        $swm = InspectionPlan::find($request->id);
        if (!$swm) {
            $log = [
                'employee_id' => auth()->user()->id,
                'message' => 'Report not found. Please try again.',
                'report_id' => $request->id,
                'report_type' => 'inspection_report',
                'error_type' => 'Not Found error',
            ];
            storeReportsLogs($log);
            return $this->message('Report not found. Please try again.', 404);
        }
        $userTable = $this->getUserTable();
        if ($userTable === "customer" && ($swm->customer_id != auth()->id() || $swm->workspace_id != auth()->user()->current_workspace_id)) {
            return $this->message('Unauthorized access. You do not have permission to modify this report.', 403);
        }
        if ($userTable === "emp" && ($swm->customer_id != auth()->user()->customer_id || $swm->workspace_id != auth()->user()->workspace_id)) {
            return $this->message('Unauthorized access. You do not have permission to modify this report.', 403);
        }
        $swm->is_published = $swm->is_published == '1' ? '0' : '1';
        $swm->save();
        $log = [
            'employee_id' => auth()->user()->id,
            'message' => 'Report status updated successfully',
            'report_id' => $request->id,
            'report_type' => 'inspection_report'
        ];
        storeReportsLogs($log);
        if ($swm->is_published == '1') {
            return $this->inspectionReportDownloadPdf($request);
        }
        return $this->message('Report status updated successfully.', 200);
    }

    public function inspectionReportDownloadPdfRegenerate(Request $request)
    {
        $id = $request->id;
        if (!$id) {
            $log = [
                'employee_id' => auth()->user()->id,
                'message' => 'Inspection plan report PDF download ID is required ',
                'report_id' => $id,
                'report_type' => 'inspection_report',
                'error_type' => 'Validation error',
            ];
            storeReportsLogs($log);
            return $this->message('ID of plan is required', 422);
        }
        try {
            $report =  DB::table('inspection_plans')
                ->leftJoin('sites', 'inspection_plans.site_id', '=', 'sites.id')
                ->leftJoin('emp_personal_details as authorised_by_details', 'inspection_plans.authorised_by', '=', 'authorised_by_details.id')
                ->leftJoin('emp_personal_details as emp_personal_supervisor_detail', 'sites.supervisor_id', '=', 'emp_personal_supervisor_detail.id')
                ->leftJoin('emp_personal_details as emp_personal_forman_detail', 'sites.forman_id', '=', 'emp_personal_forman_detail.id')
                ->where('inspection_plans.id', $id)
                ->select(
                    'inspection_plans.*',
                    'sites.title as sites_title',
                    'authorised_by_details.first_name as authorised_by_first_name',
                    'authorised_by_details.last_name as authorised_by_last_name',
                    'emp_personal_supervisor_detail.first_name as emp_personal_supervisor_detail_f_name',
                    'emp_personal_supervisor_detail.last_name as emp_personal_supervisor_detail_last_name',
                    'emp_personal_forman_detail.first_name as emp_personal_forman_detail_f_name',
                    'emp_personal_forman_detail.last_name as emp_personal_forman_detail_last_name',
                )
                ->first();

            if (!$report) {
                $log = [
                    'employee_id' => auth()->user()->id,
                    'message' => 'Inspection plan to this ID not found',
                    'report_id' => $id,
                    'report_type' => 'inspection_report'
                ];
                storeReportsLogs($log);
                return $this->message('Inspection plan to this ID not found', 404);
            }
            $userTable = $this->getUserTable();
            if (
                $userTable === "customer" &&
                ($report->customer_id != auth()->id() || $report->workspace_id != auth()->user()->current_workspace_id)
            ) {
                return $this->message('Unauthorized access to this inspection plan', 403);
            }

            if (
                $userTable === "emp" &&
                ($report->customer_id != auth()->user()->customer_id || $report->workspace_id != auth()->user()->workspace_id)
            ) {
                return $this->message('Unauthorized access to this inspection plan', 403);
            }
            $forms = InspectionPlanChecklist::leftJoin('inspection_plan_generals', 'inspection_plan_checklists.id', '=', 'inspection_plan_generals.iptc_id')
                ->leftJoin('inspection_plan_metas', 'inspection_plan_checklists.id', '=', 'inspection_plan_metas.itpc_id')
                ->leftJoin('inspection_plan_signatures', 'inspection_plan_checklists.id', '=', 'inspection_plan_signatures.itpc_id')
                ->leftJoin('emp_personal_details as inspection_plan_signatures_details', 'inspection_plan_signatures.employee_id', '=', 'inspection_plan_signatures_details.emp_id')
                ->where('inspection_plan_checklists.inspection_plan_id', $id)
                ->where('inspection_plan_checklists.status', '=', '1')
                ->select(
                    'inspection_plan_checklists.name as inspection_plan_step_name',
                    'inspection_plan_generals.*',
                    'inspection_plan_metas.title as meta_title',
                    'inspection_plan_metas.value as meta_value',
                    'inspection_plan_metas.is_table',
                    'inspection_plan_signatures.signature as inspection_plan_signature',
                    'inspection_plan_signatures.date as inspection_plan_date',
                    'inspection_plan_signatures.time as inspection_plan_time',
                    'inspection_plan_signatures_details.id as inspection_plan_employee_id',
                    'inspection_plan_signatures_details.first_name as inspection_plan_first_name',
                    'inspection_plan_signatures_details.last_name as inspection_plan_last_name'
                )
                ->get()
                ->groupBy('inspection_plan_step_name');
            $report_images =  DB::table('inspection_images')
                ->where('inspection_plan_id', $id)
                ->get();
            $viewPath = 'InspectionPlan.InspectionPlanPDF.InspectionPlanPdf';
            $html = view($viewPath, compact('report', 'forms', 'report_images'))->render();
            $report_title = isset($report->title)
                ? preg_replace('/[^A-Za-z0-9_]+/', '_', trim(str_replace(' ', '_', $report->title)))
                : 'No_Title';
            $report_document_number = isset($report->document_number)
                ? preg_replace('/[^A-Za-z0-9_]+/', '', trim(str_replace(' ', '_', $report->document_number)))
                : 'No_Document_Number';
            $report_revision_number = isset($report->revision_number)
                ? preg_replace('/[^A-Za-z0-9_]+/', '', trim(str_replace(' ', '_', $report->revision_number)))
                : '00';
            $filename = $report_title . $report_document_number . "R" . $report_revision_number . ".pdf";
            $directory = 'inspectionPlanReport';
            $path = public_path("$directory/$filename");
            $fileUrl = url("$directory/$filename");
            // Check if file already exists and delete it if necessary
            $already_file_status = GeneratedPdfReport::where('report_type', 'inspection_report')
                ->where('report_id', $id)
                ->where('path', $fileUrl)
                ->first();
            if (file_exists($path) && $already_file_status && $already_file_status->path == $fileUrl) {
                GeneratedPdfReport::where('report_type', 'inspection_report')
                    ->where('report_id', $id)
                    ->where('path', $fileUrl)
                    ->delete();
                unlink($path);
            }
            // Save the file
            if (!file_exists(public_path($directory))) {
                mkdir(public_path($directory), 0777, true);
            }
            $pdf = PDF::loadHTML($html)->setPaper('A4', 'portrait');
            $pdf->save($path);

            GeneratedPdfReport::updateOrCreate(
                ['report_id' => $id],
                [
                    'report_id' => $id,
                    'path' => $fileUrl,
                    'report_type' => 'inspection_report',
                ]
            );
            return $this->success($fileUrl, 'PDF generated and uploaded successfully');
        } catch (\Exception $e) {
            $shortMessage = substr($e->getMessage(), 0, 200) . (strlen($e->getMessage()) > 200 ? '...' : '');
            $log = [
                'employee_id' => auth()->user()->id,
                'message' => 'Inspection plan report download error:' .   $shortMessage,
                'report_id' => $id,
                'report_type' => 'inspection_report',
                'error_type' => 'Exception error'

            ];
            storeReportsLogs($log);
            return $this->message($e->getMessage(), 500);
        }
    }
    public function inspectionReportViewSignature($id)
    {
        if (!$id) {
            Session::put('error', 'Invalid Request');
            return;
        }
        $inspection_plans =  DB::table('inspection_plans')
            ->leftJoin('emp_company_details as emp_company_details_details', 'inspection_plans.authorised_by', '=', 'emp_company_details_details.id')
            ->leftJoin('emp_personal_details as authorised_by_details', 'inspection_plans.authorised_by', '=', 'authorised_by_details.emp_id')
            ->where('inspection_plans.id', $id)
            ->select(
                'inspection_plans.*',
                'emp_company_details_details.access_role as emp_company_details_details_access_role',
                'authorised_by_details.first_name as authorised_by_first_name',
                'authorised_by_details.last_name as authorised_by_last_name',
            )
            ->first();
        $inspection_plan_signatures =  DB::table('inspection_plan_signatures')
            ->leftJoin('emp_personal_details as inspection_plan_signatures_details', 'inspection_plan_signatures.employee_id', '=', 'inspection_plan_signatures_details.emp_id')
            ->where('inspection_plan_signatures.inspection_plan_id', $id)
            ->get();
        return view('InspectionPlan.inspecion_report_signature', compact('inspection_plans', 'inspection_plan_signatures'));
    }

    public function imageStore(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'inspection_plan_id' => 'required',
            'image' => 'required|image|mimes:jpeg,png,jpg,gif,svg|max:5048',
        ]);
        if ($validator->fails()) {
            $log = [
                'employee_id' => auth()->user()->id,
                'message' => 'Inspection images store: ' . $validator->errors()->first(),
                'report_id' => $request->inspection_plan_id,
                'report_type' => 'inspection_report',
                'error_type' => 'Validation error',
            ];
            storeReportsLogs($log);
            return $this->message($validator->errors()->first(), 422);
        }
        // Customer or employee check
        $userTable = $this->getUserTable();
        $inspectionPlan = InspectionPlan::where('id', $request->inspection_plan_id);
        if ($userTable === 'customer') {
            $inspectionPlan->where('customer_id', auth()->id())
                ->where('workspace_id', auth()->user()->current_workspace_id);
        }
        if ($userTable === 'emp') {
            $inspectionPlan->where('customer_id', auth()->user()->customer_id)
                ->where('workspace_id', auth()->user()->workspace_id);
        }
        if (!$inspectionPlan->exists()) {
            return $this->message('You are not authorized to perform this action.', 403);
        }
        if ($request->hasFile('image') && $request->file('image')->isValid()) {
            $imagePath = $this->handleFileImageUpload($request, 'inspectionPlanImages')['path'] ?? null;
            if (!$imagePath) {
                return $this->message('Failed to upload image.', 422);
            }
            $inspectionPlanChecklist = InspectionPlanChecklist::where('inspection_plan_id', $request->inspection_plan_id)
                ->where('name', $request->step_name)
                ->first();
            if (!$inspectionPlanChecklist) {
                return $this->message('Invalid step name or inspection plan ID.', 404);
            }
            $image = new InspectionImage();
            $image->image = $imagePath;
            $image->inspection_plan_id = $request->inspection_plan_id;
            $image->itpc_id = $inspectionPlanChecklist->id;
            $image->save();
            $images = InspectionImage::where('inspection_plan_id', $request->inspection_plan_id)
                ->where('itpc_id', $inspectionPlanChecklist->id)
                ->get();
            return $this->success($images, 'Image uploaded successfully!');
        }
        return $this->message('Invalid or no image uploaded.', 422);
    }


    public function getImages(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'inspection_plan_id' => 'required',
            'step_name' => 'required|string',
        ]);
        if ($validator->fails()) {
            $log = [
                'employee_id' => auth()->user()->id,
                'message' => 'Inspection images fetch: ' . $validator->errors()->first(),
                'report_id' => $request->inspection_plan_id,
                'report_type' => 'inspection_report',
                'error_type' => 'Validation error',
            ];
            storeReportsLogs($log);
            return $this->message($validator->errors()->first(), 422);
        }
        // Check customer or employee access
        $userTable = $this->getUserTable();
        $inspectionPlanQuery = InspectionPlan::where('id', $request->inspection_plan_id);
        if ($userTable === 'customer') {
            $inspectionPlanQuery->where('customer_id', auth()->id())
                ->where('workspace_id', auth()->user()->current_workspace_id);
        }
        if ($userTable === 'emp') {
            $inspectionPlanQuery->where('customer_id', auth()->user()->customer_id)
                ->where('workspace_id', auth()->user()->workspace_id);
        }
        if (!$inspectionPlanQuery->exists()) {
            return $this->message('You are not authorized to view these images.', 403);
        }

        $inspectionPlanChecklist = InspectionPlanChecklist::where('name', $request->step_name)
            ->where('inspection_plan_id', $request->inspection_plan_id)
            ->first();
        if (!$inspectionPlanChecklist) {
            return $this->message('Invalid step name or inspection plan ID.', 404);
        }
        $images = InspectionImage::where('itpc_id', $inspectionPlanChecklist->id)->get();
        return $this->success($images, 'Images retrieved successfully.');
    }

    public function getDescription(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'inspection_plan_id' => 'required',
        ]);
        if ($validator->fails()) {
            $log = [
                'employee_id' => auth()->user()->id,
                'message' => 'Inspection  images store:' . $validator->errors()->first(),
                'report_id' =>   $request->inspection_plan_id,
                'report_type' => 'inspection_report',
                'error_type' => 'Validation error',
            ];
            storeReportsLogs($log);
            return $this->message($validator->errors()->first(), 422);
        }
        $userTable = $this->getUserTable();
        $inspection_plan = InspectionPlan::where('id', $request->inspection_plan_id)->first();
        if ($userTable === 'customer') {
            $inspection_plan->where('customer_id', auth()->id())
                ->where('workspace_id', auth()->user()->current_workspace_id);
        }
        if ($userTable === 'emp') {
            $inspection_plan->where('customer_id', auth()->user()->customer_id)
                ->where('workspace_id', auth()->user()->workspace_id);
        }
        if (!$inspection_plan->exists()) {
            return $this->message('You are not authorized to view these images.', 403);
        }
        return $this->success($inspection_plan, 'Data get successfully!');
    }

    public function deleteImage($id)
    {
        $image = InspectionImage::find($id);
        if (!$image) {
            return $this->message('Image not found.', 404);
        }
        // Check customer or employee access
        $userTable = $this->getUserTable();
        $inspectionPlan = InspectionPlan::where('id', $image->inspection_plan_id);
        if ($userTable === 'customer') {
            $inspectionPlan->where('customer_id', auth()->id())
                ->where('workspace_id', auth()->user()->current_workspace_id);
        }
        if ($userTable === 'emp') {
            $inspectionPlan->where('customer_id', auth()->user()->customer_id)
                ->where('workspace_id', auth()->user()->workspace_id);
        }
        if (!$inspectionPlan->exists()) {
            return $this->message('You are not authorized to delete this image.', 403);
        }
        $imagePath = public_path($image->image);
        // Delete the image file from storage if it exists
        if (file_exists($imagePath)) {
            unlink($imagePath);
        }
        $image->delete();
        return $this->messge('Image deleted successfully.');
    }

    public function step4Store(Request $request)
    {
        try {
            // Validate the request
            $validator = Validator::make($request->all(), [
                'inspection_plan_id' => 'required',
                'description_box_for_message' => 'nullable|string',
            ]);
            if ($validator->fails()) {
                $log = [
                    'employee_id' => auth()->user()->id,
                    'message' => 'Inspection description store failed: ' . $validator->errors()->first(),
                    'report_id' => $request->inspection_plan_id,
                    'report_type' => 'inspection_report',
                    'error_type' => 'Validation error',
                ];
                storeReportsLogs($log);
                return $this->message($validator->errors()->first(), 422);
            }
            // Check if the inspection plan exists and if the user is authorized
            $userTable = $this->getUserTable();
            $inspectionPlanQuery = InspectionPlan::where('id', $request->inspection_plan_id)->first();
            if (!$inspectionPlanQuery) {
                return $this->message('Inspection plan not found.', 404);
            }
            if (
                $userTable === 'customer' && ($inspectionPlanQuery->customer_id !== auth()->id() ||
                    $inspectionPlanQuery->workspace_id !== auth()->user()->current_workspace_id
                )
            ) {
                return $this->message('You are not authorized to access this inspection plan.', 403);
            }
            if (
                $userTable === 'emp' &&  ($inspectionPlanQuery->customer_id !== auth()->user()->customer_id ||
                    $inspectionPlanQuery->workspace_id !== auth()->user()->workspace_id
                )
            ) {
                return $this->message('You are not authorized to access this inspection plan.', 403);
            }
            // Update the description
            $inspectionPlanQuery->description = $request->description_box_for_message;
            $inspectionPlanQuery->save();
            return $this->success('inspectionPlanQuery', 'Data saved successfully!');
        } catch (\Exception $e) {
            $shortMessage = substr($e->getMessage(), 0, 200) . (strlen($e->getMessage()) > 200 ? '...' : '');
            $log = [
                'employee_id' => auth()->user()->id,
                'message' => 'Error storing inspection description: ' . $shortMessage,
                'report_id' => $request->inspection_plan_id ?? null,
                'report_type' => 'inspection_report',
                'error_type' => 'Exception error',
            ];
            storeReportsLogs($log);
            return $this->message('An error occurred while saving the description.', 500);
        }
    }
}
