<?php

namespace App\Http\Controllers;

use App\Models\InspectionPlan;
use Illuminate\Http\Request;
use App\Models\InspectionPlanMeta;
use App\Models\InspectionPlanTable;
use App\Models\InspectionPlanChecklist;
use Illuminate\Support\Facades\Validator;

class InspectionPlanSetttingsController extends Controller
{
   
    public function indexedit($id)
    {
        try {
            // Determine user type (customer or employee)
            $userTable = $this->getUserTable();
            // Find the inspection plan setting by ID
            $inspection_plan_setting = InspectionPlanTable::orderBy('step_order', 'asc')->where('id', $id)->first();
            // Check if the record exists
            if (!$inspection_plan_setting) {
                return $this->message('Record not found.', 404);
            }
            // Customer and Employee Access Check
            if ($userTable === 'customer' && ($inspection_plan_setting->customer_id !== auth()->id() || $inspection_plan_setting->workspace_id !== auth()->user()->current_workspace_id)) {
                return $this->message('You do not have access to this record.', 403);
            }
            if ($userTable === 'emp' && ($inspection_plan_setting->customer_id !== auth()->user()->customer_id || $inspection_plan_setting->workspace_id !== auth()->user()->workspace_id)) {
                return $this->message('You do not have access to this record.', 403);
            }
            // Extract the necessary details from the record
            $first_title = $inspection_plan_setting->title ?? '';
            $step_order = $inspection_plan_setting->step_order ?? '';
            $first_value = $inspection_plan_setting->value ?? '';
            $id = $inspection_plan_setting->id ?? '';
            // Fetch related values with the same title
            $related_values = InspectionPlanTable::where('title', $first_title)
                ->get(['id', 'value']);
            // Check if related values exist
            if ($related_values->isEmpty()) {
                return $this->message('No related values found.', 404);
            }
            // Return data in a JSON response
            return $this->success([
                'first_title' => $first_title,
                'step_order' => $step_order,
                'first_value' => $first_value,
                'related_values' => $related_values,
                'id' => $id,
            ], 'Data retrieved successfully.');
        } catch (\Exception $e) {
            // Handle exceptions and return an error message
            return $this->message('An error occurred: ' . $e->getMessage(), 500);
        }
    }
    
    public function settingsTableEditData($id)
    {
        try {
            // Determine user type (customer or employee)
            $userTable = $this->getUserTable();
            // Find the inspection plan setting by ID
            $inspection_plan_setting = InspectionPlanTable::find($id);
            // Check if the record exists
            if (!$inspection_plan_setting) {
                return $this->message('Record not found.', 404);
            }
            // Customer and Employee Access Check
            if ($userTable === 'customer' && ($inspection_plan_setting->customer_id !== auth()->id() || $inspection_plan_setting->workspace_id !== auth()->user()->current_workspace_id)) {
                return $this->message('You do not have access to this record.', 403);
            }
            if ($userTable === 'emp' && ($inspection_plan_setting->customer_id !== auth()->user()->customer_id || $inspection_plan_setting->workspace_id !== auth()->user()->workspace_id)) {
                return $this->message('You do not have access to this record.', 403);
            }
            // Get the title for related values
            $first_title = $inspection_plan_setting->title;
            // Fetch related values with the same title
            $related_values = InspectionPlanTable::where('title', $first_title)
                ->get(['id', 'value']);
            // Check if related values exist
            if ($related_values->isEmpty()) {
                return $this->message('No related values found.', 404);
            }
            // Return the related values
            return $this->success($related_values, 'Data retrieved successfully.');
        } catch (\Exception $e) {
            // Return a custom JSON response in case of an error
            return $this->message('An error occurred: ' . $e->getMessage(), 500);
        }
    }



    public function inspectionPlanSettingGet()
    {
        try {
            // Determine user type (customer or employee)
            $userTable = $this->getUserTable();
            // Start building the query
            $inspectionPlanSettingsQuery = InspectionPlanTable::orderBy('step_order', 'asc');
            // Add customer or employee access filters
            if ($userTable === 'customer') {
                $inspectionPlanSettingsQuery->where('customer_id', auth()->id())
                    ->where('workspace_id', auth()->user()->current_workspace_id);
            } elseif ($userTable === 'emp') {
                $inspectionPlanSettingsQuery->where('customer_id', auth()->user()->customer_id)
                    ->where('workspace_id', auth()->user()->workspace_id);
            }
            // Fetch the filtered results
            $inspection_plan_settings = $inspectionPlanSettingsQuery->get()
                ->groupBy('title')
                ->map(function ($group) {
                    return [
                        'id' => $group->first()->id, // Include the ID here
                        'title' => $group->first()->title,
                        'step_order' => $group->first()->step_order,
                        'values' => $group->pluck('value')->implode(', ')
                    ];
                })
                ->values(); // Re-index the collection
            // Check if the result is empty
            if ($inspection_plan_settings->isEmpty()) {
                return $this->message('No inspection plan settings found.', 404);
            }
            // Return the data if it's not empty
            return $this->success($inspection_plan_settings, 'Data retrieved successfully.');
        } catch (\Exception $e) {
            // Return a custom JSON response in case of an error
            return $this->message('An error occurred: ' . $e->getMessage(), 500);
        }
    }



    public function inspectionPlanFormOptions()
    {
        try {
            // Determine user type (customer or employee)
            $userTable = $this->getUserTable();
            // Start building the query
            $inspectionPlanSettingsQuery = InspectionPlanTable::distinct('title')
                ->orderBy('title', 'asc');
            // Add customer or employee access filters
            if ($userTable === 'customer') {
                $inspectionPlanSettingsQuery->where('customer_id', auth()->id())
                    ->where('workspace_id', auth()->user()->current_workspace_id);
            } elseif ($userTable === 'emp') {
                $inspectionPlanSettingsQuery->where('customer_id', auth()->user()->customer_id)
                    ->where('workspace_id', auth()->user()->workspace_id);
            }
            // Fetch the filtered results
            $inspectionPlanSettings = $inspectionPlanSettingsQuery->get(['title']);
            // Check if the result is empty
            if ($inspectionPlanSettings->isEmpty()) {
                return $this->message('Please create form steps in settings to proceed after.', 422);
            }
            // Return the data if it's not empty
            return $this->success($inspectionPlanSettings, 'Fetched successfully.');
        } catch (\Exception $e) {
            // Return a custom JSON response in case of an error
            return $this->error('An error occurred: ' . $e->getMessage(), 500);
        }
    }

    public function indexDelete($id)
    {
        if (!$id) {
            return $this->message('ID is required. Please refresh the page.', 422);
        }
        // Step 1: Find the Record
        $record = InspectionPlanTable::find($id);
        if (!$record) {
            return $this->message('Record not found. Please refresh the page.', 404);
        }
        // Step 2: Fetch User Type and Perform Access Check
        $userTable = $this->getUserTable();
        if ($userTable === 'customer' && ($record->customer_id !== auth()->id() || $record->workspace_id !== auth()->user()->current_workspace_id)) {
            return $this->message('You do not have access to this inspection plan.', 403);
        }
        if ($userTable === 'emp' && ($record->customer_id !== auth()->user()->customer_id || $record->workspace_id !== auth()->user()->workspace_id)) {
            return $this->message('You do not have access to this inspection plan.', 403);
        }
        // Step 3: Delete Related Records
        $status = InspectionPlanTable::where('title', $record->title)->delete();
        InspectionPlanMeta::where('name', $record->title)->delete();
        InspectionPlanChecklist::where('name', $record->title)->delete();
        // Step 4: Respond Based on Deletion Status
        if (!$status) {
            return $this->message('Delete Failed.', 500);
        }
        return $this->message('Deleted Successfully.', 200);
    }


    public function inspectionPlanSettingStore(Request $request)
    {
        // Validating the request
        $validator = Validator::make($request->all(), [
            'step_name' => 'required|string',
            'step_order' => 'required|integer',
            'table_title' => 'nullable|array',
            'table_title.*' => 'nullable|string'
        ], [
            'step_name.required' => 'Checklist is required',
        ]);
        if ($validator->fails()) {
            return $this->message($validator->errors()->first(), 422);
        }
        $userTable = $this->getUserTable(); // Determine user type (customer or emp)
        $customerId = null;
        $workspaceId = null;
        if ($userTable === 'customer') {
            $customerId = auth()->id();
            $workspaceId = auth()->user()->current_workspace_id;
        } elseif ($userTable === 'emp') {
            $customerId = auth()->user()->customer_id;
            $workspaceId = auth()->user()->workspace_id;
        } else {
            return $this->message('You are not authorized to perform this action.', 403);
        }
        $step_name = $request->input('step_name');
        $step_order = $request->input('step_order');
        $updated_name = str_replace(' ', '_', $step_name);
        try {
            // If table titles are provided
            if ($request->table_title) {
                foreach ($request->table_title as $title) {
                    // Save data to InspectionPlanTable
                    $test = InspectionPlanTable::create([
                        'title' => $updated_name,
                        'step_order' => $step_order,
                        'customer_id' => $customerId,
                        'workspace_id' => $workspaceId,
                        'value' => $title,
                    ]);
                    // Save data to InspectionPlanMeta
                    InspectionPlanMeta::create([
                        'name' => $updated_name,
                        'title' => $title,
                        'is_table' => 1,

                    ]);
                }
            } else {
                // Save data without table titles
                InspectionPlanTable::create([
                    'title' => $updated_name,
                    'step_order' => $step_order,
                    'customer_id' => $customerId,
                    'workspace_id' => $workspaceId,
                ]);
                InspectionPlanMeta::create([
                    'name' => $updated_name,
                    'is_table' => 1,
                ]);
            }
            return $this->message('Created Successfully');
        } catch (\Exception $e) {
            $log = [
                'employee_id' => auth()->user()->id,
                'message' => 'Error creating inspection plan setting: ' . $e->getMessage(),
                'report_type' => 'inspection_plan',
                'error_type' => 'Exception error',
            ];
            storeReportsLogs($log);
            return $this->message('An error occurred while saving the inspection plan setting.', 500);
        }
    }

    public function inspectionPlanSettingUpdate(Request $request)
    {
        // Validate incoming request data
        $validator = Validator::make($request->all(), [
            'id' => 'required|integer',
            'table_name' => 'required|string|max:255',
            'step_order' => 'required|integer',
            'table_title' => 'required|array',
            'new_titles' => 'nullable|array',
        ]);

        if ($validator->fails()) {
            return $this->message($validator->errors()->first(), 422);
        }
        // Get user type
        $userTable = $this->getUserTable();
        // Find the existing record
        $existingRecord = InspectionPlanTable::find($request->id);

        if (!$existingRecord) {
            return $this->message('Record not found.', 404);
        }
        // Initialize customer and workspace IDs
        $customerId = null;
        $workspaceId = null;
        // Customer and Employee Access Check
        if ($userTable === 'customer') {
            $customerId = auth()->id();
            $workspaceId = auth()->user()->current_workspace_id;

            if ($existingRecord->customer_id !== $customerId || $existingRecord->workspace_id !== $workspaceId) {
                return $this->message('You do not have access to this project', 403);
            }
        }
        if ($userTable === 'emp') {
            $customerId = auth()->user()->customer_id;
            $workspaceId = auth()->user()->workspace_id;

            if ($existingRecord->customer_id !== $customerId || $existingRecord->workspace_id !== $workspaceId) {
                return $this->message('You do not have access to this project', 403);
            }
        }
        // Update related tables with the new table name
        $table_name = $request->table_name;
        $existing_title = $existingRecord->title;
        $step_order = $request->step_order;
        $newTitle = str_replace(' ', '_', $table_name);
        // Update InspectionPlanMeta and InspectionPlanChecklist
        InspectionPlanMeta::where('name', $existing_title)->update(['name' => $newTitle]);
        InspectionPlanChecklist::where('name', $existing_title)->update(['name' => $newTitle]);
        // Update existing table titles
        foreach ($request->table_title as $recordId => $newValue) {
            $relatedRecord = InspectionPlanTable::find($recordId);
            if ($relatedRecord) {
                $relatedRecord->title = $newTitle;
                $relatedRecord->step_order = $step_order;
                $relatedRecord->value = $newValue;
                $relatedRecord->save();
                // Update meta records
                InspectionPlanMeta::where('name', $newTitle)
                    ->where('title', $relatedRecord->value)
                    ->where('is_table', 1)
                    ->update([
                        'title' => $newValue,
                    ]);
            }
        }
        // Add new titles
        if (!empty($request->new_titles)) {
            foreach ($request->new_titles as $newTitleValue) {
                // Only add if the value does not already exist
                if (!InspectionPlanTable::where('value', $newTitleValue)->exists()) {
                    InspectionPlanMeta::create([
                        'title' => $newTitleValue,
                        'name' => $newTitle,
                        'is_table' => '1',
                    ]);
                    InspectionPlanTable::create([
                        'title' => $newTitle,
                        'value' => $newTitleValue,
                        'customer_id' => $customerId,
                        'workspace_id' => $workspaceId,
                    ]);
                }
            }
        }
        return $this->message('Updated successfully.', 200);
    }

    public function inspectionPlanSettingMetaTable(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'name' => 'required|string|max:255',
            'inspection_plan_id' => 'required|integer',
        ]);
        if ($validator->fails()) {
            return $this->message($validator->errors()->first(), 422);
        }
        $name = $request->name;
        $inspection_plan_id = $request->inspection_plan_id;
        // Fetch user type
        $userTable = $this->getUserTable();
        // Step 1: Fetch the Inspection Plan
        $inspectionPlan = InspectionPlan::find($inspection_plan_id);
        if (!$inspectionPlan) {
            return $this->message('Inspection plan not found.', 404);
        }
        // Step 2: Customer and Employee Access Check
        if ($userTable === 'customer' && ($inspectionPlan->customer_id !== auth()->id() || $inspectionPlan->workspace_id !== auth()->user()->current_workspace_id)) {
            return $this->message('You do not have access to this inspection plan.', 403);
        }
        if ($userTable === 'emp' && ($inspectionPlan->customer_id !== auth()->user()->customer_id || $inspectionPlan->workspace_id !== auth()->user()->workspace_id)) {
            return $this->message('You do not have access to this inspection plan.', 403);
        }
        // Step 3: Fetch the Inspection Plan Checklist
        $inspection_plan_checklist = InspectionPlanChecklist::where('name', $name)
            ->where('inspection_plan_id', $inspection_plan_id)
            ->first();
        if (!$inspection_plan_checklist) {
            return $this->message('Checklist not found.', 404);
        }
        // Step 4: Fetch Table Heads and Form Data
        $data = [
            'inspection_plan_table_heads' => InspectionPlanMeta::where('name', $name)
                ->where('is_table', 1)
                ->pluck('title'),
            'inspection_plan_table_form_data' => InspectionPlanMeta::where('itpc_id', $inspection_plan_checklist->id)
                ->where('is_table', 0)
                ->select('id', 'title', 'value') // Select relevant columns
                ->get(),
        ];
        return $this->success($data, 'Data retrieved successfully.');
    }


    public function inspectionPlanSettingMetaTableSave(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'thead' => 'required|array',             // Headers of the table
            'table_values' => 'required|array',     // Table values
            'row_ids' => 'nullable|array',          // Row IDs
            'name' => 'required|string|max:255',    // Name for identification
            'inspection_plan_id' => 'required|integer',
        ]);
        if ($validator->fails()) {
            return $this->message($validator->errors()->first(), 422);
        }
        $thead = $request->input('thead');
        $table_values = $request->input('table_values');
        $row_ids = $request->input('row_ids');
        $name = $request->input('name');
        $inspection_plan_id = $request->input('inspection_plan_id');
        // Fetch user type
        $userTable = $this->getUserTable();
        // Step 1: Fetch the Inspection Plan
        $inspectionPlan = InspectionPlan::find($inspection_plan_id);
        if (!$inspectionPlan) {
            return $this->message('Inspection plan not found.', 404);
        }
        // Step 2: Customer and Employee Access Check
        if ($userTable === 'customer' && ($inspectionPlan->customer_id !== auth()->id() || $inspectionPlan->workspace_id !== auth()->user()->current_workspace_id)) {
            return $this->message('You do not have access to this inspection plan.', 403);
        }
        if ($userTable === 'emp' && ($inspectionPlan->customer_id !== auth()->user()->customer_id || $inspectionPlan->workspace_id !== auth()->user()->workspace_id)) {
            return $this->message('You do not have access to this inspection plan.', 403);
        }
        // Step 3: Find the Checklist
        $inspection_plan_checklist = InspectionPlanChecklist::where('inspection_plan_id', $inspection_plan_id)
            ->where('name', $name)
            ->first();
        if (!$inspection_plan_checklist) {
            return $this->message('Checklist not found.', 404);
        }
        // Step 4: Delete old meta records
        InspectionPlanMeta::where('inspection_plan_id', $inspection_plan_id)
            ->where('itpc_id', $inspection_plan_checklist->id)
            ->where('is_table', 0)
            ->delete();
        // Step 5: Insert new records
        foreach ($table_values as $rowIndex => $values) {
            if (!is_array($values)) {
                continue; // Skip if the row is not an array
            }
            foreach ($values as $colIndex => $value) {
                // Ensure the header index exists
                $title = $thead[$colIndex] ?? '';
                $inspection_plan_meta = new InspectionPlanMeta();
                $inspection_plan_meta->inspection_plan_id = $inspection_plan_id;
                $inspection_plan_meta->itpc_id = $inspection_plan_checklist->id;
                $inspection_plan_meta->title = $title;
                $inspection_plan_meta->value = $value;
                $inspection_plan_meta->name = $name;
                $inspection_plan_meta->save();
            }
        }
        // Step 6: Respond with success message
        return $this->success([
            'thead' => $thead,
            'table_values' => $table_values,
            'row_ids' => $row_ids,
            'name' => $name,
        ], 'Table record submitted and saved successfully.');
    }


    public function inspectionPlanSettingMetaTableFormDataDelete(Request $request)
    {
        $ids = $request->input('ids'); // Expecting an array of IDs
        if (!is_array($ids)) {
            return $this->message('Invalid data format', 400);
        }
        $deletedCount = InspectionPlanMeta::whereIn('id', $ids)->delete();
        if ($deletedCount === 0) {
            return $this->message('No records found for deletion', 404);
        }
        return $this->success($deletedCount, 'Deleted successfully');
    }

    public function inspectionPlanChecklistGet(Request $request)
    {
        try {
            $validator = Validator::make($request->all(), [
                'inspection_plan_id' => 'required',
            ]);
            if ($validator->fails()) {
                return $this->message($validator->errors()->first(), 422);
            }
            $selected_steps = InspectionPlanChecklist::where('inspection_plan_id', $request->inspection_plan_id)
                ->where('status', "1")
                ->get();
            if ($selected_steps->isEmpty()) {
                return $this->message('No previous form was selected.', 404);
            }
            $inspectionPlan = InspectionPlan::find($request->inspection_plan_id);
            $userTable = $this->getUserTable();
            // Customer and Employee Access Check
            if ($userTable === 'customer' && ($inspectionPlan->customer_id !== auth()->id() || $inspectionPlan->workspace_id !== auth()->user()->current_workspace_id)) {
                return $this->message('You do not have access.', 403);
            }
            if ($userTable === 'emp' && ($inspectionPlan->customer_id !== auth()->user()->customer_id || $inspectionPlan->workspace_id !== auth()->user()->workspace_id)) {
                return $this->message('You do not have access .', 403);
            }
            if ($selected_steps->isEmpty()) {
                return $this->success($selected_steps, 'No previous form was selected.', 433);
            }
            return $this->success($selected_steps, 'Steps retrieved successfully');
        } catch (\Exception $e) {
            return $this->error($e->getMessage());
        }
    }
    // 
    public function inspectionPlanSettingsDelete($id)
    {
        // Find the item by ID
        $deletedItem = InspectionPlanTable::find($id);
        if (!$deletedItem) {
            return $this->message('Record not found.', 404);
        }
        $userTable = $this->getUserTable();
        // Customer and Employee Access Check
        if ($userTable === 'customer' && ($deletedItem->customer_id !== auth()->id() || $deletedItem->workspace_id !== auth()->user()->current_workspace_id)) {
            return $this->message('You do not have access.', 403);
        }
        if ($userTable === 'emp' && ($deletedItem->customer_id !== auth()->user()->customer_id || $deletedItem->workspace_id !== auth()->user()->workspace_id)) {
            return $this->message('You do not have access .', 403);
        }
        // Delete associated meta records
        $deletedCount = InspectionPlanMeta::where('title', $deletedItem->value)->delete();
        // Delete the item itself
        $deletedItem->delete();
        return $this->success($deletedCount, 'Deleted successfully.', 200,);
    }
}
