<?php

namespace App\Http\Controllers;

use Illuminate\Http\Request;
use DB;
use Auth;
use Illuminate\Support\Facades\Validator;
use App\Models\PublicHoliday;
use Illuminate\Validation\Rule;
class LeaveController extends Controller
{
    
    public function index(Request $request)
    {
        $userTable = $this->getUserTable();
        $query = PublicHoliday::query();
        if ($userTable === "customer") {
            $query->where('customer_id', Auth::user()->id)
                ->where('workspace_id', Auth::user()->current_workspace_id);
        }
        if ($userTable === "emp") {
            $query->where('customer_id', auth()->user()->customer_id) // Assuming customer_id links employee to projects
                ->where('workspace_id', auth()->user()->workspace_id);
        }
        if($request->filled('filter'))
        {
          $query =  $this->filter(json_decode($request->filter, true),$query);
        }
  
        $count = $query->count();
        $start = $request->input('from', 0);
        $query->offset($start)->limit(10);
        $query->orderBy('id', 'desc');
        $query_result = $query->get();

        $data =  [
            'holidays' => $query_result,
        ];
        $array_filter =  json_decode($request->filter, true);
        $filters = [
          'first_name' => $array_filter['first_name'] ?? '', 
          'middle_name' => $array_filter['middle_name'] ?? '',
          'last_name' => $array_filter['first_name'] ?? '',
          'employee_email' => $array_filter['last_name'] ?? '',
          'compeleted' => $array_filter['first_name'] ?? '',
          'status' => $array_filter['compeleted'] ?? '',
        ];
        return $this->WithFilter($query_result, $filters,'Get Holiday List Successfully');
    }

    public function filter($filters,$query)
    {
     
        foreach ($filters as $filterName => $filterValue) {
              if ($filterValue != null ||  $filterValue !="" ) {
                  switch ($filterName) {
                  case 'company':
                    $query->whereHas('company', function ($subquery) use ($filterValue) {
                    $subquery->where('fname', 'like', '%' . $filterValue . '%');
                  });
                  break;
                  case 'title':
                      $query->where('title', 'like', '%' . $filterValue . '%');
                  break;
                  case 'status':
                      $filterValue =   $filterValue ? 1 : 0;
                      $query->where('status', 'like', '%' . $filterValue . '%');
                  break;
                  case 'latitude':
                    $query->where('latitude', 'like', '%' . $filterValue . '%');
                  break;  
                  case 'longitude':
                    $query->where('longitude', 'like', '%' . $filterValue . '%');
                  break;                     
                  case 'area_radius':
                    $query->where('area_radius', 'like', '%' . $filterValue . '%');
                  break; 
                  case 'State':
                    $query->where('State', 'like', '%' . $filterValue . '%');
                  break;                  
                  }
              }
        }

        return $query;
    } 

    public function store(Request $request)
    {
        $userTable = $this->getUserTable();
        $auth_id =0;
        $workspace_id = 0;
        if ($userTable === "customer") {
            $auth_id = Auth::user()->id;
            $workspace_id = Auth::user()->current_workspace_id;   
        }
        if ($userTable === "emp") {
            $auth_id = auth()->user()->customer_id;
            $workspace_id = auth()->user()->workspace_id;  
        }
        $validator = Validator::make($request->all(), [
            'holiday_name' => [
                'required',
                'string',
                Rule::unique('public_holidays', 'title')
                    ->where(function ($query) use ($auth_id, $workspace_id) {
                        return $query->where('customer_id', $auth_id)
                                     ->where('workspace_id', $workspace_id);
                    }),
            ],
            'description' => 'required|string',
            'start_date' => 'required|date',
            'end_date' => 'required|date|after_or_equal:start_date',
        ]);
 
        if ($validator->fails()) {
            return $this->handleValidationFailure($validator);
        } else {
           $validatedData =  $validator->validated();
           PublicHoliday::insert([
            'title' =>$validatedData['holiday_name'],
            'description' => $validatedData['description'],
            'from' => $validatedData['start_date'],
            'to' => $validatedData['end_date'],
            'customer_id' => $auth_id,
            'workspace_id' => $workspace_id,
        ]);
        $data = PublicHoliday::where('customer_id',$auth_id)->where('workspace_id',$workspace_id)->get();
        return $this->success($data, 'Holiday Saved Successfully');
        }      

    }

    public function edit($id)
    {
        $publicholiday = PublicHoliday::find($id);
        if (!$publicholiday) {
            return $this->message('The Public Holiday with the specified ID does not exist.',404); 
        }
       
        $userTable = $this->getUserTable();
        if ($userTable == "customer" && ($publicholiday->workspace_id != auth()->user()->current_workspace_id || $publicholiday->customer_id != auth()->user()->id)) 
        {
            return $this->message('You do not have access to this Public Holiday', 403);
        }
        if ($userTable == "emp" && ($publicholiday->customer_id != auth()->user()->customer_id || $publicholiday->workspace_id != auth()->user()->workspace_id)) {
            return $this->message('You do not have access to this Public Holiday', 403);
        }
        return $this->success($publicholiday, ' Public Holiday Successfully List');
       
    }

    public function updatePublicHoliday(Request $request, $id)
    {
      
        $validator = Validator::make($request->all(), [
            'holiday_name' => 'required|string|unique:public_holidays,title,' . $id,
            'description' => 'required|string',
            'start_date' => 'required|date',
            'end_date' => 'required|date|after_or_equal:start_date',
        ]);
    
        if ($validator->fails()) {
            return $this->handleValidationFailure($validator);
        } else {
             $publicholiday = PublicHoliday::find($id);
             if (!$publicholiday) {
                return $this->message('The Leave Type  with the specified ID does not exist.',404); 
            }
            $userTable = $this->getUserTable();
            if ($userTable == "customer" && ($publicholiday->workspace_id != auth()->user()->current_workspace_id || $publicholiday->customer_id != auth()->user()->id)) 
            {
                return $this->message('You do not have access to this Public Holiday', 403);
            }
            if ($userTable == "emp" && ($publicholiday->customer_id != auth()->user()->customer_id || $publicholiday->workspace_id != auth()->user()->workspace_id)) {
                return $this->message('You do not have access to this Public Holiday', 403);
            }
            $auth_id =0;
            $workspace_id = 0;
            if ($userTable === "customer") {
                $auth_id = Auth::user()->id;
                $workspace_id = Auth::user()->current_workspace_id;   
            }
            if ($userTable === "emp") {
                $auth_id = auth()->user()->customer_id;
                $workspace_id = auth()->user()->workspace_id;  
            }
           
            $validatedData =  $validator->validated();
            $publicholiday->update([
                'title' =>$validatedData['holiday_name'],
                'description' => $validatedData['description'],
                'from' => $validatedData['start_date'],
                'to' => $validatedData['end_date'],
                'customer_id' => $auth_id,
                'workspace_id' => $workspace_id,
            ]);
            $data = PublicHoliday::where('customer_id',$auth_id)->where('workspace_id',$workspace_id)->get();
            return $this->success($data, 'Holiday Saved Successfully');
        }
    }
    
    public function destroy($id)
    {
        $publicholiday = PublicHoliday::where('id',$id)->first();
        if (!$publicholiday) {
            return $this->message('The Public Holiday with the specified ID does not exist.',404); 
        }
        $userTable = $this->getUserTable();
        if ($userTable == "customer" && ($publicholiday->workspace_id != auth()->user()->current_workspace_id || $publicholiday->customer_id != auth()->user()->id)) 
        {
            return $this->message('You do not have access to this Public Holiday', 403);
        }
        if ($userTable == "emp" && ($publicholiday->customer_id != auth()->user()->customer_id || $publicholiday->workspace_id != auth()->user()->workspace_id)) {
            return $this->message('You do not have access to this Public Holiday', 403);
        }
        PublicHoliday::where('id',$id)->delete();
        return $this->message('Holiday Deleted Successfully', 200);    
    }

    public function holidaydesc($id)
    {
        
        $data = PublicHoliday::findOrFail($id);
        return response()->json([
            'data'   => $data
        ],200);
    }
}
