<?php

namespace App\Http\Controllers;

use App\Models\LeaveRequest;
use App\Models\LeavePackage;
use App\Models\TierPackage;
use App\Models\RosterAssign;
use App\Models\EmpPersonalDetails;
use App\Models\EmployeeAttendance;
use App\Models\EmpCompanyDetails;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Validator;
use Carbon\Carbon;
use Illuminate\Support\Facades\DB;
use Illuminate\Http\Request;
use App\Http\Controllers\Traits\EmailTrait;

class LeaveRequestController extends Controller
{   
    use EmailTrait;

    public function __construct()
    {
        $this->middleware('Permissions:Leave Request Maintain')->only(['create', 'edit']);
    }
    public function index(Request $request)
    {
        $userTable = $this->getUserTable();
        $query = LeaveRequest::query();
        if ($userTable === "customer") {
            $query->where('customer_id', Auth::user()->id)
                ->where('workspace_id', Auth::user()->current_workspace_id);
        }
        if ($userTable === "emp") {
            $query->where('customer_id', auth()->user()->customer_id) // Assuming customer_id links employee to projects
                ->where('workspace_id', auth()->user()->workspace_id);
        }
        if($request->filled('filter'))
        {
          $query =  $this->filter(json_decode($request->filter, true),$query);
        }
        $query->with('LeavePackage')->with('EmpPersonalDetails');

        $count = $query->count();
        $start = $request->input('from', 0);
        $query->offset($start)->limit(10);
        $query->orderBy('id', 'desc');
        $query_result = $query->get();

        $data = [
            'leaverequest' => $query_result,
        ];
        
        $array_filter =  json_decode($request->filter, true);
        $filters = [
          'first_name' => $array_filter['first_name'] ?? '', 
          'middle_name' => $array_filter['middle_name'] ?? '',
          'last_name' => $array_filter['first_name'] ?? '',
          'employee_email' => $array_filter['last_name'] ?? '',
          'compeleted' => $array_filter['first_name'] ?? '',
          'status' => $array_filter['compeleted'] ?? '',
        ];
        return $this->WithFilter($query_result, $filters,'Get Leave Request List Successfully');

        return response()->json([
            'message' => 'Get Leave Package List Successfully',
            'data' => $data,
            'count' => $count,
            'filters' => $filters
        ],200);
    }

    public function filter($filters,$query)
    {
        foreach ($filters as $filterName => $filterValue) {
              if ($filterValue != null ||  $filterValue !="" ) {
                  switch ($filterName) {
                    case 'first_name':
                        $filterValue  = explode(" ",$filterValue);
                        if(count($filterValue) == 1)
                        {
                            $query->whereHas('empDetails', function ($subquery) use ($filterValue) {
                                $subquery->where('first_name', 'like', '%' . $filterValue[0] . '%');
                            });
                        }
    
                        if(count($filterValue) == 2)
                        {
                            $query->whereHas('empDetails', function ($subquery) use ($filterValue) {
                                $subquery->where('first_name', 'like', '%' . $filterValue[0] . '%');
                                $subquery->where('last_name', 'like', '%' . $filterValue[1] . '%');
                            });
                        }
    
                        if(count($filterValue) == 3)
                        {
                            $query->whereHas('empDetails', function ($subquery) use ($filterValue) {
                                $subquery->where('first_name', 'like', '%' . $filterValue[0] . '%');
                                $subquery->where('middle_name', 'like', '%' . $filterValue[1] . '%');
                                $subquery->where('last_name', 'like', '%' . $filterValue[2] . '%');
                            });
                        }
    
                      break;
                  case 'leave_package_id':
                      $query->where('leave_package_id',$filterValue);
                  break;
                  case 'to':
                    $query->where('to',date('Y-m-d',strtotime($filterValue)));
                break;
                case 'from':
                    $query->where('from',date('Y-m-d',strtotime($filterValue)));
                break;
                  case 'status':
                      $query->where('status',$filterValue);
                  break;                 
                  }
              }
        }

        return $query;
    } 


    public function create()
    {
        $leave_packages =  LeavePackage::orderBy('id', 'DESC')->get();
      
        $emp_details =  EmpCompanyDetails::with('EmpPersonalDetails')->where('del',0)->where('compeleted', 1)->orderBy('id', 'DESC')->get();
        return view('LeaveRequest.create',compact('leave_packages','emp_details'));
    }

   
    public function store(Request $request)
    {
        $userTable = $this->getUserTable();
        $auth_id =0;
        $workspace_id = 0;
        if ($userTable === "customer") {
            $auth_id = Auth::user()->id;
            $workspace_id = Auth::user()->current_workspace_id;   
        }
        if ($userTable === "emp") {
            $auth_id = auth()->user()->customer_id;
            $workspace_id = auth()->user()->workspace_id;  
        }
        $validator = Validator::make($request->all(), [
            'employee' => 'required',
            'leave_package' => 'required',
            'start_date' => 'required|date',
            'end_date' => 'required|date|after_or_equal:start_date',
            'reason' => 'required',
            'full_day' => 'nullable',
            'start_time' => 'nullable',
            'end_time' => 'nullable'
        ]);
 
        if ($validator->fails()) {
            return $this->handleValidationFailure($validator);
        } else {
          
        $validatedData = $validator->validated();
        // employee_id
        if(
            (LeaveRequest::where('from','>=',$validatedData['start_date'] )->where('status','!=','2')->where('employee_id',$validatedData['employee'])->count()
             && LeaveRequest::where('to','<=',$validatedData['end_date'] )->where('status','!=','2')->where('employee_id',$validatedData['employee'])->count())
            ||LeaveRequest::where('from',$validatedData['start_date'] )->where('status','!=','2')->where('employee_id',$validatedData['employee'])->count()
          )
        {
            return $this->message('You already have leave scheduled for this date',400);
        }

        $userTable = $this->getUserTable();
            $auth_id =0;
            $workspace_id = 0;
            if ($userTable === "customer") {
                $auth_id = Auth::user()->id;
                $workspace_id = Auth::user()->current_workspace_id;        
            }
            if ($userTable === "emp") {
                $auth_id = auth()->user()->customer_id;
                $workspace_id = auth()->user()->workspace_id; 
            }
      
        $insertedRecord = LeaveRequest::create([
            'employee_id' =>$validatedData['employee'],
            'leave_package_id' => $validatedData['leave_package'],
            'from' => $validatedData['start_date'],
            'to' => $validatedData['end_date'],
            'reason' => $validatedData['reason'],
            'full_day' => $validatedData['full_day'],
            'start_time' => $validatedData['start_time'],
            'end_time' => $validatedData['end_time'],
            'applied_by' => Auth::user()->id,
            'customer_id' => $auth_id,
            'workspace_id' => $workspace_id,
        ]);
        
        $id = $insertedRecord->id;
        $data = LeaveRequest::find($id);
        return $this->success($data, 'Leave Request Saved Successfully');    
        } 
    }
////////////////leave Request Api///////////////
    public function applie_leave_request(Request $request)
{
    $validator = Validator::make($request->all(), [
        'employee_id'       => 'required',
        'leave_package_id'  => 'required',
        'from'              => 'required|date',
        'mail_to'           => 'required',
        'mail_cc'           => 'required|array',
        'to'                => 'required|date|after_or_equal:from',
        'reason'            => 'required'
    ], [
        // Here you specify your custom messages
        'from.required' => 'Start date is required.',  // Custom message for `from` field
        'to.required'   => 'End date is required.',    // Custom message for `to` field
    ]);

    if ($validator->fails()) {
        return response()->json([
            'message' => $validator->errors()->first()
        ], 422);
    }
    $validatedData = $validator->validated();

    // Calculate number of leave days
    $fromDate = Carbon::parse($validatedData['from']);
    $toDate = Carbon::parse($validatedData['to']);
    $numberOfDays = $fromDate->diffInDays($toDate) + 1; // Include start and end dates
    // Check if leave overlaps with existing leaves
    $existingLeaves = LeaveRequest::where('employee_id', $validatedData['employee_id'])
        ->where('status', '!=', 2)
        ->where(function ($query) use ($validatedData) {
            $query->whereBetween('from', [$validatedData['from'], $validatedData['to']])
                ->orWhereBetween('to', [$validatedData['from'], $validatedData['to']])
                ->orWhere(function ($q) use ($validatedData) {
                    $q->where('from', '<=', $validatedData['from'])
                        ->where('to', '>=', $validatedData['to']);
                });
        })
        ->exists();

    if ($existingLeaves) {
        return response()->json([
            'message' => 'You already have a leave scheduled during this period'
        ], 400);
    }

    // Fetching details
    $leavePackage = LeavePackage::find($validatedData['leave_package_id']);
    $empDetail = EmpPersonalDetails::where('emp_id',$validatedData['employee_id'])->first();
    // $mailTo = EmpCompanyDetails::find($validatedData['mail_to'])->employee_email;
    $employee = EmpCompanyDetails::with('empTeamsMembers.empTeamsList')
        ->where('id', $validatedData['employee_id'])
        ->first();
    $teamTitle = null;
    if ($employee) {
        foreach ($employee->empTeamsMembers as $teamMember) {
            if (isset($teamMember->empTeamsList)) {
                $teamTitle = $teamMember->empTeamsList->title;
                break;
            }
        }
    }

    // Handling CCs
    $ccEmails = EmpCompanyDetails::whereIn('id', $validatedData['mail_cc'])->pluck('employee_email')->toArray();

    // Email data
    $emailData = [
        'name' => $empDetail->first_name . ' ' . $empDetail->middle_name . ' ' . $empDetail->last_name,
        'leave_pkg' => $leavePackage->title,
        'team' => $teamTitle,
        'start_date' => $validatedData['from'],
        'end_date' => $validatedData['to'],
        'reason' => $validatedData['reason']
    ];

    // Send email with CC support
    // $emailSent = $this->sendEmail_cc($mailTo, $leavePackage->title . ' | ' . env("APP_NAME"), $emailData, 'Emails.Leave-Request', $ccEmails);

    // Store leave request
    $validatedData['number_of_leaves'] = $numberOfDays;
    $insertedRecord = LeaveRequest::create($validatedData);
    $data = LeaveRequest::find($insertedRecord->id);

    return response()->json([
        'message' => 'Leave Saved Successfully',
        'email_status' =>  'Email sending failed.',
        'data' => $data
    ], 200);
}
    
    public function get_leave_request(Request $request)
    {
        
        $validator = Validator::make($request->all(), [
            'employee_id' => 'required'
        ]);
    
        if ($validator->fails()) {
            $error = $validator->errors()->first();
            return response()->json([
                'message' => $error
            ], 422);
        } else {
            $validatedData = $validator->validated();
           
            $data = LeaveRequest::where('employee_id', $validatedData['employee_id'])
            ->with(['empCompanyDetails.empPersonalDetails'])
            ->with('LeavePackage')
            ->get();

            if ($data->isNotEmpty()) {
            return $this->success($data, 'Leave Retrieved Successfully');
            } else {
            return $this->error('Leave not found for the given employee ');
            }
        }
    }

    public function edit_leave_request(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'leave_id' => 'required'
        ]);
    
        if ($validator->fails()) {
            $error = $validator->errors()->first();
            return response()->json([
                'message' => $error
            ], 422);
        } else {
            $validatedData = $validator->validated();
            $data = LeaveRequest::where('id', $validatedData['leave_id'])->first();
    
            if ($data !== null) {
                return response()->json([
                    'message' => 'Leave Retrieved Successfully',
                    'data'   => $data
                ], 200);
            } else {
                return response()->json([
                    'message' => 'Leave not found',
                ], 404);
            }
        }
    }

    public function update_leave_request(Request $request, $id)
    {
        $validator = Validator::make($request->all(), [
            'employee_id' => 'required',
            'leave_package_id' => 'required',
            'from' => 'required|date',
            'to' => 'required|date|after_or_equal:from',
            'reason' => 'required'
        ]);

        if ($validator->fails()) {
            $error = $validator->errors()->first();
            return response()->json([
                'message' => $error
            ], 422);
        } else {
            $validatedData = $validator->validated();

            $leaveRequest = LeaveRequest::find($id);

            if ($leaveRequest) {
                $leaveRequest->update($validatedData);

                $data = LeaveRequest::find($id);
                return response()->json([
                    'message' => 'Leave Updated Successfully',
                    'data'   => $data
                ], 200);
            } else {
                return response()->json([
                    'message' => 'Leave not found',
                ], 404);
            }
        }
    }

    public function employee_leave_pkg(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'employee_id' => 'required|integer',
        ]);

        if ($validator->fails()) {
            return response()->json([
                'message' => $validator->errors()->first()
            ], 422);
        }

        $validatedData = $validator->validated();
        $employeeId = $validatedData['employee_id'];

        $empDetails = EmpCompanyDetails::where('id', $employeeId)->first();

        if (!$empDetails || !$empDetails->accessRole) {
            return response()->json(['error' => 'Employee or employee role not found'], 404);
        }

        $accessRoleId = $empDetails->accessRole->id;

        $leavePackages = LeavePackage::whereRaw("FIND_IN_SET(?, employee_roles)", [$accessRoleId])
                                     ->where('customer_id', auth()->user()->customer_id)
                                     ->where('workspace_id', auth()->user()->workspace_id)
                                     ->get();

        if ($leavePackages->isEmpty()) {
            return response()->json(['message' => 'No leave packages found for this employee role'], 404);
        }

        $leavePackagesWithRemaining = $leavePackages->map(function ($package) use ($employeeId) {
            $totalAvailableLeaves = $package->leave_number;

            $approvedLeaves = LeaveRequest::where('employee_id', $employeeId)
                ->where('leave_package_id', $package->id)
                ->where('status', 1) 
                ->get();

            $totalApprovedDays = 0;
            foreach ($approvedLeaves as $leave) {
                $fromDate = \Carbon\Carbon::parse($leave->from);
                $toDate = \Carbon\Carbon::parse($leave->to);
                $totalApprovedDays += $fromDate->diffInDays($toDate) + 1;
            }

            $remainingLeave = $totalAvailableLeaves - $totalApprovedDays;

            $package->remaining_leave = $remainingLeave;

            return $package;
        });

        return response()->json([
            'message' => 'Employee leave packages with remaining leave retrieved successfully',
            'data' => $leavePackagesWithRemaining,
        ], 200);
    }

////////////////leave Request Api///////////////

      
    

    public function edit($id)
    {
        $leave_packages =  LeavePackage::orderBy('id', 'DESC')->get();
        $emp_details =  EmpCompanyDetails::with('EmpPersonalDetails')->where('del',0)->where('compeleted', 1)->orderBy('id', 'DESC')->get();
        $leaverquest = LeaveRequest::where('id',$id)->first();
        return view('LeaveRequest.edit', compact('leave_packages', 'emp_details','leaverquest'));
    }


    public function edit_leave_for_roster(Request $request)
    {

        $leave_rquest    =   LeaveRequest::where('id',$request->id)->first();

        $data['leave_rquest'] = $leave_rquest;

        return response()->json([
            'message' => 'Get Leave Successfully',
            'data'   => $data
        ],200);
    }

    public function update(Request $request)
    {
        $id = $request->id;
        
        $validator = Validator::make($request->all(), [
            'employee' => 'required',
            'leave_package' => 'required',
            'start_date' => 'required|date',
            'end_date' => 'required|date|after_or_equal:start_date',
            'reason' => 'required',
            'full_day' => 'nullable',
            'start_time' => 'nullable',
            'end_time' => 'nullable'
        ]);
 
        if ($validator->fails()) {
            return $this->handleValidationFailure($validator);
        } else {

            $leaveRequest = LeaveRequest::find($id);
            if (!$leaveRequest) {
                return $this->message('The Leave Request with the specified ID does not exist.',404); 
            }
            $userTable = $this->getUserTable();
            if ($userTable == "customer" && ($leaveRequest->workspace_id != auth()->user()->current_workspace_id || $leaveRequest->customer_id != auth()->user()->id)) 
            {
                return $this->message('You do not have access to this Leave Request', 403);
            }
            if ($userTable == "emp" && ($leaveRequest->customer_id != auth()->user()->customer_id || $leaveRequest->workspace_id != auth()->user()->workspace_id)) {
                return $this->message('You do not have access to this Leave Request', 403);
            }
            
            $auth_id =0;
            $workspace_id = 0;
            if ($userTable === "customer") {
                $auth_id = Auth::user()->id;
                $workspace_id = Auth::user()->current_workspace_id;   
            }
            if ($userTable === "emp") {
                $auth_id = auth()->user()->customer_id;
                $workspace_id = auth()->user()->workspace_id;  
            }
            $user = Auth::user();
            $validatedData = $validator->validated();
            $validatedData['applied_by'] = $auth_id;
            $leaveRequest->update([
                'employee_id' =>$validatedData['employee'],
                'leave_package_id' => $validatedData['leave_package'],
                'from' => $validatedData['start_date'],
                'to' => $validatedData['end_date'],
                'reason' => $validatedData['reason'],
                'full_day' => $validatedData['full_day'] ?? 0,
                'start_time' => $validatedData['start_time'],
                'end_time' => $validatedData['end_time'],
                'customer_id' => $auth_id,
                'workspace_id' => $workspace_id,
            ]);
           $data = LeaveRequest::where('id',$id)->get();
           return $this->success($data, 'Leave Request Saved Successfully');  
        } 
    }

    public function destroy($id)
    {
      
        $LeaveRequest = LeaveRequest::find($id);
        if (!$LeaveRequest) {
            return $this->message('The Leave Request  with the specified ID does not exist.',404); 
        }
        $userTable = $this->getUserTable();
        if ($userTable == "customer" && ($LeaveRequest->workspace_id != auth()->user()->current_workspace_id || $LeaveRequest->customer_id != auth()->user()->id)) 
        {
            return $this->message('You do not have access to this Leave Request', 403);
        }
        if ($userTable == "emp" && ($LeaveRequest->customer_id != auth()->user()->customer_id || $LeaveRequest->workspace_id != auth()->user()->workspace_id)) {
            return $this->message('You do not have access to this Leave Request', 403);
        }
        LeaveRequest::where('id',$id)->delete();
        return response()->json([
            'message' => 'Leave Request Deleted Successfully'
        ],200);
    }

    public function updateStatus(Request $request, $id)
    {
        $validator = Validator::make($request->all(), [
            'status' => 'required|integer',
        ]);
        if ($validator->fails()) {
          return $this->handleValidationFailure($validator);
        } 
       
        $leaveRequest = LeaveRequest::find($id);
        if (!$leaveRequest) {
            return $this->message('The Leave Request  with the specified ID does not exist.',404); 
        }
        $userTable = $this->getUserTable();
        if ($userTable == "customer" && ($leaveRequest->workspace_id != auth()->user()->current_workspace_id || $leaveRequest->customer_id != auth()->user()->id)) 
        {
            return $this->message('You do not have access to this Leave Request', 403);
        }
        if ($userTable == "emp" && ($leaveRequest->customer_id != auth()->user()->customer_id || $leaveRequest->workspace_id != auth()->user()->workspace_id)) {
            return $this->message('You do not have access to this Leave Request', 403);
        }
        
        // Update leave request status
        $leaveRequest->status = $request->input('status');
        $leaveRequest->approved_by = Auth::user()->id;
        $leaveRequest->save();
        
        // Get leave package and employee details
        $leave_pkg = LeavePackage::where('id', $leaveRequest->leave_package_id)->first();
        $emp_company = EmpCompanyDetails::where('id', $leaveRequest->employee_id)->first();
        $emp_detail = EmpPersonalDetails::where('emp_id', $leaveRequest->employee_id)->first();
        
        // If status is approved (1) or rejected (2), send email notification
        if ($leaveRequest->status == 1 || $leaveRequest->status == 2) {
            // Set status configuration based on approval/rejection
            $status_config = [];
            if ($leaveRequest->status == 1) {
                $status_text = 'Approved';
                $status_config = [
                    'status_text' => 'Approved',
                    'color' => '#047857', // Green
                    'background' => 'linear-gradient(135deg, #ECFDF5 0%, #D1FAE5 100%)',
                    'icon' => '✅',
                    'message' => 'Your leave request has been approved.',
                ];
            } else {
                $status_text = 'Rejected';
                $status_config = [
                    'status_text' => 'Rejected',
                    'color' => '#DC2626', // Red
                    'background' => 'linear-gradient(135deg, #FEF2F2 0%, #FEE2E2 100%)',
                    'icon' => '❌',
                    'message' => 'Your leave request has been rejected.',
                ];
            }
            
            // Format date range
            $from_date = Carbon::parse($leaveRequest->from)->format('d M, Y');
            $to_date = Carbon::parse($leaveRequest->to)->format('d M, Y');
            $date_range = $from_date . ' to ' . $to_date;
            
            // Email parameters
            $params = [
                'subject' => 'Leave Request ' . $status_text . ' | ' . env("APP_NAME"),
                'to' => $emp_company->employee_email,
                'msg' => view('Emails.leave-notification', [
                    'name' => $emp_detail->first_name . ' ' . $emp_detail->middle_name . ' ' . $emp_detail->last_name,
                    'subject' => 'Leave Request ' . $status_text,
                    'status_config' => $status_config,
                    'ticket_details' => [
                        'Leave Type' => $leave_pkg->title,
                        'Date Range' => $date_range,
                        'Status' => $status_text,
                        'Reason' => $leaveRequest->reason
                    ],
                    'message' => 'This is to inform you that your leave request has been ' . strtolower($status_text) . '.'
                ])->render()
            ];
            
            // Send email
            $emailSent = $this->SendInstantEmail($params);
            
            if ($emailSent) {
                return $this->message('Status updated successfully and notification email has been sent.', 200);
            } else {
                return $this->message('Status updated successfully but failed to send notification email.', 200);
            }
        } else {
            return $this->message('Status updated successfully', 200);
        }
    }

    public function leavereason($id)
    {
        $data = LeaveRequest::find($id);
        if (!$data) {
            return $this->message('The Leave Request with the specified ID does not exist.',404); 
        }
        $userTable = $this->getUserTable();
        if ($userTable == "customer" && ($data->workspace_id != auth()->user()->current_workspace_id || $data->customer_id != auth()->user()->id)) 
        {
            return $this->message('You do not have access to this Leave Request', 403);
        }
        if ($userTable == "emp" && ($data->customer_id != auth()->user()->customer_id || $data->workspace_id != auth()->user()->workspace_id)) {
            return $this->message('You do not have access to this Leave Request', 403);
        }
        return response()->json([
            'data'   => $data
        ],200);
    }

    public function remainingleave(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'employee_id' => 'required|integer',
            'leave_package_id' => 'required|integer',
        ]);

        if ($validator->fails()) {
            $error = $validator->errors()->first();
            return response()->json([
                'message' => $error

            ], 422);
        } else {
            $leave_request = LeaveRequest::where('leave_package_id', $request->leave_package_id)->where('employee_id',$request->employee_id)->first();

            if (!$leave_request) {
                return response()->json([
                    'message' => 'Leave request not found',
                ], 404);
            }

            $from = new \DateTime($leave_request->from);
            $to = new \DateTime($leave_request->to);

            $no_of_leaves = $from->diff($to)->days + 1;


            $_leave_pkg = LeavePackage::where('id', $request->leave_package_id)->first();

            if (!$_leave_pkg) {
                return response()->json([
                    'message' => 'Leave package not found',
                ], 404);
            }

            $total_pkg_no = $_leave_pkg->leave_number;

            $remaining_leave = $total_pkg_no - $no_of_leaves;
            return response()->json([
                'message' => 'Remaining Leave',
                'data'   => $remaining_leave
            ], 200);
        }
    }
    public function leave_view(Request $request)
    {
        $id = $request->id;
        $leave_request = LeaveRequest::where('id', $id)->first();
        $leave_pgk = LeavePackage::where('id', $leave_request->leave_package_id)->first();
        $emp_details = EmpCompanyDetails::with('EmpPersonalDetails')->where('id', $leave_request->employee_id)->first();        
        $all_pkgs = LeavePackage::whereRaw("FIND_IN_SET(?, employee_roles)", [$emp_details->accessRole->id])->get();

        $emp_role = LeaveRequest::where('leave_requests.id', $id)
        ->join('emp_company_details', 'leave_requests.employee_id', '=', 'emp_company_details.id')
        ->join('roles', 'emp_company_details.access_role', '=', 'roles.code')
        ->select('roles.*')
        ->first();

        $currentDate = Carbon::today();
        $today_on_leave_employes = DB::table('leave_requests')
        ->where('leave_requests.status',1)
        ->whereDate('leave_requests.created_at', $currentDate)
        ->join('emp_company_details', 'leave_requests.employee_id', '=', 'emp_company_details.id')
        ->join('emp_personal_details','leave_requests.employee_id', '=', 'emp_personal_details.emp_id')
        ->join('roles', 'emp_company_details.access_role', '=', 'roles.code')
        ->select('roles.title', 'emp_personal_details.*')
        ->get();

        $data = [
            'leave_request'             => $leave_request,
            'leave_package'             => $leave_pgk,
            'employee_details'          => $emp_details,
            'all_packages_for_role'     => $all_pkgs,
            'employee_role'             => $emp_role,
            'today_on_leave_employees'  => $today_on_leave_employes,
        ];
    
        return response()->json([
            'success' => true,
            'data'    => $data,
        ], 200); 
       }
  public function view($id)
{
    try {
        \Log::info('Leave Request view started', ['id' => $id]);
        
        $LeaveRequest = LeaveRequest::find($id);
        if (!$LeaveRequest) {
            \Log::warning('Leave Request not found', ['id' => $id]);
            return $this->message('The Leave Request with the specified ID does not exist.', 404); 
        }
        
        \Log::info('Leave Request found', [
            'id' => $id,
            'employee_id' => $LeaveRequest->employee_id ?? 'null',
            'workspace_id' => $LeaveRequest->workspace_id ?? 'null',
            'customer_id' => $LeaveRequest->customer_id ?? 'null'
        ]);
        
        $userTable = $this->getUserTable();
        \Log::info('User table determined', ['user_table' => $userTable]);
        
        if ($userTable == "customer" && ($LeaveRequest->workspace_id != auth()->user()->current_workspace_id || $LeaveRequest->customer_id != auth()->user()->id)) 
        {
            \Log::warning('Customer access denied', [
                'user_workspace' => auth()->user()->current_workspace_id,
                'request_workspace' => $LeaveRequest->workspace_id,
                'user_id' => auth()->user()->id,
                'request_customer_id' => $LeaveRequest->customer_id
            ]);
            return $this->message('You do not have access ', 403);
        }
        
        if ($userTable == "emp" && ($LeaveRequest->customer_id != auth()->user()->customer_id || $LeaveRequest->workspace_id != auth()->user()->workspace_id)) {
            \Log::warning('Employee access denied', [
                'user_customer_id' => auth()->user()->customer_id,
                'request_customer_id' => $LeaveRequest->customer_id,
                'user_workspace' => auth()->user()->workspace_id,
                'request_workspace' => $LeaveRequest->workspace_id
            ]);
            return $this->message('You do not have access ', 403);
        }
        
        // Get leave request details
        $leave_request = LeaveRequest::where('id', $id)->first();
        \Log::info('Leave request details retrieved', ['found' => $leave_request ? true : false]);
        
        // Get leave package with null safety
        $leave_pgk = null;
        if ($LeaveRequest->leave_package_id) {
            $leave_pgk = LeavePackage::where('id', $LeaveRequest->leave_package_id)->first();
            \Log::info('Leave package retrieved', [
                'package_id' => $LeaveRequest->leave_package_id,
                'found' => $leave_pgk ? true : false
            ]);
        } else {
            \Log::info('No leave package ID found');
        }
        
        // Get employee details with null safety
        $emp_details = null;
        if ($LeaveRequest->employee_id) {
            $emp_details = EmpCompanyDetails::with('EmpPersonalDetails')->where('id', $LeaveRequest->employee_id)->first();
            \Log::info('Employee details retrieved', [
                'employee_id' => $LeaveRequest->employee_id,
                'found' => $emp_details ? true : false,
                'has_access_role' => ($emp_details && $emp_details->accessRole) ? true : false
            ]);
        } else {
            \Log::warning('No employee ID found in leave request');
        }
        
        // Get all packages with proper null checking
        $all_pkgs = null;
        if ($emp_details && $emp_details->accessRole && $emp_details->accessRole->id) {
            try {
                $all_pkgs = LeavePackage::whereRaw("FIND_IN_SET(?, employee_roles)", [$emp_details->accessRole->id])->get();
                \Log::info('All packages retrieved', [
                    'access_role_id' => $emp_details->accessRole->id,
                    'packages_count' => $all_pkgs ? $all_pkgs->count() : 0
                ]);
            } catch (\Exception $e) {
                \Log::error('Error retrieving leave packages', [
                    'access_role_id' => $emp_details->accessRole->id ?? 'null',
                    'error' => $e->getMessage()
                ]);
                $all_pkgs = collect(); // Return empty collection instead of null
            }
        } else {
            \Log::info('Cannot retrieve packages - missing employee details or access role', [
                'emp_details_exists' => $emp_details ? true : false,
                'access_role_exists' => ($emp_details && $emp_details->accessRole) ? true : false,
                'access_role_id' => ($emp_details && $emp_details->accessRole && $emp_details->accessRole->id) ? $emp_details->accessRole->id : 'null'
            ]);
            $all_pkgs = collect(); // Return empty collection instead of null
        }

        // Get employee role with null safety
        $emp_role = null;
        try {
            $emp_role = LeaveRequest::where('leave_requests.id', $id)
                ->join('emp_company_details', 'leave_requests.employee_id', '=', 'emp_company_details.id')
                ->join('roles', 'emp_company_details.access_role', '=', 'roles.code')
                ->select('roles.*')
                ->first();
            
            \Log::info('Employee role retrieved', ['found' => $emp_role ? true : false]);
        } catch (\Exception $e) {
            \Log::error('Error retrieving employee role', [
                'error' => $e->getMessage(),
                'leave_request_id' => $id
            ]);
        }

        // Get today's employees on leave with error handling
        $today_on_leave_employes = collect();
        try {
            $currentDate = Carbon::today();
            $today_on_leave_employes = DB::table('leave_requests')
                ->where('leave_requests.status', 1)
                ->whereDate('leave_requests.created_at', $currentDate)
                ->join('emp_company_details', 'leave_requests.employee_id', '=', 'emp_company_details.id')
                ->join('emp_personal_details', 'leave_requests.employee_id', '=', 'emp_personal_details.emp_id')
                ->join('roles', 'emp_company_details.access_role', '=', 'roles.code')
                ->select('roles.title', 'emp_personal_details.*')
                ->get();
            
            \Log::info('Today on leave employees retrieved', ['count' => $today_on_leave_employes->count()]);
        } catch (\Exception $e) {
            \Log::error('Error retrieving today on leave employees', [
                'error' => $e->getMessage()
            ]);
        }

        $response_data = [
            'leave_request' => $leave_request,
            'emp_details' => $emp_details,
            'leave_pgk' => $leave_pgk,
            'all_pkgs' => $all_pkgs,
            'emp_role' => $emp_role,
            'today_on_leave_employees' => $today_on_leave_employes
        ];
        
        \Log::info('Leave Request view completed successfully', [
            'id' => $id,
            'response_data_keys' => array_keys($response_data)
        ]);
        
        return response()->json($response_data);
        
    } catch (\Exception $e) {
        \Log::error('Fatal error in Leave Request view', [
            'id' => $id,
            'error' => $e->getMessage(),
            'file' => $e->getFile(),
            'line' => $e->getLine(),
            'trace' => $e->getTraceAsString()
        ]);
        
        return $this->message('An error occurred while retrieving the leave request', 500);
    }
}
    
    public function employee_pkg(Request $request, $id)
    {
        
        $emp_details = EmpCompanyDetails::where('id', $id)->first();
        $leave_packages    = LeavePackage::whereRaw("FIND_IN_SET(?, employee_roles)", [$emp_details->accessRole->id])->get();
        

        if (!$leave_packages) {
            return response()->json(['error' => 'Leave package not found'], 400);
        }
    
  
        return response()->json([
            'all_packages' => $leave_packages
        ], 200);
    }
    public function leave_count(Request $request, $id)
    {
        $leavePackage = LeavePackage::find($id);
    
        if (!$leavePackage) {
            return response()->json(['error' => 'Leave package not found'], 404);
        }
    
        $remaining_leave = $leavePackage->leave_number;
        $leave_title = $leavePackage->title;
        return response()->json([
            'remaining_leave' => $remaining_leave,
            'title_leave' => $leave_title
        ], 200);
    }

    public function leave_pkg(Request $request)
    {
        $leave = LeavePackage::all();
    
        if (!$leave) {
            return response()->json(['error' => 'Leave packages not found'], 404);
        }
    
        return response()->json([
            'message' => 'All Leave Packages',
            'data' => $leave
        ], 200);
    }


    
    public function leave_portal(Request $request)
    {
        $userTable = $this->getUserTable();
                $auth_id =0;
                $workspace_id = 0;
                if ($userTable === "customer") {
                    $auth_id = Auth::user()->id;
                    $workspace_id = Auth::user()->current_workspace_id; 
                }
                if ($userTable === "emp") {
                    $auth_id = auth()->user()->customer_id;
                    $workspace_id = auth()->user()->workspace_id;
                }
        $current_date = date('Y-m-d');
        // $all_emp = EmpCompanyDetails::where('status',1)->where('compeleted',1)->where('del',0)->count();
        $employee_ids = RosterAssign::join('emp_company_details','roster_assigns.assign_to','emp_company_details.id')
                    ->where('emp_company_details.del',0)->where('emp_company_details.status',1)
                    ->where('roster_assigns.schedule_date', $current_date)
                    ->where('roster_assigns.customer_id', $auth_id) // Added filter
                    ->where('roster_assigns.workspace_id', $workspace_id) // Added filter
                    ->groupBy('roster_assigns.assign_to')
                    ->pluck('roster_assigns.assign_to');
        $all_emp = count($employee_ids);
        $present_employee_ids = EmployeeAttendance::where('date',$current_date)
                                    ->wherein('employee_id',$employee_ids)
                                    ->where('customer_id', $auth_id) // Added filter
                                    ->where('workspace_id', $workspace_id) // Added filter
                                    ->groupBy('employee_id')
                                    ->pluck('employee_id')
                                    ->toArray();
        $present_employees = count($present_employee_ids);

        $leave_employee_ids = LeaveRequest::wherein('employee_id',$employee_ids)
                                    ->where('from',">=",$current_date)
                                    ->where('status',1)
                                    ->where('customer_id', $auth_id) // Added filter
                                    ->where('workspace_id', $workspace_id) // Added filter
                                    ->pluck('employee_id')
                                    ->toArray();
        $leave_employees = count($leave_employee_ids);
        return response()->json([
            'all_emp' => $all_emp,
            'leave_employees' => $leave_employees,
            'present_employees' => $present_employees,
        ]);
    }

    public function upcoming_leave(Request $request)
    {
       
        $userTable = $this->getUserTable();
        $today = Carbon::today();
        $query = LeaveRequest::query();
        $query->where('status','0');
        $query->where('from', '>=', $today);
        if ($userTable === "customer") {
            $query->where('customer_id', Auth::user()->id)
                ->where('workspace_id', Auth::user()->current_workspace_id);
        }
        if ($userTable === "emp") {
            $query->where('customer_id', auth()->user()->customer_id) // Assuming customer_id links employee to projects
                ->where('workspace_id', auth()->user()->workspace_id);
        }
        if($request->filled('filter'))
        {
          $query =  $this->filter(json_decode($request->filter, true),$query);
        }
        
        $query->with('LeavePackage')->with('EmpPersonalDetails');
        $count = $query->count();
        $start = $request->input('from', 0);
        $query->offset($start)->limit(10);
        $query->orderBy('id', 'desc');
        $query_result = $query->get();
         $query_result;
        $i = 0;
        foreach ($query_result as $query_resul) {
            $from = \Carbon\Carbon::parse($query_resul->from);
            $to = \Carbon\Carbon::parse($query_resul->to);
            $days = $to->diffInDays($from) + 1;
        
           
            $query_result[$i]->total_day = $days;
            $i++;
        }
        $data = [
            'all_leaves' => $query_result,
        ];
        $array_filter =  json_decode($request->filter, true);
        return $this->WithFilter($query_result, $array_filter,'Get Upcoming Leave List Successfully');
       
    }

    
    public function approved_leave(Request $request)
    {
        $userTable = $this->getUserTable();
        $query = LeaveRequest::query();
        $query->where('status','1');
        
        if ($userTable === "customer") {
            $query->where('customer_id', Auth::user()->id)
                ->where('workspace_id', Auth::user()->current_workspace_id);
            $customer_id = Auth::user()->id;
            $workspace_id = Auth::user()->current_workspace_id;
        }
        if ($userTable === "emp") {
            $query->where('customer_id', auth()->user()->customer_id)
                ->where('workspace_id', auth()->user()->workspace_id);
            $customer_id = auth()->user()->customer_id;
            $workspace_id = auth()->user()->workspace_id;
        }
        
        if($request->filled('filter')) {
            $query = $this->filter(json_decode($request->filter, true), $query);
        }
        
        $query->with('LeavePackage')->with('EmpPersonalDetails');
    
        $count = $query->count();
        $start = $request->input('from', 0);
        $query->offset($start)->limit(10);
        $query->orderBy('id', 'desc');
        $query_result = $query->get();
        
        $i = 0;
        foreach ($query_result as $query_resul) {
            $from = \Carbon\Carbon::parse($query_resul->from);
            $to = \Carbon\Carbon::parse($query_resul->to);
            $days = $to->diffInDays($from) + 1;
            $query_result[$i]->total_day = $days;
            $i++;
        }
        
        // Get total employee count from EmpPersonalDetails
        $total_employees = EmpCompanyDetails::where('customer_id', $customer_id)
            ->where('workspace_id', $workspace_id)
            ->count();
        
        // Count unique employees with approved leave from the current result set
        $employees_with_approved_leave = $query_result->pluck('EmpPersonalDetails.emp_id')->unique()->count();
        
        // Add employee counts to the response
        $response_data = [
            'leave_requests' => $query_result,
            'total_employees' => $total_employees,
            'employees_with_approved_leave' => $employees_with_approved_leave,
            'total_leave_requests' => $count
        ];
        
        $array_filter = json_decode($request->filter, true);
        return $this->WithFilter($response_data, $array_filter, 'Get Leave Package List Successfully');
    }
    public function rejected_leave(Request $request)
    {
        $userTable = $this->getUserTable();
        $query = LeaveRequest::query();
        $query->where('status','2');
        if ($userTable === "customer") {
            $query->where('customer_id', Auth::user()->id)
                ->where('workspace_id', Auth::user()->current_workspace_id);
        }
        if ($userTable === "emp") {
            $query->where('customer_id', auth()->user()->customer_id) // Assuming customer_id links employee to projects
                ->where('workspace_id', auth()->user()->workspace_id);
        }
        if($request->filled('filter'))
        {
          $query =  $this->filter(json_decode($request->filter, true),$query);
        }
        $query->with('LeavePackage')->with('EmpPersonalDetails');

        $count = $query->count();
        $start = $request->input('from', 0);
        $query->offset($start)->limit(10);
        $query->orderBy('id', 'desc');
        $query_result = $query->get();

        $i = 0;

        foreach ($query_result as $query_resul) {
            

            $from = \Carbon\Carbon::parse($query_resul->from);
            $to = \Carbon\Carbon::parse($query_resul->to);
           
            $days = $to->diffInDays($from) + 1;
        
           
            $query_result[$i]->total_day = $days;
            $i++;
        }
        

        $data = [
            'rejected_leaves' => $query_result
        ];
        $array_filter =  json_decode($request->filter, true);
        return $this->WithFilter($query_result, $array_filter,'Get Rejected Leave List Successfully');
    }
   
    public function exipired_leave(Request $request)
    {
        $userTable = $this->getUserTable();
        $currentDate = Carbon::now();
        $query = LeaveRequest::query();
        $query->where('status','3');
        if ($userTable === "customer") {
            $query->where('customer_id', Auth::user()->id)
                ->where('workspace_id', Auth::user()->current_workspace_id);
        }
        if ($userTable === "emp") {
            $query->where('customer_id', auth()->user()->customer_id) // Assuming customer_id links employee to projects
                ->where('workspace_id', auth()->user()->workspace_id);
        }
        if($request->filled('filter'))
        {
          $query =  $this->filter(json_decode($request->filter, true),$query);
        }
        $query->with('LeavePackage')->with('EmpPersonalDetails');

        $count = $query->count();
        $start = $request->input('from', 0);
        $query->offset($start)->limit(10);
        $query->orderBy('id', 'desc');
        $query_result = $query->get();
        $i = 0;

        foreach ($query_result as $query_resul) {
            

            $from = \Carbon\Carbon::parse($query_resul->from);
            $to = \Carbon\Carbon::parse($query_resul->to);
           
            $days = $to->diffInDays($from) + 1;
        
           
            $query_result[$i]->total_day = $days;
            $i++;
        }
        $data = [
            'expired_leaves' => $query_result,
        ];
        $array_filter =  json_decode($request->filter, true);
        return $this->WithFilter($query_result, $array_filter,'Get Exipired Leave List Successfully');
    }


    public function update_expired_status()
    {
        $expiredLeaves = LeaveRequest::where('to', '<', Carbon::now())
            ->where('status', 0)
            ->where('status', '!=', 3) 
            ->get();

       foreach ($expiredLeaves as $leave) { $leave->update(['status' => 3]); }
       return 0;
    }
}
