<?php

namespace App\Http\Controllers;

use App\Models\LeaveType;
use DB;
use Auth;
use Illuminate\Support\Facades\Validator;
use Illuminate\Validation\Rule;
use Illuminate\Http\Request;

class LeaveTypeController extends Controller
{
    
    public function index(Request $request)
    {
        $userTable = $this->getUserTable();
        $query = LeaveType::query();
        if ($userTable === "customer") {
            $query->where('customer_id', Auth::user()->id)
                ->where('workspace_id', Auth::user()->current_workspace_id);
        }
        if ($userTable === "emp") {
            $query->where('customer_id', auth()->user()->customer_id) // Assuming customer_id links employee to projects
                ->where('workspace_id', auth()->user()->workspace_id);
        }
        if($request->filled('filter'))
        {
          $query =  $this->filter(json_decode($request->filter, true),$query);
        }
  
        $count = $query->count();
        $query->orderBy('id', 'desc');
        $query_result = $query->get();

        $data =  [
            'leavetypes' => $query_result,
        ];
        $array_filter =  json_decode($request->filter, true);
        $filters = [
          'first_name' => $array_filter['first_name'] ?? '', 
          'middle_name' => $array_filter['middle_name'] ?? '',
          'last_name' => $array_filter['first_name'] ?? '',
          'employee_email' => $array_filter['last_name'] ?? '',
          'compeleted' => $array_filter['first_name'] ?? '',
          'status' => $array_filter['compeleted'] ?? '',
        ];
        return $this->WithFilter($query_result, $filters,'Get Holiday List Successfully');
        // return response()->json([
        //     'message' => 'Get Holiday List Successfully',
        //     'data' => $data,
        //     'count' => $count,
        //     'filters' => $filters
        // ],200);
    }

    public function filter($filters,$query)
    {
     
        foreach ($filters as $filterName => $filterValue) {
              if ($filterValue != null ||  $filterValue !="" ) {
                  switch ($filterName) {
                  case 'company':
                    $query->whereHas('company', function ($subquery) use ($filterValue) {
                    $subquery->where('fname', 'like', '%' . $filterValue . '%');
                  });
                  break;
                  case 'title':
                      $query->where('title', 'like', '%' . $filterValue . '%');
                  break;
                  case 'status':
                      $filterValue =   $filterValue ? 1 : 0;
                      $query->where('status', 'like', '%' . $filterValue . '%');
                  break;
                  case 'latitude':
                    $query->where('latitude', 'like', '%' . $filterValue . '%');
                  break;  
                  case 'longitude':
                    $query->where('longitude', 'like', '%' . $filterValue . '%');
                  break;                     
                  case 'area_radius':
                    $query->where('area_radius', 'like', '%' . $filterValue . '%');
                  break; 
                  case 'State':
                    $query->where('State', 'like', '%' . $filterValue . '%');
                  break;                  
                  }
              }
        }

        return $query;
    } 
 
    public function create()
    {
        //
    }

   
    public function store(Request $request)
    {
        $userTable = $this->getUserTable();
        $auth_id =0;
        $workspace_id = 0;
        if ($userTable === "customer") {
            $auth_id = Auth::user()->id;
            $workspace_id = Auth::user()->current_workspace_id;   
        }
        if ($userTable === "emp") {
            $auth_id = auth()->user()->customer_id;
            $workspace_id = auth()->user()->workspace_id;  
        }
        //  $validator = Validator::make($request->all(), [
        //     'leave_type' => 'required|string|unique:leave_types,title',
        //     'description' => 'required|string',
           
        // ]);
        $validator = Validator::make($request->all(), [
            'leave_type' => [
                'required',
                'string',
                Rule::unique('leave_types', 'title')
                    ->where(function ($query) use ($auth_id, $workspace_id) {
                        return $query->where('customer_id', $auth_id)
                                     ->where('workspace_id', $workspace_id);
                    }),
            ],
            'description' => 'required|string',
        ]);
 
        if ($validator->fails()) {
            return $this->handleValidationFailure($validator);

        } else {
          
           $validatedData =  $validator->validated();
           
           
           LeaveType::insert([
            'title' =>$validatedData['leave_type'],
            'description' => $validatedData['description'],
            'customer_id' => $auth_id,
            'workspace_id' => $workspace_id,
        ]);

            $data = LeaveType::where('customer_id',$auth_id)->where('workspace_id',$workspace_id)->get();
            return $this->success($data, 'Leave Type Saved Successfully');
        }      
    }

  
    public function show(LeaveType $leaveType)
    {
        //
    }

    
    public function edit($id)
    {
        $publicholiday = LeaveType::where('id',$id)->first();
        if (!$publicholiday) {
            return $this->message('The Leave Type  with the specified ID does not exist.',404); 
        }
       
        $userTable = $this->getUserTable();
        if ($userTable == "customer" && ($publicholiday->workspace_id != auth()->user()->current_workspace_id || $publicholiday->customer_id != auth()->user()->id)) 
        {
            return $this->message('You do not have access to this Leave Type', 403);
        }
        if ($userTable == "emp" && ($publicholiday->customer_id != auth()->user()->customer_id || $publicholiday->workspace_id != auth()->user()->workspace_id)) {
            return $this->message('You do not have access to this Leave Type', 403);
        }
        return $this->success($publicholiday, ' Leave Type Successfully List');
    }

    public function updatePublicHoliday(Request $request, $id)
    {
       
        $validator = Validator::make($request->all(), [
            'leave_type' => 'required|string|unique:leave_types,title,' . $id,
            'description' => 'required|string',
        ]);
    
        if ($validator->fails()) {
            return $this->handleValidationFailure($validator);
           
        } else {
             $publicholiday = LeaveType::find($id);
             if (!$publicholiday) {
                return $this->message('The Leave Type  with the specified ID does not exist.',404); 
            }
            $userTable = $this->getUserTable();
            if ($userTable == "customer" && ($publicholiday->workspace_id != auth()->user()->current_workspace_id || $publicholiday->customer_id != auth()->user()->id)) 
            {
                return $this->message('You do not have access to this Leave Type', 403);
            }
            if ($userTable == "emp" && ($publicholiday->customer_id != auth()->user()->customer_id || $publicholiday->workspace_id != auth()->user()->workspace_id)) {
                return $this->message('You do not have access to this Leave Type', 403);
            }
            $auth_id =0;
            $workspace_id = 0;
            if ($userTable === "customer") {
                $auth_id = Auth::user()->id;
                $workspace_id = Auth::user()->current_workspace_id;   
            }
            if ($userTable === "emp") {
                $auth_id = auth()->user()->customer_id;
                $workspace_id = auth()->user()->workspace_id;  
            }
           
             $validatedData =  $validator->validated();
            $publicholiday->update([
                'title' =>$validatedData['leave_type'],
                'description' => $validatedData['description'],
                'customer_id' => $auth_id,
                'workspace_id' => $workspace_id,
            ]);
            $data = LeaveType::where('customer_id',$auth_id)->where('workspace_id',$workspace_id)->get();
            return $this->success($data, 'Leave Type Updated Successfully');
        }
    }

  
    public function destroy($id)
    {
        $publicholiday = LeaveType::where('id',$id)->first();
        if (!$publicholiday) {
            return $this->message('The Leave Type with the specified ID does not exist.',404); 
        }
        $userTable = $this->getUserTable();
        if ($userTable == "customer" && ($publicholiday->workspace_id != auth()->user()->current_workspace_id || $publicholiday->customer_id != auth()->user()->id)) 
        {
            return $this->message('You do not have access to this Leave Type', 403);
        }
        if ($userTable == "emp" && ($publicholiday->customer_id != auth()->user()->customer_id || $publicholiday->workspace_id != auth()->user()->workspace_id)) {
            return $this->message('You do not have access to this Leave Type', 403);
        }
        LeaveType::where('id',$id)->delete();
        return $this->message('Leave Type Deleted Successfully', 200);
    }
    public function leavetypedesc($id)
    {
        $data = LeaveType::findOrFail($id);
        return response()->json([
            'data'   => $data
        ],200);
    }
}
