<?php

namespace App\Http\Controllers;

use DB;
use DateTime;
use App\Models\Role;
use App\Models\Tier;
use App\Models\User;
use App\Models\EmpTeam;
use Illuminate\Http\Request;
use App\General\SettingsClass;
use App\Models\LinkManagement;
use App\Models\EmpPersonalDetails;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Validator;

class LinkManagementController extends Controller
{

    public function index(Request $request)
{
    // Determine auth_id and workspace_id
    $userTable = $this->getUserTable();
    $auth_id = 0;
    $workspace_id = 0;

    if ($userTable === "customer") {
        $auth_id = Auth::user()->id;
        $workspace_id = Auth::user()->current_workspace_id;
    }

    if ($userTable === "emp") {
        $auth_id = auth()->user()->customer_id;
        $workspace_id = auth()->user()->workspace_id;
    }

    // Initialize the query
    $query = LinkManagement::query();

    // Apply customer_id and workspace_id filters
    $query->where('customer_id', $auth_id)
          ->where('workspace_id', $workspace_id);

    // Apply additional filters if provided
    if ($request->filled('filter')) {
        $query = $this->filter(json_decode($request->filter, true), $query);
    }

    $query->with(['roleId', 'tierId']);

    // Pagination and ordering
    $count = $query->count();
    $start = $request->input('from', 0);
    $query->offset($start)->limit(20);
    $query->orderBy('id', 'desc');
    $query_result = $query->get();

    $links = $query_result;


    return $this->success($links,'link Get successfully');
}


    public function create()
    {
        $roles = Role::get();
        $tiers = Tier::get();
        $teams = EmpTeam::get();
        return view('LinkManagement.create')->with(['roles' => $roles, 'tiers' => $tiers, 'teams' => $teams]);
    }

    public function link_edit(Request $request)
    {
        $id=$request->id;
        $userTable = $this->getUserTable();
        $link_management = LinkManagement::find($id);
        if (!$link_management) {
            // Return a JSON response with a 404 status code
            return $this->message('Link record not found.', 404);
        }
        
        if (
            $userTable == "customer" && ($link_management->workspace_id != auth()->user()->current_workspace_id || $link_management->customer_id != auth()->user()->id)
        ) {
            return $this->message('You do not have access to this Link', 403);
        }

        if ($userTable == "emp" && ($link_management->customer_id != auth()->user()->customer_id || $link_management->workspace_id != auth()->user()->workspace_id)) {
            return $this->message('You do not have access to this Link', 403);
        }
        $data['link_management']=$link_management;

        return $this->success($data,'Get link successfully');
    }

    public function store(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'title' => 'required|string|max:255',
            'paid_break' => 'required|integer|min:0',
            'color_code' => 'nullable|regex:/^#[0-9a-fA-F]{6}$/',
            'role' => 'required|integer|exists:roles,id',
            'tier' => 'required|integer|exists:tiers,id',
            'start_time' => 'required',
            'end_time' => 'required',
            'description' => 'nullable',
            'working_days' => 'required|array',
            'number_of_days' => 'required|integer|max:365',
            'teams' => 'required|exists:emp_teams,id',
        ],
        [
            'working_days.required' => 'Please select one or more working days'
        ]
        );

        if ($validator->fails()) {

            $error = $validator->errors()->first();
            return $this->error($error,422);
        }
        $validatedData = $validator->validated();
        $emp_teams = implode(",",$request->teams);
        $userTable = $this->getUserTable();
        $auth_id =0;
        $workspace_id = 0;

        if ($userTable === "customer") {
            $auth_id = Auth::user()->id;
            $workspace_id = Auth::user()->current_workspace_id;  
            $authPersonalDetails = User::where('id', Auth::user()->id)->first(); 
        }
        if ($userTable === "emp") {
            $auth_id = auth()->user()->customer_id;
            $workspace_id = auth()->user()->workspace_id;
            $authPersonalDetails = EmpPersonalDetails::where('emp_id', Auth::user()->id)->first();  
        }

        $link = LinkManagement::insert([
            'title' => $validatedData['title'],
            'description' => $validatedData['description'],
            'role_id' => $validatedData['role'],
            'tier_id' => $validatedData['tier'],
            'customer_id' => $auth_id,
            'workspace_id' => $workspace_id,

            'working_days' => implode(",",$validatedData['working_days']),
            'number_of_days' => $validatedData['number_of_days'],
            'start_time' => $validatedData['start_time'],
            'end_time' => $validatedData['end_time'],
            'paid_break' => $validatedData['paid_break'],
            'color_code' => $validatedData['color_code'],
            'secret_key' => $this->generateSecretKey(),
            'created_by' => auth()->user()->id,
            'status' => '1',
            'team_ids' => $emp_teams,
        ]);

        return $this->message('Link created Successfully');

    }

    public function link_update(Request $request)
    {
        // Validation
        $validator = Validator::make($request->all(), [
            'id' => 'required',
            'title' => 'required|string|max:255',
            'paid_break' => 'required|integer|min:0',
            'color_code' => [
                'nullable',
                'regex:/^#[0-9a-fA-F]{6}$/'
            ],
            'role' => 'required|integer|exists:roles,id',
            'tier' => 'required|integer|exists:tiers,id',
            'start_time' => 'required',
            'end_time' => 'required',
            'description' => 'nullable|string',
            'working_days' => 'required|array',
            'number_of_days' => 'required|integer|max:365',
            'teams' => 'required|array|exists:emp_teams,id',
        ], [
            'working_days.required' => 'Please select one or more working days',
        ]);
    
        if ($validator->fails()) {
            $error = $validator->errors()->first();
            return $this->error($error, 422);
        }
        $id=$request->id;

        // Validate LinkManagement existence
        $link_management = LinkManagement::find($id);
        if (!$link_management) {
            return $this->message('Link not found', 404);
        }
    
        // Authorisation checks
        $userTable = $this->getUserTable();
        if (
            ($userTable == "customer" && 
            ($link_management->workspace_id != auth()->user()->current_workspace_id || $link_management->customer_id != auth()->user()->id))
            || 
            ($userTable == "emp" && 
            ($link_management->customer_id != auth()->user()->customer_id || $link_management->workspace_id != auth()->user()->workspace_id))
        ) {
            return $this->message('You do not have access to this Link', 403);
        }
    
        // Update the record
        $link_management->update([
            'title' => $validator->validated()['title'],
            'description' => $validator->validated()['description'],
            'role_id' => $validator->validated()['role'],
            'tier_id' => $validator->validated()['tier'],
            'working_days' => implode(",", $validator->validated()['working_days']),
            'number_of_days' => $validator->validated()['number_of_days'],
            'start_time' => $validator->validated()['start_time'],
            'end_time' => $validator->validated()['end_time'],
            'paid_break' => $validator->validated()['paid_break'],
            'color_code' => $validator->validated()['color_code'],
            'team_ids' => implode(",", $validator->validated()['teams']),
            'status' => '1',
        ]);
    
        return $this->message('Link Updated Successfully');
    }
    

    public function filter($filters,$query)
    {
     
        foreach ($filters as $filterName => $filterValue) {
              if ($filterValue != null ||  $filterValue !="" ) {
                  switch ($filterName) {
                  case 'title':
                      $query->where('title', 'like', '%' . $filterValue . '%');
                  break;
                  case 'tier_id':
                    $query->where('tier_id', 'like', '%' . $filterValue . '%');
                break;
                case 'role_id':
                    $query->where('role_id', 'like', '%' . $filterValue . '%');
                break;
                case 'working_days':
                    $query->where('working_days', 'like', '%' . $filterValue . '%');
                break;
                case 'number_of_days':
                    $query->where('number_of_days', 'like', '%' . $filterValue . '%');
                break;
                case 'start_time':
                    $sTime = new DateTime($filterValue);
                    $query->where('start_time', 'like', '%' .  $sTime->format('H:i:s') . '%');
                break;
                case 'end_time':
                    $eTime = new DateTime($filterValue);
                    $query->where('end_time', 'like', '%' . $eTime->format('H:i:s') . '%');
                break;
                case 'status':
                    $filterValue =   $filterValue ? 1 : 0;
                    $query->where('status', 'like', '%' . $filterValue . '%');
                break;                       
                  }
              }
        }

        return $query;
    }

    public function link_delete(Request $request)
    {
      $id=$request->id;
      $userTable = $this->getUserTable();
      $LinkManagement = LinkManagement::find($id);

        if (!$LinkManagement) {
            // Return a JSON response with a 404 status code
            return $this->message('Link record not found.', 404);
        }
        
        if ($userTable == "customer" && ($LinkManagement->workspace_id != auth()->user()->current_workspace_id || $LinkManagement->customer_id != auth()->user()->id)) {
            return $this->message('You do not have access to this Link', 403);
        }

        if ($userTable == "emp" && ($LinkManagement->customer_id != auth()->user()->customer_id || $LinkManagement->workspace_id != auth()->user()->workspace_id)) {
            return $this->message('You do not have access to this Link', 403);
        }
        $LinkManagement->delete();
        return $this->message('Link Deleted successfully');

    }

    public function updateStatus(Request $request)
    {     
        $id=$request->id;
        $userTable = $this->getUserTable();
        $link = LinkManagement::find($id);
        if ( $userTable == "customer" && ($link->workspace_id != auth()->user()->current_workspace_id || $link->customer_id != auth()->user()->id)) {
            return $this->message('You do not have access to this Link', 403);
        }
        if ($userTable == "emp" && ($link->customer_id != auth()->user()->customer_id || $link->workspace_id != auth()->user()->workspace_id)) {
            return $this->message('You do not have access to this Link', 403);
        }
        $status = $link->status == '1' ? '0' : '1';
        LinkManagement::where('id',$id)->update(['status'=> $status]);
        $data['status'] = $status;

        return $this->success($data,'Link status update Successfully');
    }

    public function generateSecretKey()
    {
        $characters = 'abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789';
        $length = 20;
    
        do {
            $secretKey = substr(str_shuffle(str_repeat($characters, ceil($length / strlen($characters)))), 0, $length);
        } while (LinkManagement::where('secret_key', $secretKey)->exists());
    
        return $secretKey;
    }
}
