<?php

namespace App\Http\Controllers;

use DB;
use Session;
use App\Models\User;
use App\Models\Sites;
use App\Models\Notification;
use Illuminate\Http\Request;
use App\Models\EmpCompanyDetails;
use App\Models\EmpPersonalDetails;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Validator;

class NotificationController extends Controller
{
    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'employee_id' => 'required|exists:emp_company_details,id',
        ]);

        if ($validator->fails()) {
            $error = $validator->errors()->first();
            return $this->error($error, 422);
        }

        $notifications = Notification::where('receiver_id', $request->employee_id)->get();

        foreach ($notifications as $notify) {
            $notify->short_description = strip_tags($notify->short_description);
        }

        return $this->success($notifications, 'Notification Get Successfully');
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function clear_all_notifications(Request $request)
    {
        Notification::where('receiver_id', Auth::user()->id)->update(['read' => '1']);

        return $this->message('Notification Clear successfully');
    }

    public function create(Request $request)
    {
        $employees = EmpCompanyDetails::where('emp_company_details.del', 0)->join('emp_personal_details as epd', 'emp_company_details.id', 'epd.emp_id')->select('epd.id', 'epd.emp_id', 'epd.first_name', 'epd.middle_name', 'epd.last_name')->get();
        $sites = sitesList();
        $teams = DB::table('emp_teams')->get();
        return view('Notification.create', compact('employees', 'sites', 'teams'));
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */

    public function store(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'title' => 'required',
            'message' => 'required',
        ]);

        if ($validator->fails()) {
            $errors = $validator->errors();
            return $this->error($errors);
        }
        $validatedData = $validator->validated();
        $userTable = $this->getUserTable();
        $auth_id = 0;
        $workspace_id = 0;
        if ($userTable === 'customer') {
            $auth_id = Auth::user()->id;
            $workspace_id = Auth::user()->current_workspace_id;
            $authPersonalDetails = User::where('id', Auth::user()->id)->first();
        }
        if ($userTable === 'emp') {
            $auth_id = auth()->user()->customer_id;
            $workspace_id = auth()->user()->workspace_id;
            $authPersonalDetails = EmpPersonalDetails::where('emp_id', Auth::user()->id)->first();
        }
        // Start transaction
        DB::beginTransaction();

        try {
            // Save notifications for employees
            if (isset($request->employee_ids) && count($request->employee_ids) > 0) {
                foreach ($request->employee_ids as $id) {
                    if ($id == 0) {
                        $employee_ids = EmpCompanyDetails::where('customer_id', $auth_id)->where('workspace_id', $workspace_id)->where('emp_company_details.del', 0)->join('emp_personal_details as epd', 'emp_company_details.id', 'epd.emp_id')->pluck('epd.emp_id');
                        dd($employee_ids);
                        foreach ($employee_ids as $emp_id) {
                            $notification = new Notification();
                            $notification->title = $request->title;
                            $notification->short_description = $request->message;
                            $notification->sender_id = Auth::user()->id;
                            $notification->receiver_id = $emp_id;
                            $notification->type = 'custom';
                            $notification->read = '0';
                            $notification->customer_id = $auth_id;
                            $notification->workspace_id = $workspace_id;
                            $notification->save();
                        }
                    } else {
                        $notification = new Notification();
                        $notification->title = $request->title;
                        $notification->short_description = $request->message;
                        $notification->sender_id = Auth::user()->id;
                        $notification->receiver_id = $id;
                        $notification->type = 'custom';
                        $notification->read = '0';
                        $notification->customer_id = $auth_id;
                        $notification->workspace_id = $workspace_id;
                        $notification->save();
                    }
                }
            }

            // Save notifications for sites
            if (isset($request->site_ids) && count($request->site_ids) > 0) {
                foreach ($request->site_ids as $id) {
                    if ($id == 0) {
                        $site_ids = Sites::where('active', 1)->pluck('id');
                        foreach ($site_ids as $site_id) {
                            $notification = new Notification();
                            $notification->title = $request->title;
                            $notification->short_description = $request->message;
                            $notification->sender_id = Auth::user()->id;
                            $notification->customer_id = $auth_id;
                            $notification->workspace_id = $workspace_id;
                            $notification->type = 'custom';
                            $notification->read = '0';
                            $notification->site_id = $site_id;
                            $notification->save();
                        }
                    } else {
                        $notification = new Notification();
                        $notification->title = $request->title;
                        $notification->short_description = $request->message;
                        $notification->sender_id = Auth::user()->id;
                        $notification->type = 'custom';
                        $notification->read = '0';
                        $notification->customer_id = $auth_id;
                        $notification->workspace_id = $workspace_id;
                        $notification->site_id = $id;
                        $notification->save();
                    }
                }
            }
            if (isset($request->notification_team_id) && count($request->notification_team_id) > 0) {
                // Use a collection to keep track of unique employee IDs and their team IDs
                $unique_emp_ids = collect();

                // Convert notification_team_id to a collection for easy manipulation
                $team_ids = collect($request->notification_team_id);

                // Loop through each team ID
                foreach ($team_ids as $id) {
                    if ($id == 0) {
                        // Get all unique team members when "0" is selected
                        // Here we get all employee IDs along with their team IDs
                        $emp_team_member_ids = DB::table('emp_teams_members')
                            ->where('del', '0')
                            ->get(['emp_id', 'team_id']); // Fetch both emp_id and team_id

                        // Add each emp_id with its corresponding team_id
                        foreach ($emp_team_member_ids as $member) {
                            $unique_emp_ids->push(['emp_id' => $member->emp_id, 'team_id' => $member->team_id]);
                        }
                    } else {
                        // Get unique team members for the selected team
                        $emp_team_member_ids = DB::table('emp_teams_members')
                            ->where('team_id', $id)
                            ->get(['emp_id']); // Only fetch emp_id, as team_id is known

                        // Add each emp_id with the corresponding team_id
                        foreach ($emp_team_member_ids as $member) {
                            $unique_emp_ids->push(['emp_id' => $member->emp_id, 'team_id' => $id]);
                        }
                    }
                }

                // Create notifications for each unique employee ID
                foreach ($unique_emp_ids->unique('emp_id') as $data) {
                    $notification = new Notification();
                    $notification->title = $request->title;
                    $notification->short_description = $request->message;
                    $notification->sender_id = Auth::user()->id;
                    $notification->receiver_id = $data['emp_id'];
                    $notification->type = 'custom';
                    $notification->read = '0';
                    $notification->customer_id = $auth_id;
                    $notification->workspace_id = $workspace_id;

                    // Store the corresponding team_id
                    $notification->team_id = $data['team_id']; // Ensure this is not null
                    $notification->save();
                }
            }

            // Commit transaction if all records were inserted successfully
            DB::commit();
            Session::put('success', 'Notification Saved Successfull.');
            return $this->message('Notifications Saved Succefully');
        } catch (\Exception $e) {
            // Rollback transaction if there is any error
            DB::rollBack();

            return $this->error($e->getMessage(), 'Faild To Save the notification');
        }
    }

    /**
     * Display the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function show($id)
    {
        //
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function edit($id)
    {
        //
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request, $id)
    {
        //
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function destroy($id)
    {
        //
    }

    function customNotification(Request $request)
    {
        $userTable = $this->getUserTable();
        $auth_id = 0;
        $workspace_id = 0;
        if ($userTable === 'customer') {
            $auth_id = Auth::user()->id;
            $workspace_id = Auth::user()->current_workspace_id;
            $authPersonalDetails = User::where('id', Auth::user()->id)->first();
        }
        if ($userTable === 'emp') {
            $auth_id = auth()->user()->customer_id;
            $workspace_id = auth()->user()->workspace_id;
            $authPersonalDetails = EmpPersonalDetails::where('emp_id', Auth::user()->id)->first();
        }
        $query = Notification::query();

        if ($request->filled('filter')) {
            $query = $this->filter(json_decode($request->filter, true), $query);
        }

        $query->with(['senderDetails', 'receiverDetails', 'sites', 'teams']);
        $start = $request->input('from', 0);
        $query->offset($start)->limit(10);
        $query->orderBy('id', 'desc');
        $query->where('customer_id', $auth_id);
        $query->where('workspace_id', $workspace_id);
        $notifications = $query->get();

        $sites = $sites = Sites::where('customer_id', $auth_id)->where('workspace_id', $workspace_id)->where('active', 1)->select('id', 'title', 'longitude', 'latitude', 'area_radius')->get();

        $data = [
            'notifications' => $notifications,
            'sites' => $sites,
        ];

        return $this->success($data, 'Tiers related data get successfully');
    }

    public function filter($filters, $query)
    {
        foreach ($filters as $filterName => $filterValue) {
            if ($filterValue != null || $filterValue != '') {
                switch ($filterName) {
                    case 'sender':
                        $filterValue = explode(' ', $filterValue);
                        if (count($filterValue) == 1) {
                            $query->whereHas('senderDetails', function ($subquery) use ($filterValue) {
                                $subquery->where('first_name', 'like', '%' . $filterValue[0] . '%');
                            });
                        }
                        if (count($filterValue) == 2) {
                            $query->whereHas('senderDetails', function ($subquery) use ($filterValue) {
                                $subquery->where('first_name', 'like', '%' . $filterValue[0] . '%');
                                $subquery->where('last_name', 'like', '%' . $filterValue[1] . '%');
                            });
                        }
                        if (count($filterValue) == 3) {
                            $query->whereHas('senderDetails', function ($subquery) use ($filterValue) {
                                $subquery->where('first_name', 'like', '%' . $filterValue[0] . '%');
                                $subquery->where('middle_name', 'like', '%' . $filterValue[1] . '%');
                                $subquery->where('last_name', 'like', '%' . $filterValue[2] . '%');
                            });
                        }
                        break;
                    case 'receiver':
                        $filterValue = explode(' ', $filterValue);
                        if (count($filterValue) == 1) {
                            $query->whereHas('receiverDetails', function ($subquery) use ($filterValue) {
                                $subquery->where('first_name', 'like', '%' . $filterValue[0] . '%');
                            });
                        }
                        if (count($filterValue) == 2) {
                            $query->whereHas('receiverDetails', function ($subquery) use ($filterValue) {
                                $subquery->where('first_name', 'like', '%' . $filterValue[0] . '%');
                                $subquery->where('last_name', 'like', '%' . $filterValue[1] . '%');
                            });
                        }
                        if (count($filterValue) == 3) {
                            $query->whereHas('receiverDetails', function ($subquery) use ($filterValue) {
                                $subquery->where('first_name', 'like', '%' . $filterValue[0] . '%');
                                $subquery->where('middle_name', 'like', '%' . $filterValue[1] . '%');
                                $subquery->where('last_name', 'like', '%' . $filterValue[2] . '%');
                            });
                        }
                        break;
                    case 'title':
                        $query->where('title', 'like', '%' . $filterValue . '%');
                        break;
                    case 'message':
                        $query->where('short_description', 'like', '%' . $filterValue . '%');
                        break;
                    case 'type':
                        $query->where('type', 'like', '%' . $filterValue . '%');
                        break;
                    case 'site':
                        $query->whereHas('sites', function ($subquery) use ($filterValue) {
                            $subquery->where('title', 'like', '%' . $filterValue[0] . '%');
                        });
                        break;
                    case 'team':
                        $query->whereHas('teams', function ($subquery) use ($filterValue) {
                            $subquery->where('title', 'like', '%' . $filterValue[0] . '%');
                        });
                        break;
                }
            }
        }
        return $query;
    }

    function notificationFiltersData()
    {
        $userTable = $this->getUserTable();
        $auth_id = 0;
        $workspace_id = 0;
        if ($userTable === 'customer') {
            $auth_id = Auth::user()->id;
            $workspace_id = Auth::user()->current_workspace_id;
            $authPersonalDetails = User::where('id', Auth::user()->id)->first();
        }
        if ($userTable === 'emp') {
            $auth_id = auth()->user()->customer_id;
            $workspace_id = auth()->user()->workspace_id;
            $authPersonalDetails = EmpPersonalDetails::where('emp_id', Auth::user()->id)->first();
        }
        $sites = Sites::where('customer_id', $auth_id)->where('workspace_id', $workspace_id)->where('active', 1)->select('id', 'title', 'longitude', 'latitude', 'area_radius')->get();
        $employees = EmpCompanyDetails::where('customer_id', $auth_id)->where('workspace_id', $workspace_id)->where('emp_company_details.del', 0)->join('emp_personal_details as epd', 'emp_company_details.id', 'epd.emp_id')->select('epd.id', 'epd.emp_id', 'epd.first_name', 'epd.middle_name', 'epd.last_name')->get();
        $data['sites'] = $sites;
        $employees = $employees;
        return $this->success($employees, 'Get Notification Successfully');
    }

    /**
     * Get all notifications for the authenticated employee
     *
     * @return \Illuminate\Http\JsonResponse
     */
public function getEmployeeNotifications()
{
    $userId = Auth::user()->id;

    $notifications = Notification::with(['senderDetails', 'receiverDetails', 'sites', 'teams'])
        ->where('receiver_id', $userId)
        ->orderBy('created_at', 'desc')
        ->get();

    $notifications->transform(function ($notification) {
        if (isset($notification->short_description)) {
            $notification->short_description = strip_tags($notification->short_description);
        }
        return $notification;
    });

    return $this->success($notifications, 'Notifications retrieved successfully');
}


    /**
     * Mark a specific notification as read
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\JsonResponse
     */
    public function markNotificationAsRead(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'notification_id' => 'required|exists:all_notifications,id',
        ]);
        if ($validator->fails()) {
            $error = $validator->errors()->first();
            return $this->error($error, 422);
        }
        $userId = Auth::user()->id;
        $notification = Notification::where('id', $request->notification_id)
            ->where('receiver_id', $userId)
            ->first();
            
        if (!$notification) {
            return $this->error('Notification not found or does not belong to you', 404);
        }
        if($notification->read == 1){
            return $this->error('Notification already read', 400);
        }
        $notification->read = '1';
        $notification->save();
        return $this->message('Notification marked as read successfully');
    }

    /**
     * Mark all notifications as read for the authenticated employee
     *
     * @return \Illuminate\Http\JsonResponse
     */
    public function markAllNotificationsAsRead()
    {
        $userId = Auth::user()->id;
        $count = Notification::where('receiver_id', $userId)
            ->where('read', '0')
            ->update(['read' => '1']);
        return $this->message("$count notifications marked as read successfully");
    }
}
