<?php

namespace App\Http\Controllers;

use DB;
use App\Models\User;
use Illuminate\Http\Request;
use App\Models\RequiredDocument;
use App\Models\EmpPersonalDetails;
use Illuminate\Support\Facades\Auth;
use App\Models\RequiredDocumentField;
use Illuminate\Support\Facades\Validator;

class RequiredDocumentController extends Controller
{
    public function index()
{
    $userTable = $this->getUserTable();
    $query = RequiredDocument::query()->with('requiredDocumentField'); // Eager load the relationship

    if ($userTable === "customer") {
        $query->where('del', '0')
              ->where('customer_id', auth()->id())
              ->where('workspace_id', auth()->user()->current_workspace_id);
    }

    if ($userTable === "emp") {
        $query->where('del', '0')
              ->where('customer_id', auth()->user()->customer_id) // Assuming customer_id links employee to projects
              ->where('workspace_id', auth()->user()->workspace_id);
    }
    
    $required_documents = $query->get();

    return $this->success($required_documents, 'Required Documents Get Successfully');
}

    public function store(Request $request)
    {
     
        $user = auth()->user();

        $validator = Validator::make($request->all(), [
            'title'      => 'required',
            'for_who'    => 'required',
            'field_name' => 'required|array',
            'field_type' => 'required|array',
            'field_required' => 'required|array',
            'priority' => 'required|array',
        ]);

        if ($validator->fails()) {

            $error = $validator->errors()->first();

            return $this->error($error,422);

        } else {

            $validatedData =  $validator->validated();

            $RequiredDocumentData = [
                'title'   =>  $validatedData['title'],
                'for_who' =>  $validatedData['for_who']
            ];

            $userTable = $this->getUserTable();
            $auth_id =0;
            $workspace_id = 0;
            if ($userTable === "customer") {
                $auth_id = Auth::user()->id;
                $workspace_id = Auth::user()->current_workspace_id;  
                $authPersonalDetails = User::where('id', Auth::user()->id)->first();  
            }
            if ($userTable === "emp") {
                $auth_id = auth()->user()->customer_id;
                $workspace_id = auth()->user()->workspace_id;
                $authPersonalDetails = EmpPersonalDetails::where('emp_id', Auth::user()->id)->first();  
            }
            $RequiredDocumentData['customer_id'] = $auth_id;
            $RequiredDocumentData['workspace_id'] = $workspace_id;

           $insertGetId =  RequiredDocument::insertGetId($RequiredDocumentData);

           $RequiredDocumentDataArr = [];

            $i = 0;
           foreach ($validatedData['field_name'] as $key => $value)
           { 
                $RequiredDocumentDataArr[$i] = [
                    'field_type' => $validatedData['field_type'][$key],
                    'field_name' => $validatedData['field_name'][$key],
                    'doc_id'     => $insertGetId,
                    'field_required' => $validatedData['field_required'][$key],
                    'priority' => $validatedData['priority'][$key]
                ];

                $i++;
           }

           RequiredDocumentField::insert($RequiredDocumentDataArr);

           $required_documents =  RequiredDocument::where('customer_id', $auth_id)
                                                    ->where('workspace_id', $workspace_id)
                                                    ->where('del','0')->get();

           $data['required_documents'] = $required_documents;
           return $this->success($data, 'Required Documents Store Successfully');
        }
    }

    public function document_edit(Request $request)
    {
        $id=$request->id;
        $userTable = $this->getUserTable();
        $required_documents = RequiredDocument::find($id);
        if (
            $userTable == "customer" && ($required_documents->workspace_id != auth()->user()->current_workspace_id || $required_documents->customer_id != auth()->user()->id)
        ) {
            return $this->message('You do not have access to this Employee Requird Document', 403);
        }

        if ($userTable == "emp" && ($required_documents->customer_id != auth()->user()->customer_id || $required_documents->workspace_id != auth()->user()->workspace_id)) {
            return $this->message('You do not have access to this Employee Requird Document', 403);
        }
        return $this->success($required_documents, 'Get Employee Requird Document Details Successfully');
    }

    public function update(Request $request)
    {
        $user = auth()->user();
        $validator = Validator::make($request->all(), [
            'title'      => 'required',
            'for_who'    => 'required',
            'document_id'    => 'required',
            'field_name' => 'required|array',
            'field_type' => 'required|array',
            'field_required' => 'required|array',
            'status' => 'required|array',
            'has_data' => 'required|array',
            'id_list' => 'required|array',
            'priority' => 'required|array',
            ]);
        
        if ($validator->fails()) {
            
            $errors = $validator->errors()->first();
            return $this->error($errors,422);

        } else {
          
            $validatedData =  $validator->validated();
            $document = RequiredDocument::where('id', $validatedData['document_id'])->first();
            $userTable = $this->getUserTable();
        
            if ($userTable == "customer" && ($document->workspace_id != auth()->user()->current_workspace_id || $document->customer_id != auth()->user()->id)
            ) {
                return $this->message('You do not have access to this document', 403);
            }

            if ($userTable == "emp" && ($document->customer_id != auth()->user()->customer_id || $document->workspace_id != auth()->user()->workspace_id)) {
                return $this->message('You do not have access to this document', 403);
            }
            if (!$document) {
                return $this->notFound('document Not Found', 404);
            }

            RequiredDocument::where('id', $validatedData['document_id'])
            ->update([
                'title' => $validatedData['title'],
                'for_who' => $validatedData['for_who']
            ]);
            $RequiredDocumentDataArr = [];

            $i = 0;
            foreach ($validatedData['field_name'] as $key => $value)
            {
                if($validatedData['id_list'][$key] == 0)
                {
                    $RequiredDocumentDataArr= [
                        'field_type' => $validatedData['field_type'][$key],
                        'field_name' => $validatedData['field_name'][$key],
                        'doc_id'     => $validatedData['document_id'],
                        'field_required' => $validatedData['field_required'][$key],
                        'status' => $validatedData['status'][$key],
                        'priority' => $validatedData['priority'][$key]
                    ];
                    RequiredDocumentField::insert($RequiredDocumentDataArr);

                }
                else
                {
                    if($validatedData['has_data'][$key] == 0)
                    {
                        $RequiredDocumentDataArr= [
                            'field_type' => $validatedData['field_type'][$key],
                            'field_name' => $validatedData['field_name'][$key],
                            'doc_id'     => $validatedData['document_id'],
                            'field_required' => $validatedData['field_required'][$key],
                            'status' => $validatedData['status'][$key],
                            'priority' => $validatedData['priority'][$key]
                        ];
                        RequiredDocumentField::where(['id' => $validatedData['id_list'][$key]])->update($RequiredDocumentDataArr);
                    }
                    else
                    {
                        $RequiredDocumentDataArr= [
                            'field_name' => $validatedData['field_name'][$key],
                            'doc_id'     => $validatedData['document_id'],
                            'field_required' => $validatedData['field_required'][$key],
                            'status' => $validatedData['status'][$key],
                            'priority' => $validatedData['priority'][$key]
                        ];
                        RequiredDocumentField::where(['id' => $validatedData['id_list'][$key]])->update($RequiredDocumentDataArr);

                    }
                }
                $i++;
            }
 
            $userTable = $this->getUserTable();
            $auth_id =0;
            $workspace_id = 0;
            if ($userTable === "customer") {
                $auth_id = Auth::user()->id;
                $workspace_id = Auth::user()->current_workspace_id;  
                $authPersonalDetails = User::where('id', Auth::user()->id)->first(); 
            }
            if ($userTable === "emp") {
                $auth_id = auth()->user()->customer_id;
                $workspace_id = auth()->user()->workspace_id;
                $authPersonalDetails = EmpPersonalDetails::where('emp_id', Auth::user()->id)->first();  
            }
            

            $required_documents =  RequiredDocument::where('customer_id', $auth_id)
            ->where('workspace_id', $workspace_id)
            ->where('del','0')->get();

            $data['required_documents'] = $required_documents;
           
            return $this->success($data, 'Required Documents Updated Successfully');   
        }        

    }

    public function document_delete(Request $request)
    {
        $id=$request->id;
        $userTable = $this->getUserTable();
        $document = RequiredDocument::find($id);
    
        if ($userTable == "customer" && ($document->workspace_id != auth()->user()->current_workspace_id || $document->customer_id != auth()->user()->id)) {
            return $this->message('You do not have access to this Document', 403);
        }

        if ($userTable == "emp" && ($document->customer_id != auth()->user()->customer_id || $document->workspace_id != auth()->user()->workspace_id)) {
            return $this->message('You do not have access to this Document', 403);
          
        }
        $document->del='1';
        $document->save();
         
        $user = auth()->user();
            if($userTable == "customer"){
                    $data['document'] = RequiredDocument::where('customer_id', $user->id)
                    ->where('workspace_id', $user->current_workspace_id)
                    ->where('del', '0')
                    ->get();
                }
                elseif($userTable == "emp"){
                    $data['document'] = RequiredDocument::where('customer_id',auth()->user()->customer_id)
                    ->where('workspace_id', auth()->user()->workspace_id)
                    ->where('del', '0')
                    ->get();
                }

        return $this->success($data, 'Required Documents Deleted Successfully');   
    }


    public function delete_field(Request $request)
    {
        RequiredDocumentField::where('id', $request->id)->delete();
       
        return $this->message('Field Deleted Successfully');   
    }

  
}
