<?php

namespace App\Http\Controllers;

use Illuminate\Http\Request;
use App\Models\Sites;
use App\Models\Company;
use App\Models\Project;
use App\Models\EmpCompanyDetails;
use App\Models\SiteHistory;
use App\Models\EmpPersonalDetails;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Validator;


class SiteController extends Controller
{
    public function __construct()
    {
        $this->middleware('Permissions:Sites Maintain')->only(['create', 'edit']);
    }
    public function index(Request $request)
    {
        $userTable = $this->getUserTable();
        $query = Sites::query();
        // $query->where('del','0');
        // $query->where('customer_id',Auth::user()->id);
        // $query->where('workspace_id',Auth::user()->current_workspace_id);
        if ($userTable === "customer") {
            $query->where('del', '0')->where('customer_id', Auth::user()->id)
                ->where('workspace_id', Auth::user()->current_workspace_id);
        }
        if ($userTable === "emp") {
            $query->where('del', '0')->where('customer_id', auth()->user()->customer_id) // Assuming customer_id links employee to projects
                ->where('workspace_id', auth()->user()->workspace_id);
        }
        if ($request->filled('filter')) {
            $query =  $this->filter(json_decode($request->filter, true), $query);
        }
        $query->with('supervisor')->with('Company')->with('project');
        $count = $query->count();
        $start = $request->input('from', 0);
        $query->orderBy('id', 'desc');
        $query_result = $query->get();

        $array_filter =  json_decode($request->filter, true);
        $filters = [
            'first_name' => $array_filter['first_name'] ?? '',
            'middle_name' => $array_filter['middle_name'] ?? '',
            'last_name' => $array_filter['first_name'] ?? '',
            'employee_email' => $array_filter['last_name'] ?? '',
            'compeleted' => $array_filter['first_name'] ?? '',
            'status' => $array_filter['compeleted'] ?? '',
            'access_role' => $array_filter['access_role'] ?? '',
        ];
        return $this->WithFilter($query_result, $filters, 'Data get Successfully');
    }
    public function filter($filters, $query)
    {
        foreach ($filters as $filterName => $filterValue) {
            if ($filterValue != null ||  $filterValue != "") {
                switch ($filterName) {
                    case 'company':
                        $query->whereHas('company', function ($subquery) use ($filterValue) {
                            $subquery->where('fname', 'like', '%' . $filterValue . '%');
                        });
                        break;
                    case 'first_name':
                        $query->whereHas('emppersonaldetails', function ($subquery) use ($filterValue) {
                            $subquery->where('first_name', 'like', '%' . $filterValue . '%');
                        });
                        break;
                    case 'title':
                        $query->where('title', 'like', '%' . $filterValue . '%');
                        break;
                    case 'area_radius':
                        $query->where('area_radius', 'like', '%' . $filterValue . '%');
                        break;
                    case 'active':
                        $query->where('active', 'like', '%' . $filterValue . '%');
                        break;
                }
            }
        }
        return $query;
    }

    public function create(Request $request)
    {
        $get_employes = EmpCompanyDetails::with('EmpPersonalDetails')->where('del', 0)->where('compeleted', 1)->where('status', 1)->where('access_role', 'spv')->get();
        $get_companies = Company::where('del', 0)->where('active', 1)->get();
        return view('Sites.create', compact('get_employes', 'get_companies'));
    }

    public function ssoList()
    {
        $userTable = $this->getUserTable();
        // Base query for employee list
        $query = EmpCompanyDetails::where('compeleted', '1')
            ->where('approved', '1')
            ->where('status', '1')
            ->where('del', '0')
            ->with([
                'empPersonalDetails' => function ($query) {
                    $query->select('emp_id', 'first_name', 'middle_name', 'last_name', 'customer_id', 'workspace_id');
                },
                'accessRole' => function ($query) {
                    $query->select('id', 'title', 'code');
                },
                'empTier' => function ($query) {
                    $query->select('id', 'title');
                },
                'accessTier' => function ($query) {
                    $query->select('id', 'title', 'tier_key');
                },
            ])
            ->select('id', 'access_role', 'tier_id');
        // Add Customer and Employee Access Checks
        if ($userTable === 'customer') {
            $query->whereHas('empPersonalDetails', function ($q) {
                $q->where('customer_id', auth()->id())
                    ->where('workspace_id', auth()->user()->current_workspace_id);
            });
        }
        if ($userTable === 'emp') {
            $query->whereHas('empPersonalDetails', function ($q) {
                $q->where('customer_id', auth()->user()->customer_id)
                    ->where('workspace_id', auth()->user()->workspace_id);
            });
        }
        // Execute query
        $empList = $query->get();
        // Filter only SSO roles
        $data = $empList->where('access_role', 'SSO')->values()->toArray();
        // Return response
        return $this->success($data, 'SSO List fetched successfully');
    }

    public function store(Request $request)
    {
       
        $validator = Validator::make($request->all(), [
            'title' => 'required',
            'state' => 'required',
            'external_id' => 'nullable',
            'supervisor' => 'required',
            'company_client' => 'required',
            'description' => 'nullable',
            'address' => 'required',
            'latitude' => 'required',
            'longitude' => 'required',
            'area_radius' => 'required',
            'project' => 'nullable',
            'forman_id' => 'required',
            'sso_id'=>'required'
        ]);
        if ($validator->fails()) {
            return $this->handleValidationFailure($validator);
        }
        try {
            $userTable = $this->getUserTable();
            $auth_id = 0;
            $workspace_id = 0;
            if ($userTable === "customer") {
                $auth_id = Auth::user()->id;
                $workspace_id = Auth::user()->current_workspace_id;
            }
            if ($userTable === "emp") {
                $auth_id = auth()->user()->customer_id;
                $workspace_id = auth()->user()->workspace_id;
            }
            $validatedData = $validator->validated();
            $Data = Sites::create([
                'title' => $validatedData['title'],
                'site_state' => $validatedData['state'],
                'external_id' => $validatedData['external_id'],
                'supervisor_id' => $validatedData['supervisor'],
                'company_client' => $validatedData['company_client'],
                'forman_id' => $validatedData['forman_id'],
                'sso_id' => $validatedData['sso_id'],   
                'description' => $validatedData['description'],
                'street_address' => $validatedData['address'],
                'latitude' => $validatedData['latitude'],
                'longitude' => $validatedData['longitude'],
                'area_radius' => $validatedData['area_radius'],
                'project_id' => $validatedData['project'],
                'customer_id' => $auth_id,
                'workspace_id' => $workspace_id,
                'created_by' => auth()->user()->id
            ]);
            return $this->success($Data, 'Site Created Successfully');
        } catch (QueryException $exception) {
            $errorMessage = $exception->getMessage();
            preg_match("/'([^']+)'/", $errorMessage, $matches);
            $errorField = count($matches) > 1 ? $matches[1] : 'unknown';
            return $this->message('The ' . $errorField . ' field is required.', 500);
        }
    }


    public function siteEdit($id)
    {
        $site = Sites::with(['emppersonaldetails', 'company','siteSafetyOfficer'])->where('id', $id)->where('del', '0')->first();
        if (!$site) {
            return $this->message('Site not Found.', 404);
        }
        $userTable = $this->getUserTable();
        if (
            $userTable == "customer" && ($site->workspace_id != auth()->user()->current_workspace_id || $site->customer_id != auth()->user()->id)
        ) {
            return $this->message('You do not have access to this Site', 403);
        }

        if ($userTable == "emp" && ($site->customer_id != auth()->user()->customer_id || $site->workspace_id != auth()->user()->workspace_id)) {
            return $this->message('You do not have access to this Site', 403);
        }
        return $this->success($site, 'Site data retrieved successfully');
    }

    public function siteUpdate(Request $request, $id)
    {

        $validator = Validator::make($request->all(), [
            'title' => 'required',
            'state' => 'required',
            'external_id' => 'nullable',
            'supervisor' => 'required',
            'company_client' => 'required',
            'description' => 'nullable',
            'address' => 'required',
            'latitude' => 'required',
            'longitude' => 'required',
            'area_radius' => 'required',
            'project' => 'nullable',
            'forman_id' => 'required',
            'sso_id'=>'required'
        ]);

        if ($validator->fails()) {
            return $this->handleValidationFailure($validator);
        } else {

            $validatedData = $validator->validated();
            $site = Sites::where('id', $id)->where('del', '0')->first();
            if (!$site) {
                return $this->message('Site not found or already deleted', 404);
            }
            $updateData = [
                'title' => $validatedData['title'],
                'site_state' => $validatedData['state'],
                'external_id' => $validatedData['external_id'],
                'supervisor_id' => $validatedData['supervisor'],
                'company_client' => $validatedData['company_client'],
                'description' => $validatedData['description'],
                'street_address' => $validatedData['address'],
                'latitude' => $validatedData['latitude'],
                'longitude' => $validatedData['longitude'],
                'area_radius' => $validatedData['area_radius'],
                'forman_id' => $validatedData['forman_id'],
                'sso_id' => $validatedData['sso_id'],
                // 'project_id' => $validatedData['project']
            ];
            $userTable = $this->getUserTable();
            if (
                $userTable == "customer" && ($site->workspace_id != auth()->user()->current_workspace_id || $site->customer_id != auth()->user()->id)
            ) {
                return $this->message('You do not have access to this Site', 403);
            }
            if ($userTable == "emp" && ($site->customer_id != auth()->user()->customer_id || $site->workspace_id != auth()->user()->workspace_id)) {
                return $this->message('You do not have access to this Site', 403);
            }
            $site->update($updateData);
            $getuserdetail = DB::table('users')->where('id', auth()->user()->id)->first();
            $description = "<a href='" . url('/user-profile/' . Auth::id()) . "' style='text-transform: capitalize;' role='button' class='primary text-decoration-none'>" . e($getuserdetail->name ?? '') . "</a> changed the site detail: '" . e($site->title) . "'.";
            $history = [
                'site_id' => $site->id,
                'description' => $description,
                'updated_by' => auth()->user()->id,
            ];
            SiteHistory::create($history);
            return $this->success($updateData, 'Site Updated Successfully');
        }
    }

    public function updateStatus($id)
    {
        $site = Sites::where('id', $id)->where('del', '0')->first();
        if (!$site) {
            return $this->message('Site not found', 404);
        }
        $userTable = $this->getUserTable();
        if (
            $userTable == "customer" && ($site->workspace_id != auth()->user()->current_workspace_id || $site->customer_id != auth()->user()->id)
        ) {
            return $this->message('You do not have access to this Site', 403);
        }

        if ($userTable == "emp" && ($site->customer_id != auth()->user()->customer_id || $site->workspace_id != auth()->user()->workspace_id)) {
            return $this->message('You do not have access to this Site', 403);
        }
        // Toggle the active status
        $site->active = $site->active == 1 ? 0 : 1;
        $site->save();
        return $this->success($site, 'Status Updated Successfully');
    }

    public function destroy($id)
    {
        $site = Sites::where('id', $id)->where('del', '0')->first();
        if (!$site) {
            return $this->message('Site not found or already deleted', 404);
        }
        $userTable = $this->getUserTable();
        if (
            $userTable == "customer" && ($site->workspace_id != auth()->user()->current_workspace_id || $site->customer_id != auth()->user()->id)
        ) {
            return $this->message('You do not have access to this Site', 403);
        }

        if ($userTable == "emp" && ($site->customer_id != auth()->user()->customer_id || $site->workspace_id != auth()->user()->workspace_id)) {
            return $this->message('You do not have access to this Site', 403);
        }
        Sites::where('id', $id)->update(['del' => '1']);
        $getuserdetail = DB::table('users')->where('id', auth()->user()->id)->first();
        if (!$getuserdetail) {
            return $this->message('User details not found');
        }
        $description = "<a href='" . url('/') . "/user-profile/" . Auth::user()->id . "' style='text-transform: capitalize;' role='button' class='primary text-decoration-none'> {$getuserdetail->name} </a>  Deleted that site: '{$site->title}'.";

        $history = [
            'site_id' => $site->id,
            'description' => $description,
            'updated_by' => auth()->user()->id,
        ];
        SiteHistory::create($history);
        return $this->message('User details not found', 200);
    }


    public function view($id)
    {
        $site = Sites::where('id', $id)->where('del', '0')->first();
        if (!$site) {
            return $this->message('Site not found or deleted', 404);
        }
        $userTable = $this->getUserTable();
        if (
            $userTable == "customer" && ($site->workspace_id != auth()->user()->current_workspace_id || $site->customer_id != auth()->user()->id)
        ) {
            return $this->message('You do not have access to this Site', 403);
        }

        if ($userTable == "emp" && ($site->customer_id != auth()->user()->customer_id || $site->workspace_id != auth()->user()->workspace_id)) {
            return $this->message('You do not have access to this Site', 403);
        }
        // $site = Sites::where('id', $id)->first();
        $sitehistories = SiteHistory::where('site_id', $id)->get();
        $company = Company::where('id', $site->company_client)->first();
        $project = Project::where('id', $site->project_id)->first();
        $emppersonal = EmpPersonalDetails::where('id', $site->created_by)->first();
        $empsuper = EmpPersonalDetails::where('emp_id', $site->supervisor_id)->first();
        return response()->json([
            'site' => $site,
            'company' => $company,
            'emppersonal' => $emppersonal,
            'empsuper' => $empsuper,
            'sitehistories' => $sitehistories
        ]);
        // return view('Sites.view', compact('site','company','emppersonal','empsuper','sitehistories'));
    }
}
