<?php

namespace App\Http\Controllers;

use PDF;
use Exception;
use App\Models\Swms;
use App\Models\Sites;
use App\Models\Company;
use App\Models\SwmsMeta;
use App\Models\SwmsImage;
use App\Models\HazardRisk;
use App\Models\SwmsAwnswers;
use Illuminate\Http\Request;
use App\Models\SwmsQuestions;
use App\Models\SwmsSignature;
use App\Models\SwmsApprovedBy;
use App\Models\SwmsTaskDetail;
use App\Models\swmsDevelopedBy;
use App\Models\SwmsGeneralNote;
use App\Models\EmpCompanyDetails;
use App\Models\GeneratedPdfReport;
use App\Models\SwmsProtectionTool;
use Illuminate\Support\Facades\DB;
use App\Models\SwmsHazardRiskPivot;
use App\Models\EmpPersonalDetails;
use Illuminate\Pagination\Paginator;
use App\Models\SwmsReviewResponsible;
use App\Models\SwmsGeneralNoteAnserws;
use App\Models\SwmsGeneralNoteAnswers;
use App\Models\SwmsProtectionToolPivot;
use App\Models\SwmsComplianceResponsible;
use App\Models\SwmsMonitoringResponsible;
use App\Models\SwmsSafetyPlanningAnswers;
use Illuminate\Support\Facades\Validator;
use App\Models\SwmsSafetyPlanningQuestion;
use Illuminate\Pagination\LengthAwarePaginator;

class SwmsStepsController extends Controller
{

    public function swmsReportListData(Request $request)
    {
        try {
            $auth_id = auth()->user()->id;
            if ($auth_id != 1) {
                $sites = Sites::where('sso_id', $auth_id)->pluck('id');
                $query = Swms::query();
                $query->orderBy('id', 'DESC');
                $query->where('del', '0');
                $query->with('authorizedBy');
                $query->whereIn('site_id', $sites);
                $query_result = $query->get();

                if ($request->filled('filter')) {
                    $query =  $this->filter(json_decode($request->filter, true), $query);
                }

                $count = $query->count();
                $start = $request->input('from', 0);
                $query->offset($start)->limit(20);
                $query->orderBy('id', 'desc');
                $query_result = $query->get();

                $i = 0;
                $transformed_data = $query_result->map(function ($swms) {
                    $swmsData = $swms->toArray();

                    $swmsData['authorizedBy'] = [];

                    if (!empty($swms->site_manager_or_forman)) {
                        $formanIds = explode(',', $swms->site_manager_or_forman);

                        $employeeDetails = EmpPersonalDetails::whereIn('emp_id', $formanIds)->get();

                        $swmsData['authorizedBy'] = $employeeDetails;
                    }

                    return $swmsData;
                });
                $data = [
                    'swms' => $transformed_data,
                ];

                return response()->json([
                    'message' => 'Get SWMS Reports List Successfully',
                    'data' => $data,
                    'count' => $count,
                    'filters' => []
                ], 200);
            } else {
                $query = Swms::query();
                $query->orderBy('id', 'DESC');
                $query->where('del', '0');
                $query->with('authorizedBy');
                $query_result = $query->get();

                if ($request->filled('filter')) {
                    $query =  $this->filter(json_decode($request->filter, true), $query);
                }

                $count = $query->count();
                $start = $request->input('from', 0);
                $query->offset($start)->limit(20);
                $query->orderBy('id', 'desc');
                $query_result = $query->get();

                $i = 0;
                $transformed_data = $query_result->map(function ($swms) {
                    $swmsData = $swms->toArray();

                    $swmsData['authorizedBy'] = [];

                    if (!empty($swms->site_manager_or_forman)) {
                        $formanIds = explode(',', $swms->site_manager_or_forman);

                        $employeeDetails = EmpPersonalDetails::whereIn('emp_id', $formanIds)->get();

                        $swmsData['authorizedBy'] = $employeeDetails;
                    }

                    return $swmsData;
                });
                $data = [
                    'swms' => $transformed_data,
                ];

                return response()->json([
                    'message' => 'Get SWMS Reports List Successfully',
                    'data' => $data,
                    'count' => $count,
                    'filters' => []
                ], 200);
            }
        } catch (Exception $e) {
            $shortMessage = substr($e->getMessage(), 0, 200) . (strlen($e->getMessage()) > 200 ? '...' : '');
            $log = [
                'employee_id' => auth()->user()->id,
                'message' => 'SWMS Repport List Failed to get:' . $shortMessage,
                'report_id' =>  0,
                'report_type' => 'swms_report'
            ];
            storeReportsLogs($log);
            return response()->json([
                'message' => 'An error occurred : ' . $e->getMessage(),
            ], 500);
        }
    }

    public function filter($filters, $query)
    {

        foreach ($filters as $filterName => $filterValue) {
            if ($filterValue != null ||  $filterValue != "") {
                switch ($filterName) {
                    case 'authorised_by':
                        $filterValue  = explode(" ", $filterValue);

                        if (count($filterValue) == 1) {
                            $query->whereHas('authorizedBy', function ($subquery) use ($filterValue) {
                                $subquery->where('first_name', 'like', '%' . $filterValue[0] . '%');
                            });
                        }

                        if (count($filterValue) == 2) {
                            $query->whereHas('authorizedBy', function ($subquery) use ($filterValue) {
                                $subquery->where('first_name', 'like', '%' . $filterValue[0] . '%');
                                $subquery->where('last_name', 'like', '%' . $filterValue[1] . '%');
                            });
                        }

                        if (count($filterValue) == 3) {

                            $query->whereHas('authorizedBy', function ($subquery) use ($filterValue) {
                                $subquery->where('first_name', 'like', '%' . $filterValue[0] . '%');
                                $subquery->where('middle_name', 'like', '%' . $filterValue[1] . '%');
                                $subquery->where('last_name', 'like', '%' . $filterValue[2] . '%');
                            });
                        }

                        break;
                    case 'title':
                        $query->where('title', 'like', '%' . $filterValue . '%');
                        break;
                    case 'swms_number':
                        $query->where('swms_number', 'like', '%' . $filterValue . '%');
                        break;
                    case 'swms_version_number':
                        $query->where('version_number', 'like', '%' . $filterValue . '%');
                        break;
                    case 'start_date':
                        $query->where('start_date', 'like', '%' . $filterValue . '%');
                        break;
                    case 'process':
                        $query->where('process', 'like', '%' . $filterValue . '%');
                        break;
                    case 'publish':
                        $query->where('published', 'like', '%' . $filterValue . '%');
                        break;
                }
            }
        }

        return $query;
    }
    public function swmsReportRecordDelete($id)
    {
        try {
            if (!$id) {
                return response()->json([
                    'message' => 'Report ID is required',
                ], 422);
            }
            Swms::where('id', $id)->update(['del' => '1']);
            $log = [
                'employee_id' => auth()->user()->id,
                'message' => 'SWMS Repport ID ' . $id . ' Deleted successfully',
                'report_id' =>  0,
                'report_type' => 'swms_report',
                'error_type' => 'Validation error',
            ];
            storeReportsLogs($log);
            return response()->json([
                'message' => 'Deleted successfully',
            ], 200);
        } catch (Exception $e) {
            $shortMessage = substr($e->getMessage(), 0, 200) . (strlen($e->getMessage()) > 200 ? '...' : '');
            $log = [
                'employee_id' => auth()->user()->id,
                'message' => 'SWMS Report delete Failed . Please retry.',
                'report_id' =>  0,
                'report_type' => 'swms_report',
                'error_type' => 'Exception error'
            ];
            storeReportsLogs($log);
            return response()->json([
                'message' => 'An error occurred : ' . $e->getMessage(),
            ], 500);
        }
    }
    public function swmsStep1Store(Request $request)
    {
        $isUpdate = $request->swms_report_id;
        $rules = [
            'title' => 'required|string|max:255',
            'site_manager_or_forman' => 'nullable|string|max:255',
            'revision_number' => 'nullable|string|max:255',
            'version_number' => 'nullable|string|max:255',
            'company_name' => 'nullable|string|max:255',
            'contact_number' => 'nullable|max:20',
            'company_address' => 'nullable|string|max:255',
            'scope_of_work' => 'nullable|string|max:255',
            'fast_address' => 'nullable|string|max:255',
            'principal_contractor' => 'nullable|string|max:255',
            'site_id' => 'required',
            'start_date' => 'nullable|date',
        ];
        $validator = Validator::make($request->all(), $rules);
        if ($validator->fails()) {
            $log = [
                'employee_id' => auth()->user()->id,
                'message' => 'SWMS Step 1 error: ' . $validator->errors()->first(),
                'report_id' => $request->swms_report_id,
                'report_type' => 'swms_report',
                'error_type' => 'Validation error',
            ];
            storeReportsLogs($log);
            return $this->message($validator->errors()->first(), 422);
        }
        // Determine the appropriate message
        $message = $isUpdate ? 'Updated Successfully' : 'Saved Successfully';
        try {
            $userTable = $this->getUserTable();
            if ($userTable === "customer") {
                $customerId = auth()->id();
                $workspaceId = auth()->user()->current_workspace_id;
            } elseif ($userTable === "emp") {
                $customerId = auth()->user()->customer_id;
                $workspaceId = auth()->user()->workspace_id;
            } else {
                return $this->error('You do not have permission to create this record.', 403);
            }
            $data = [
                'swms_number' => $request->swms_number,
                'title' => $request->title,
                'site_manager_or_forman' => $request->site_manager_or_forman,
                'revision_number' => $request->revision_number,
                'version_number' => $request->version_number,
                'start_date' => $request->start_date ?? null,
                'company_name' => $request->company_name,
                'site_id' => $request->site_id,
                'contact_number' => $request->contact_number,
                'company_address' => $request->company_address,
                'scope_of_work' => $request->scope_of_work,
                'fast_address' => $request->fast_address,
                'principal_contractor' => $request->principal_contractor,
                'principal_contractor_address' => $request->principal_contractor_address,
                'principal_contractor_number' => $request->principal_contractor_number,
                'description' => $request->description,
                'customer_id' => $customerId,
                'workspace_id' => $workspaceId,
            ];
            $already_report = Swms::where('id', $request->swms_report_id)
                ->where('swms_number', $request->swms_number)
                ->where('del', 0)
                ->first();

            if ($already_report && $already_report->revision_number != $request->revision_number) {
                $already_report_to_swms_number = Swms::where('id', '!=', $request->swms_report_id)
                    ->where('swms_number', $request->swms_number)
                    ->where('revision_number', $request->revision_number)
                    ->where('del', 0)
                    ->first();

                if ($already_report_to_swms_number) {
                    return $this->message('This revision number for this SWMS number already exists.', 422);
                } else {
                    $data['process'] = $already_report->process ?? '0';
                    $new_report_id = Swms::insertGetId($data);
                    $this->swmsReportDuplicate($new_report_id, $request->swms_report_id);
                    $data['swms_report_id'] = $new_report_id;
                }
            } else {
                $swms = Swms::updateOrCreate(
                    ['id' => $request->swms_report_id],
                    $data
                );
                $data['swms_report_id'] = $swms->id;
            }
            $user_site_forman_by = EmpCompanyDetails::with(['EmpPersonalDetails', 'accessRole', 'accessTier'])
                ->find($request->site_manager_or_forman);
            $accessRoleId = $user_site_forman_by->accessRole->id ?? null;

            SwmsSignature::updateOrCreate(
                [
                    'swms_id' => $data['swms_report_id'],
                    'tier_id' => $accessRoleId,
                ],
                [
                    'swms_id' => $data['swms_report_id'],
                    'tier_id' => $accessRoleId,
                ]
            );
            $log = [
                'employee_id' => auth()->user()->id,
                'message' => 'SWMS Step 1 ' . $message,
                'report_id' => $request->swms_report_id,
                'report_type' => 'swms_report'
            ];
            storeReportsLogs($log);
            return $this->success($data, $message);
        } catch (Exception $e) {
            $shortMessage = substr($e->getMessage(), 0, 200) . (strlen($e->getMessage()) > 200 ? '...' : '');
            $log = [
                'employee_id' => auth()->user()->id,
                'message' => 'SWMS Report Step 1 error: ' . $shortMessage,
                'report_id' => $request->swms_report_id,
                'report_type' => 'swms_report',
                'error_type' => 'Exception error'
            ];
            storeReportsLogs($log);
            return $this->error('An error occurred: ' . $shortMessage);
        }
    }


    public function swmsStep1Edit(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'swms_report_id' => 'required',

        ]);
        if ($validator->fails()) {
            $log = [
                'employee_id' => auth()->user()->id,
                'message' => 'SWMS  Step 1 edit:' . $validator->errors()->first(),
                'report_id' =>   $request->swms_report_id,
                'report_type' => 'swms_report',
                'error_type' => 'Validation error',
            ];
            storeReportsLogs($log);
            return $this->message($validator->errors()->first(), 422);
        }
        try {
            // $swms_number = rand(99999, 999999);
            $swms = Swms::find($request->swms_report_id);
            if (!$swms) {
                return $this->error('SWMS report not found.', 404);
            }
            $userTable = $this->getUserTable();
            if ($userTable === 'customer' && ($swms->customer_id !== auth()->id() || $swms->workspace_id !== auth()->user()->current_workspace_id)) {
                return $this->message('You do not have access to this report.', 403);
            }
            if ($userTable === 'emp' && ($swms->customer_id !== auth()->user()->customer_id || $swms->workspace_id !== auth()->user()->workspace_id)) {
                return $this->message('You do not have access to this report.', 403);
            }
            // $emp_list = EmpCompanyDetails::where('compeleted', '1')
            // ->where('approved', '1')
            // ->where('status', '1')
            // ->where('del', '0')
            // ->where('customer_id', $swms->customer_id)
            // ->where('workspace_id', $swms->workspace_id)
            // ->with([
            //     'empPersonalDetails' => function ($query) {
            //         $query->select('emp_id', 'first_name', 'middle_name', 'last_name', 'mobile');
            //     },
            //     'accessRole' => function ($query) {
            //         $query->select('id', 'title', 'code');
            //     },
            //     'empTier' => function ($query) {
            //         $query->select('id', 'title');
            //     },
            //     'accessTier' => function ($query) {
            //         $query->select('id', 'title', 'tier_key');
            //     },
            // ])
            // ->select('id', 'access_role', 'tier_id')
            // ->whereIn('tier_id', function ($query) {
            //     $query->select('id')
            //         ->from('tiers')
            //         ->whereIn('tier_key', ['T-1', 'T-2', 'T-3']);
            // })
            // ->get();
            // $data['authorised_by_list'] = $emp_list;
            // $data['swms_report_id'] = $request->swms_report_id;
            // $data['swms_number'] = $swms_number;

            // if ($request->swms_report_id == 0) {
            //     $data['sites'] = DB::table('sites')
            //         ->where('del', '0')
            //         ->where('active', '1')
            //         ->get(['id', 'title']);
            // } else {
            //     $swms = Swms::with('siteData')
            //     ->where('id', $request->swms_report_id)
            //     ->first();
            //     $data['sites'] = DB::table('sites')
            //         ->where(function ($query) use ($swms) {
            //             $query->where('active', '1')
            //                   ->orWhere(function ($subQuery) use ($swms) {
            //                       if ($swms && isset($swms->site_id)) {
            //                           $subQuery->where('id', $swms->site_id)
            //                                    ->where('del', '0'); 
            //                       }
            //                   });
            //         })
            //         ->get(['id', 'title']);
            // }
            // $data['company_name'] = Company::get(['id','fname','mname','lname']);
            return $this->success($swms->load(['siteData', 'authorizedBy']), 'Reocrd get Successfully');
        } catch (Exception $e) {
            $shortMessage = substr($e->getMessage(), 0, 200) . (strlen($e->getMessage()) > 200 ? '...' : '');
            $log = [
                'employee_id' => auth()->user()->id,
                'message' => 'SWMS Report Step 1 error:' . $shortMessage,
                'report_id' =>  $request->swms_report_id,
                'report_type' => 'swms_report',
                'error_type' => 'Exception error'
            ];
            storeReportsLogs($log);
            return $this->error('An error occurred : ' . $shortMessage);
        }
    }
    public function swmsReportDuplicate($new_report_id, $old_report_id)
    {
        if (!$new_report_id) {
            $log = [
                'employee_id' => auth()->user()->id,
                'message' => 'SWMS Report Step 1 duplicate not found.',
                'report_id' =>  $new_report_id,
                'report_type' => 'swms_report',
                'error_type' => 'Validation error',
            ];
            storeReportsLogs($log);
            return response()->json([
                'message' => 'New version Report id not found. Please try again',
                'new_report_id' => $new_report_id,
                'old_report_id' => $old_report_id,
            ], 433);
        }
        try {
            // Retrieve existing records
            $swms_meta_step2 = SwmsMeta::where('swms_id', $old_report_id)->get();
            $swms_general_notes_step2 = SwmsGeneralNoteAnswers::where('swms_id', $old_report_id)->get();
            $swms_developed_by_step3 = swmsDevelopedBy::where('swms_id', $old_report_id)->get();
            $swms_approved_by_step3 = SwmsApprovedBy::where('swms_id', $old_report_id)->get();
            $swms_review_responsible_step3 = SwmsReviewResponsible::where('swms_id', $old_report_id)->get();
            $swms_monitoring_responsible_step3 = SwmsMonitoringResponsible::where('swms_id', $old_report_id)->get();
            $swms_task_details_step4 = SwmsTaskDetail::where('swms_id', $old_report_id)->get();
            $swms_hazard_risk_pivot_step5 = SwmsHazardRiskPivot::where('swms_id', $old_report_id)->get();
            $swms_protection_tool_pivot_step5 = SwmsProtectionToolPivot::where('swms_id', $old_report_id)->get();
            $swms_question_answers_step5 = SwmsAwnswers::where('swms_id', $old_report_id)->get();
            $swms_safety_planning = SwmsSafetyPlanningAnswers::where('swms_id', $old_report_id)->get();
            $swms_signatures = SwmsSignature::where('swms_id', $old_report_id)->get();
            // Duplicate records with new swms_id
            $this->duplicateRecords($swms_meta_step2, $new_report_id);
            $this->duplicateRecords($swms_general_notes_step2, $new_report_id);
            $this->duplicateRecords($swms_developed_by_step3, $new_report_id);
            $this->duplicateRecords($swms_approved_by_step3, $new_report_id);
            $this->duplicateRecords($swms_review_responsible_step3, $new_report_id);
            $this->duplicateRecords($swms_monitoring_responsible_step3, $new_report_id);
            $this->duplicateRecords($swms_task_details_step4, $new_report_id);
            $this->duplicateRecords($swms_hazard_risk_pivot_step5, $new_report_id);
            $this->duplicateRecords($swms_protection_tool_pivot_step5, $new_report_id);
            $this->duplicateRecords($swms_question_answers_step5, $new_report_id);
            $this->duplicateRecords($swms_signatures, $new_report_id);

            $this->duplicateRecords($swms_safety_planning, $new_report_id);
            $log = [
                'employee_id' => auth()->user()->id,
                'message' => 'SWMS Report ID' . $old_report_id . ' duplicated successfully',
                'report_id' =>  $new_report_id,
                'report_type' => 'swms_report'
            ];
            storeReportsLogs($log);
            return response()->json([
                'message' => 'Report duplicated successfully',
                'new_report_id' => $new_report_id,
                'old_report_id' => $old_report_id,
            ], 200);
        } catch (Exception $e) {
            $shortMessage = substr($e->getMessage(), 0, 200) . (strlen($e->getMessage()) > 200 ? '...' : '');
            $log = [
                'employee_id' => auth()->user()->id,
                'message' => 'Report duplicated error:' . $shortMessage,
                'report_id' =>  $request->swms_report_id,
                'report_type' => 'swms_report',
                'error_type' => 'Exception error'
            ];
            storeReportsLogs($log);
            return response()->json([
                'message' => 'An error occurred : ' . $shortMessage,
            ], 500);
        }
    }

    private function duplicateRecords($records, $new_report_id)
    {
        foreach ($records as $record) {
            $recordClone = $record->replicate();
            $recordClone->swms_id = $new_report_id;
            $recordClone->save();
        }
    }


    // Step 2
    public function swmsStep2Store(Request $request)
    {
        $rules = [
            'swms_report_id' => 'required|exists:swms,id',
            'consultation_method' => 'nullable|string|max:255',
            'review_frequency' => 'nullable|string|max:255',
            'worker_consultation' => 'nullable|string|max:255',
            'hsr_consultation' => 'nullable|string|max:255',
            'is_based_on_risk' => 'nullable|string|max:255',
            'received_date' => 'nullable|date',
            'date_received_by_reviewer' => 'nullable|date'
        ];
        $messages = [
            'received_date.required' => 'Received date is required.',
            'received_date.date' => 'Received date must be a valid date.',
            'date_received_by_reviewer.required' => 'Date received by reviewer is required.',
            'date_received_by_reviewer.date' => 'Date received by reviewer must be a valid date.',
        ];
        // Validate the request
        $validator = Validator::make($request->all(), $rules, $messages);
        if ($validator->fails()) {
            $log = [
                'employee_id' => auth()->user()->id,
                'message' => 'SWMS Step 2 error: ' . $validator->errors()->first(),
                'report_id' =>  $request->swms_report_id,
                'report_type' => 'swms_report',
                'error_type' => 'Validation error',
            ];
            storeReportsLogs($log);
            return $this->message($validator->errors()->first(), 422);
        }
        // Check if SWMS report exists
        $swmsReport = SWMS::findOrFail($request->swms_report_id);
        if (!$swmsReport) {
            return $this->error('No matching record found.', 404);
        }
        $userTable = $this->getUserTable();
        $authUser = auth()->user();
        // Store customer_id & workspace_id for reuse
        $customerId = ($userTable === 'customer') ? $authUser->id : $authUser->customer_id;
        $workspaceId = ($userTable === 'customer') ? $authUser->current_workspace_id : $authUser->workspace_id;
        // Access check for both customer and employee
        if (
            ($userTable === 'customer' && ($swmsReport->customer_id !== $customerId || $swmsReport->workspace_id !== $workspaceId)) ||
            ($userTable === 'emp' && ($swmsReport->customer_id !== $customerId || $swmsReport->workspace_id !== $workspaceId))
        ) {
            return $this->message('You do not have access to this report.', 403);
        }
        // Prepare data for storage
        $data = [
            'swms_id' => $request->swms_report_id,
            'consultation_method' => $request->consultation_method,
            'review_frequency' => $request->review_frequency,
            'recieved_date' => $request->received_date,
            'date_recieved_by_reviewer' => $request->date_received_by_reviewer,
            'is_based_on_risk' => $request->is_based_on_risk,
            'is_worker_consulted' => $request->worker_consultation,
            'is_hsr_consulted' => $request->hsr_consultation,
        ];
        try {
            // Update or create SWMS metadata
            SwmsMeta::updateOrCreate(['swms_id' => $request->swms_report_id], $data);
            // Handle InputAnswers JSON data
            if ($request->has('InputAnswers')) {
                $inputAnswers = is_array($request->InputAnswers) ? $request->InputAnswers : json_decode($request->input('InputAnswers'), true);
                if (is_array($inputAnswers)) {
                    foreach ($inputAnswers as $noteId => $answer) {
                        // Check if swms_general_notes_id exists before inserting
                        $noteExists = SwmsGeneralNote::where('id', $noteId)->where('customer_id', $customerId)->where('workspace_id', $workspaceId)->exists();
                        if ($noteExists) {
                            SwmsGeneralNoteAnswers::updateOrCreate(
                                ['swms_id' => $request->swms_report_id, 'swms_general_notes_id' => $noteId],
                                ['answer' => $answer]
                            );
                        } else {
                            return $this->error("Invalid swms_general_notes_id: {$noteId}. No matching record found.", 422);
                        }
                    }
                }
            }
            $data['swms_report_id'] = $request->swms_report_id;
            $message = 'Updated Successfully';
            // Store logs
            $log = [
                'employee_id' => auth()->user()->id,
                'message' => 'SWMS Step 2 ' . $message,
                'report_id' =>  $request->swms_report_id,
                'report_type' => 'swms_report'
            ];
            storeReportsLogs($log);
            return $this->success($data, $message);
        } catch (Exception $e) {
            $shortMessage = substr($e->getMessage(), 0, 200) . (strlen($e->getMessage()) > 200 ? '...' : '');
            $log = [
                'employee_id' => auth()->user()->id,
                'message' => 'SWMS Step 2 error: ' . $shortMessage,
                'report_id' =>  $request->swms_report_id,
                'report_type' => 'swms_report',
                'error_type' => 'Exception error'
            ];
            storeReportsLogs($log);
            return $this->error('An error occurred: ' . $shortMessage);
        }
    }

    public function swmsStep2Edit(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'swms_report_id' => 'required',
        ]);
        if ($validator->fails()) {
            $log = [
                'employee_id' => auth()->user()->id,
                'message' => 'SWMS Step 2 edit :' . $validator->errors()->first(),
                'report_id' =>  $request->swms_report_id,
                'report_type' => 'swms_report',
                'error_type' => 'Validation error',
            ];
            storeReportsLogs($log);
            return $this->message($validator->errors()->first(), 422);
        }
        try {
            $swmsReport = SWMS::find($request->swms_report_id);
            if (!$swmsReport) {
                return $this->error('No matching record found.', 404);
            }
            $userTable = $this->getUserTable();
            $authUser = auth()->user();
            // Store customer_id & workspace_id for reuse
            $customerId = ($userTable === 'customer') ? $authUser->id : $authUser->customer_id;
            $workspaceId = ($userTable === 'customer') ? $authUser->current_workspace_id : $authUser->workspace_id;
            // Access check for both customer and employee
            if (
                ($userTable === 'customer' && ($swmsReport->customer_id !== $customerId || $swmsReport->workspace_id !== $workspaceId)) ||
                ($userTable === 'emp' && ($swmsReport->customer_id !== $customerId || $swmsReport->workspace_id !== $workspaceId))
            ) {
                return $this->message('You do not have access to this report.', 403);
            }
            $data['swms_meta'] = SwmsMeta::where('swms_id', $request->swms_report_id)->first();
            $data['GeneralNotes'] = SwmsGeneralNote::where('customer_id', $customerId)->where('workspace_id', $workspaceId)->get();
            $answers = SwmsGeneralNoteAnswers::where('swms_id', $request->swms_report_id)->get()->keyBy('swms_general_notes_id');
            $data['SwmsGeneralNoteAnswers'] = $answers;
            $data['swms_report_id'] = $request->swms_report_id;
            return $this->success($data, 'Reocrd get Successfully');
        } catch (Exception $e) {
            $shortMessage = substr($e->getMessage(), 0, 200) . (strlen($e->getMessage()) > 200 ? '...' : '');
            $log = [
                'employee_id' => auth()->user()->id,
                'message' => 'SWMS Step 2 error:' . $shortMessage,
                'report_id' =>  $request->swms_report_id,
                'report_type' => 'swms_report',
                'error_type' => 'Exception error'
            ];
            storeReportsLogs($log);
            return $this->error('An error occurred : ' . $shortMessage,);
        }
    }
    public function swmsStep3Store(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'swms_report_id' => 'required|exists:swms,id',
        ]);
        if ($validator->fails()) {
            $log = [
                'employee_id' => auth()->user()->id,
                'message' => 'SWMS Step 3 store :' . $validator->errors()->first(),
                'report_id' => $request->swms_report_id,
                'report_type' => 'swms_report',
                'error_type' => 'Validation error'
            ];
            storeReportsLogs($log);
            return $this->message($validator->errors()->first(), 422);
        }
        $swms_report_id = $request->swms_report_id;
        $swmsReport = SWMS::find($swms_report_id);
        if (!$swmsReport) {
            return $this->error('No matching record found.', 404);
        }
        // Retrieve customer and workspace IDs
        $userTable = $this->getUserTable();
        $authUser = auth()->user();
        $customerId = ($userTable === 'customer') ? $authUser->id : $authUser->customer_id;
        $workspaceId = ($userTable === 'customer') ? $authUser->current_workspace_id : $authUser->workspace_id;
        // Validate access permissions
        if ($swmsReport->customer_id !== $customerId || $swmsReport->workspace_id !== $workspaceId) {
            return $this->message('You do not have access to this report.', 403);
        }
        try {
            // Save all signatures using a reusable method
            $signatureData = [
                ['employees' => 'column_employee_id', 'signatures' => 'column_signature', 'dates' => 'column_date', 'model' => swmsDevelopedBy::class],
                ['employees' => 'columns_approved_by_emp_id', 'signatures' => 'columns_approved_by_signature', 'dates' => 'columns_approved_by_date', 'model' => SwmsApprovedBy::class],
                ['employees' => 'review-responsible_employee_id', 'signatures' => 'review-responsible_signature', 'dates' => 'review-responsible_by_date', 'model' => SwmsReviewResponsible::class],
                ['employees' => 'monitoring-responsible_employee_id', 'signatures' => 'monitoring-responsible_signature', 'dates' => 'monitoring-responsible_by_date', 'model' => SwmsMonitoringResponsible::class],
                ['employees' => 'compliance-responsible_employee_id', 'signatures' => 'compliance-responsible_signature', 'dates' => 'compliance-responsible_by_date', 'model' => SwmsComplianceResponsible::class]
            ];
            foreach ($signatureData as $data) {
                $this->saveSignatures(
                    $swms_report_id,
                    $request->input($data['employees'], []),
                    $request->file($data['signatures'], []),
                    $request->input($data['dates'], []),
                    $data['model']
                );
            }
            $log = [
                'employee_id' => auth()->user()->id,
                'message' => 'SWMS Step 3 Saved Successfully.',
                'report_id' => $swms_report_id,
                'report_type' => 'swms_report'
            ];
            storeReportsLogs($log);
            return $this->success(['swms_report_id' => $swms_report_id], 'Saved Successfully');
        } catch (Exception $e) {
            $shortMessage = substr($e->getMessage(), 0, 200) . (strlen($e->getMessage()) > 200 ? '...' : '');
            $log = [
                'employee_id' => auth()->user()->id,
                'message' => 'SWMS Step 3 error:' . $shortMessage,
                'report_id' => $swms_report_id,
                'report_type' => 'swms_report',
                'error_type' => 'Exception error'
            ];
            storeReportsLogs($log);
            return $this->error('An error occurred: ' . $shortMessage);
        }
    }

    // Generalized function to save signature data
    private function saveSignatures($swms_report_id, $employee_ids, $signature_files, $dates, $modelClass)
    {
        try {
            $employee_count = count($employee_ids);
            for ($i = 0; $i < $employee_count; $i++) {
                $employee_id = $employee_ids[$i];
                $signature_date = isset($dates[$i]) ? $dates[$i] : null;
                $signature_file = isset($signature_files[$i]) ? $signature_files[$i] : null;
                if (empty($employee_id)) {
                    continue;
                }
                $signature_path = null;
                if ($signature_file && $signature_file->isValid()) {
                    $imageName = $employee_id . time() . '.' . $signature_file->getClientOriginalExtension();
                    $signature_file->move(public_path('SWMSReportDocuments'), $imageName);
                    $signature_path = 'SWMSReportDocuments/' . $imageName;
                }
                $signature = $modelClass::where('swms_id', $swms_report_id)
                    ->where('employee_id', $employee_id)
                    ->where('del', '0')
                    ->first();
                // dd($signature);
                if ($signature) {
                    if ($signature_path) {
                        $signature->signature = $signature_path;
                    }
                    if ($signature_date) {
                        $signature->date = $signature_date;
                    }
                    $signature->save();
                } else {
                    $modelClass::create([
                        'swms_id' => $swms_report_id,
                        'employee_id' => $employee_id,
                        'signature' => $signature_path,
                        'date' => $signature_date,
                    ]);
                }
            }
        } catch (Exception $e) {
            $shortMessage = substr($e->getMessage(), 0, 200) . (strlen($e->getMessage()) > 200 ? '...' : '');
            $log = [
                'employee_id' => auth()->user()->id,
                'message' => 'SWMS Step 3 error:' . $shortMessage,
                'report_id' =>  $request->swms_report_id,
                'report_type' => 'swms_report',
                'error_type' => 'Exception error'
            ];
            storeReportsLogs($log);
            return response()->json([
                'message' => 'An error occurred : ' . $shortMessage,
            ], 500);
        }
    }
    public function swmsStep3Edit(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'swms_report_id' => 'required',

        ]);
        if ($validator->fails()) {
            $log = [
                'employee_id' => auth()->user()->id,
                'message' => 'SWMS Step 3 error:' . $validator->errors()->first(),
                'report_id' =>  $request->swms_report_id,
                'report_type' => 'swms_report',
                'error_type' => 'Validation error',
            ];
            storeReportsLogs($log);
            return $this->message($validator->errors()->first(), 422);
        }
        try {
            $swmsReport = SWMS::find($request->swms_report_id);
            if (!$swmsReport) {
                return $this->message('No SWMS report found ', 404);
            }
            $userTable = $this->getUserTable();
            if ($userTable === 'customer' && ($swmsReport->customer_id !== auth()->id() || $swmsReport->workspace_id !== auth()->user()->current_workspace_id)) {
                return $this->message('You do not have access to this report.', 403);
            }
            if ($userTable === 'emp' && ($swmsReport->customer_id !== auth()->user()->customer_id || $swmsReport->workspace_id !== auth()->user()->workspace_id)) {
                return $this->message('You do not have access to this report.', 403);
            }
            $data['swmsDevelopedBy'] = swmsDevelopedBy::where('swms_id', $request->swms_report_id)->get();
            $data['SwmsApprovedBy'] = SwmsApprovedBy::where('swms_id', $request->swms_report_id)->get();
            $data['SwmsReviewResponsible'] = SwmsReviewResponsible::where('swms_id', $request->swms_report_id)->get();
            $data['SwmsMonitoringResponsible'] = SwmsMonitoringResponsible::where('swms_id', $request->swms_report_id)->get();
            $data['SwmsComplianceResponsible'] = SwmsComplianceResponsible::where('swms_id', $request->swms_report_id)->get();

            // $authorised_by_list = EmpCompanyDetails::where('compeleted', '1')
            //     ->where('approved', '1')
            //     ->where('status', '1')
            //     ->where('del', '0')
            //     ->with([
            //         'empPersonalDetails' => function ($query) {
            //             $query->select('emp_id', 'first_name', 'middle_name', 'last_name');
            //         },
            //         'accessRole' => function ($query) {
            //             $query->select('id', 'title', 'code');
            //         },
            //         'empTier' => function ($query) {
            //             $query->select('id', 'title');
            //         },
            //         'accessTier' => function ($query) {
            //             $query->select('id', 'title', 'tier_key');
            //         },
            //     ])
            //     ->select('id', 'access_role', 'tier_id')
            //     ->whereIn('tier_id', function ($query) {
            //         $query->select('id')
            //             ->from('tiers')
            //             ->whereIn('tier_key', ['T-1', 'T-2', 'T-3']);
            //     })
            //     ->get();
            // $data['authorised_by_list'] = $authorised_by_list;
            $data['swms_report_id'] = $request->swms_report_id;
            return $this->success($data, 'Reocrd get Successfully');
        } catch (Exception $e) {
            $shortMessage = substr($e->getMessage(), 0, 200) . (strlen($e->getMessage()) > 200 ? '...' : '');
            $log = [
                'employee_id' => auth()->user()->id,
                'message' => 'SWMS Step 3 error:' . $shortMessage,
                'report_id' =>  $request->swms_report_id,
                'report_type' => 'swms_report',
                'error_type' => 'Exception error'
            ];
            storeReportsLogs($log);
            return $this->error('An error occurred : ' . $shortMessage);
        }
    }
    public function swmsStep3DevelopedByDelete($id)
    {
        try {
            $developedBy = SwmsDevelopedBy::with('swms')->find($id);
            if (!$developedBy) {
                return $this->message('Record not found.', 404);
            }
            $swmsReport = $developedBy->swms;
            if (!$swmsReport) {
                return $this->message('SWMS Report not found.', 404);
            }
            $userTable = $this->getUserTable();
            if (
                ($userTable === 'customer' && ($swmsReport->customer_id !== auth()->id() || $swmsReport->workspace_id !== auth()->user()->current_workspace_id)) ||
                ($userTable === 'emp' && ($swmsReport->customer_id !== auth()->user()->customer_id || $swmsReport->workspace_id !== auth()->user()->workspace_id))
            ) {
                return $this->message('You do not have permission to delete this record.', 403);
            }
            if ($developedBy->signature && file_exists(public_path($developedBy->signature))) {
                unlink(public_path($developedBy->signature));
            }
            $developedBy->delete();
            $log = [
                'employee_id' => auth()->user()->id,
                'message' => 'SWMS Step 3 Developed By Deleted Successfully.',
                'report_id' => $id,
                'report_type' => 'swms_report'
            ];
            storeReportsLogs($log);
            return $this->message('Deleted Successfully');
        } catch (Exception $e) {
            $shortMessage = substr($e->getMessage(), 0, 200) . (strlen($e->getMessage()) > 200 ? '...' : '');
            $log = [
                'employee_id' => auth()->user()->id,
                'message' => 'SWMS Step 3 error: ' . $shortMessage,
                'report_id' => $id,
                'report_type' => 'swms_report',
                'error_type' => 'Exception error'
            ];
            storeReportsLogs($log);
            return $this->error('An error occurred: ' . $shortMessage);
        }
    }

    public function swmsStep3ApprovedByDelete($id)
    {
        try {
            $approvedBy = SwmsApprovedBy::with('swms')->find($id);
            if (!$approvedBy) {
                return $this->message('Record not found.', 404);
            }
            $swmsReport = $approvedBy->swms;
            if (!$swmsReport) {
                return $this->message('SWMS Report not found.', 404);
            }
            $userTable = $this->getUserTable();
            if (
                ($userTable === 'customer' && ($swmsReport->customer_id !== auth()->id() || $swmsReport->workspace_id !== auth()->user()->current_workspace_id)) ||
                ($userTable === 'emp' && ($swmsReport->customer_id !== auth()->user()->customer_id || $swmsReport->workspace_id !== auth()->user()->workspace_id))
            ) {
                return $this->message('You do not have permission to delete this record.', 403);
            }
            if ($approvedBy->signature && file_exists(public_path($approvedBy->signature))) {
                unlink(public_path($approvedBy->signature));
            }
            $approvedBy->delete();
            $log = [
                'employee_id' => auth()->user()->id,
                'message' => 'SWMS Step 3 Approved by Deleted Successfully.',
                'report_id' => $id,
                'report_type' => 'swms_report'
            ];
            storeReportsLogs($log);
            return $this->message('Deleted Successfully');
        } catch (Exception $e) {
            $shortMessage = substr($e->getMessage(), 0, 200) . (strlen($e->getMessage()) > 200 ? '...' : '');
            $log = [
                'employee_id' => auth()->user()->id,
                'message' => 'SWMS Step 3 error: ' . $shortMessage,
                'report_id' => $id,
                'report_type' => 'swms_report',
                'error_type' => 'Exception error'
            ];
            storeReportsLogs($log);
            return $this->error('An error occurred: ' . $shortMessage);
        }
    }


    public function swmsStep3ReviewByDelete($id)
    {
        try {
            $reviewResponsible = SwmsReviewResponsible::with('swms')->find($id);
            if (!$reviewResponsible) {
                return $this->message('Record not found.', 404);
            }
            $swmsReport = $reviewResponsible->swms;
            if (!$swmsReport) {
                return $this->message('SWMS Report not found.', 404);
            }
            $userTable = $this->getUserTable();
            if (
                ($userTable === 'customer' && ($swmsReport->customer_id !== auth()->id() || $swmsReport->workspace_id !== auth()->user()->current_workspace_id)) ||
                ($userTable === 'emp' && ($swmsReport->customer_id !== auth()->user()->customer_id || $swmsReport->workspace_id !== auth()->user()->workspace_id))
            ) {
                return $this->message('You do not have permission to delete this record.', 403);
            }
            // Check and delete the image if it exists
            if ($reviewResponsible->signature && file_exists(public_path($reviewResponsible->signature))) {
                unlink(public_path($reviewResponsible->signature));
            }
            $reviewResponsible->delete();
            $log = [
                'employee_id' => auth()->user()->id,
                'message' => 'SWMS Step 3 Review by Deleted Successfully.',
                'report_id' => $id,
                'report_type' => 'swms_report'
            ];
            storeReportsLogs($log);

            return $this->message('Deleted Successfully');
        } catch (Exception $e) {
            $shortMessage = substr($e->getMessage(), 0, 200) . (strlen($e->getMessage()) > 200 ? '...' : '');
            $log = [
                'employee_id' => auth()->user()->id,
                'message' => 'SWMS Step 3 error: ' . $shortMessage,
                'report_id' => $id,
                'report_type' => 'swms_report',
                'error_type' => 'Exception error'
            ];
            storeReportsLogs($log);
            return $this->error('An error occurred: ' . $shortMessage);
        }
    }
    public function swmsStep3MonitoringByDelete($id)
    {
        try {
            // Find the monitoring responsible record with its related SWMS report
            $monitoringResponsible = SwmsMonitoringResponsible::with('swms')->find($id);
            if (!$monitoringResponsible) {
                return $this->message('Record not found.', 404);
            }
            $swmsReport = $monitoringResponsible->swms;
            if (!$swmsReport) {
                return $this->message('SWMS Report not found.', 404);
            }
            $userTable = $this->getUserTable();
            if (
                ($userTable === 'customer' && ($swmsReport->customer_id !== auth()->id() || $swmsReport->workspace_id !== auth()->user()->current_workspace_id)) ||
                ($userTable === 'emp' && ($swmsReport->customer_id !== auth()->user()->customer_id || $swmsReport->workspace_id !== auth()->user()->workspace_id))
            ) {
                return $this->message('You do not have permission to delete this record.', 403);
            }
            if ($monitoringResponsible->signature && file_exists(public_path($monitoringResponsible->signature))) {
                unlink(public_path($monitoringResponsible->signature));
            }
            $monitoringResponsible->delete();
            $log = [
                'employee_id' => auth()->user()->id,
                'message' => 'SWMS Step 3 Monitoring by Deleted Successfully.',
                'report_id' => $id,
                'report_type' => 'swms_report'
            ];
            storeReportsLogs($log);
            return $this->message('Deleted Successfully');
        } catch (Exception $e) {
            $shortMessage = substr($e->getMessage(), 0, 200) . (strlen($e->getMessage()) > 200 ? '...' : '');
            $log = [
                'employee_id' => auth()->user()->id,
                'message' => 'SWMS Step 3 error: ' . $shortMessage,
                'report_id' => $id,
                'report_type' => 'swms_report',
                'error_type' => 'Exception error'
            ];
            storeReportsLogs($log);
            return $this->error('An error occurred: ' . $shortMessage);
        }
    }
    public function swmsStep3ComplianceByDelete($id)
    {
        try {
            $complianceResponsible = SwmsComplianceResponsible::with('swms')->find($id);
            if (!$complianceResponsible) {
                return $this->message('Record not found.', 404);
            }
            $swmsReport = $complianceResponsible->swms;
            if (!$swmsReport) {
                return $this->message('SWMS Report not found.', 404);
            }
            $userTable = $this->getUserTable();
            if (
                ($userTable === 'customer' && ($swmsReport->customer_id !== auth()->id() || $swmsReport->workspace_id !== auth()->user()->current_workspace_id)) ||
                ($userTable === 'emp' && ($swmsReport->customer_id !== auth()->user()->customer_id || $swmsReport->workspace_id !== auth()->user()->workspace_id))
            ) {
                return $this->message('You do not have permission to delete this record.', 403);
            }
            if ($complianceResponsible->signature && file_exists(public_path($complianceResponsible->signature))) {
                unlink(public_path($complianceResponsible->signature));
            }
            $complianceResponsible->delete();
            $log = [
                'employee_id' => auth()->user()->id,
                'message' => 'SWMS Step 3 Compliance by Deleted Successfully.',
                'report_id' => $id,
                'report_type' => 'swms_report'
            ];
            storeReportsLogs($log);

            return $this->message('Deleted Successfully');
        } catch (Exception $e) {
            $shortMessage = substr($e->getMessage(), 0, 200) . (strlen($e->getMessage()) > 200 ? '...' : '');
            $log = [
                'employee_id' => auth()->user()->id,
                'message' => 'SWMS Step 3 error: ' . $shortMessage,
                'report_id' => $id,
                'report_type' => 'swms_report',
                'error_type' => 'Exception error'
            ];
            storeReportsLogs($log);
            return $this->error('An error occurred: ' . $shortMessage);
        }
    }


    public function swmsStep4Store(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'swms_report_id' => 'required|exists:swms,id',
            'column_job_task' => 'nullable|array',
            'column_job_task.*' => 'nullable|string',
            'column_risk' => 'nullable|array',
            'column_risk.*' => 'nullable|string',
            'column_initial_risk_rating' => 'nullable|array',
            'column_initial_risk_rating.*' => 'nullable|integer',
            'column_hazard' => 'nullable|array',
            'column_hazard.*' => 'nullable|string',
            'column_control_risk_rating' => 'nullable|array',
            'column_control_risk_rating.*' => 'nullable|integer',
            'control_measure' => 'nullable|array',
            'control_measure.*' => 'nullable|string',
            'column_final_risk_rating' => 'nullable|array',
            'column_final_risk_rating.*' => 'nullable|integer',
            'column_task_detail_id' => 'nullable|array',
            'column_task_detail_id.*' => 'nullable|integer',
            'responsible_person_ids' => 'nullable|array',
            'responsible_person_ids.*' => 'nullable|integer',
            'column_task_details_image.*' => 'nullable|image|mimes:png,jpeg,jpg|max:2048'
        ]);
        if ($validator->fails()) {
            $log = [
                'employee_id' => auth()->user()->id,
                'message' => 'SWMS Step 4 error: ' . $validator->errors()->first(),
                'report_id' => $request->swms_report_id,
                'report_type' => 'swms_report',
                'error_type' => 'Validation error',
            ];
            storeReportsLogs($log);
            return $this->message($validator->errors()->first(), 422);
        }
        $swms_report_id = $request->swms_report_id;
        try {
            $swmsReport = Swms::find($swms_report_id);
            if (!$swmsReport) {
                return $this->message('SWMS Report not found.', 404);
            }
            $userTable = $this->getUserTable();
            if (
                ($userTable === 'customer' && ($swmsReport->customer_id !== auth()->id() || $swmsReport->workspace_id !== auth()->user()->current_workspace_id)) ||
                ($userTable === 'emp' && ($swmsReport->customer_id !== auth()->user()->customer_id || $swmsReport->workspace_id !== auth()->user()->workspace_id))
            ) {
                return $this->message('You do not have access to this report.', 403);
            }
            $jobTasks = $request->input('column_job_task', []);
            $risks = $request->input('column_risk', []);
            $initialRiskRatings = $request->input('column_initial_risk_rating', []);
            $hazards = $request->input('column_hazard', []);
            $controlRiskRatings = $request->input('column_control_risk_rating', []);
            $controlMeasures = $request->input('control_measure', []);
            $finalRiskRatings = $request->input('column_final_risk_rating', []);
            $taskDetailImages = $request->file('column_task_details_image', []);
            $taskDetailIds = $request->input('column_task_detail_id', []);
            $responsiblePersonIds = $request->input('responsible_person_ids', []);
            foreach ($responsiblePersonIds as $index => $responsiblePersonId) {
                $data = [
                    'swms_id' => $swms_report_id,
                    'responsible_preson_id' => $responsiblePersonId,
                    'job_task' => $jobTasks[$index] ?? null,
                    'risk' => $risks[$index] ?? null,
                    'initial_risk_rating' => $initialRiskRatings[$index] ?? null,
                    'hazard' => $hazards[$index] ?? null,
                    'controle_risk_ratting' => $controlRiskRatings[$index] ?? null,
                    'control_measure' => $controlMeasures[$index] ?? null,
                    'final_risk_ratting' => $finalRiskRatings[$index] ?? null
                ];
                // Handle Image Upload
                if (!empty($taskDetailImages[$index]) && $taskDetailImages[$index]->isValid()) {
                    $imageName = time() . '_' . $taskDetailImages[$index]->getClientOriginalName();
                    $taskDetailImages[$index]->move(public_path('SWMSReportDocuments'), $imageName);
                    $data['image'] = 'SWMSReportDocuments/' . $imageName;
                }
                // Check if updating an existing record
                $taskDetailId = $taskDetailIds[$index] ?? null;
                if ($taskDetailId) {
                    $existingRecord = SwmsTaskDetail::where('swms_id', $swms_report_id)->where('id', $taskDetailId)->first();
                    if ($existingRecord) {
                        $existingRecord->update($data);
                        continue;
                    }
                }
                // Create new record if no existing one
                SwmsTaskDetail::create($data);
            }
            $log = [
                'employee_id' => auth()->user()->id,
                'message' => 'SWMS Step 4 Saved Successfully.',
                'report_id' => $request->swms_report_id,
                'report_type' => 'swms_report'
            ];
            storeReportsLogs($log);
            return $this->success(['swms_report_id' => $swms_report_id], 'Saved Successfully');
        } catch (Exception $e) {
            $shortMessage = substr($e->getMessage(), 0, 200) . (strlen($e->getMessage()) > 200 ? '...' : '');
            $log = [
                'employee_id' => auth()->user()->id,
                'message' => 'SWMS Step 4 error:' . $shortMessage,
                'report_id' => $request->swms_report_id,
                'report_type' => 'swms_report',
                'error_type' => 'Exception error'
            ];
            storeReportsLogs($log);
            return $this->error('An error occurred: ' . $shortMessage);
        }
    }



    public function swmsStep4Edit(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'swms_report_id' => 'required',
        ]);
        if ($validator->fails()) {
            $log = [
                'employee_id' => auth()->user()->id,
                'message' => 'SWMS Step 4 error:' . $validator->errors()->first(),
                'report_id' =>  $request->swms_report_id,
                'report_type' => 'swms_report',
                'error_type' => 'Validation error',
            ];
            storeReportsLogs($log);
            return $this->message($validator->errors()->first(), 422);
        }
        try {
            $swmsReport = Swms::find($request->swms_report_id);
            if (!$swmsReport) {
                return $this->message('SWMS Report not found.', 404);
            }
            $userTable = $this->getUserTable();
            if (
                ($userTable === 'customer' && ($swmsReport->customer_id !== auth()->id() || $swmsReport->workspace_id !== auth()->user()->current_workspace_id)) ||
                ($userTable === 'emp' && ($swmsReport->customer_id !== auth()->user()->customer_id || $swmsReport->workspace_id !== auth()->user()->workspace_id))
            ) {
                return $this->message('You do not have access to this report.', 403);
            }
            $data = SwmsTaskDetail::where('swms_id', $request->swms_report_id)->get();
            // $authorised_by_list = EmpCompanyDetails::where('compeleted', '1')
            //     ->where('approved', '1')
            //     ->where('status', '1')
            //     ->where('del', '0')
            //     ->with(
            //         [
            //             'empPersonalDetails' => function ($query) {
            //                 $query->select('emp_id', 'first_name', 'middle_name', 'last_name');
            //             },
            //             'accessRole' => function ($query) {
            //                 $query->select('id', 'title', 'code');
            //             },
            //             'empTier' => function ($query) {
            //                 $query->select('id', 'title');
            //             },
            //         ]
            //     )
            //     ->select('id', 'access_role', 'tier_id')
            //     ->get();
            // $data['responsible_person_list'] = $authorised_by_list;
            // $data['swms_report_id'] = $request->swms_report_id;
            return $this->success($data, 'Reocrd get Successfully');
        } catch (Exception $e) {
            $shortMessage = substr($e->getMessage(), 0, 200) . (strlen($e->getMessage()) > 200 ? '...' : '');
            $log = [
                'employee_id' => auth()->user()->id,
                'message' => 'SWMS Step 4 edit error:' . $shortMessage,
                'report_id' =>  $request->swms_report_id,
                'report_type' => 'swms_report',
                'error_type' => 'Exception error'
            ];
            storeReportsLogs($log);
            return $this->error('An error occurred : ' . $shortMessage);
        }
    }
    public function swmsStep3TaskDetailDelete($id)
    {
        try {
            $taskDetail = SwmsTaskDetail::with('swms')->find($id);
            if (!$taskDetail) {
                return $this->message('Record not found.', 404);
            }
            $swmsReport = $taskDetail->swms;
            if (!$swmsReport) {
                return $this->message('SWMS Report not found.', 404);
            }
            $userTable = $this->getUserTable();
            if (
                ($userTable === 'customer' && ($swmsReport->customer_id !== auth()->id() || $swmsReport->workspace_id !== auth()->user()->current_workspace_id)) ||
                ($userTable === 'emp' && ($swmsReport->customer_id !== auth()->user()->customer_id || $swmsReport->workspace_id !== auth()->user()->workspace_id))
            ) {
                return $this->message('You do not have permission to delete this record.', 403);
            }
            if (!empty($taskDetail->image) && file_exists(public_path($taskDetail->image))) {
                unlink(public_path($taskDetail->image)); // Remove the file from storage
            }
            $taskDetail->delete();
            $log = [
                'employee_id' => auth()->user()->id,
                'message' => 'SWMS Step 3 Task Detail Deleted Successfully.',
                'report_id' => $id,
                'report_type' => 'swms_report'
            ];
            storeReportsLogs($log);
            return $this->message('Deleted Successfully');
        } catch (Exception $e) {
            $shortMessage = substr($e->getMessage(), 0, 200) . (strlen($e->getMessage()) > 200 ? '...' : '');
            $log = [
                'employee_id' => auth()->user()->id,
                'message' => 'SWMS Step 3 Task Detail Delete error: ' . $shortMessage,
                'report_id' => $id,
                'report_type' => 'swms_report',
                'error_type' => 'Exception error'
            ];
            storeReportsLogs($log);
            return $this->error('An error occurred: ' . $shortMessage);
        }
    }


    public function swmsStep5Edit(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'swms_report_id' => 'required',
        ]);
        if ($validator->fails()) {
            return $this->message($validator->errors()->first(), 422);
        }
        $swmsReport = Swms::find($request->swms_report_id);
        if (!$swmsReport) {
            return $this->message('SWMS Report not found.', 404);
        }
        $userTable = $this->getUserTable();
        $authUser = auth()->user();
        $customerId = ($userTable === 'customer') ? $authUser->id : $authUser->customer_id;
        $workspaceId = ($userTable === 'customer') ? $authUser->current_workspace_id : $authUser->workspace_id;
        // Validate access permissions
        if ($swmsReport->customer_id !== $customerId || $swmsReport->workspace_id !== $workspaceId) {
            return $this->message('You do not have access to this report.', 403);
        }
        $data['HazardRisk'] = HazardRisk::where('status', '1')->where('customer_id', $customerId)->where('workspace_id', $workspaceId)->get();
        $data['selectedHazardRiskIds'] = SwmsHazardRiskPivot::where('swms_id', $request->swms_report_id)->get();
        $data['SwmsProtectionToolPivotSelectedIds'] = SwmsProtectionToolPivot::where('swms_id', $request->swms_report_id)->get();
        $data['SwmsProtectionTool'] = SwmsProtectionTool::where('customer_id', $customerId)->where('workspace_id', $workspaceId)->get();
        $data['SwmsQuestions'] = SwmsQuestions::where('status', '1')->where('customer_id', $customerId)->where('workspace_id', $workspaceId)->get();
        $answers = SwmsAwnswers::where('swms_id', $request->swms_report_id)->get()->keyBy('swms_question_id');
        $data['SwmsAwnswers'] = $answers;
        return $this->success($data, 'Get Successfully');
    }

    public function swmsStep5Store(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'swms_report_id' => 'required|exists:swms,id',
            'selectedHazards' => 'nullable|array|min:1',
            'selectedHazards.*' => 'integer|exists:hazard_risks,id',
            'selectedPivots' => 'nullable|array|min:1',
            'selectedPivots.*' => 'integer|exists:swms_protection_tools,id',
            'InputAnswers' => 'nullable|array',
            'InputAnswers.*' => 'string|nullable'
        ], [
            'selectedHazards.required' => 'At least one Hazard risk is required.',
            'selectedPivots.required' => 'At least one Protection tool is required.',
        ]);

        if ($validator->fails()) {
            $log = [
                'employee_id' => auth()->user()->id,
                'message' => 'SWMS Step 5 store error: ' . $validator->errors()->first(),
                'report_id' => $request->swms_report_id,
                'report_type' => 'swms_report'
            ];
            storeReportsLogs($log);
            return $this->message($validator->errors()->first(), 422);
        }

        try {
            $swms_report_id = $request->input('swms_report_id');
            $swmsReport = Swms::findOrFail($swms_report_id);

            // Validate user access
            $userTable = $this->getUserTable();
            if (
                ($userTable === 'customer' && ($swmsReport->customer_id !== auth()->id() || $swmsReport->workspace_id !== auth()->user()->current_workspace_id)) ||
                ($userTable === 'emp' && ($swmsReport->customer_id !== auth()->user()->customer_id || $swmsReport->workspace_id !== auth()->user()->workspace_id))
            ) {
                return $this->message('You do not have access to this report.', 403);
            }

            // Prepare Data
            $data['swms_report_id'] = $swms_report_id;

            // Delete old records
            SwmsHazardRiskPivot::where('swms_id', $swms_report_id)->delete();
            SwmsProtectionToolPivot::where('swms_id', $swms_report_id)->delete();
            SwmsAwnswers::where('swms_id', $swms_report_id)->delete();

            // Insert Hazard Risks
            $selectedHazards = $request->input('selectedHazards', []);
            foreach ($selectedHazards as $hazardId) {
                SwmsHazardRiskPivot::create([
                    'swms_id' => $swms_report_id,
                    'hazard_risks_id' => $hazardId
                ]);
            }

            // Insert Protection Tools
            $selectedPivots = $request->input('selectedPivots', []);
            foreach ($selectedPivots as $protectionToolId) {
                SwmsProtectionToolPivot::create([
                    'swms_id' => $swms_report_id,
                    'protection_tool_id' => $protectionToolId
                ]);
            }

            // Insert Input Answers
            $inputAnswers = $request->input('InputAnswers', []);
            $existingQuestionIds = SwmsQuestions::pluck('id')->toArray(); // Fetch valid question IDs

            foreach ($inputAnswers as $questionId => $answer) {
                if (in_array($questionId, $existingQuestionIds)) { // ✅ Only insert if question exists
                    SwmsAwnswers::updateOrCreate(
                        ['swms_id' => $swms_report_id, 'swms_question_id' => $questionId],
                        ['answer' => $answer]
                    );
                }
            }

            // Logging
            $log = [
                'employee_id' => auth()->user()->id,
                'message' => 'SWMS Step 5 Saved successfully.',
                'report_id' => $swms_report_id,
                'report_type' => 'swms_report'
            ];
            storeReportsLogs($log);

            return $this->success($data, 'Saved successfully.');
        } catch (Exception $e) {
            $shortMessage = substr($e->getMessage(), 0, 200) . (strlen($e->getMessage()) > 200 ? '...' : '');
            $log = [
                'employee_id' => auth()->user()->id,
                'message' => 'SWMS Step 5 store error: ' . $shortMessage,
                'report_id' => $request->swms_report_id,
                'report_type' => 'swms_report',
                'error_type' => 'Exception error'
            ];
            storeReportsLogs($log);
            return $this->error('An error occurred: ' . $shortMessage);
        }
    }
    public function SwmsSignature(Request $request)
    {
        try {
        $swms_report_id = $request->input('swms_report_id');

        // Check if a SWMS Signature record already exists for the given SWMS ID
        $swmsSignatureExists = SwmsSignature::where('swms_id', $swms_report_id)->exists();

        $validator = Validator::make($request->all(), [
            'swms_report_id' => 'required|integer',

        ]);
        $inductance_dates = (array) $request->input('inductance_dates', []);

        if ($inductance_dates || $inductance_dates != null) {
            $validator->sometimes('inductance_dates', 'required|array|min:1', function ($input) use ($swmsSignatureExists) {
                return !$swmsSignatureExists;
            });

            $validator->sometimes('inductance_dates.*', 'required|date|after_or_equal:today', function ($input) use ($swmsSignatureExists) {
                return !$swmsSignatureExists;
            });
        }

        // if ($validator->fails()) {
        //     return response()->json([
        //         'message' => $validator->errors()->first(),
        //     ], 422);
        // }

        $swms_report_id = $request->swms_report_id;

    
            $access_role_ids = (array) $request->input('access_role_ids', []);
            $job_descriptions = (array) $request->input('job_descriptions', []);
            $inductance_dates = (array) $request->input('inductance_dates', []);
            $record_ids = (array) $request->input('record_ids', []);

            // Check if the arrays have matching lengths
            if (count($access_role_ids) !== count($job_descriptions) || count($access_role_ids) !== count($inductance_dates) || count($access_role_ids) !== count($record_ids)) {
                return response()->json([
                    'message' => 'Mismatched input  please complete all inputs.',
                ], 422);
            }

            // Loop through the input arrays or handle single items
            foreach ($access_role_ids as $index => $access_role_id) {
                $jobDescription = $job_descriptions[$index] ?? null;
                $inductanceDate = $inductance_dates[$index] ?? null;
                $recordId = $record_ids[$index] ?? null;

                if ($recordId) {
                    // Update existing record
                    $swmsSignature = SwmsSignature::where('swms_id', $swms_report_id)->find($recordId);
                    if ($swmsSignature) {
                        $swmsSignature->update([
                            'swms_id' => $swms_report_id,
                            'tier_id' => $access_role_id,
                            'job_desription' => $jobDescription,
                            'inductance_date' => $inductanceDate,
                        ]);
                    } else {
                        SwmsSignature::create([
                            'swms_id' => $swms_report_id,
                            'tier_id' => $access_role_id,
                            'job_desription' => $jobDescription,
                            'inductance_date' => $inductanceDate,
                        ]);
                    }
                } else {
                    // Create new record
                    SwmsSignature::create([
                        'swms_id' => $swms_report_id,
                        'tier_id' => $access_role_id,
                        'job_desription' => $jobDescription,
                        'inductance_date' => $inductanceDate,
                    ]);
                }
            }
            $SwmsSafetyPlanningAnswersinputAnswers = json_decode($request->input('InputAnswers', '{}'), true);

            // Loop through each input answer and save or update in the database
            foreach ($SwmsSafetyPlanningAnswersinputAnswers as $noteId => $answer) {
                SwmsSafetyPlanningAnswers::updateOrCreate(
                    [
                        'swms_id' => $request->swms_report_id,
                        'swms_safety_planning_id' => $noteId
                    ],
                    [
                        'answer' => $answer
                    ]
                );
            }
            
            Swms::where('id', $swms_report_id)->update(['process' => '1']);
            $log = [
                'employee_id' => auth()->user()->id,
                'message' => 'SWMS Step 6 store Data saved successfully.' ,
                'report_id' =>  $request->swms_report_id,
                'report_type' => 'swms_report'
            ];  
            storeReportsLogs($log);
            return response()->json([
                'message' => 'Data saved successfully.',
                'data' => ['swms_report_id' => $swms_report_id],
            ], 200);
        } catch (Exception $e) {
            $shortMessage = substr($e->getMessage(), 0, 200) . (strlen($e->getMessage()) > 200 ? '...' : '');
            
            storeReportsLogs([
                'employee_id' => auth()->user()->id,
                'message' => 'SWMS Step 6 store error: ' . $shortMessage,
                'report_id' => $request->swms_report_id,
                'report_type' => 'swms_report',
                'error_type' => 'Exception error'
            ]);
            
            return $this->error('An error occurred: ' . $shortMessage);
        }
    }
    
    public function swmsStep6Store(Request $request)
    {
        try {
            $swms_report_id = $request->input('swms_report_id');
            // Check if a SWMS Signature record already exists for the given SWMS ID
            $swmsSignatureExists = SwmsSignature::where('swms_id', $swms_report_id)->exists();
            $validator = Validator::make($request->all(), [
                'swms_report_id' => 'required|integer',

            ]);
            $inductance_dates = (array) $request->input('inductance_dates', []);
            if ($inductance_dates || $inductance_dates != null) {
                $validator->sometimes('inductance_dates', 'required|array|min:1', function ($input) use ($swmsSignatureExists) {
                    return !$swmsSignatureExists;
                });

                $validator->sometimes('inductance_dates.*', 'required|date|after_or_equal:today', function ($input) use ($swmsSignatureExists) {
                    return !$swmsSignatureExists;
                });
            }
            $swms_report_id = $request->swms_report_id;
            
            $access_role_ids = (array) $request->input('access_role_ids', []);
            $job_descriptions = (array) $request->input('job_descriptions', []);
            $inductance_dates = (array) $request->input('inductance_dates', []);
            $record_ids = (array) $request->input('record_ids', []);
            // Check if the arrays have matching lengths
            if (count($access_role_ids) !== count($job_descriptions) || count($access_role_ids) !== count($inductance_dates) || count($access_role_ids) !== count($record_ids)) {
                return $this->message('Mismatched input  please complete all inputs.', 422);
            }
            // Loop through the input arrays or handle single items
            foreach ($access_role_ids as $index => $access_role_id) {
                $jobDescription = $job_descriptions[$index] ?? null;
                $inductanceDate = $inductance_dates[$index] ?? null;
                $recordId = $record_ids[$index] ?? null;
                if ($recordId) {
                    // Update existing record
                    $swmsSignature = SwmsSignature::where('swms_id', $swms_report_id)->find($recordId);
                    if ($swmsSignature) {
                        $swmsSignature->update([
                            'swms_id' => $swms_report_id,
                            'tier_id' => $access_role_id,
                            'job_desription' => $jobDescription,
                            'inductance_date' => $inductanceDate,
                        ]);
                    } else {
                        SwmsSignature::create([
                            'swms_id' => $swms_report_id,
                            'tier_id' => $access_role_id,
                            'job_desription' => $jobDescription,
                            'inductance_date' => $inductanceDate,
                        ]);
                    }
                } else {
                    // Create new record
                    SwmsSignature::create([
                        'swms_id' => $swms_report_id,
                        'tier_id' => $access_role_id,
                        'job_desription' => $jobDescription,
                        'inductance_date' => $inductanceDate,
                    ]);
                }
            }
            $SwmsSafetyPlanningAnswersinputAnswers = json_decode($request->input('InputAnswers', '{}'), true);
            // Loop through each input answer and save or update in the database
            foreach ($SwmsSafetyPlanningAnswersinputAnswers as $noteId => $answer) {
                SwmsSafetyPlanningAnswers::updateOrCreate(
                    [
                        'swms_id' => $request->swms_report_id,
                        'swms_safety_planning_id' => $noteId
                    ],
                    [
                        'answer' => $answer
                    ]
                );
            }
            Swms::where('id', $swms_report_id)->update(['process' => '1']);
            $log = [
                'employee_id' => auth()->user()->id,
                'message' => 'SWMS Step 6 store Data saved successfully.',
                'report_id' =>  $request->swms_report_id,
                'report_type' => 'swms_report'
            ];
            storeReportsLogs($log);
            return $this->sucess(['swms_report_id' => $swms_report_id] , 'Data saved successfully.');
        } catch (Exception $e) {
            $shortMessage = substr($e->getMessage(), 0, 200) . (strlen($e->getMessage()) > 200 ? '...' : '');
            $log = [
                'employee_id' => auth()->user()->id,
                'message' => 'SWMS Step 6 store error:' . $shortMessage,
                'report_id' =>  $request->swms_report_id,
                'report_type' => 'swms_report',
                'error_type' => 'Exception error'
            ];
            storeReportsLogs($log);
            return $this->error('An error occurred : ' . $shortMessage);
        }
    }

    public function swmsStep6Edit(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'swms_report_id' => 'required',
        ]);

        if ($validator->fails()) {
            $log = [
                'employee_id' => auth()->user()->id,
                'message' => 'SWMS Step 6 edit error:',
                'report_id' =>  $request->swms_report_id,
                'report_type' => 'swms_report',
                'error_type' => 'Validation error',
            ];
            storeReportsLogs($log);
            return response()->json([
                'message' => $validator->errors()->first(),
            ], 422);
        }
        try {
            $data['SwmsSignature'] = SwmsSignature::where('swms_id', $request->swms_report_id)->get();
            $data['SwmsSafetyPlanningQuestion'] = SwmsSafetyPlanningQuestion::get();
            $data['SwmsSafetyPlanningAnswers'] = SwmsSafetyPlanningAnswers::where('swms_id', $request->swms_report_id)->get();


            // $authorised_by_list = EmpCompanyDetails::where('compeleted', '1')
            //     ->where('approved', '1')
            //     ->where('status', '1')
            //     ->where('del', '0')
            //     ->with(
            //         [
            //             'empPersonalDetails' => function ($query) {
            //                 $query->select('emp_id', 'first_name', 'middle_name', 'last_name');
            //             },
            //             'accessRole' => function ($query) {
            //                 $query->select('id', 'title', 'code');
            //             },
            //             'empTier' => function ($query) {
            //                 $query->select('id', 'title');
            //             },
            //         ]
            //     )
            //     ->select('id', 'access_role', 'tier_id')
            //     ->get();
            $role = DB::table('roles')
                ->where('del', '0')
                ->get();
            $data['role'] = $role;
            // dd($role);
            return response()->json([
                'message' => 'Get successfully.',
                'data' => $data,
            ], 200);
        } catch (Exception $e) {
            $shortMessage = substr($e->getMessage(), 0, 200) . (strlen($e->getMessage()) > 200 ? '...' : '');
            $log = [
                'employee_id' => auth()->user()->id,
                'message' => 'SWMS Step 6 store error:' . $shortMessage,
                'report_id' =>  $request->swms_report_id,
                'report_type' => 'swms_report',
                'error_type' => 'Exception error'
            ];
            storeReportsLogs($log);
            return response()->json([
                'message' => 'An error occurred : ' . $shortMessage,
            ], 500);
        }
    }
    public function swmsStep6SignatureDelete($id)
    {
        try {
            SwmsSignature::where('id', $id)->delete();
            $log = [
                'employee_id' => auth()->user()->id,
                'message' => 'SWMS Step 6 Signature with ID ' . $id . ' deleted:',
                'report_id' =>  $request->swms_report_id,
                'report_type' => 'swms_report',
                'error_type' => 'Validation error',
            ];
            storeReportsLogs($log);
            return response()->json([
                'message' => 'Deleted Successfully. ',
            ], 200);
        } catch (Exception $e) {
            $shortMessage = substr($e->getMessage(), 0, 200) . (strlen($e->getMessage()) > 200 ? '...' : '');
            $log = [
                'employee_id' => auth()->user()->id,
                'message' => 'SWMS Step 6 store error:' . $shortMessage,
                'report_id' =>  $request->swms_report_id,
                'report_type' => 'swms_report',
                'error_type' => 'Exception error'
            ];
            storeReportsLogs($log);
            return response()->json([
                'message' => 'An error occurred : ' . $shortMessage,
            ], 500);
        }
    }


    // PDF
    public function swmsReportDownloadPdf($id)
    {

        if (!$id) {
            $log = [
                'employee_id' => auth()->user()->id,
                'message' => 'SWMS Report Download ID is required :',
                'report_id' => 0,
                'report_type' => 'swms_report',
                'error_type' => 'Validation error',
            ];
            storeReportsLogs($log);
            return response()->json([
                'message' => 'Report ID is Required.'
            ], 422);
        }
        try {
            $swms_data = DB::table('swms')
                ->leftJoin('sites', 'swms.site_id', '=', 'sites.id')

                ->leftJoin('swms_metas', 'swms.id', '=', 'swms_metas.swms_id')
                ->leftJoin('company', 'swms.company_name', '=', 'company.id')
                ->leftJoin('emp_personal_details as swms_site_manager_or_forman', 'swms.site_manager_or_forman', '=', 'swms_site_manager_or_forman.emp_id')
                ->leftJoin('emp_personal_details as emp_personal_supervisor_detail', 'sites.supervisor_id', '=', 'emp_personal_supervisor_detail.emp_id')
                ->leftJoin('emp_personal_details as emp_personal_forman_detail', 'sites.forman_id', '=', 'emp_personal_forman_detail.emp_id')

                ->where('swms.id', "=", $id)
                ->select(
                    'swms.*',
                    'sites.title as sites_title',
                    'swms_metas.*',
                    'sites.supervisor_id as site_supervisor_id',
                    'swms_site_manager_or_forman.first_name as swms_site_manager_or_forman_first_name',
                    'swms_site_manager_or_forman.last_name as swms_site_manager_or_forman_last_name',
                    'company.fname as compnay_fname',
                    'company.mname as compnay_mname',
                    'company.lname as compnay_lname',
                    'emp_personal_supervisor_detail.first_name as emp_personal_supervisor_detail_f_name',
                    'emp_personal_supervisor_detail.last_name as emp_personal_supervisor_detail_last_name',
                    'emp_personal_forman_detail.first_name as emp_personal_forman_detail_f_name',
                    'emp_personal_forman_detail.last_name as emp_personal_forman_detail_last_name',

                )
                ->first();

            $swms_data_forman = DB::table('swms')
                ->leftJoin('swms_metas', 'swms.id', '=', 'swms_metas.swms_id')
                ->leftJoin('company', 'swms.company_name', '=', 'company.id')
                ->leftJoin('emp_personal_details as swms_site_manager_or_forman', function ($join) {
                    $join->on(DB::raw("FIND_IN_SET(swms_site_manager_or_forman.emp_id, swms.site_manager_or_forman)"), '>', DB::raw('0'));
                })
                ->where('swms.id', "=", $id)
                ->select(
                    'swms.*',
                    'swms_metas.*',
                    'swms_site_manager_or_forman.first_name as swms_site_manager_or_forman_first_name',
                    'swms_site_manager_or_forman.last_name as swms_site_manager_or_forman_last_name',
                    'company.fname as compnay_fname',
                    'company.mname as compnay_mname',
                    'company.lname as compnay_lname',
                )
                ->get();

            $swms_developed_bies_data = DB::table('swms_developed_bies')
                ->leftJoin('emp_personal_details as emp_personal_details_developed_bies', 'swms_developed_bies.employee_id', '=', 'emp_personal_details_developed_bies.emp_id')

                ->where('swms_developed_bies.swms_id', "=", $id)
                ->select(
                    'swms_developed_bies.signature as swms_developed_bies_signature',
                    'swms_developed_bies.date as swms_developed_bies_date',

                    'emp_personal_details_developed_bies.first_name as emp_personal_details_developed_bies_first_name',
                    'emp_personal_details_developed_bies.last_name as emp_personal_details_developed_bies_last_name',

                )
                ->get();
            $swms_approved_bies_data = DB::table('swms_approved_bies')
                ->leftJoin('emp_personal_details as emp_personal_details_approved_bies', 'swms_approved_bies.employee_id', '=', 'emp_personal_details_approved_bies.emp_id')

                ->where('swms_approved_bies.swms_id', "=", $id)
                ->select(
                    'swms_approved_bies.signature as swms_approved_bies_signature',
                    'swms_approved_bies.date as swms_approved_bies_date',

                    'emp_personal_details_approved_bies.first_name as emp_personal_details_approved_bies_first_name',
                    'emp_personal_details_approved_bies.last_name as emp_personal_details_approved_bies_last_name',

                )
                ->get();
            $swms_approved_bies_data = DB::table('swms_approved_bies')
                ->leftJoin('emp_personal_details as emp_personal_details_approved_bies', 'swms_approved_bies.employee_id', '=', 'emp_personal_details_approved_bies.emp_id')

                ->where('swms_approved_bies.swms_id', "=", $id)
                ->select(
                    'swms_approved_bies.signature as swms_approved_bies_signature',
                    'swms_approved_bies.date as swms_approved_bies_date',

                    'emp_personal_details_approved_bies.first_name as emp_personal_details_approved_bies_first_name',
                    'emp_personal_details_approved_bies.last_name as emp_personal_details_approved_bies_last_name',

                )
                ->get();
            $swms_review_responsibles_data = DB::table('swms_review_responsibles')
                ->leftJoin('emp_personal_details as emp_personal_details_review_responsibles', 'swms_review_responsibles.employee_id', '=', 'emp_personal_details_review_responsibles.emp_id')

                ->where('swms_review_responsibles.swms_id', "=", $id)
                ->select(
                    'swms_review_responsibles.signature as swms_review_responsibles_signature',
                    'swms_review_responsibles.date as swms_review_responsibles_date',
                    'emp_personal_details_review_responsibles.first_name as emp_personal_details_review_responsibles_first_name',
                    'emp_personal_details_review_responsibles.last_name as emp_personal_details_review_responsibles_last_name',

                )
                ->get();
            $swms_monitoring_data = DB::table('swms_monitoring_responsibles')
                ->leftJoin('emp_personal_details as emp_personal_details_monitoring_responsibles', 'swms_monitoring_responsibles.employee_id', '=', 'emp_personal_details_monitoring_responsibles.emp_id')
                ->where('swms_monitoring_responsibles.swms_id', "=", $id)
                ->select(
                    'swms_monitoring_responsibles.signature as swms_monitoring_responsibles_signature',
                    'swms_monitoring_responsibles.date as swms_monitoring_responsibles_date',

                    'emp_personal_details_monitoring_responsibles.first_name as emp_personal_details_monitoring_responsibles_first_name',
                    'emp_personal_details_monitoring_responsibles.last_name as emp_personal_details_monitoring_responsibles_last_name',

                )
                ->get();
            $swms_compliance_responsibles_data = DB::table('swms_compliance_responsibles')
                ->leftJoin('emp_personal_details as emp_personal_details_compliance_responsibles', 'swms_compliance_responsibles.employee_id', '=', 'emp_personal_details_compliance_responsibles.emp_id')

                ->where('swms_compliance_responsibles.swms_id', "=", $id)
                ->select(
                    'swms_compliance_responsibles.signature as swms_compliance_responsibles_signature',
                    'swms_compliance_responsibles.date as swms_compliance_responsibles_date',

                    'emp_personal_details_compliance_responsibles.first_name as emp_personal_details_compliance_responsibles_first_name',
                    'emp_personal_details_compliance_responsibles.last_name as emp_personal_details_compliance_responsibles_last_name',

                )
                ->get();
            $hazard_risks_data = DB::table('swms_hazard_risk_pivots')
                ->leftJoin('hazard_risks as hazard_risks', 'swms_hazard_risk_pivots.hazard_risks_id', '=', 'hazard_risks.id')

                ->where('swms_hazard_risk_pivots.swms_id', "=", $id)
                ->select(
                    'hazard_risks.title as hazard_risks_title',
                    'hazard_risks.description as hazard_risks_description',

                )
                ->get();
            $swms_task_details_data = DB::table('swms_task_details')
                ->leftJoin('emp_personal_details as swms_task_details_personal_details', 'swms_task_details.responsible_preson_id', '=', 'swms_task_details_personal_details.emp_id')

                ->where('swms_task_details.swms_id', "=", $id)
                ->select(
                    'swms_task_details.*',

                    'swms_task_details_personal_details.first_name as swms_task_details_personal_details_first_name',
                    'swms_task_details_personal_details.last_name as swms_task_details_personal_details_last_name',

                )
                ->get();
            $swms_protection_tool_pivots_data = DB::table('swms_protection_tool_pivots')
                ->leftJoin('swms_protection_tools', 'swms_protection_tool_pivots.protection_tool_id', '=', 'swms_protection_tools.id')
                ->where('swms_protection_tool_pivots.swms_id', "=", $id)
                ->select(
                    'swms_protection_tools.*',
                )
                ->get();
            $swms_safety_planning_answers_data = DB::table('swms_safety_planning_answers')
                ->leftJoin('swms_safety_planning_questions', 'swms_safety_planning_answers.swms_safety_planning_id', '=', 'swms_safety_planning_questions.id')
                ->where('swms_safety_planning_answers.swms_id', "=", $id)
                ->select(
                    'swms_safety_planning_questions.*',
                    'swms_safety_planning_answers.answer as swms_safety_planning_answers_answer'
                )
                ->get();
            $swms_questions_answers_data = DB::table('swms_awnswers')
                ->leftJoin('swms_questions', 'swms_awnswers.swms_question_id', '=', 'swms_questions.id')

                ->where('swms_awnswers.swms_id', "=", $id)
                ->select(
                    'swms_questions.*',
                    'swms_awnswers.*'
                )
                ->get();
            $swms_signatures_data = DB::table('swms_signatures')
                ->leftJoin('swms', 'swms_signatures.swms_id', '=', 'swms.id')
                ->leftJoin('roles', 'swms_signatures.tier_id', '=', 'roles.id')

                ->where('swms_signatures.swms_id', "=", $id)
                ->select(
                    'swms_signatures.*',
                    'roles.title as roles_signatures_title',

                )
                ->get();
            $swms_general_notes_data = DB::table('swms_general_note_answers')
                ->leftJoin('swms_general_notes', 'swms_general_note_answers.swms_general_notes_id', '=', 'swms_general_notes.id')

                ->where('swms_general_note_answers.swms_id', "=", $id)
                ->select(
                    'swms_general_notes.*',
                    'swms_general_note_answers.*'

                )
                ->get();
            $supervisorIds = explode(',', $swms_data->site_supervisor_id);
            $supervisors = DB::table('emp_personal_details')->whereIn('emp_id', $supervisorIds)
                ->select(
                    'emp_personal_details.first_name as emp_personal_supervisor_detail_f_name',
                    'emp_personal_details.last_name as emp_personal_supervisor_detail_last_name'
                )
                ->get();
            $viewPath = 'Swms.NewSwmsReport.PdfReport.swmsReport';
            $html = view($viewPath, compact(
                'swms_data',
                'swms_developed_bies_data',
                'swms_approved_bies_data',
                'swms_review_responsibles_data',
                'swms_monitoring_data',
                'swms_compliance_responsibles_data',
                'hazard_risks_data',
                'swms_data_forman',
                'swms_task_details_data',
                'swms_protection_tool_pivots_data',
                'swms_safety_planning_answers_data',
                'swms_questions_answers_data',
                'swms_signatures_data',
                'swms_general_notes_data',
                'supervisors'
            ))->render();

            $report_title = isset($swms_data->title)
                ? preg_replace('/[^A-Za-z0-9_]+/', '_', trim(str_replace(' ', '_', $swms_data->title)))
                : 'No_named';
            $report_swms_number = isset($swms_data->swms_number)
                ? preg_replace('/[^A-Za-z0-9_]+/', '', trim(str_replace(' ', '_', $swms_data->swms_number)))
                : 'NO_Swms';

            $report_version_number = isset($swms_data->version_number)
                ? preg_replace('/[^A-Za-z0-9_]+/', '', trim(str_replace(' ', '_', $swms_data->version_number)))
                : 'No_Version';

            $filename =  $report_title . "N" . $report_swms_number . "R" . $report_version_number . ".pdf";

            $path = public_path("Files_manager/swms_reports/{$filename}");
            $report_full_path = public_path('Files_manager/swms_reports/' . $filename);
            $already_file_status = GeneratedPdfReport::where('report_type', 'swms_report')->where('report_id', $id)->where('path', url('Files_manager/swms_reports/' . $filename))->first();

            if (file_exists($report_full_path) && $already_file_status && $already_file_status->path == url('Files_manager/swms_reports/' . $filename)) {
                return response()->download($report_full_path, $filename);
            }

            $pdf = PDF::loadHTML($html)
                ->setPaper('A4', 'portrait');
            $pdf->save($path);
            $generated_report = new GeneratedPdfReport();
            $generated_report->report_id = $id;
            $generated_report->path = url('Files_manager/swms_reports/' . $filename);
            $generated_report->report_type = 'swms_report';
            $generated_report->save();
            //  dd( $generated_report);


            return response()->download($path, $filename);
        } catch (Exception $e) {
            $shortMessage = substr($e->getMessage(), 0, 200) . (strlen($e->getMessage()) > 200 ? '...' : '');
            $log = [
                'employee_id' => auth()->user()->id,
                'message' => 'SWMS step download error:' . $shortMessage,
                'report_id' =>  $id,
                'report_type' => 'swms_report',
                'error_type' => 'Exception error'
            ];
            storeReportsLogs($log);
            return response()->json([
                'message' => 'An error occurred : ' . $shortMessage,
            ], 500);
        }
    }
    public function swmsReportPdfFilesManager(Request $request)
    {
        // Get the current page
        $currentPage = Paginator::resolveCurrentPage();

        // Number of items per page
        $perPage = 7;

        // Fetch SWMS report files from the database
        $query = GeneratedPdfReport::where('report_type', 'swms_report');

        // Count total items
        $totalReports = $query->count();

        // Get the reports for the current page
        $reports = $query->skip(($currentPage - 1) * $perPage)->take($perPage)->get();

        $paginator = new LengthAwarePaginator(
            $reports,
            $totalReports,
            $perPage,
            $currentPage,
            ['path' => Paginator::resolveCurrentPath()]
        );

        // Return the view with the paginator
        return view('FileManger.SwmsReportPdfFiles', ['files' => $paginator]);
    }

    public function downloadSwmsPdfFile($filename)
    {
        $path = public_path("Files_manager/swms_reports/");
        if (!file_exists($path . $filename)) {
            abort(404, 'SWMS File not found.');
        }
        return response()->download($path . $filename);
    }
    public function swmsReportPublishReport($id)
    {

        $swm = Swms::find($id);
        if ($swm) {
            $swm->published = $swm->published == '1' ? '0' : '1';
            $swm->save();
            $log = [
                'employee_id' => auth()->user()->id,
                'message' => 'Report status updated successfully',
                'report_id' =>  0,
                'report_type' => 'swms_report'
            ];
            storeReportsLogs($log);
            if ($swm->published == '1') {
                $this->swmsReportDownloadPdf($id);
            } else {
                return response()->json([
                    'message' => 'Report status updated successfully.'
                ], 200);
            }
        } else {
            $log = [
                'employee_id' => auth()->user()->id,
                'message' => 'Report Not Found please try again.',
                'report_id' =>  0,
                'report_type' => 'swms_report',
                'error_type' => 'Not found error',
            ];
            storeReportsLogs($log);
            return response()->json([
                'message' => 'Report Not Found please try again.'
            ], 433);
        }
    }
    public function swmsReportDownloadPdfRegenerate($id)
    {

        if (!$id) {
            $log = [
                'employee_id' => auth()->user()->id,
                'message' => 'SWMS Report Download ID is required :',
                'report_id' => 0,
                'report_type' => 'swms_report',
                'error_type' => 'Validation error',
            ];
            storeReportsLogs($log);
            return response()->json([
                'message' => 'Report ID is Required.'
            ], 422);
        }
        try {
            $swms_data = DB::table('swms')
                ->leftJoin('swms_metas', 'swms.id', '=', 'swms_metas.swms_id')
                ->leftJoin('sites', 'swms.site_id', '=', 'sites.id')
                ->leftJoin('company', 'swms.company_name', '=', 'company.id')
                ->leftJoin('emp_personal_details as swms_site_manager_or_forman', 'swms.site_manager_or_forman', '=', 'swms_site_manager_or_forman.emp_id')
                ->leftJoin('emp_personal_details as emp_personal_supervisor_detail', 'sites.supervisor_id', '=', 'emp_personal_supervisor_detail.emp_id')
                ->leftJoin('emp_personal_details as emp_personal_forman_detail', 'sites.forman_id', '=', 'emp_personal_forman_detail.emp_id')

                ->where('swms.id', "=", $id)
                ->select(
                    'swms.*',
                    'sites.title as sites_title',
                    'swms_metas.*',
                    'sites.supervisor_id as site_supervisor_id',
                    'swms_site_manager_or_forman.first_name as swms_site_manager_or_forman_first_name',
                    'swms_site_manager_or_forman.last_name as swms_site_manager_or_forman_last_name',
                    'company.fname as compnay_fname',
                    'company.mname as compnay_mname',
                    'company.lname as compnay_lname',
                    'emp_personal_supervisor_detail.first_name as emp_personal_supervisor_detail_f_name',
                    'emp_personal_supervisor_detail.last_name as emp_personal_supervisor_detail_last_name',
                    'emp_personal_forman_detail.first_name as emp_personal_forman_detail_f_name',
                    'emp_personal_forman_detail.last_name as emp_personal_forman_detail_last_name',



                )
                ->first();

            $swms_data_forman = DB::table('swms')
                ->leftJoin('swms_metas', 'swms.id', '=', 'swms_metas.swms_id')
                ->leftJoin('company', 'swms.company_name', '=', 'company.id')
                ->leftJoin('emp_personal_details as swms_site_manager_or_forman', function ($join) {
                    $join->on(DB::raw("FIND_IN_SET(swms_site_manager_or_forman.emp_id, swms.site_manager_or_forman)"), '>', DB::raw('0'));
                })
                ->where('swms.id', "=", $id)
                ->select(
                    'swms.*',
                    'swms_metas.*',
                    'swms_site_manager_or_forman.first_name as swms_site_manager_or_forman_first_name',
                    'swms_site_manager_or_forman.last_name as swms_site_manager_or_forman_last_name',
                    'company.fname as compnay_fname',
                    'company.mname as compnay_mname',
                    'company.lname as compnay_lname',
                )
                ->get();

            $swms_developed_bies_data = DB::table('swms_developed_bies')
                ->leftJoin('emp_personal_details as emp_personal_details_developed_bies', 'swms_developed_bies.employee_id', '=', 'emp_personal_details_developed_bies.emp_id')

                ->where('swms_developed_bies.swms_id', "=", $id)
                ->select(
                    'swms_developed_bies.signature as swms_developed_bies_signature',
                    'swms_developed_bies.date as swms_developed_bies_date',

                    'emp_personal_details_developed_bies.first_name as emp_personal_details_developed_bies_first_name',
                    'emp_personal_details_developed_bies.last_name as emp_personal_details_developed_bies_last_name',

                )
                ->get();
            $swms_approved_bies_data = DB::table('swms_approved_bies')
                ->leftJoin('emp_personal_details as emp_personal_details_approved_bies', 'swms_approved_bies.employee_id', '=', 'emp_personal_details_approved_bies.emp_id')

                ->where('swms_approved_bies.swms_id', "=", $id)
                ->select(
                    'swms_approved_bies.signature as swms_approved_bies_signature',
                    'swms_approved_bies.date as swms_approved_bies_date',

                    'emp_personal_details_approved_bies.first_name as emp_personal_details_approved_bies_first_name',
                    'emp_personal_details_approved_bies.last_name as emp_personal_details_approved_bies_last_name',

                )
                ->get();
            $swms_approved_bies_data = DB::table('swms_approved_bies')
                ->leftJoin('emp_personal_details as emp_personal_details_approved_bies', 'swms_approved_bies.employee_id', '=', 'emp_personal_details_approved_bies.emp_id')

                ->where('swms_approved_bies.swms_id', "=", $id)
                ->select(
                    'swms_approved_bies.signature as swms_approved_bies_signature',
                    'swms_approved_bies.date as swms_approved_bies_date',

                    'emp_personal_details_approved_bies.first_name as emp_personal_details_approved_bies_first_name',
                    'emp_personal_details_approved_bies.last_name as emp_personal_details_approved_bies_last_name',

                )
                ->get();
            $swms_review_responsibles_data = DB::table('swms_review_responsibles')
                ->leftJoin('emp_personal_details as emp_personal_details_review_responsibles', 'swms_review_responsibles.employee_id', '=', 'emp_personal_details_review_responsibles.emp_id')

                ->where('swms_review_responsibles.swms_id', "=", $id)
                ->select(
                    'swms_review_responsibles.signature as swms_review_responsibles_signature',
                    'swms_review_responsibles.date as swms_review_responsibles_date',

                    'emp_personal_details_review_responsibles.first_name as emp_personal_details_review_responsibles_first_name',
                    'emp_personal_details_review_responsibles.last_name as emp_personal_details_review_responsibles_last_name',

                )
                ->get();
            $swms_monitoring_data = DB::table('swms_monitoring_responsibles')
                ->leftJoin('emp_personal_details as emp_personal_details_monitoring_responsibles', 'swms_monitoring_responsibles.employee_id', '=', 'emp_personal_details_monitoring_responsibles.emp_id')
                ->where('swms_monitoring_responsibles.swms_id', "=", $id)
                ->select(
                    'swms_monitoring_responsibles.signature as swms_monitoring_responsibles_signature',
                    'swms_monitoring_responsibles.date as swms_monitoring_responsibles_date',

                    'emp_personal_details_monitoring_responsibles.first_name as emp_personal_details_monitoring_responsibles_first_name',
                    'emp_personal_details_monitoring_responsibles.last_name as emp_personal_details_monitoring_responsibles_last_name',

                )
                ->get();
            $swms_review_responsibles_data = DB::table('swms_review_responsibles')
                ->leftJoin('emp_personal_details as emp_personal_details_review_responsibles', 'swms_review_responsibles.employee_id', '=', 'emp_personal_details_review_responsibles.emp_id')

                ->where('swms_review_responsibles.swms_id', "=", $id)
                ->select(
                    'swms_review_responsibles.signature as swms_review_responsibles_signature',
                    'swms_review_responsibles.date as swms_review_responsibles_date',

                    'emp_personal_details_review_responsibles.first_name as emp_personal_details_review_responsibles_first_name',
                    'emp_personal_details_review_responsibles.last_name as emp_personal_details_review_responsibles_last_name',

                )
                ->get();
            $swms_compliance_responsibles_data = DB::table('swms_compliance_responsibles')
                ->leftJoin('emp_personal_details as emp_personal_details_compliance_responsibles', 'swms_compliance_responsibles.employee_id', '=', 'emp_personal_details_compliance_responsibles.emp_id')

                ->where('swms_compliance_responsibles.swms_id', "=", $id)
                ->select(
                    'swms_compliance_responsibles.signature as swms_compliance_responsibles_signature',
                    'swms_compliance_responsibles.date as swms_compliance_responsibles_date',

                    'emp_personal_details_compliance_responsibles.first_name as emp_personal_details_compliance_responsibles_first_name',
                    'emp_personal_details_compliance_responsibles.last_name as emp_personal_details_compliance_responsibles_last_name',

                )
                ->get();
            $hazard_risks_data = DB::table('swms_hazard_risk_pivots')
                ->leftJoin('hazard_risks as hazard_risks', 'swms_hazard_risk_pivots.hazard_risks_id', '=', 'hazard_risks.id')

                ->where('swms_hazard_risk_pivots.swms_id', "=", $id)
                ->select(
                    'hazard_risks.title as hazard_risks_title',
                    'hazard_risks.description as hazard_risks_description',

                )
                ->get();
            $swms_task_details_data = DB::table('swms_task_details')
                ->leftJoin('emp_personal_details as swms_task_details_personal_details', 'swms_task_details.responsible_preson_id', '=', 'swms_task_details_personal_details.emp_id')

                ->where('swms_task_details.swms_id', "=", $id)
                ->select(
                    'swms_task_details.*',

                    'swms_task_details_personal_details.first_name as swms_task_details_personal_details_first_name',
                    'swms_task_details_personal_details.last_name as swms_task_details_personal_details_last_name',

                )
                ->get();
            $swms_protection_tool_pivots_data = DB::table('swms_protection_tool_pivots')
                ->leftJoin('swms_protection_tools', 'swms_protection_tool_pivots.protection_tool_id', '=', 'swms_protection_tools.id')
                ->where('swms_protection_tool_pivots.swms_id', "=", $id)
                ->select(
                    'swms_protection_tools.*',
                )
                ->get();
            $swms_safety_planning_answers_data = DB::table('swms_safety_planning_answers')
                ->leftJoin('swms_safety_planning_questions', 'swms_safety_planning_answers.swms_safety_planning_id', '=', 'swms_safety_planning_questions.id')
                ->where('swms_safety_planning_answers.swms_id', "=", $id)
                ->select(
                    'swms_safety_planning_questions.*',
                    'swms_safety_planning_answers.answer as swms_safety_planning_answers_answer'
                )
                ->get();
            $swms_questions_answers_data = DB::table('swms_awnswers')
                ->leftJoin('swms_questions', 'swms_awnswers.swms_question_id', '=', 'swms_questions.id')

                ->where('swms_awnswers.swms_id', "=", $id)
                ->select(
                    'swms_questions.*',
                    'swms_awnswers.*'
                )
                ->get();
            $swms_signatures_data = DB::table('swms_signatures')
                ->leftJoin('swms', 'swms_signatures.swms_id', '=', 'swms.id')
                ->leftJoin('roles', 'swms_signatures.tier_id', '=', 'roles.id')

                ->where('swms_signatures.swms_id', "=", $id)
                ->select(
                    'swms_signatures.*',
                    'roles.title as roles_signatures_title',

                )
                ->get();
            $swms_general_notes_data = DB::table('swms_general_note_answers')
                ->leftJoin('swms_general_notes', 'swms_general_note_answers.swms_general_notes_id', '=', 'swms_general_notes.id')

                ->where('swms_general_note_answers.swms_id', "=", $id)
                ->select(
                    'swms_general_notes.*',
                    'swms_general_note_answers.*'

                )
                ->get();
            $supervisorIds = explode(',', $swms_data->site_supervisor_id);
            $supervisors = DB::table('emp_personal_details')->whereIn('emp_id', $supervisorIds)
                ->select(
                    'emp_personal_details.first_name as emp_personal_supervisor_detail_f_name',
                    'emp_personal_details.last_name as emp_personal_supervisor_detail_last_name'
                )
                ->get();
            $viewPath = 'Swms.NewSwmsReport.PdfReport.swmsReport';
            $html = view($viewPath, compact(
                'swms_data',
                'swms_developed_bies_data',
                'swms_approved_bies_data',
                'swms_review_responsibles_data',
                'swms_monitoring_data',
                'swms_compliance_responsibles_data',
                'hazard_risks_data',
                'swms_task_details_data',
                'swms_protection_tool_pivots_data',
                'swms_safety_planning_answers_data',
                'swms_data_forman',
                'swms_questions_answers_data',
                'swms_signatures_data',
                'swms_general_notes_data',
                'supervisors'
            ))->render();

            $report_title = isset($swms_data->title)
                ? preg_replace('/[^A-Za-z0-9_]+/', '_', trim(str_replace(' ', '_', $swms_data->title)))
                : 'No_named';
            $report_swms_number = isset($swms_data->swms_number)
                ? preg_replace('/[^A-Za-z0-9_]+/', '', trim(str_replace(' ', '_', $swms_data->swms_number)))
                : 'NO_Swms';

            $report_version_number = isset($swms_data->version_number)
                ? preg_replace('/[^A-Za-z0-9_]+/', '', trim(str_replace(' ', '_', $swms_data->version_number)))
                : 'No_Version';

            $filename =  $report_title . "N" . $report_swms_number . "R" . $report_version_number . ".pdf";

            $path = public_path("Files_manager/swms_reports/{$filename}");
            $report_full_path = public_path('Files_manager/swms_reports/' . $filename);
            $already_file_status = GeneratedPdfReport::where('report_type', 'swms_report')->where('report_id', $id)->where('path', url('Files_manager/swms_reports/' . $filename))->first();
            if (file_exists($report_full_path) && $already_file_status && $already_file_status->path == url('Files_manager/swms_reports/' . $filename)) {
                $already_file_status = GeneratedPdfReport::where('report_type', 'swms_report')->where('report_id', $id)->where('path', url('Files_manager/swms_reports/' . $filename))->delete();
                unlink($report_full_path);
            }

            $pdf = PDF::loadHTML($html)
                ->setPaper('A4', 'portrait');
            $pdf->save($path);

            GeneratedPdfReport::updateOrCreate(
                ['report_id' => $id],
                [
                    'report_id' => $id,
                    'path' => url('Files_manager/swms_reports/' . $filename),
                    'report_type' => 'swms_report',
                ]
            );

            return response()->download($path, $filename);
        } catch (Exception $e) {
            $shortMessage = substr($e->getMessage(), 0, 200) . (strlen($e->getMessage()) > 200 ? '...' : '');
            $log = [
                'employee_id' => auth()->user()->id,
                'message' => 'SWMS step download error:' . $shortMessage,
                'report_id' =>  $id,
                'report_type' => 'swms_report',
                'error_type' => 'Exception error'
            ];
            storeReportsLogs($log);
            return response()->json([
                'message' => 'An error occurred : ' . $shortMessage,
            ], 500);
        }
    }

    public function swmsReportViewSignature($id)
    {

        if ($id == "") {
            Session::put('error', 'Invalid Request');
            return redirect()->back();
        }
        $swms_data_forman = DB::table('swms')
            ->leftJoin('emp_personal_details as authorised_by', function ($join) {
                $join->on(DB::raw("FIND_IN_SET(authorised_by.emp_id, swms.site_manager_or_forman)"), '>', DB::raw('0'));
            })
            ->leftJoin('company as company', 'swms.company_name', '=', 'company.id')
            ->where('swms.id', "=", $id)
            ->select(
                'swms.*',
                'company.fname as company_first_name',
                'company.lname as company_last_name',
                'authorised_by.first_name as first_name',
                'authorised_by.last_name as last_name'
            )
            ->get();

        $swms_data = DB::table('swms')
            ->leftJoin('emp_personal_details as authorised_by', function ($join) {
                $join->on(DB::raw("FIND_IN_SET(authorised_by.emp_id, swms.site_manager_or_forman)"), '>', DB::raw('0'));
            })->leftJoin('company as company', 'swms.company_name', '=', 'company.id')
            ->where('swms.id', "=", $id)
            ->select(
                'swms.*',
                'company.fname as company_first_name',
                'company.lname as company_last_name',
                'authorised_by.first_name as first_name',
                'authorised_by.last_name as last_name',
            )
            ->first();

        $whs_signature_data = DB::table('swms_signatures')
            ->leftJoin('roles as roles', 'swms_signatures.tier_id', '=', 'roles.id')
            ->leftJoin('emp_company_details as emp_company_details', 'roles.code', '=', 'emp_company_details.access_role')
            ->leftJoin('emp_personal_details as emp_personal_details', 'emp_company_details.id', '=', 'emp_personal_details.emp_id')
            ->where('swms_signatures.swms_id', "=", $id)

            ->select(
                'swms_signatures.*',
                'emp_personal_details.first_name as first_name',
                'emp_personal_details.last_name as last_name',

            )
            ->get();
        return view('Swms.NewSwmsReport.swms_signature', compact('swms_data', 'whs_signature_data', 'swms_data_forman'));
    }
    public function swmsStep7Store(Request $request)
    {
        $isUpdate = $request->swms_report_id;
        $message = $isUpdate ? 'Updated Successfully' : 'Saved Successfully';
        try {
            $data = [
                'status' => 1,
            ];
            // Update or create the SWMS record
            Swms::where('id', $request->swms_report_id)->update(['process' => '1']);

            $data['swms_report_id'] = "";
            $log = [
                'employee_id' => auth()->user()->id,
                'message' => 'Swms  Step 5 :' . $message,
                'report_id' =>   $request->swms_report_id,
                'report_type' => 'swms_report'
            ];
            storeReportsLogs($log);
            return response()->json([
                'message' => $message,
                'data' => $data,
            ], 200);
        } catch (\Exception $e) {
            // Handle exceptions and return an error response
            $shortMessage = substr($e->getMessage(), 0, 200) . (strlen($e->getMessage()) > 200 ? '...' : '');
            $log = [
                'employee_id' => auth()->user()->id,
                'message' => 'Swms Step 5 error:' . $shortMessage,
                'report_id' =>  $request->swms_report_id,
                'report_type' => 'swms_report'
            ];
            storeReportsLogs($log);
            return response()->json([
                'message' => 'An error occurred while saving  data: ' . $e->getMessage(),
            ], 500);
        }
    }

    public function getImages(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'swms_report_id' => 'required',
        ]);

        if ($validator->fails()) {
            $log = [
                'employee_id' => auth()->user()->id,
                'message' => 'WHS  Step 3 store:' . $validator->errors()->first(),
                'report_id' =>   $request->swms_report_id,
                'report_type' => 'swms_report',
                'error_type' => 'Validation error',
            ];
            storeReportsLogs($log);
            return response()->json([
                'message' => $validator->errors()->first()
            ], 422);
        }
        $images = SwmsImage::where('swms_id', $request->swms_report_id)->get();
        return response()->json([
            'success' => 'Image uploaded successfully!',
            'images' => $images  // List of all images
        ]);
    }

    public function imageStore(Request $request)
    {

        $validator = Validator::make($request->all(), [
            'swms_report_id' => 'required',
            'image' => 'required|image|mimes:jpeg,png,jpg,gif,svg|max:2048',

        ]);

        if ($validator->fails()) {
            $log = [
                'employee_id' => auth()->user()->id,
                'message' => 'WHS  Step 3 store:' . $validator->errors()->first(),
                'report_id' =>   $request->whs_report_id,
                'report_type' => 'swms_report',
                'error_type' => 'Validation error',
            ];
            storeReportsLogs($log);
            return response()->json([
                'message' => $validator->errors()->first()
            ], 422);
        }
        ////////////////////////////////////
        // Validate the uploaded file

        // Store the image in the public storage
        $imagePath = $request->file('image')->store('images', 'public');

        // Save the image path in the database
        $image = new SwmsImage();
        $image->image = $imagePath;
        $image->swms_id = $request->swms_report_id;
        $image->save();

        // Fetch all images from the database to return the updated list
        $images = SwmsImage::where('swms_id', $request->swms_report_id)->get();

        // Return a JSON response with the uploaded image URL and the updated image list
        return response()->json([
            'success' => 'Image uploaded successfully!',
            'image_url' => asset('storage/' . $imagePath),
            'images' => $images  // List of all images
        ]);
    }

    public function deleteImage($id)
    {
        // Find the image by ID
        $image = SwmsImage::find($id);

        // Check if the image exists
        if (!$image) {
            return response()->json(['success' => false, 'message' => 'Image not found.']);
        }

        // Delete the image file from storage
        $imagePath = storage_path('app/public/' . $image->image);
        if (file_exists($imagePath)) {
            unlink($imagePath);  // Delete the file
        }

        // Delete the image record from the database
        $image->delete();

        // Return success response
        return response()->json(['success' => true]);
    }
}
