<?php

namespace App\Http\Controllers\Traits;

trait JsonResponseTrait
{

    public function success($data, $message = '', $statusCode = 200)
    {
        $data = $this->prepareData($data, $message);
        if (empty($data)) {
            return $this->notFound("Record Not Found", 404);
        }
        return response()->json([
            'data' => $data,
            'message' => $message ? $message : "success",
            'statusCode' => $statusCode, // Add the status key here
        ], $statusCode);
    }


    public function errorWithData($data, $message = '', $statusCode = 500)
    {
        $data = $this->prepareData($data, $message);
        if (empty($data)) {
            return $this->notFound("Record Not Found", 404);
        }
        return response()->json([
            'data' => $data,
            'message' => $message ? $message : "success",
            'statusCode' => $statusCode, // Add the status key here
        ], $statusCode);
    }
    public function withToken($data, $token = '', $message = '', $statusCode = 200, $permissions = [], $settings = []) 
    {
        $data = $this->prepareData($data, $message);
        if (empty($data)) {
            return $this->notFound("Record Not Found", 404);
        }
        return response()->json([
            'data' => $data,
            'token' => $token,
            'message' => $message ? $message : "success",
            'statusCode' => $statusCode,
            'permissions' => $permissions,
            'settings' => $settings // ← THIS MUST BE HERE
        ], $statusCode);
    }
    public function withCompanyToken($data, $token = '',$admin = '', $message = '', $statusCode = 200)
    {
        $data = $this->prepareData($data, $message);
        if (empty($data)) {
            return $this->notFound("Record Not Found", 404);
        }
        return response()->json([
            'data' => $data,
            'token' => $token,
            'admin' => $admin,
            'message' => $message ? $message : "success",
            'statusCode' => $statusCode, // Add the status key here
        ], $statusCode);
    }

    public function withCompanyTokenWithSubscription($data, $token = '',$admin = '',  $subscription = '', $permissions = '', $settings = '', $message = '', $statusCode = 200)
    {
        $data = $this->prepareData($data, $message);
        if (empty($data)) {
            return $this->notFound("Record Not Found", 404);
        }
        return response()->json([
            'data' => $data,
            'token' => $token,
            'admin' => $admin,
            'subscription' => $subscription ?? null,
            'permissions' => $permissions ?? null,
            'settings' => $settings ?? null,
            'message' => $message ? $message : "success",
            'statusCode' => $statusCode, // Add the status key here
        ], $statusCode);
    }

    public function WithFilter($data, $filter = null, $message = '', $statusCode = 200 )
    {
        $data = $this->prepareData($data, $message);
        if (empty($data)) {
            return $this->notFound("Record Not Found", 404);
        }
        $count = count($data);       
        return response()->json([
            'data' => $data,
            'filter' => $filter,
            'message' => $message ? $message : "success",
            'totalCount' => $count,
            'statusCode' => $statusCode, // Add the status key here
        ], $statusCode);
    }

    public function withCount($data, $message = '', $statusCode = 200 )
    {
        $data = $this->prepareData($data, $message);
        if (empty($data)) {
            return $this->notFound("Record Not Found", 404);
        }
        // Local defensive check - only sort if data structure is valid for sorting
        $canSort = true;
        if (!empty($data)) {
            // Check if first item has the required structure for sorting
            $firstItem = reset($data);
            if (!is_array($firstItem) && !is_object($firstItem)) {
                $canSort = false;
            } elseif (is_array($firstItem) && !isset($firstItem['created_at'])) {
                $canSort = false;
            } elseif (is_object($firstItem) && !property_exists($firstItem, 'created_at') && !isset($firstItem->created_at)) {
                $canSort = false;
            }
        }
        
        // Only sort if the data structure supports it
        if ($canSort) {
            usort($data, function ($a, $b) {
                // Handle both array and object access safely
                $aTime = null;
                $bTime = null;
                
                if (is_array($a) && isset($a['created_at'])) {
                    $aTime = $a['created_at'];
                } elseif (is_object($a) && isset($a->created_at)) {
                    $aTime = $a->created_at;
                }
                
                if (is_array($b) && isset($b['created_at'])) {
                    $bTime = $b['created_at'];
                } elseif (is_object($b) && isset($b->created_at)) {
                    $bTime = $b->created_at;
                }
                
                // If either timestamp is missing, don't change order
                if (!$aTime || !$bTime) {
                    return 0;
                }
                
                return strtotime($bTime) - strtotime($aTime);
            });
        }
        
        $count = count($data);
        return response()->json([
            'data' => $data,
            'message' => $message ? $message : "success",
            'totalCount' => $count,
            'statusCode' => $statusCode,
        ], $statusCode);
    }
    

    public function error( $message = '', $statusCode = 500)
    {
        return response()->json([
            'message' => $message ? $message : "something went wrong",
            'statusCode' => $statusCode, // Add the status key here
        ], $statusCode);
    }

    public function notFound( $message = '', $statusCode = 404)
    {
        return response()->json([
            'message' => $message ? $message : 'Record Not Found',
            'statusCode' => $statusCode, // Add the status key here
        ], $statusCode);
    }
    public function unauth( $message = '', $statusCode = 401)
    {
        return response()->json([
            'message' => $message ? $message : 'Record Not Found',
            'statusCode' => $statusCode, // Add the status key here
        ], $statusCode);
    }
    public function message( $message = '', $statusCode = 200)
    {
        return response()->json([
            'message' => $message ? $message : 'Record Not Found',
            'statusCode' => $statusCode, // Add the status key here
        ], $statusCode);
    }

    private function prepareData($data, $message)
    {
        if ($data instanceof \Illuminate\Database\Eloquent\Model) {
            $data = $data->toArray();
        } elseif ($data instanceof \Illuminate\Database\Eloquent\Builder) {
            $data = $data->get()->toArray();
        } elseif ($data instanceof \Illuminate\Support\Collection) {
            if ($data->isEmpty()) {
                return null;
            }
            $data = $data->toArray();
        } elseif ($data instanceof \Illuminate\Database\Eloquent\Relations\HasMany) {
            $data = $data->get()->toArray();
        }
        return empty($data) ? null : $data;
    }

}
