<?php
namespace App\Http\Controllers\Traits;
use Illuminate\Http\Request;
use Illuminate\Validation\Rule;
use Illuminate\Support\Facades\Validator;

trait ValidationTrait{

    protected function adminLoginValidationRequest(Request $request){
        return Validator::make($request->all(), [
            'email' => 'nullable|email', // For User table
            'password' => 'required|string',
        ]);
    }

    protected function subscriptionPlanValidationRequest(Request $request){
        return Validator::make($request->all(), [
            'name' => 'nullable|string|max:255',
            'number_of_user' => 'nullable|integer',
            'custom_plan' => 'nullable|string',
            'active' => 'nullable|integer',
            'is_free_plan' => 'nullable|integer',
            'status' => 'nullable|integer',
            'package_price_monthly' => 'nullable|integer',
            'package_price_yearly' => 'nullable|integer',
            'price_per_user_monthly' => 'nullable|integer',
            'price_per_user_yearly' => 'nullable|integer',
            'price_per_workspace_monthly' => 'nullable|integer',
            'price_per_workspace_yearly' => 'nullable|integer',
            'module_id' => 'nullable|string',
            'number_of_workspace' => 'nullable|integer',
        ]);

    }

     protected function couponValidationRequest(Request $request, $isUpdate = false)
    {
        $rules = [
            'name' => 'nullable|string|max:255',
            'code' => 'nullable|string|max:255', // No need for unique check when updating
            'discount' => 'nullable|integer|min:0|max:100',
            'limit' => 'nullable|integer|min:0',
            'type' => 'nullable|string|in:percentage,flat,fixed_plan_wise',
            'minimum_spend' => 'nullable|integer|min:0',
            'maximum_spend' => 'nullable|integer|min:0',
            'limit_per_user' => 'nullable|integer|min:1',
            'expiry_date' => 'nullable|date|after:today',
            'included_plan' => 'nullable|array',
            'included_plan.*' => 'nullable|integer|exists:plans,id',
            'excluded_plan' => 'nullable|array',
            'description' => 'nullable|string|max:500',
            'is_active' => 'nullable|boolean',
        ];
    
        if (!$isUpdate) {
            // Apply unique validation for 'code' only when creating a coupon
            $rules['code'] = 'nullable|string|unique:coupons,code|max:255';
        }
    
        return Validator::make($request->all(), $rules);
    }
    
    
    

    protected function moduleValidationRequest(Request $request){
        return Validator::make($request->all(), [
            'name' => 'required|string|max:255',
            'status' => 'nullable|integer',
            'image' => 'nullable|image|mimes:jpeg,png,jpg,gif|max:2048', 
        ]);
    }

    protected function subModuleValidationRequest(Request $request){
        return Validator::make($request->all(), [
            'name' => 'required|string|max:255',
            'module_id'=>'required|integer',
            'status' => 'nullable|integer',
            'image' => 'nullable|image|mimes:jpeg,png,jpg,gif|max:2048', 
        ]);
    }
    private function customerSignupValidation(Request $request)
    {
        return Validator::make($request->all(), [
            'name' => 'required|string|max:255',
            'email' => 'required|string|email|max:255|unique:users,email',
            'password' => 'required|string|min:8|confirmed',
            'workspace_name' => 'required|string|max:255',
            'mobile_number' => 'nullable|string',
            'terms_accepted' => 'required|boolean|accepted',
        ]);
    }
    protected function customerValidationRequest(Request $request)
    {
        return Validator::make($request->all(), [
            'workspace_name' => 'required|string|max:255',
            'enable_domain' => 'nullable|string|max:255',
            'domain_type' => 'nullable|string|max:255',
            'domain' => 'nullable|string|max:255',
            'subdomain' => 'nullable|string|max:255',
            'status' => 'nullable|integer',
            'slug' => 'nullable|string|max:255',
            'is_disable' => 'nullable|integer',
    
            'customer_name' => 'required|string|max:255',
            'email' => ['required','email', Rule::unique('emp_company_details', 'employee_email') ->ignore($request->emp_id, 'id'), Rule::unique('users', 'email')->ignore($request->emp_id, 'id'),  ],
            'mobile_number' => 'nullable|string|max:20',
            'password' => 'nullable|string|min:6',
            'is_enable_login' => 'nullable|boolean',
            'user_type' => 'nullable|integer',
            'active_status' => 'nullable|integer',
            'avatar' => 'nullable|image|mimes:jpeg,png,jpg,gif|max:2048',
            'active_plan_id' => 'nullable|integer',
            'plan_expire_date' => 'nullable|date', 
        ]);
    }

    protected function adminResetPasswordValidationRequest(Request $request){
        return Validator::make($request->all(), [
            'password' => 'required|string|min:6',
            'password_confirmation' => 'required|same:password',
        ]);
    }

    protected function loginToggleValidationRequest(Request $request)
    {
        return Validator::make($request->all(), [
            'customer_id' => 'required|integer|max:255',
            'is_enable_login' => 'required|boolean', 
        ]);
    }
    protected function companyLoginValidationRequest(Request $request)
    {
        return Validator::make($request->all(), [
            'customer_id' => 'required|integer|max:255', 
        ]);
    }

    protected function AssignPlanValidationRequest(Request $request)
    {
        return Validator::make($request->all(), [
            'customer_id' => 'required|exists:users,id',
            'plan_id' => 'required|exists:plans,id',
            'price' => 'required|numeric',
            'expiration_date' => 'nullable|date|after:today', 
        ]);
    }


    protected function helpdeskCategoryValidationRequest(Request $request)
    {
        return Validator::make($request->all(), [
           'name' => 'required|string|max:255',
           'color' => 'required|string|max:255', 
        ]);
    }
    protected function adminCustomerEditValidationRequest(Request $request){

        return Validator::make($request->all(), [
            'name' => 'nullable|string|min:3',
            'email' => 'nullable|email',  // Add email validation

        ]);
    }

    protected function customDomainValidation(Request $request){
        return Validator::make($request->all(), [
            'domain' => 'required',
        ]);
    }


    protected function helpdeskValidationRequest(Request $request)
    {
        return Validator::make($request->all(), [
            'user_id' => 'required|integer|max:255',
            'category_id' => 'required|integer|max:255',
            'email' => 'required|string|max:255', 
            'subject' => 'required|string|max:255',
            'descriptions' => 'required|string|max:255',
            'status' => 'nullable|string|max:255', // Accepts null, default handled below
            'image' => 'nullable|image|mimes:jpeg,png,jpg,gif|max:2048',
        ]);
    }

    

    protected function handleValidationFailure($validator)
    {
        $errors = $validator->errors()->toArray();
        $message = reset($errors)[0];
        return $this->error($message, 422);
    }

    protected function customDomainStatusValidation(Request $request)
    {
        return Validator::make($request->all(), [
            'id' => 'required|integer|exists:custom_domain_requests,id',
            'status' => 'required|integer|in:0,1,2', // Adjust the 'in' values based on your status codes
        ]);
    }

    protected function adminGeneralSettingValidationRequest(Request $request)
    {
        return Validator::make($request->all(), [
            'settings' => 'required|array',
            'settings.*.key' => 'required|string',
            'settings.*.value' => 'nullable', // We'll validate the type in a custom way later
            'workspace' => 'nullable|integer',
            'created_by' => 'nullable|integer',
        ]);
    }
    

    protected function replyValidationRequest(Request $request){

        return Validator::make($request->all(), [
            'ticket_id' => 'required|integer|max:255',
            'description' => 'required|string|max:255',
            'attachments' => 'nullable|image|mimes:jpeg,png,jpg,gif|max:2048',  
        ]);
    }

    protected function customerSubscriptionRequestValidationRequest(Request $request){

        return Validator::make($request->all(), [
            'plan_id' => 'required|string|max:255',
            'name' => 'nullable|string|max:255',
            'email' => 'nullable|string|max:255',
            'months' => 'nullable|string|max:255',
            'plan_name' => 'nullable|string|max:255',
            'price' => 'nullable|string|max:255',
            'price_curency' => 'nullable|string|max:255',
            'payment_status' => 'nullable|string|max:255',
            'txn_id' => 'nullable|string|max:255',
            'receipt' => 'nullable|image|mimes:jpeg,png,jpg,gif|max:2048',
            'user_id' => 'nullable|string|max:255',
            'is_refund' => 'nullable|string|max:255',
            'payment_type' => 'nullable|integer',
            'price' => 'nullable|string|max:255',
            'price_currency' => 'nullable|string|max:255',
            'type' => 'nullable|string|max:255',
            'bank_account_id' => 'nullable|string|max:255',
            'request' => 'nullable|string|max:255',
            'is_recurring' => 'nullable',
            'attachments' => 'nullable|image|mimes:jpeg,png,jpg,gif|max:2048',  
            'interval' => 'nullable|integer',
        ]);


    }

    protected function BankTransferRequestStatusValidationRequest(Request $request){
        return Validator::make($request->all(), [
            'bank_id' => 'required|integer|max:255',
            'status' => 'required|integer|max:255',
            'customer_id' => 'required|integer|max:255',
        ]);
    }

    protected function BankTransferRequestDeleteValidationRequest(Request $request){

        return Validator::make($request->all(), [
            'id' => 'required|integer|max:255',
          
            
        ]);
    }

    protected function companyStoreValidationRequest(Request $request){
        return Validator::make($request->all(), [
            'first_name' => 'required',
            'middle_name' => 'nullable',
            'last_name' => 'required',
            'mobile' => 'required',
            'phone' => 'required',
            'email' => 'required|email',
            'logo_img' => 'nullable|image|mimes:jpeg,png,jpg,gif,svg'
        ]);
    }

    protected function bulkScheduleRoasterCreateValidationRequest(Request $request){
        return Validator::make($request->all(), [
            'start_date' => 'required',
            'end_date' => 'required|after_or_equal:start_date',
            'start_time' => 'required',
            'end_time' => 'required|after:start_time',
            'color_code' => 'required',
            'break_minutes' => 'required',
            'shift_notes' => 'required',
            'users_ids' => ['required', 'array', 'min:1'],
            'working_days_hidden' => 'required'
        ],[

            'end_time.after' => 'End time must be after start time',
            'users_ids.required' => 'Please select atleast one employee',
            'working_days_hidden.required' => 'Please select atleast one day'
        ]); 
    }


    protected function customerProfileValidationRequest(Request $request)
    {
        $rules = [
            'name' => 'nullable|string|max:255',
            'email' => 'nullable|email|max:255',
            'mobile_number' => 'nullable|string|max:20',
            'avatar' => 'nullable|image|mimes:jpeg,png,jpg,gif|max:2048',
        ];
        
        // Add password validation rules if password fields are present
        if ($request->filled('old_password') || $request->filled('new_password') || $request->filled('new_password_confirmation')) {
            $rules = array_merge($rules, [
                'old_password' => 'required|string|min:6',
                'new_password' => 'required|string|min:6|confirmed',
                'new_password_confirmation' => 'required|string|min:6',
            ]);
        }
        
        return Validator::make($request->all(), $rules);
    }
   
}