<?php

namespace App\Models;

use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Database\Eloquent\Model;

class Order extends Model
{
    use HasFactory;
    protected $fillable = [
        'name',
        'email',
        'order_id',
        'card_number',
        'card_exp_month',
        'card_exp_year',
        'plan_name',
        'plan_id',
        'is_recurring',
        'price',
        'original_price',
        'price_curency',
        'payment_status',
        'payment_type',
        'months',
        'workspace',
        'receipt',
        'txn_id',
        'stripe_subscription_id',
        'stripe_customer_id',
        'stripe_price_id',
        'user_id',
        'request',
        'is_refund',
        'coupon_code',
        'discount_amount',
        'interval'
    ];


    protected $with = ['plan'];

    /**
     * Get the plan associated with the order.
     */
    public function plan()
    {
        return $this->belongsTo(Plan::class, 'plan_id', 'id');
    }

    /**
     * Get the subscription associated with the order.
     */
    public function subscription()
    {
        return $this->hasOne(Subscription::class);
    }

    /**
     * Get the user associated with the order.
     */
    public function user()
    {
        return $this->belongsTo(User::class);
    }

    /**
     * Get the coupon associated with the order.
     */
    public function coupon()
    {
        return $this->belongsTo(Coupon::class);
    }

    /**
     * Determine if the order has a subscription.
     */
    public function hasSubscription()
    {
        return !is_null($this->stripe_subscription_id);
    }

    /**
     * Determine if the order is recurring.
     */
    public function isRecurring()
    {
        return (bool) $this->is_recurring;
    }
}
