<?php

namespace App\Models;

use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Database\Eloquent\Model;

class Swms extends Model
{
    use HasFactory;
    protected $table = 'swms'; // Explicit table name

    protected $fillable = [
        'customer_id',
        'workspace_id',
        'swms_number',
        'title',
        'description',
        'revision_number',
        'version_number',
        'start_date',
        'company_name',
        'company_address',
        'contact_number',
        'scope_of_work',
        'site_manager_or_forman',
        'principal_contractor',
        'principal_contractor_address',
        'principal_contractor_number',
        'fast_address',
        'site_id',
        'parent_id',
        'del',
        'process',
        'published'
    ];


    protected $hidden = [
        'customer_id',
        'workspace_id',

    ];

    public function parent()
    {
        return $this->belongsTo(Swms::class, 'parent_id');
    }
    public function authorizedBy()
    {
        return $this->belongsTo(EmpPersonalDetails::class, 'site_manager_or_forman', 'emp_id');
    }
    public function monitoringResponsibles()
    {
        return $this->hasMany(SwmsMonitoringResponsible::class, 'swms_id');
    }

    public function complianceResponsibles()
    {
        return $this->hasMany(SwmsComplianceResponsible::class, 'swms_id');
    }

    public function reviewResponsibles()
    {
        return $this->hasMany(SwmsReviewResponsible::class, 'swms_id');
    }

    public function developedBies()
    {
        return $this->hasMany(SwmsDevelopedBy::class, 'swms_id');
    }

    public function approvedBies()
    {
        return $this->hasMany(SwmsApprovedBy::class, 'swms_id');
    }

    public function signatures()
    {
        return $this->hasMany(SwmsSignature::class, 'swms_id');
    }

    public function siteData()
    {
        return $this->belongsTo(Sites::class, 'site_id', 'id');
    }
}
