@php
  use App\General\SettingsClass;
  use Illuminate\Support\Facades\DB;
  use Illuminate\Support\Facades\Route;
  use Illuminate\Support\Facades\Config;
  use App\Models\User;
  

  

  $logoPath = null;
  $companyName = env('APP_NAME', 'WMS');
  $companyAddress = '';
  $companyCity = '';
  $companyState = '';
  $companyPostCode = '';
  $companyCountry = '';
  $companyPhone = 'Contact Information';
  $companyEmail = env('EMAIL_ADDRESS', 'support@example.com');
  
  // Get admin settings data
    $admin = User::where('user_type', config('constants.user_types.admin'))->first();
    if ($admin) {
      $settings = DB::table('adminsettings')
          ->where('customer_id', $admin->id)
          ->get();
          
      // Transform into key-value pairs for easier access
      $settingsData = [];
      foreach ($settings as $setting) {
          $settingsData[$setting->key] = $setting->value;
      }
      
      // Get logo
      if (isset($settingsData['brand_logo_dark'])) {
          $logoPath = $settingsData['brand_logo_dark'];
      }
      
      // Get company information
      if (isset($settingsData['company_company_name'])) {
          $companyName = $settingsData['company_company_name'];
      }
      if (isset($settingsData['company_company_address'])) {
          $companyAddress = $settingsData['company_company_address'];
      }
      
      if (isset($settingsData['company_company_city'])) {
          $companyCity = $settingsData['company_company_city'];
      }
      
      if (isset($settingsData['company_company_state'])) {
          $companyState = $settingsData['company_company_state'];
      }
      
      if (isset($settingsData['company_company_post_code'])) {
          $companyPostCode = $settingsData['company_company_post_code'];
      }
      
      if (isset($settingsData['company_company_country'])) {
          $companyCountry = $settingsData['company_company_country'];
      }
      
      if (isset($settingsData['company_company_telephone'])) {
          $companyPhone = $settingsData['company_company_telephone'];
      }
      
      if (isset($settingsData['company_company_email'])) {
          $companyEmail = $settingsData['company_company_email'];
      }
    }
  
  
  // Format full address
  $fullAddress = $companyAddress;
  if (!empty($companyCity)) {
      $fullAddress .= ', ' . $companyCity;
  }
  if (!empty($companyState)) {
      $fullAddress .= ', ' . $companyState;
  }
  if (!empty($companyPostCode)) {
      $fullAddress .= ' ' . $companyPostCode;
  }
  if (!empty($companyCountry)) {
      $fullAddress .= ', ' . $companyCountry;
  }
  // Helper function to determine if a date is valid (not 1970-01-01)
  function isValidDate($date) {
    if (empty($date)) return false;
    if (strpos($date, '1970-01-01') !== false) return false;
    return true;
  }
  
  // Format the next billing date with a fallback
  $formattedNextBillingDate = isValidDate($next_billing_date) ? 
    $next_billing_date : 
    date('Y-m-d', strtotime('+' . $interval . ' months'));
    

@endphp
<!DOCTYPE html>
<html lang="en">
<head>
  <meta charset="UTF-8">
  <meta name="viewport" content="width=device-width, initial-scale=1.0">
  <title>{{$subject}}</title>
  <link rel="preconnect" href="https://fonts.googleapis.com">
<link rel="preconnect" href="https://fonts.gstatic.com" crossorigin>
<link href="https://fonts.googleapis.com/css2?family=Inter:wght@100;200;300;400;500;600;700;800;900&display=swap" rel="stylesheet">
  <link rel="stylesheet" href="css/emails_style.css">
<style>
  *{
  padding: 0;
  margin: 0;
  font-family: Inter !important;
}
#email-body{
  min-width: 600px;
  width: 600px;
  max-width: 600px;
  margin: 0 auto;
  font-family: Inter !important;

}
#email-header{
  padding: 18px 25px;
  background-color: #22C55E;
}
#email-content{
  padding: 42px 34px;
}
.greeting-section{
  font-size: 14px;
  margin-bottom: 18px;
}
#tagline-section{
  background-color:#EDEDED;
  padding: 14px;
  text-align: center;
}
.small-tagline{
  font-size: 13px;
}
.big-tagline{
  font-size: 14px;
}
.big-tagline a{
  color: #019FF3;
  font-weight: 700;
  text-decoration: none;
}
.footer-logo{
  margin-top: 16px;
  height: 35px;
  filter: invert(65%);
}
.company-title{
  font-size: 14px;
  font-weight: 600;
}
.company-address-contact{
  color: gray;
  font-size: 12px;
}
.email-content-area p{
  font-size: 14px;
  margin-bottom: 15px;
}
.thank-you-section{
  margin-bottom: 0px !important;
}
.subscription-title{
  font-size: 22px !important;
  font-weight: 700;
  margin-bottom: 20px;
}
.subscription-details{
  background-color: #f9f9f9;
  padding: 15px;
  border-radius: 5px;
  margin-bottom: 20px;
}
.subscription-details p{
  margin-bottom: 8px;
}
.subscription-details strong{
  font-weight: 600;
}
</style>
</head>
<body>
  <div id="email-body">
    <div id="email-header">
      @if(!empty($logoPath) && file_exists(public_path($logoPath)))
        <img src="{{ url($logoPath) }}" height="54px" width="auto" alt="Logo">
      @else
        <h2 style="color: white; margin: 0;">{{ $companyName }}</h2>
      @endif
    </div>
    <div id="email-content">
      <div class="greeting-section">Hi, <b>{{$name}}</b></div>
      <div class="email-content-area">
        <p class="subscription-title">Your Subscription is Active!</p>
        <div class="subscription-details">
          <p><strong>Plan:</strong> {{$plan_name}}</p>
          <p><strong>Order ID:</strong> {{$order_id}}</p>
          <p><strong>Amount:</strong> {{$price}} {{$currency}}</p>
          <p><strong>Subscription Period:</strong> {{$interval}} Month(s)</p>
          <p><strong>Next Billing Date:</strong> {{$formattedNextBillingDate}}</p>
        </div>
        <p>Thank you for subscribing to our service. Your subscription is now active and you can start using all the premium features right away.</p>
        <p>If you have any questions about your subscription, please contact our customer support team.</p>
        <p class="thank-you-section">Thank You</p>
      </div>
    </div>
    <div id="email-footer">
      <div id="tagline-section">
        <p class="small-tagline">This email is auto generated, don't reply on this Email.</p>
        <p class="big-tagline">For any question and query, consider
           {{-- <a href="mailto:{{getEnv('EMAIL_ADDRESS', 'support@example.com')}}">{{getEnv('EMAIL_ADDRESS', 'support@example.com')}}</a> --}}
          </p>
      </div>
      <div class="company-details">
        @if(!empty($logoPath) && file_exists(public_path($logoPath)))
          <img src="{{ url($logoPath) }}" class="footer-logo" alt="Logo">
        @endif
        <div class="company-title">{{ $companyName }}</div>
        <div class="company-address-contact">
        {{ $fullAddress }}<br>
        Call : {{ $companyPhone }}
        </div>
      </div>
    </div>
  </div>
</body>
</html> 